#include <linux/device.h>
#include <linux/pci.h>
#include <linux/usb.h>
#include <linux/tty.h>
#include <linux/platform_device.h>
#include <linux/cred.h>
#include <linux/fs.h>
#include <linux/kthread.h>
#include <linux/netdevice.h>
#include <linux/sysfs.h>
#include <linux/kobject.h>
#include <linux/slab.h>
#include <linux/proc_fs.h>

#include <linux/sm_devctl.h>

#define NETNAME_SIZE            32
DEFINE_RWLOCK(devt_rw_lock);
DEFINE_MUTEX(devctl_mutex);
DEFINE_MUTEX(devctl_lst_mutex);
static char devctl_policy_buf[PAGE_SIZE * 2];
#define AQTJ_DIR "nfs_aqtj"
#define AQTJ_DEVCTL "devctl_policy"
static struct proc_dir_entry *devctl_policy_file;
static struct proc_dir_entry *nfs_aqtj_dir;

enum perm_stat {
	PERM_CONFIGURED,
	PERM_UNCONFIGURED
};

struct class_addr {
	struct list_head 	list;
	struct class *		cls;
};

struct sm_devctl {
	int 	inf_type; 	// 见上面的接口枚举
	int 	dev_type; 	// 见上面的设备枚举
	int 	usb_type; 	// 见上面的 usb 设备枚举
	u32 	vid; 		// 指定设备的 vid
	u32 	pid; 		// 指定设备的 pid
	u32 	major; 		// 指定设备的主设备号
	int 	perm;
    	char    netcards[NETNAME_SIZE]; // 网卡名字
};

struct sm_devctl_vidpid {
	int 	usb_class; 	// 见上面的 usb 设备枚举
	u32 	vid; 		// 指定设备的 vid
	u32 	pid; 		// 指定设备的 pid
	int 	perm;
};

struct sm_devctl_vm {
	char 	vm_path[VM_PATH_LEN];
	int	perm;
};

struct sm_pkg_protect {
	char	pkg_name[PKG_NAME_LEN];
	int	perm;
};

/* tempory for file protect */
struct sm_file_protect {
	char	process_name[TASK_COMM_LEN];
	int	perm;
};

/* dev control */
struct sm_devctl_internal {
	struct list_head 	list;
	struct sm_devctl 	devctl;
	struct device *		device; 
	struct device_driver *	driver; 
};

/* dev control - by vid pid */
struct sm_devctl_vidpid_internal {
	struct list_head 	list;
	struct sm_devctl_vidpid devctl;
	struct device *		device; 
	struct device_driver *	driver; 
};

/* dev control - by major, minor */
struct sm_devctl_devt_internal {
	struct 	list_head 	list;
	struct 	sm_devctl 	devctl;
};

/* virtual machine control */
struct sm_devctl_vm_internal {
	struct 	list_head 	list;
	struct 	sm_devctl_vm 	devctl;
};

/* package unintall protect */
struct sm_pkg_protect_internal {
	struct 	list_head 	list;
	struct 	sm_pkg_protect 	pkg;
};

/* file access protect for tempory */
struct sm_file_protect_internal {
	struct 	list_head 	list;
	struct 	sm_file_protect file;
};

struct devctl_info {
	int type;
	char *type_name;
};

static const struct devctl_info inf_info[] = {
	{SM_INF_SERIAL, 	"INF_SERIAL"},
	{SM_INF_PARALLEL,	"INF_PARALLEL"},
	{SM_INF_1394,		"INF_1394"},
	{SM_INF_PCMCIA,		"INF_PCMCIA"},
	{SM_INF_TYPEC,		"INF_TYPEC"},
	{SM_INF_HDMI,		"INF_HDMI"},
	{SM_INF_DISPLAYPORT,	"INF_DISPLAYPORT"},
	{-1,			"INF_UNKNOW"}
};

static const struct devctl_info dev_info[] = {
	{SM_DEV_BLUETOOTH,	"DEV_BLUETOOTH"},
	{SM_DEV_INFRARED,	"DEV_INFRARED"},
	{SM_DEV_MODEM,		"DEV_MODEM"},
	{SM_DEV_STORAGE_DRIVE,	"DEV_STORAGE_DRIVE"},
	{SM_DEV_SCSI_RAID,	"DEV_SCSI_RAID"},
	{SM_DEV_WINCE,		"DEV_WINCE"},
	{SM_DEV_FLOPPY_DRIVE,	"DEV_FLOPPY_DRIVE"},
	{SM_DEV_IMAGE,		"DEV_IMAGE"},
	{SM_DEV_MULTI_SERIAL_CARD,	"DEV_MULTI_SERIAL_CARD"},
	{SM_DEV_ETHERNET,	"DEV_ETHERNET"},
	{SM_DEV_WIRELESS,	"DEV_WIRELESS"},
	{SM_DEV_PRINTER,		"DEV_PRINTER"},
	{SM_DEV_PRINTER_LOCAL,	"DEV_PRINTER_LOCAL"},
	{SM_DEV_PRINTER_NET,	"DEV_PRINTER_NET"},
	{SM_DEV_CDROM,		"DEV_CDROM"},
	{DEV_VOLUMESHADOW,	"DEV_VOLUMESHADOW"},
	{DEV_TAPE,		"DEV_TAPE"},
	{-1,			"DEV_UNKNOW"}
};

static const struct devctl_info usb_info[] = {
	{SM_USB_CLASS_PER_INTERFACE,	"USB_PER_INTERFACE"},
	{SM_USB_CLASS_AUDIO,		"USB_AUDIO"},
	{SM_USB_CLASS_COMM,		"USB_COMM"},
	{SM_USB_CLASS_HID,		"USB_HID"},
	{SM_USB_CLASS_PHYSICAL,		"USB_PHYSICAL"},
	{SM_USB_CLASS_STILL_IMAGE,	"USB_STILL_IMAGE"},
	{SM_USB_CLASS_PRINTER,		"USB_PRINTER"},
	{SM_USB_CLASS_MASS_STORAGE,	"USB_MASS_STORAGE"},
	{SM_USB_CLASS_HUB,		"USB_HUB"},
	{SM_USB_CLASS_CDC_DATA,		"USB_CDC_DATA"},
	{SM_USB_CLASS_CSCID,		"USB_CSCID"},
	{SM_USB_CLASS_CONTENT_SEC,	"USB_CONTENT_SEC"},
	{SM_USB_CLASS_VIDEO,		"USB_VIDEO"},
	{SM_USB_CLASS_WIRELESS_CONTROLLER,	"USB_WIRELESS_CONTROLLER"},
	{SM_USB_CLASS_MISC,		"USB_MISC"},
	{SM_USB_CLASS_APP_SPEC,		"USB_APP_SPEC"},
	{SM_USB_CLASS_VENDOR_SPEC,	"USB_VENDOR_SPEC"},
	{SM_USB_CLASS_PERSONAL_HEALTH,	"USB_PERSONAL_HEALTH"},
	{SM_USB_CLASS_DIAGNOSTIC,	"USB_DIAGNOSTIC"},
	{-1,				"USB_UNKNOW"}
};



static char cur_cfg_inf[INF_MAX] = {0};
static char cur_cfg_dev[DEV_MAX] = {0};
static char cur_cfg_usb[USB_MAX] = {0};

static LIST_HEAD(devctl_lst);
static LIST_HEAD(devctl_devt_lst);
static LIST_HEAD(devctl_vidpid_lst);
static LIST_HEAD(class_addr_lst);
static LIST_HEAD(devctl_vm_lst);
static LIST_HEAD(pkg_protect_lst);
/*file access protect for tempory */
static LIST_HEAD(file_protect_lst);
/* from net/dsa/dsa.c */
static char * devctl_info_decode(struct sm_devctl *info){
	int ix;

	if(info->inf_type > 0)
		for(ix = 0; inf_info[ix].type != -1; ix++)
			if(inf_info[ix].type == info->inf_type)
				return inf_info[ix].type_name;

	if(info->dev_type > 0)
		for(ix = 0; dev_info[ix].type != -1; ix++)
			if(dev_info[ix].type == info->dev_type)
				return dev_info[ix].type_name;

	if(info->usb_type > 0)
		for(ix = 0; usb_info[ix].type != -1; ix++)
			if(usb_info[ix].type == info->usb_type)
				return usb_info[ix].type_name;

	if(strlen(info->netcards) > 0)
		return "NETCARDS";

	return "UNKNOW_TYPE";
}


static char *perm_decode(int perm){
	switch(perm){
		case 0:
			return "disable";
		case 1:
			return "enable";
		case 5:
			return "rdonly";
		default:
			break;
	}

	return "unknow_perm";
}


static int dev_is_class(struct device *dev, void *class)
{
        if (dev->class != NULL && !strcmp(dev->class->name, class))
                return 1;

        return 0;
}

/* from net/dsa/dsa.c */
static struct device *dev_find_class(struct device *parent, char *class)
{
	if(parent == NULL)
		return NULL;

        if (dev_is_class(parent, class)) 
	{
                get_device(parent);
                return parent;
        }

        return device_find_child(parent, class, dev_is_class);
}

static u8 usb_enum_to_class(enum usb_subtype usb_type)
{
	static u8 usb_array[USB_MAX] = {0x00, 0x00, 0x01,
	       		0x02, 0x03, 0x05, 0x06, 0x07,
			0x08, 0x09, 0x0a, 0x0b, 0x0d,
			0x0e, 0xe0, 0xef, 0xfe, 0xff,
			0x0f, 0xdc};
	return usb_array[usb_type];	
}

static int usb_class_to_enum(u8 usb_class)
{
	static u8 usb_array[USB_MAX] = {0x00, 0x00, 0x01,
	       		0x02, 0x03, 0x05, 0x06, 0x07,
			0x08, 0x09, 0x0a, 0x0b, 0x0d,
			0x0e, 0xe0, 0xef, 0xfe, 0xff,
			0x0f, 0xdc};

	int i = 1;

	for(; i < sizeof(usb_array); i++)
	{
		if(usb_array[i] == usb_class)
			return i;
	}
	return -1;	
}

static struct class * class_addr_get(const char *cls_name)
{
	struct list_head *pos = NULL;
	struct class_addr *tmp = NULL;
	
	list_for_each(pos, &class_addr_lst)
	{
		tmp = list_entry(pos, struct class_addr, 
				list);
		
		/* found it */
		if(strcmp(tmp->cls->name, cls_name) == 0)
			return tmp->cls;
	}
	return NULL;
}

static void print_class(void)
{
	struct list_head *pos = NULL;
	struct class_addr *tmp = NULL;
	
	list_for_each(pos, &class_addr_lst)
	{
		tmp = list_entry(pos, struct class_addr, 
				list);
	}
}

static int perm_match_class(struct device *dev, 
		void * data)
{
	struct sm_devctl *devctl = (struct sm_devctl *)data;
	struct usb_interface *intf = NULL;
	struct usb_interface_descriptor *desc = NULL;
 	if (devctl == NULL || dev == NULL) {
		return 0;
	}	
	unsigned char usb_class = usb_enum_to_class(
			devctl->usb_type);
	int result = strcmp(dev->type->name, "usb_endpoint");
        if (result == 0)
                return 0;
    	if((intf = to_usb_interface(dev)) == NULL)
		return 0;

    	if (intf->cur_altsetting == NULL)
		return 0;
        	
	desc = &intf->cur_altsetting->desc;
        if (desc == NULL)
		return 0;
        if (desc->bInterfaceClass == usb_class)
                return 1;
 	return 0;
}
 
static int perm_match_class1(struct device *dev, 
		void * data)
{
	return perm_match_class(dev, data);
}
 
static int perm_match_class2(struct device *dev, 
		void *class)
{
        if (dev_is_class(dev, (void *)class))
		return 1;	

        return device_find_child(dev, (void *)class, 
			dev_is_class) != NULL ? 1 : 0;
}

static int perm_match_vidpid(struct device *dev, 
		void * data)
{
	struct sm_devctl_vidpid *devctl = 
			(struct sm_devctl_vidpid *)data;
	
	if((to_usb_device(dev)->descriptor.idVendor == 
			devctl->vid) && (to_usb_device(dev)->
			descriptor.idProduct == devctl->pid))
		return 1;

	return 0;
}

static int device_match_major(struct device *dev, 
	const void *major)
{
	return MAJOR(dev->devt) == *(dev_t *)major;
}

/* for serial port -- ugly but use tempory */
static int device_match_minor(struct device *dev, 
	const void *minor)
{	
	return MAJOR(dev->devt) == 4 && MINOR(
			dev->devt) > *(int *)minor;
}

static int device_match_devname(struct device *dev, 
	const void *devname)
{	
	return strstr(dev_name(dev), devname) != NULL ?
		1 : 0;
}

static int device_match_drv(struct device *dev, 
		void *drv_name)
{
	if(dev->driver == NULL)
		return 0;

	if(dev->driver->name == NULL)
		return 0;

	return strcmp(dev->driver->name, drv_name) == 0 ?
		1 : 0;
}

static int device_match_intf(struct device *dev, 
		void * data)
{
	if(to_usb_device(dev)->descriptor.idVendor == 
			0xffe0 && to_usb_device(dev)->
			descriptor.idProduct == 0xffff)
		return 1;
 	return 0;
}

static int device_match_mtp(struct device *dev, 
		void * data)
{
	/* 手机有只充电模式，这时actconfig是NULL，
	 * 无需进一步配置 */
	if(to_usb_device(dev)->actconfig == NULL)
		return 0;

	/* ptp/mtp protocol不能禁止，否则会死机 
	 * 应用层处理这个问题 */
	if(to_usb_device(dev)->actconfig->desc.
			iConfiguration == 4)
		return 1;
 	return 0;
}

static inline struct device *bus_find_device_by_class(
		struct device *start, struct sm_devctl 
		*devctl)
{
	return bus_find_device(&usb_bus_type, start, devctl, 
			perm_match_class);
}	

static inline struct device *usb_find_class(struct device *parent, 
		void *data)
{
	return device_find_child(parent, data, perm_match_class1);
}

static inline struct device *bus_find_device_by_vidpid(
		struct device *start, struct 
		sm_devctl_vidpid *devctl)
{
	return bus_find_device(&usb_bus_type, start, devctl, 
			perm_match_vidpid);
}	

static inline struct device *bus_find_device_by_drv(
		struct device *start, 
		const char *drv_name)
{
	return bus_find_device(&usb_bus_type, start, drv_name, 
			device_match_drv);
}

static inline struct device *class_find_device_by_major(
		struct class *class, struct device * start,
		int major)
{
        return class_find_device(class, start, &major, 
			device_match_major);
}

static inline struct device *class_find_device_by_minor(
		struct class *class, struct device * start,
		int minor)
{
        return class_find_device(class, start, &minor, 
			device_match_minor);
}

/* devname match -- support wildcard */
static inline struct device *class_find_device_by_devname(
		struct class *class, struct device * start,
		char *devname)
{
        return class_find_device(class, start, devname, 
			device_match_devname);
}

static inline struct device *bus_find_net_device(
		struct bus_type *bus, struct device *start, 
		char * class)
{
	return bus_find_device(bus, start, class, 
			perm_match_class2);
}	

static inline struct device *bus_find_mtp_device(
		struct device *start)
{
	return bus_find_device(&usb_bus_type, start, 
			NULL, device_match_mtp);
}	

/* perm = 0 - disable driver 
 * perm = 1 - enable driver */
static int perm_setup(struct device *dev, struct device_driver 
		*drv, int perm)
{
	if(dev == NULL)
	{
		return 0;
	}
	
	if(perm == 0)
	{
		if(dev->class){
			printk("INFO: devname=%s device DELETE\n", (dev_name(dev))? dev_name(dev):"NULL");
			device_del(dev);
		}
		else{
			printk("INFO: devname=%s driver RELEASE\n", (dev_name(dev))? dev_name(dev):"NULL");
			device_release_driver(dev);
		}
		return 0;
	}

	if(dev->class){
		printk("INFO: devname=%s device ADD\n", (dev_name(dev))? dev_name(dev):"NULL");
		return device_add(dev);
	}
		
	printk("INFO: devname=%s driver ATTACH\n", (dev_name(dev))? dev_name(dev):"NULL");
	return device_attach(dev);
}

static struct sm_devctl_internal * lookup_by_name(
		const char *devname)
{
	struct list_head *pos = NULL;
	struct sm_devctl_internal *tmp = NULL;
	
	mutex_lock(&devctl_lst_mutex);
	list_for_each(pos, &devctl_lst)
	{
		tmp = list_entry(pos, struct sm_devctl_internal, 
				list);
		
		/* found it */
		if(strcmp(devname, dev_name(
				tmp->device)) == 0){
			mutex_unlock(&devctl_lst_mutex);
			return tmp;
		}
	}
	mutex_unlock(&devctl_lst_mutex);
	return NULL;
}

static struct sm_devctl_internal * lookup_by_vidpid1(
		struct sm_devctl *devctl)
{
	struct list_head *pos = NULL;
	struct sm_devctl_internal *tmp = NULL;
	
	if(devctl->vid == 0 || devctl->pid == 0)
		return tmp;

	mutex_lock(&devctl_lst_mutex);
	list_for_each(pos, &devctl_lst)
	{
		tmp = list_entry(pos, struct sm_devctl_internal, 
				list);
		
		/* found it */
		if(devctl->vid == tmp->devctl.vid &&
				devctl->pid == tmp->devctl.pid){
			mutex_unlock(&devctl_lst_mutex);
			return tmp;
		}
	}
	mutex_unlock(&devctl_lst_mutex);

	return NULL;
}

static struct sm_devctl_internal * lookup_by_major(
		struct sm_devctl *devctl)
{
	struct list_head *pos = NULL;
	struct sm_devctl_internal *tmp = NULL;
	
	if(devctl->major == 0)
		return tmp;

	mutex_lock(&devctl_lst_mutex);
	list_for_each(pos, &devctl_lst)
	{
		tmp = list_entry(pos, struct sm_devctl_internal, 
				list);
		
		/* found it */
		if(devctl->major == tmp->devctl.major){
			mutex_unlock(&devctl_lst_mutex);
			return tmp;
		}
	}
	mutex_unlock(&devctl_lst_mutex);

	return NULL;
}

static int lookup_chrblk(dev_t rdev)
{
	struct list_head *pos = NULL;
	struct sm_devctl_devt_internal *tmp = NULL;

	//read_lock(&devt_rw_lock);
	mutex_lock(&devctl_mutex);
	list_for_each(pos, &devctl_devt_lst)
	{
		tmp = list_entry(pos, struct sm_devctl_devt_internal, 
				list);
		if (IS_ERR(&tmp->devctl) || IS_ERR(tmp->devctl.inf_type))
			goto ERROR;
		switch(tmp->devctl.inf_type)
		{
		case SM_INF_SERIAL:
			if(tmp->devctl.major == MAJOR(rdev) && 
					(MINOR(rdev) > 63 || 
					 MAJOR(rdev) == 188))
				goto FALSE_V;
			break;
		default:
			if(tmp->devctl.major == MAJOR(rdev))
			{
				if(tmp->devctl.major == 11)
				{
					if(tmp->devctl.perm == 5)
					{

						goto TRUE_V;
					}
					else
					{
						goto FALSE_V;
					}
				}
				else
				{
					goto FALSE_V;
				}
			}
			break;
		}
	}
ERROR:
	//read_unlock(&devt_rw_lock);
	mutex_unlock(&devctl_mutex);
	return -1;
TRUE_V:
	//read_unlock(&devt_rw_lock);
	mutex_unlock(&devctl_mutex);
	printk("INFO:chrblk true %s %s\n", devctl_info_decode(&(tmp->devctl)), perm_decode(tmp->devctl.perm));
	return 1;
FALSE_V:
	//read_unlock(&devt_rw_lock);
	mutex_unlock(&devctl_mutex);
	printk("INFO:chrblk false %s %s\n", devctl_info_decode(&(tmp->devctl)), perm_decode(tmp->devctl.perm));
	return 0;
}

static struct sm_devctl_vidpid_internal *
		lookup_by_vidpid2(struct 
		sm_devctl_vidpid *devctl)
{
	struct list_head *pos = NULL;
	struct sm_devctl_vidpid_internal *tmp = NULL;
	
	list_for_each(pos, &devctl_vidpid_lst)
	{
		tmp = list_entry(pos, struct 
				sm_devctl_vidpid_internal, 
				list);
		
		/* found it */
		if(devctl->vid == tmp->devctl.vid && 
				devctl->pid == 
				tmp->devctl.pid)
			return tmp;
	}
	return NULL;
}

static struct sm_devctl_internal * add_to_lst(struct 
		sm_devctl *devctl, struct device *dev)
{
	struct sm_devctl_internal *tmp = NULL;

	if((tmp = lookup_by_vidpid1(devctl)) != NULL)
		goto out;

	/* fixme: 本机串口ttyS0-ttyS31 加入会有问题，
	 * 只剩下ttyS31 */
	if((tmp = lookup_by_major(devctl)) != NULL)
		goto out;

	if((tmp = lookup_by_name(dev_name(dev))) != 
			NULL)
		goto out;

	/* not found, add to list */
	tmp = kzalloc(sizeof(struct sm_devctl_internal), 
			GFP_KERNEL);
	if(!tmp){
		printk("kzalloc failed\n");
		return NULL;
	}

	mutex_lock(&devctl_lst_mutex);
	list_add(&tmp->list, &devctl_lst);
	memcpy(&tmp->devctl, devctl, 
			sizeof(struct sm_devctl));
	tmp->device = dev;
	tmp->driver = dev->driver;
	mutex_unlock(&devctl_lst_mutex);
	printk("INFO:SET vid=%04x pid=%04x type=%s major=%d netcards=%s devname=%s driv_name=%s perm=%s\n", 
			devctl->vid, devctl->pid, devctl_info_decode(devctl),
			devctl->major, devctl->netcards, (dev_name(dev))? dev_name(dev):"NULL", 
			(dev->driver && dev->driver->name)? dev->driver->name:"NULL", perm_decode(devctl->perm));
	return tmp;
out:		
	memcpy(&tmp->devctl, devctl, 
			sizeof(struct sm_devctl));
	tmp->device = dev;
	tmp->driver = dev->driver;
	return tmp;
}

static int add_to_vidpid_lst(struct 
		sm_devctl_vidpid *devctl, 
		struct device *dev)
{
	struct sm_devctl_vidpid_internal *tmp = NULL;

	if((tmp = lookup_by_vidpid2(devctl)) != NULL)
	{
		memcpy(&tmp->devctl, devctl, 
				sizeof(struct 
				sm_devctl_vidpid));
		tmp->device = dev;
		tmp->driver = dev != NULL ? dev->driver : NULL;
		return 0;
	}
			
	/* not found, add to list */
	tmp = kzalloc(sizeof(struct 
			sm_devctl_vidpid_internal), 
			GFP_KERNEL);
	memcpy(&tmp->devctl, devctl, sizeof(struct 
			sm_devctl_vidpid));
	tmp->device = dev;
	tmp->driver = dev != NULL ? dev->driver : NULL;
	list_add(&tmp->list, &devctl_vidpid_lst);
	printk("INFO:SET vid=%04x pid=%04x cls=%04x perm=%s\n", devctl->vid, devctl->pid, devctl->usb_class, perm_decode(devctl->perm));
	return 0;
}

static int lookup_setup(const struct sm_devctl *devctl)
{
	struct list_head *pos = NULL;
	struct sm_devctl_internal *tmp = NULL;

	mutex_lock(&devctl_lst_mutex);
	list_for_each(pos, &devctl_lst)
	{
		tmp = list_entry(pos, struct sm_devctl_internal, 
				list);
		
		if(devctl->inf_type > 0 && devctl->inf_type ==
				tmp->devctl.inf_type)
		{
			perm_setup(tmp->device, tmp->driver, 
					devctl->perm);
			continue;
		}
		
		if(devctl->dev_type > 0 && devctl->dev_type ==
				tmp->devctl.dev_type)
		{
			perm_setup(tmp->device, tmp->driver, 
					devctl->perm);
			continue;
		}

		if(devctl->usb_type > 0 && devctl->usb_type ==
				tmp->devctl.usb_type)
		{
			perm_setup(tmp->device, tmp->driver, 
					devctl->perm);
			continue;
		}

        	if((devctl->netcards[0] != '\0') && (tmp->
				devctl.netcards[0] != '\0'))
        	{
            		if(strcmp(tmp->devctl.netcards, 
					devctl->netcards) == 0)
                		perm_setup(tmp->device, tmp->driver, 
						devctl->perm);
        	}

	}
	mutex_unlock(&devctl_lst_mutex);
	return 0;
}

static int lookup_vidpid_setup(const struct 
		sm_devctl_vidpid *devctl)
{
	int ret = -1;

	struct list_head *pos = NULL;
	struct sm_devctl_vidpid_internal *tmp = NULL;

	list_for_each(pos, &devctl_vidpid_lst)
	{
		tmp = list_entry(pos, struct 
				sm_devctl_vidpid_internal, 
				list);
		
		if(devctl->vid == tmp->devctl.vid && 
				devctl->pid == 
				tmp->devctl.pid)
		{
			memcpy(&tmp->devctl, devctl, sizeof(
					struct sm_devctl_vidpid));
			ret = perm_setup(tmp->device, tmp->driver, 
					devctl->perm);
		}
	}
	return 0;
}

static int sm_devctl_devt_setup(struct sm_devctl *devctl) 
{
	struct list_head *pos = NULL;
	struct list_head *q = NULL;
	struct sm_devctl_devt_internal *tmp = NULL;

	/* add item */
	if(devctl->perm == 0 || devctl->perm == 5)
	{
		//write_lock(&devt_rw_lock);
		mutex_lock(&devctl_mutex);
		list_for_each(pos, &devctl_devt_lst)
		{
			tmp = list_entry(pos, struct 
					sm_devctl_devt_internal, 
					list);
			/* alread setup, skip */
			if(tmp->devctl.major == devctl->major)
			{
				tmp->devctl.perm = devctl->perm;
				goto DONE;
			}
		}
	
		printk("INFO:SET major=%d perm=%s\n" ,devctl->major, perm_decode(devctl->perm));	
		tmp = kzalloc(sizeof(struct 
				sm_devctl_devt_internal), 
				GFP_KERNEL);
		memcpy(&tmp->devctl, devctl, sizeof(struct 
				sm_devctl));
		list_add(&tmp->list, &devctl_devt_lst);
		goto DONE;
	}

	/* del item */
	//write_lock(&devt_rw_lock);
	mutex_lock(&devctl_mutex);
	list_for_each_safe(pos, q, &devctl_devt_lst)
	{
		tmp = list_entry(pos, struct 
				sm_devctl_devt_internal, list);
		if(tmp->devctl.major == devctl->major)
		{
			printk("INFO:SET major=%d perm=enable\n" ,devctl->major);	
			list_del(pos);
			kfree(tmp);
			goto DONE;
		}
	}
DONE:
	//write_unlock(&devt_rw_lock);
	mutex_unlock(&devctl_mutex);
	return 0;
}

static int sm_devctl_vm_setup(struct sm_devctl_vm *devctl) 
{
	struct list_head *pos = NULL;
	struct list_head *q = NULL;
	struct sm_devctl_vm_internal *tmp = NULL;

	/* add item */
	if(devctl->perm == 0)
	{
		list_for_each(pos, &devctl_vm_lst)
		{
			tmp = list_entry(pos, struct 
					sm_devctl_vm_internal, 
					list);
			/* alread setup, skip */
			if(strcmp(tmp->devctl.vm_path, devctl->
					vm_path) == 0)
				return 0;
		}
	
		printk("disable vm: vm_path = %s\n",
				devctl->vm_path);	
		tmp = kzalloc(sizeof(struct 
				sm_devctl_vm_internal), 
				GFP_KERNEL);
		memcpy(&tmp->devctl, devctl, sizeof(struct 
				sm_devctl_vm));
		list_add(&tmp->list, &devctl_vm_lst);

		return 0;	
	}

	/* del item */
	list_for_each_safe(pos, q, &devctl_vm_lst)
	{
		tmp = list_entry(pos, struct 
				sm_devctl_vm_internal, list);
		if(strcmp(tmp->devctl.vm_path, devctl->
				vm_path) == 0)
		{
			printk("enable vm: vm_path = %s\n",
					devctl->vm_path);	
			list_del(pos);
			kfree(tmp);
			return 0;
		}
	}
	return 0;
}

static int sm_pkg_protect_setup(struct sm_pkg_protect *pkg) 
{
	struct list_head *pos = NULL;
	struct list_head *q = NULL;
	struct sm_pkg_protect_internal *tmp = NULL;

	/* add item */
	if(pkg->perm == 0)
	{
		list_for_each(pos, &pkg_protect_lst)
		{
			tmp = list_entry(pos, struct 
					sm_pkg_protect_internal, 
					list);
			/* alread setup, skip */
			if(strcmp(tmp->pkg.pkg_name, pkg->
					pkg_name) == 0)
				return 0;
		}
	
		printk("disable unintall: pkg name = %s\n",
				pkg->pkg_name);	
		tmp = kzalloc(sizeof(struct 
				sm_pkg_protect_internal), 
				GFP_KERNEL);
		memcpy(&tmp->pkg, pkg, sizeof(struct 
				sm_pkg_protect));
		list_add(&tmp->list, &pkg_protect_lst);

		return 0;	
	}

	/* del item */
	list_for_each_safe(pos, q, &pkg_protect_lst)
	{
		tmp = list_entry(pos, struct 
				sm_pkg_protect_internal, list);
		if(strcmp(tmp->pkg.pkg_name, pkg->
				pkg_name) == 0)
		{
			printk("enable unintall: pkg name = %s\n",
					pkg->pkg_name);	
			list_del(pos);
			kfree(tmp);
			return 0;
		}
	}
	return 0;
}

/* file access protect tempory */
static int sm_file_protect_setup(struct sm_file_protect *file) 
{
	struct list_head *pos = NULL;
	struct list_head *q = NULL;
	struct sm_file_protect_internal *tmp = NULL;

	/* add item */
	if(file->perm == 1)
	{
		list_for_each(pos, &file_protect_lst)
		{
			tmp = list_entry(pos, struct 
					sm_file_protect_internal, 
					list);
			/* alread setup, skip */
			if(strcmp(tmp->file.process_name, file->
					process_name) == 0)
				return 0;
		}
	
		printk("enable file access: process name = %s\n",
				file->process_name);	
		tmp = kzalloc(sizeof(struct 
				sm_file_protect_internal), 
				GFP_KERNEL);
		memcpy(&tmp->file, file, sizeof(struct 
				sm_file_protect));
		list_add(&tmp->list, &file_protect_lst);

		return 0;	
	}

	/* del item */
	list_for_each_safe(pos, q, &file_protect_lst)
	{
		tmp = list_entry(pos, struct 
				sm_file_protect_internal, list);
		if(strcmp(tmp->file.process_name, file->
				process_name) == 0)
		{
			printk("disable file access: process name = %s\n",
					file->process_name);	
			list_del(pos);
			kfree(tmp);
			return 0;
		}
	}
	return 0;
}

static int inf_setup(struct sm_devctl *devctl)
{
	if(devctl->inf_type == 0)
		return -EINVAL;
	
	switch(devctl->inf_type)
	{
	case SM_INF_SERIAL:
		/* 188 -- ttyUSB major */
		/* 4 -- ttyS major */
		devctl->major = 188;
		sm_devctl_devt_setup(devctl);
			
		/* ttyS0 */
		devctl->major = 4;
		sm_devctl_devt_setup(devctl);

		break;

	case SM_INF_PARALLEL:
		/* 99 -- parport major */
		devctl->major = 99;
		sm_devctl_devt_setup(devctl);
			
		break;
	case SM_INF_1394:
		/* 171 -- char 1394 major, need know
		 * device class */
		devctl->major = 171;
		sm_devctl_devt_setup(devctl);
			
		break;
	case SM_INF_PCMCIA:
		/* 191 -- char pcmcia major, need know
		 * device class */
		devctl->major = 191;
		sm_devctl_devt_setup(devctl);
			
		break;
	case SM_INF_TYPEC:
		break;
	case SM_INF_HDMI:
		break;
	case SM_INF_DISPLAYPORT:
		break;
	default:
		break;
			
	}

	return 0;
	/* 搜索队列，统一设置 */
	// return lookup_setup(devctl);
}

static int dev_setup(struct sm_devctl *devctl)
{
	struct class *tmp = NULL;
	struct device *find = NULL;
	struct device *cls_dev = NULL;
	struct usb_interface *intf = NULL;

	if(devctl->dev_type == 0)
		return -EINVAL;

	/* disable - 查找设备，看是否已经在列表中，如果不在，
	 * 需要添加到队列 */
	if(devctl->perm == 0)
	{
		switch(devctl->dev_type)
		{
		case SM_DEV_BLUETOOTH:
			while((find = bus_find_device_by_drv(
					find, "btusb")) != NULL)			
			{
				intf = to_usb_interface(find);
		
				devctl->vid = interface_to_usbdev(intf)->
						descriptor.idVendor;
				devctl->pid = interface_to_usbdev(intf)->
						descriptor.idProduct;
			
				add_to_lst(devctl, &interface_to_usbdev(intf)->dev);
			}
			
			break;

		case SM_DEV_INFRARED:
			/* need know more info */
			return 0;
		case SM_DEV_MODEM:
			/* need know more info */
			return 0;
		case SM_DEV_STORAGE_DRIVE:
			devctl->major = 179;
			sm_devctl_devt_setup(devctl);
			break;

		case SM_DEV_SCSI_RAID:
			if((tmp = class_addr_get("scsi_generic")) == 
					NULL)
				return 0;

			/* 21 --chr device major */
			while((find = class_find_device_by_major(
					tmp, find, 21)) != 
					NULL)	
			{	
				devctl->major = 21;	
				add_to_lst(devctl, find);
			}
			break;
		case SM_DEV_WINCE:
			cur_cfg_dev[SM_DEV_WINCE] = 1;
			#if 0
			while((find = bus_find_mtp_device(
					find)) != NULL)			
			{
				intf = to_usb_interface(find);
		
				devctl->vid = to_usb_device(find)->
						descriptor.idVendor;
				devctl->pid = to_usb_device(find)->
						descriptor.idProduct;
			
				add_to_lst(devctl, find);
			}
			#endif
			break;
		case SM_DEV_FLOPPY_DRIVE:
			devctl->major = 2;
			return sm_devctl_devt_setup(devctl);
    		case SM_DEV_IMAGE:
			devctl->major = 81;
			return sm_devctl_devt_setup(devctl);
		case SM_DEV_MULTI_SERIAL_CARD:
			/* 172 -- char moxa serial card */
			while((find = class_find_device_by_major(
					tty_class, find, 172)) != 
					NULL)			
			{
				devctl->major = 172;
				add_to_lst(devctl, find);
			}

			/* 173 -- char moxa serial card -alternate 
			 * devices */
			while((find = class_find_device_by_major(
					tty_class, find, 173)) != 
					NULL)			
			{
				devctl->major = 173;
				add_to_lst(devctl, find);
			}

			/* 174 -- char smartio serial card */
			while((find = class_find_device_by_major(
					tty_class, find, 174)) != 
					NULL)			
			{
				devctl->major = 174;
				add_to_lst(devctl, find);
			}

			/* 175 -- char smartio serial card - alternate
			 * device */
			while((find = class_find_device_by_major(
					tty_class, find, 175)) != 
					NULL)			
			{
				devctl->major = 175;
				add_to_lst(devctl, find);
			}
			break;
		case SM_DEV_ETHERNET:
			while((find = bus_find_net_device(
					&pci_bus_type, find, 
					"net")) != NULL)
			{
				cls_dev = dev_find_class(find, "net");

				if(!(cls_dev->type != NULL && 
						cls_dev->type->name != NULL && 
						strcmp(cls_dev->type->name,
						"wlan") == 0))
					add_to_lst(devctl, find);
			}

			find = NULL;
			
			while((find = bus_find_net_device(
					&platform_bus_type, find, 
					"net")) != NULL)
			{
				cls_dev = dev_find_class(find, "net");
				if(!(cls_dev->type != NULL && 
						cls_dev->type->name != NULL && 
						strcmp(cls_dev->type->name,
						"wlan") == 0))
					add_to_lst(devctl, find);
			}

			find = NULL;

			while((find = bus_find_net_device(
					&usb_bus_type, find, 
					"net")) != NULL)
			{
				intf = to_usb_interface(find);
                        
				devctl->vid = interface_to_usbdev(intf)->
                                        descriptor.idVendor;
                        	devctl->pid = interface_to_usbdev(intf)->
                                        descriptor.idVendor;

				cls_dev = dev_find_class(find, "net");
				if(!(cls_dev->type != NULL && 
						cls_dev->type->name != NULL && 
						strcmp(cls_dev->type->name,
						"wlan") == 0))
					add_to_lst(devctl, 
						&interface_to_usbdev(intf)->
						dev);
			}

			break;
				
		case SM_DEV_WIRELESS:  
			while((find = bus_find_net_device(
					&pci_bus_type, find, 
					"net")) != NULL)
			{
				cls_dev = dev_find_class(find, "net");
				if(cls_dev->type != NULL && 
						cls_dev->type->name != NULL && 
						strcmp(cls_dev->type->name,
						"wlan") == 0)
					add_to_lst(devctl, find);
			}

			find = NULL;
			
			while((find = bus_find_net_device(
					&platform_bus_type, find, 
					"net")) != NULL)
			{
				cls_dev = dev_find_class(find, "net");
				if(cls_dev->type != NULL && 
						cls_dev->type->name != NULL && 
						strcmp(cls_dev->type->name,
						"wlan") == 0)
					add_to_lst(devctl, find);
			}

			find = NULL;

			while((find = bus_find_net_device(
					&usb_bus_type, find, 
					"net")) != NULL)
			{
				intf = to_usb_interface(find);
                        
				devctl->vid = interface_to_usbdev(intf)->
                                        descriptor.idVendor;
                        	devctl->pid = interface_to_usbdev(intf)->
                                        descriptor.idVendor;

				cls_dev = dev_find_class(find, "net");
				if(cls_dev->type != NULL && 
						cls_dev->type->name != NULL && 
						strcmp(cls_dev->type->name,
						"wlan") == 0)
					add_to_lst(devctl, 
						&interface_to_usbdev(intf)->
						dev);
			}

			break;
				
		case SM_DEV_PRINTER:
			if((tmp = class_addr_get("printer")) == 
					NULL)
				return 0;
			
			/* 6 -- parport printer major */
			while((find = class_find_device_by_major(
					tmp, find, 6)) != 
					NULL)			
			{
				devctl->major = 6;
				add_to_lst(devctl, find);
			}

			/* 180 -- usb printer major */
			while((find = class_find_device_by_major(
					tmp, find, 180)) != 
					NULL)			
			{
				devctl->major = 180;
				add_to_lst(devctl, find);
			}

			break;

		case SM_DEV_CDROM:
			devctl->major = 11;
			burning_setup(0);
			return sm_devctl_devt_setup(devctl);

			break;
		case DEV_VOLUMESHADOW:
			/* TODO: realize ... */
			return 0;
		case DEV_TAPE:
			/* TODO: realize ... */
			return 0;
		default:
			return 0;
		}
	}

	if(devctl->perm == 1)
	{
		switch(devctl->dev_type)
		{
		case SM_DEV_STORAGE_DRIVE:
			devctl->major = 179;
			return sm_devctl_devt_setup(devctl);
		case SM_DEV_FLOPPY_DRIVE:
			devctl->major = 2;
			return sm_devctl_devt_setup(devctl);
    		case SM_DEV_IMAGE:
			devctl->major = 81;
			return sm_devctl_devt_setup(devctl);
		case SM_DEV_CDROM:
			devctl->major = 11;
			burning_setup(1);
			return sm_devctl_devt_setup(devctl);
		default:
			break;
		}
	}

	/* cdrom read only, can't burn */
	if(devctl->perm == 5)
	{
		switch(devctl->dev_type)
		{
		case SM_DEV_CDROM:
			burning_setup(0);
			devctl->major = 11;
			return sm_devctl_devt_setup(devctl);
		default:
			return 0;
		}
	}

	/* 搜索队列，统一设置 */
	return lookup_setup(devctl);
}

static int usb_setup(struct sm_devctl *devctl)
{
	struct device *find = NULL;
	struct usb_interface *intf = NULL;

	if(devctl->usb_type == 0)
		return -EINVAL;
			
	/* disable - 查找设备，看是否已经在列表中，如果不在，
	 * 需要添加到队列 */
	if(devctl->perm == 0)
	{
		while((find = bus_find_device_by_class(
				find, devctl)) != NULL)			
		{
			intf = to_usb_interface(find);
			devctl->vid = interface_to_usbdev(intf)->
					descriptor.idVendor;
			devctl->pid = interface_to_usbdev(intf)->
					descriptor.idVendor;
			
			add_to_lst(devctl, &interface_to_usbdev(
					intf)->dev);
			put_device(find);
		}
	}

	/* 搜索队列，统一设置 */
	return lookup_setup(devctl);
}

static int usb_vidpid_setup(struct sm_devctl_vidpid 
		*devctl)
{
	struct device *find = NULL;

	if(devctl->usb_class == 0 || 
			devctl->vid == 0 || 
			devctl->pid == 0)
		return -EINVAL;
			
	/* disable - 查找设备，看是否已经在列表中，如果不在，
	 * 需要添加到队列 */
	find = bus_find_device_by_vidpid(
			NULL, devctl);

	/* not found, add it too */
	add_to_vidpid_lst(devctl, find);

	if(find != NULL){
		printk("INFO:put device vid=%04x pid=%04x cls=%04x perm=%s\n", devctl->vid, devctl->pid, devctl->usb_class, perm_decode(devctl->perm));
		put_device(find);
	}

	return lookup_vidpid_setup(devctl);
}

static int netname_setup(struct sm_devctl *devctl)
{
	struct device *find = NULL;
	struct net_device *net_dev = NULL;

	/* 遍历每个网络设备 */
	for_each_netdev(&init_net, net_dev)
	{
		/* 检查网卡名是否匹配 */
		if (strcmp(net_dev->name, devctl->netcards) == 0)
		{
			find = net_dev->dev.parent;

			add_to_lst(devctl, find);
			break;
		}
	}

	/* 搜索队列，统一设置 */
	return lookup_setup(devctl);
}

static int sm_devctl_setup(struct sm_devctl 
		*devctl) 
{
	if(devctl->inf_type > 0)
		return inf_setup(devctl);

	if(devctl->dev_type > 0)
		return dev_setup(devctl);

	if(devctl->usb_type > 0)
		return usb_setup(devctl);

    	if(strlen(devctl->netcards) > 0)
		return netname_setup(devctl);

	return 0;
}

static int sm_devctl_vidpid_setup(struct sm_devctl_vidpid 
		*devctl) 
{
	return usb_vidpid_setup(devctl);
}

static struct sm_devctl_vidpid_internal *lookup_usbdev(
		u16 vid, u16 pid)
{
	struct list_head *pos = NULL;
	struct sm_devctl_vidpid_internal *tmp = NULL;

	list_for_each(pos, &devctl_vidpid_lst)
	{
		tmp = list_entry(pos, struct 
				sm_devctl_vidpid_internal, list);
		if((tmp->devctl.vid == vid) && (tmp->
				devctl.pid == pid))
			return tmp;
	}
	return NULL;
}

long sm_devctl_perm_ioctl(struct socket *sock, unsigned int cmd, 
		compat_ulong_t __user *argp)
{
	struct sm_devctl devctl = { 0 };

#if 0
	/* non root user can't do this operation */
	if(current_uid().val != 0)
		return -EPERM;
#endif	
	if (copy_from_user(&devctl, argp, sizeof(
			struct sm_devctl)))
		return -EFAULT;

	printk("POLICY:%s %s\n", devctl_info_decode(&devctl), perm_decode(devctl.perm));
	if(devctl.inf_type > 0) 
	{
		/* 重复设置，返回 */
		if(devctl.perm == cur_cfg_inf[
				devctl.inf_type])
			return 0;

		cur_cfg_inf[devctl.inf_type] = 
			devctl.perm;
	}

	if(devctl.dev_type > 0) 
	{
		/* 重复设置，返回 */
		if(devctl.perm == cur_cfg_dev[
				devctl.dev_type])
			return 0;

		cur_cfg_dev[devctl.dev_type] = 
			devctl.perm;
	}
	
	if(devctl.usb_type > 0) 
	{
		/* 重复设置，返回 */
		if(devctl.perm == cur_cfg_usb[
				devctl.usb_type])
			return 0;

		cur_cfg_usb[devctl.usb_type] = 
			devctl.perm;
	}
	
	return sm_devctl_setup(&devctl);
}
EXPORT_SYMBOL(sm_devctl_perm_ioctl);

long sm_devctl_vidpid_perm_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp)
{
	struct sm_devctl_vidpid devctl;
#if 0
	/* non root user can't do this operation */
	if(current_uid().val != 0)
		return -EPERM;
#endif	
	if (copy_from_user(&devctl, argp, sizeof(
			struct sm_devctl_vidpid)))
		return -EFAULT;

	printk("POLICY:vid=%04x pid=%04x %s\n", devctl.vid, devctl.pid, perm_decode(devctl.perm));
	return sm_devctl_vidpid_setup(&devctl);
}
EXPORT_SYMBOL(sm_devctl_vidpid_perm_ioctl);

long sm_devctl_vm_perm_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp)
{
	struct sm_devctl_vm devctl;
#if 0
	/* non root user can't do this operation */
	if(current_uid().val != 0)
		return -EPERM;
#endif	
	if (copy_from_user(&devctl, argp, sizeof(
			struct sm_devctl_vm)))
		return -EFAULT;

	printk("SM_DEVCTL_VM_PERM: perm = %d\n", devctl.perm);
	return sm_devctl_vm_setup(&devctl);
}
EXPORT_SYMBOL(sm_devctl_vm_perm_ioctl);

long sm_pkg_protect_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp)
{
	struct sm_pkg_protect pkg;
#if 0
	/* non root user can't do this operation */
	if(current_uid().val != 0)
		return -EPERM;
#endif	
	if (copy_from_user(&pkg, argp, sizeof(
			struct sm_pkg_protect)))
		return -EFAULT;

	printk("SM_PKG_PROTECT_PERM: perm = %d\n", pkg.perm);
	return sm_pkg_protect_setup(&pkg);
}
EXPORT_SYMBOL(sm_pkg_protect_ioctl);

/* file access protect for tempory */
long sm_file_protect_ioctl(struct socket *sock, 
		unsigned int cmd, compat_ulong_t __user *argp)
{
	struct sm_file_protect file;
#if 0
	/* non root user can't do this operation */
	if(current_uid().val != 0)
		return -EPERM;
#endif	
	if (copy_from_user(&file, argp, sizeof(
			struct sm_file_protect)))
		return -EFAULT;

	printk("SM_PKG_PROTECT_PERM: perm = %d\n", file.perm);
	return sm_file_protect_setup(&file);
}
EXPORT_SYMBOL(sm_file_protect_ioctl);

static int bluetooth_perm(struct device *dev)
{
	struct device *find = NULL;
	struct sm_devctl devctl = {0};

	if((find = device_find_child(dev, NULL,
			device_match_intf)) == NULL)
		return PERM_UNCONFIGURED ; 
	
	if(find->driver == NULL || find->driver->name == NULL)
		return PERM_UNCONFIGURED; 

	if(strcmp(find->driver->name, "btusb") != 0)
		return PERM_UNCONFIGURED; 

	/* 启用状态，直接返回已配置 */
	if(cur_cfg_dev[SM_DEV_BLUETOOTH] == 1)
		return PERM_CONFIGURED; 
	
	devctl.vid = to_usb_device(dev)->
                        descriptor.idVendor;
	devctl.pid = to_usb_device(dev)->
                        descriptor.idProduct;

	devctl.dev_type = SM_DEV_BLUETOOTH;
	devctl.perm = cur_cfg_dev[SM_DEV_BLUETOOTH];

	printk("INFO:BLUETOOTH_SET vid=%04x pid=%04x type=%s perm=%s\n", 
			devctl.vid, devctl.pid, 
			devctl_info_decode(&devctl), 
			perm_decode(devctl.perm));
	add_to_lst(&devctl, dev);

	perm_setup(dev, dev->driver, 0);
		
	return PERM_CONFIGURED; 
}

static int net_perm(struct device *dev)
{
	struct device *cls_dev = NULL;
	struct sm_devctl devctl = {0};

	if((cls_dev = dev_find_class(device_find_child(
			dev, NULL, device_match_intf), 
			"net")) == NULL)
		return PERM_UNCONFIGURED;
	
	if(cls_dev->type != NULL && 
			cls_dev->type->name != NULL && 
			strcmp(cls_dev->type->name,
			"wlan") == 0)
	{	
		if(cur_cfg_dev[SM_DEV_WIRELESS] == 1)
			return PERM_CONFIGURED; 
	       	devctl.dev_type = SM_DEV_WIRELESS;	
	}
	else{		
		if(cur_cfg_dev[SM_DEV_ETHERNET] == 1)
			return PERM_CONFIGURED;
	       	devctl.dev_type = SM_DEV_ETHERNET;	
	}
	
	devctl.vid = to_usb_device(dev)->
			descriptor.idVendor;
	devctl.pid = to_usb_device(dev)->
                        descriptor.idProduct;
	devctl.perm = 0;

	printk("INFO:NET_SET vid=%04x pid=%04x type=%s perm=%s\n", 
			devctl.vid, devctl.pid, 
			devctl_info_decode(&devctl), 
			perm_decode(devctl.perm));
	add_to_lst(&devctl, dev);

	perm_setup(dev, dev->driver, 0);
	return PERM_CONFIGURED; 
}

static int mtp_perm(struct device *dev)
{
	struct sm_devctl devctl = {0};

	if(to_usb_device(dev)->actconfig->desc.
			iConfiguration != 4)
		return PERM_UNCONFIGURED;

	/* 启用状态，直接返回已配置 */
	if(cur_cfg_dev[SM_DEV_WINCE] == 1)
		return PERM_CONFIGURED; 
	
	devctl.vid = to_usb_device(dev)->
                        descriptor.idVendor;
	devctl.pid = to_usb_device(dev)->
                        descriptor.idProduct;

	devctl.dev_type = SM_DEV_WINCE;
	devctl.perm = cur_cfg_dev[SM_DEV_WINCE];
	printk("INFO:MTP_SET vid=%04x pid=%04x type=%s perm=%s\n", 
			devctl.vid, devctl.pid, 
			devctl_info_decode(&devctl), 
			perm_decode(devctl.perm));
	add_to_lst(&devctl, dev);

	perm_setup(dev, dev->driver, 0);
		
	return PERM_CONFIGURED; 
}

/* 设备插入时，权限判断 */
static int usb_perm(struct device *dev)
{
	int i = 1;

	struct sm_devctl devctl = {0};

	devctl.vid = to_usb_device(dev)->
                        descriptor.idVendor;
	devctl.pid = to_usb_device(dev)->
                        descriptor.idProduct;

	for(; i < USB_MAX; i++)
	{	
		devctl.usb_type = i;
		devctl.perm = cur_cfg_usb[i];

		if(usb_find_class(dev, &devctl) != NULL)
		{
			printk("INFO:USB_SET vid=%04x pid=%04x type=%s perm=%s\n", 
					devctl.vid, devctl.pid, 
					devctl_info_decode(&devctl), 
					perm_decode(devctl.perm));
			add_to_lst(&devctl, dev);

			if(devctl.perm == 1)
				return PERM_CONFIGURED;

			perm_setup(dev, dev->driver, 0);
			return PERM_CONFIGURED;
		}
	}
	return PERM_UNCONFIGURED;
}

/* 设备插入时，权限判断 */
static int vidpid_perm(struct device *dev)
{
	struct sm_devctl_vidpid_internal *tmp = NULL;
	
	if((tmp = lookup_usbdev(to_usb_device(dev)->
			descriptor.idVendor, 
			to_usb_device(dev)->
			descriptor.idProduct)) == NULL)
		return PERM_UNCONFIGURED;

	tmp->device = dev;
	tmp->driver = dev->driver;

	/* enabled, return */
	if(tmp->devctl.perm == 1)
	{
		if(bluetooth_perm(dev) == PERM_CONFIGURED)
			return PERM_CONFIGURED;

		if(net_perm(dev) == PERM_CONFIGURED)
			return PERM_CONFIGURED;

		if(mtp_perm(dev) == PERM_CONFIGURED)
			return PERM_CONFIGURED;

		return PERM_CONFIGURED;
	}

	perm_setup(dev, dev->driver, 0);
	return PERM_CONFIGURED;
}

static int netname_perm(struct device *dev)
{
	struct list_head *pos = NULL;
	struct sm_devctl_internal *tmp = NULL;

	struct net_device *net_dev = NULL;

	mutex_lock(&devctl_lst_mutex);
	list_for_each(pos, &devctl_lst)
	{
		tmp = list_entry(pos, struct sm_devctl_internal, list);

		for_each_netdev(&init_net, net_dev)
		{
			// 检查网卡名是否匹配
			if (strcmp(net_dev->name, tmp->devctl.netcards) == 0)
			{
				if(tmp->devctl.perm == 0)
					perm_setup(dev, dev->driver, 0);
				mutex_unlock(&devctl_lst_mutex);
				return PERM_CONFIGURED;
			}
		}
	}
	mutex_unlock(&devctl_lst_mutex);

	return PERM_UNCONFIGURED;
}

static int delay_setup(void *data)
{
	struct device *dev = (struct device *)data;
	
	msleep(3000);
	/* 手机有只充电模式，这时actconfig是NULL，
	 * 无需进一步配置 */
	{
        	if(to_usb_device(dev)->actconfig == NULL)
        	{
                	return PERM_CONFIGURED;
        	}
	}
	if(cur_cfg_dev[SM_DEV_WINCE] == 1){
		int icount_06 = 0;
		int icount_08 = 0;
		int icount_ff = 0;
		int i;
		struct usb_interface *intf;
		for(i=0;i<to_usb_device(dev)->actconfig->desc.bNumInterfaces;i++){
			intf = to_usb_device(dev)->actconfig->interface[i];
			if(intf){
				if(intf->cur_altsetting->desc.bInterfaceClass == USB_CLASS_STILL_IMAGE){
					icount_06++;
				}
				else if(intf->cur_altsetting->desc.bInterfaceClass == USB_CLASS_MASS_STORAGE){
					icount_08++;
				}
				else if(intf->cur_altsetting->desc.bInterfaceClass == USB_CLASS_VENDOR_SPEC){
					icount_ff++;
				}
			}
		}
		if( (icount_06>=1 && icount_08>=1) || (icount_08>=1 && icount_ff>=1) ){
			printk("remove wince device, Vid:%04x, Pid:%04x\n", to_usb_device(dev)->descriptor.idVendor, to_usb_device(dev)->descriptor.idProduct);
			perm_setup(dev, dev->driver, 0);
			return PERM_CONFIGURED;
		}
	}

	if(netname_perm(dev) == PERM_CONFIGURED)
		return PERM_CONFIGURED;

	if(vidpid_perm(dev) == PERM_CONFIGURED)
		return PERM_CONFIGURED;
	
	int ret = usb_perm(dev);
	return ret;
}

/* disabled device cannot insmod to kernel */
int dev_perm_chk(struct device *dev)
{
	/* list init flag */
	static int init = 0; 

	struct task_struct *tsk = NULL;

	if(!init)
	{
		memset(cur_cfg_inf, 1, sizeof(cur_cfg_inf));
		memset(cur_cfg_dev, 1, sizeof(cur_cfg_dev));
		memset(cur_cfg_usb, 1, sizeof(cur_cfg_usb));
		
		init = 1;
	}

	/* tempory not use */
	/* class_addr_add(dev->class); */

	/* non usb device */
	if(dev->bus != &usb_bus_type)
		return 0;
	
	if((dev->type->name == NULL ) || ((dev->type->name != 
			NULL) && (strcmp(dev->type->name, 
			"usb_device") != 0)))
		return 0;

	/* usb设备会添加很多设备结构，必须等到
	 * 最后一个设备添加完成才能卸载 */
	tsk = kthread_create(delay_setup, (void *)dev,
			"delay_setup");
	if(IS_ERR(tsk))
		printk("create delay_setup warn: %s\n", 
				__func__);
	
	wake_up_process(tsk);
	return 0;
}
EXPORT_SYMBOL(dev_perm_chk);

/* when device unregister, clean up */
void dev_perm_clean(const char * devname)
{
	struct list_head *pos = NULL;
	struct list_head *q = NULL;
	struct sm_devctl_internal *tmp = NULL;
	struct sm_devctl *devctl = NULL;

	mutex_lock(&devctl_lst_mutex);
	list_for_each_safe(pos, q, &devctl_lst)
	{
		tmp = list_entry(pos, struct sm_devctl_internal, 
				list);
		if(!tmp->device)
		{
			devctl = &(tmp->devctl);
			printk("INFO:DEL vid=%04x pid=%04x type=%s major=%d netcards=%s devname=%s driv_name=%s perm=%s\n", 
					devctl->vid, devctl->pid, devctl_info_decode(devctl),
					devctl->major, devctl->netcards, (dev_name(tmp->device))? dev_name(tmp->device):"NULL", 
					(tmp->device->driver->name)? tmp->device->driver->name:"NULL", perm_decode(devctl->perm));
			list_del(pos);
			kfree(tmp);
			continue;
		}
		if(strcmp(dev_name(tmp->device), devname) == 0)
		{
			if(!tmp->device->class){
				mutex_unlock(&devctl_lst_mutex);
				return;
			}

			devctl = &(tmp->devctl);
			printk("INFO:DEL vid=%04x pid=%04x type=%s major=%d netcards=%s devname=%s driv_name=%s perm=%s\n", 
					devctl->vid, devctl->pid, devctl_info_decode(devctl),
					devctl->major, devctl->netcards, (dev_name(tmp->device))? dev_name(tmp->device):"NULL", 
					(tmp->device->driver->name)? tmp->device->driver->name:"NULL", perm_decode(devctl->perm));
			list_del(pos);
			kfree(tmp);
			mutex_unlock(&devctl_lst_mutex);
			return;
		}
	}
	mutex_unlock(&devctl_lst_mutex);
	return;
}
EXPORT_SYMBOL(dev_perm_clean);

int fs_perm_chk(struct inode *inode)
{
	int ret = 0;
	if(S_ISBLK(inode->i_mode))
	{
		 if((ret=lookup_chrblk(inode->i_rdev)) < 0)
			return 0;
		 if(ret == 1)
		 {
		 	return 1;
		 }
		 else
		 {
		 	return -EPERM;
		 }
	}

	if(S_ISCHR(inode->i_mode))
	{
		if(lookup_chrblk(inode->i_rdev) < 0)
			return 0;
		return -EPERM;
	}

	/* major == 0 - memory file system 
	 * s_bdev == NULL - not a block dev */
	if((MAJOR(inode->i_sb->s_dev) == 0) || 
			inode->i_sb->s_bdev == NULL)
		return 0;

	if((ret = lookup_chrblk(inode->i_sb->s_dev)) < 0)
		return 0;
	if(ret == 1)
		return 1;

	// TODO: you can add uid, gid if can pass through here
	return -EPERM;
}
EXPORT_SYMBOL(fs_perm_chk);

int vm_perm_chk(const char *vm_path)
{
	struct list_head *pos = NULL;
	struct sm_devctl_vm_internal *tmp = NULL;

	list_for_each(pos, &devctl_vm_lst)
	{
		tmp = list_entry(pos, struct sm_devctl_vm_internal, 
				list);

		if(strcmp(tmp->devctl.vm_path, vm_path) == 0)
		{
			printk("run %s not permit.\n",
					vm_path);
			return -EPERM;
		}
	}

	return 0;
}
EXPORT_SYMBOL(vm_perm_chk);

int pkg_perm_chk(const char *pkg_name)
{
	struct list_head *pos = NULL;
	struct sm_pkg_protect_internal *tmp = NULL;

	list_for_each(pos, &pkg_protect_lst)
	{
		tmp = list_entry(pos, struct sm_pkg_protect_internal, 
				list);
		if(strstr(pkg_name, tmp->pkg.pkg_name) != NULL)
		{
			printk("uninstall %s not permit.\n",
					tmp->pkg.pkg_name);
			return -EPERM;
		}
	}
	return 0;
}
EXPORT_SYMBOL(pkg_perm_chk);

int file_perm_chk(const char *process_name)
{
	struct list_head *pos = NULL;
	struct sm_file_protect_internal *tmp = NULL;

	list_for_each(pos, &file_protect_lst)
	{
		tmp = list_entry(pos, struct sm_file_protect_internal, 
				list);
		if(strcmp(process_name, tmp->file.process_name) == 0)
		{
			// printk("%s can access protect file.\n",
			//		tmp->file.process_name);
			return 1;
		}
	}
	return 0;
}
EXPORT_SYMBOL(file_perm_chk);

/* record class address */
void class_addr_add(struct class *cls)
{
	struct list_head *pos = NULL;
	struct class_addr *tmp = NULL;

	if(cls == NULL)
		return;

	list_for_each(pos, &class_addr_lst)
	{
		tmp = list_entry(pos, struct class_addr, 
				list);
		if(strcmp(tmp->cls->name, cls->name) == 0)
		{
			tmp->cls = cls;
			return;
		}
	}

	tmp = kzalloc(sizeof(struct class_addr), 
			GFP_KERNEL);
	tmp->cls = cls;
	list_add(&tmp->list, &class_addr_lst);
}

EXPORT_SYMBOL(class_addr_add);

/* del class address */
void class_addr_del(struct class *cls)
{
	struct list_head *pos = NULL;
	struct list_head *q = NULL;
	struct class_addr *tmp = NULL;

	list_for_each_safe(pos, q, &class_addr_lst)
	{
		tmp = list_entry(pos, struct class_addr, 
				list);
		if(strcmp(tmp->cls->name, cls->name) == 0)
		{
			list_del(pos);
			kfree(tmp);
			return;
		}
	}
	return;
}
EXPORT_SYMBOL(class_addr_del);

LIST_HEAD(devctl_vm_blacklist);

ssize_t key_pro_show(struct kobject *object, struct kobj_attribute *attr, char *buf)
{
        struct sm_devctl_vm_driver *item, *tmp;
        char vm_path[256];
        char all_path[2560];
        int count = 0;
        list_for_each_entry_safe(item, tmp, &devctl_vm_blacklist, list) {
                        sprintf(vm_path, "%s\t", item->vm_path);
                        strcat(all_path, vm_path);
                        count++;
        }
        return sprintf(buf, "%d %s\n", count, all_path);
}

static ssize_t key_pro_store(struct kobject *kobj,struct kobj_attribute *attr, const char *buf, size_t count)
{
        struct sm_devctl_vm_driver *item, *tmp, *new_item;
        char temp_vm_path[MAX_VM_PATH_LEN];
        int ret = 0;
        int char_count;

        char_count=1;
        while(buf[char_count])
        {
                temp_vm_path[char_count-1]=buf[char_count];
                //printk("%c, %c, %d\n", buf[char_count],temp_vm_path[char_count-1], char_count);
                char_count++;
        }
        temp_vm_path[char_count-1]='\0';
        switch(buf[0])
        {
                case '+':
                        list_for_each_entry_safe(item, tmp, &devctl_vm_blacklist, list) {
                                if (strcmp(item->vm_path, temp_vm_path) == 0) {
                                        printk(KERN_INFO "%s already exist in vm_perm_blacklist.\n", temp_vm_path);
                                        return 1;
                                }
                        }
                        new_item = kmalloc(sizeof(struct sm_devctl_vm_driver), GFP_KERNEL);
                        if (!new_item)
                                return -ENOMEM;
                        strncpy(new_item->vm_path, temp_vm_path, MAX_VM_PATH_LEN-1);
                        new_item->vm_path[MAX_VM_PATH_LEN-1] = '\0';
                        list_add(&new_item->list, &devctl_vm_blacklist);
                        printk(KERN_INFO "add %s successfully.\n", temp_vm_path);
                        ret = 0;
                        break;
                case '-':
                        list_for_each_entry_safe(item, tmp, &devctl_vm_blacklist, list) {
                                if (strcmp(item->vm_path, temp_vm_path) == 0) {
                                        list_del(&item->list);
                                        kfree(item);
                                        printk(KERN_INFO "delete %s in vm_perm_blacklist.\n", temp_vm_path);
                                        return 0;
                                }
                        }
                                        printk(KERN_INFO "no %s in vm_perm_blacklist.\n", temp_vm_path);
                        break;
                default:
                        break;
        }
        return 0;
}

static struct kobj_attribute key_pro_attribute=__ATTR(vm_devctl,0644,key_pro_show,key_pro_store);
static __init int vm_devctl_init(void)
{
        int error=0;
        vm_devctlobj = kobject_create_and_add("vm_devctl", kernel_kobj);
        if (!vm_devctlobj)
                return -ENOMEM;
        error=sysfs_create_file(vm_devctlobj,&key_pro_attribute.attr);
        if (error)
        {
                printk("vm_devctl create failed\n");
        }

        return 0;
}
postcore_initcall(vm_devctl_init);

static int get_inf_dev_usb_policy(char *buf, int size){
	int i;
	int idx = 0;
	int len;

	for(i = 0; i < INF_MAX - 1; i++){
		if(size <= idx)
			return -1;
		len = snprintf(buf + idx, size - idx, "%s %s\n", inf_info[i].type_name, perm_decode(cur_cfg_inf[i + 1]));
		idx += len;
	}
	for(i = 0; i < DEV_MAX - 1; i++){
		if(size <= idx)
			return -1;
		len = snprintf(buf + idx, size - idx, "%s %s\n", dev_info[i].type_name, perm_decode(cur_cfg_dev[i + 1]));
		idx += len;
	}
	for(i = 0; i < USB_MAX - 1; i++){
		if(size <= idx)
			return -1;
		len = snprintf(buf + idx, size - idx, "%s %s\n", usb_info[i].type_name, perm_decode(cur_cfg_usb[i + 1]));
		idx += len;
	}

	//printk("line=%d ret=%d\n", __LINE__,idx);
	return idx;
}
static int get_devctl_lst_policy(char *buf, int size){
	struct list_head *pos = NULL;
	struct sm_devctl_internal *tmp = NULL;
	int len;
	int idx = 0;
	
	mutex_lock(&devctl_lst_mutex);
	list_for_each(pos, &devctl_lst)
	{
		if(size <= idx)
			return -1;
		tmp = list_entry(pos, struct sm_devctl_internal, 
				list);
		len = snprintf(buf + idx, size - idx, 
				"vid=%04x pid=%04x type=%s major=%d netcards=%s devname=%s dri_name=%s perm=%s\n",
				tmp->devctl.vid, tmp->devctl.pid, devctl_info_decode(&(tmp->devctl)),
				tmp->devctl.major, tmp->devctl.netcards, (dev_name(tmp->device))? dev_name(tmp->device):"NULL",
				(tmp->driver->name)? tmp->driver->name:"NULL", perm_decode(tmp->devctl.perm));
		idx += len;
	}
	mutex_unlock(&devctl_lst_mutex);

	return idx;
}

static int get_devctl_devt_lst_policy(char *buf, int size){
	struct list_head *pos = NULL;
	struct sm_devctl_devt_internal *tmp = NULL;
	int len;
	int idx = 0;

	mutex_lock(&devctl_mutex);
	list_for_each(pos, &devctl_devt_lst)
	{
		if(size <= idx){
			mutex_unlock(&devctl_mutex);
			return -1;
		}
		tmp = list_entry(pos, struct sm_devctl_devt_internal, list);
		len = snprintf(buf + idx, size - idx,
				"major=%d perm=%s\n",
				tmp->devctl.major, perm_decode(tmp->devctl.perm));
		idx += len;
	}
	mutex_unlock(&devctl_mutex);

	return idx;
}

static int get_devctl_vidpid_lst(char *buf, int size){
	int idx = 0;
	int len;
	struct list_head *pos = NULL;
	struct sm_devctl_vidpid_internal *tmp = NULL;

	list_for_each(pos, &devctl_vidpid_lst)
	{
		if(size <= idx)
			return -1;
		tmp = list_entry(pos, struct 
				sm_devctl_vidpid_internal, 
				list);
		len = snprintf(buf + idx, size - idx,
				"vid=%04x pid=%04x cls=%04x perm=%s\n",
				tmp->devctl.vid, tmp->devctl.pid,
				tmp->devctl.usb_class, perm_decode(tmp->devctl.perm));
		idx += len;
	}

	return idx;
}

static ssize_t devctl_policy_write(struct file *file, const char __user *ubuf, size_t count, loff_t *off){
	return -EPERM;
}
static ssize_t devctl_policy_read(struct file *file, char __user *ubuf, size_t size, loff_t *off){
	int idx = 0;
	int len;
	int policy_sz;

	policy_sz = sizeof(devctl_policy_buf);
	if(*off >= policy_sz)
		return 0;

	memset(devctl_policy_buf, 0x0, policy_sz);
	//len = snprintf(devctl_policy_buf, policy_sz - idx, "interface:\n");
	//idx += len;
	
	len = get_inf_dev_usb_policy(devctl_policy_buf + idx, policy_sz - idx);
	if(len < 0)
		return -ENOMEM;
	idx += len;

	len = get_devctl_lst_policy(devctl_policy_buf + idx, policy_sz - idx);
	if(len < 0)
		return -ENOMEM;
	idx += len;

	len = get_devctl_devt_lst_policy(devctl_policy_buf + idx, policy_sz - idx);
	if(len < 0)
		return -ENOMEM;
	idx += len;

	len = get_devctl_vidpid_lst(devctl_policy_buf + idx, policy_sz - idx);
	if(len < 0)
		return -ENOMEM;
	idx += len;
	//printk("data=\n%s\n", devctl_policy_buf);

	if(*off >= idx)
		return 0;

	if(0 != copy_to_user(ubuf, devctl_policy_buf, idx))
		return -EFAULT;
	*off = idx;

	return idx;
}
const struct file_operations devctl_policy_opt = {
	.read = devctl_policy_read,
	.write = devctl_policy_write,
};
static __init int devctl_policy_init(void){
	int ret = 0;

	nfs_aqtj_dir = proc_mkdir(AQTJ_DIR, NULL);
	if(!nfs_aqtj_dir){
		printk("cannot create /proc/%s\n", AQTJ_DIR);
		ret = -EFAULT;
		goto proc_dir_fail;
	}

	devctl_policy_file = proc_create(AQTJ_DEVCTL, 0444, nfs_aqtj_dir, &devctl_policy_opt);
	if(!devctl_policy_file){
		printk("cannot craete /proc/%s/%s\n", AQTJ_DIR, AQTJ_DEVCTL);
		ret = -EFAULT;
		goto proc_file_fail;
	}

	return ret;
proc_file_fail:
	if(nfs_aqtj_dir){
		proc_remove(nfs_aqtj_dir);
		nfs_aqtj_dir = NULL;
	}
proc_dir_fail:
	return ret;
}
postcore_initcall(devctl_policy_init);
