#include "ac_table.h"
#include "sec_path_trie.h"

static ac_trie_ext_t *__ac_table_insert_entry(void *__trie_obj,
                                              const char *path, unsigned int id)
{
    path_trie_pattern_t *pattern = NULL;
    unsigned short path_len = 0;
    ac_trie_ext_t *ext = NULL;

    path_len = (unsigned short)strlen(path);
    if (path_len < 1)
        return NULL;

    if (1 == path_len && path[0] != '/')
        return NULL;

    if (path[path_len - 1] == '/')
        path_len--;

    pattern = sec_path_trie_insert(__trie_obj, path, path_len,
                                   (void *)(unsigned long)id);
    if (!pattern) {
        return NULL;
    }

    ext = __PATH_TRIE_GET_EXT(ac_trie_ext_t, pattern);

    memset(ext, 0, sizeof(ac_trie_ext_t));

    return ext;
}

int ac_table_insert(void *__trie_obj, const char *path, unsigned int id,
                    unsigned int type, unsigned char sub_type, void *ext_data)
{
    ac_trie_ext_t *ext = NULL;

    ext = __ac_table_insert_entry(__trie_obj, path, id);
    if (!ext)
        return FALSE;

    ext->sub_type = sub_type;
    if (ext_data)
        memcpy(ext->ext_data, ext_data, sizeof(ext->ext_data));

    return TRUE;
}

void *ac_object_create(void)
{
    void *obj = NULL;

    // 1024 * 100 -> 128*100
    obj = sec_path_trie_create(128 * 100, sizeof(ac_trie_ext_t));

    return obj;
}

void ac_object_print(void *__trie_object)
{
    sec_path_trie_print(__trie_object);
    return;
}

void *ac_get_trie_object(ac_trie_t *ac_trie)
{
    void *__trie_obj = NULL;

    spin_lock_bh(&ac_trie->lock);
    __trie_obj = ac_trie->trieObject;
    if (__trie_obj)
        comm_object_addref(__trie_obj);
    spin_unlock_bh(&ac_trie->lock);

    return __trie_obj;
}

// return: TRUE means ok(information will retrieve by pAcitem)
// FALSE means no action will be token.
BOOLEAN ac_table_check(void *ply_head, const char *path,
                       sec_user_item_t *userItem, BOOLEAN b_modify,
                       BOOLEAN b_read, PID_CONTEXT *pid_ctx,
                       unsigned char sub_type, ac_result_t *p_result,
                       ac_trie_check_t ac_trie_check)
{
    BOOLEAN retval = FALSE;
    ac_ctx_t ctx;
    void *__trie_obj = NULL;

    if (!path)
        return FALSE;

    if (ply_head == NULL)
        return FALSE;

    __trie_obj = ply_head;

    memset(&ctx, 0, sizeof(ctx));
    ctx.bValid = FALSE;
    ctx.bModifying = b_modify;
    ctx.bReading = b_read;
    ctx.userItem = userItem;  // -- dont increment ref-counter
    ctx.pidCtx = pid_ctx;
    ctx.opSubType = sub_type;
    ctx.path = (char *)path;

    // tag_pr_debug("b_modify[%d], b_read[%d]\n", b_modify, b_read);

    // todo: 这里，预留了一个对于路径的递归检查，可用于系统保护时，是否递归目录
    sec_path_trie_traverse(__trie_obj, path, strlen(path), TRUE,
                    ac_trie_check, &ctx);
    if (ctx.bValid) {
        tag_pr_debug("retval true\n");

        retval = TRUE;
        *p_result = ctx.acitem;
    }

// DONE:
    return retval;
}

int ac_table_init(ac_trie_t *ac_trie)
{
    spin_lock_init(&ac_trie->lock);

    ac_trie->trieObject = ac_object_create();
    if (!ac_trie->trieObject) {
        tag_pr_err("ac object create failed\n");
        return FALSE;
    }

    return TRUE;
}
void ac_table_uninit(void * trie_obj)
{
    if (NULL != trie_obj) {
        comm_object_release(trie_obj);
        trie_obj = NULL;
    }
}
