#ifndef __AC_TABLE_H_
#define __AC_TABLE_H_

#include "common/sec_path_trie.h"
#include "common/ac_result.h"
#include "common/user_item.h"
#include "common/pid_ctx.h"

// ===================================================================
// 各种ac相关的私有数据
typedef struct __mac_obj_ext
{
    unsigned int level;     // 当前对象的等级
    unsigned int model;     // 模型
} mac_obj_ext_t;

#define PROTECT_TYPE_FILE 1
#define PROTECT_TYPE_DIR 2
typedef struct __protect_ext
{
    unsigned int type;  // 文件或目录
    // 对于目录，是否需要迭代，0/1 否/是
    unsigned int is_recursion;  
} protect_ext_t;
// ===================================================================

// todo 暂时先去掉各种类型，此处做一层封装，方便操作
// 具体里面的内容，还是有根据sub_type转成不同的格式（是否用此字段？）
// type是否有用？
typedef struct __AC_TRIE_EXT {
    unsigned int id;
    unsigned short sub_type;
    // void * ext_data;
    char ext_data[128]; // 似乎空间就够用了，省的分配空间
} __attribute__((packed)) ac_trie_ext_t;

typedef struct __ac_ctx {
    ac_result_t acitem;
    BOOLEAN bValid;

    sec_user_item_t *userItem;
    PID_CONTEXT *pidCtx;

    char *path;

    BOOLEAN bModifying;
    BOOLEAN bReading;
    unsigned char opSubType;
} ac_ctx_t;

typedef struct __ACCESS_TRIE {
    void *trieObject;
    spinlock_t lock;
} ac_trie_t;

void ac_object_print(void *__trie_object);

int ac_table_init(ac_trie_t *ac_trie);
void ac_table_uninit(void * trie_obj);

void *ac_object_create(void);

typedef PATH_TRIE_CHECK_ROUTINE_FUNC ac_trie_check_t;

BOOLEAN ac_table_check(void * ply_head, const char *tftPath, sec_user_item_t *userItem,
                              BOOLEAN bModifying, BOOLEAN bReading,
                              PID_CONTEXT *pidCtx, 
                              unsigned char sub_type,
                              ac_result_t *pAcitem,
                              ac_trie_check_t ac_trie_check);

int ac_table_insert(void *__trie_obj, const char *path, 
    unsigned int id, unsigned int type,
    unsigned char sub_type, void * ext_data);

#endif