#include <project.h>

#include "audit_map.h"

BOOLEAN audit_map_init(audit_map_t *table, HAHSTB_VALUE __get_value_ptr,
                     HAHSTB_RELEASE release_item, unsigned int valueLen)
{
    BOOLEAN retval = FALSE;
    int bucket_count = (4096 * 4);

    spin_lock_init(&table->lock);

    __list_init(&table->cacheList);

    hashtb_init(&table->lookupTB, __get_value_ptr, release_item, valueLen);

    while (bucket_count > 10) {
        if (!hashtb_set_buckets(&table->lookupTB, bucket_count)) {
        } else {
            break;
        }
        bucket_count = (bucket_count / 2);
    }
    if (bucket_count <= 10)
        return FALSE;

    retval = TRUE;
    return retval;
}

static void __audit_map_clear(audit_map_t *table)
{
    while (!__list_is_empty(&table->cacheList)) {
        LIST_HEAD *entry;
        audit_cache_hdr_t *item;

        __list_remove_tail(&table->cacheList, entry);
        item = OBJECT_PTR(entry, audit_cache_hdr_t, chead);

        comm_object_release(item);
    }
    hashtb_remove_all(&table->lookupTB);
    return;
}

void audit_map_clear(audit_map_t *table)
{
    spin_lock_bh(&table->lock);
    __audit_map_clear(table);
    spin_unlock_bh(&table->lock);
    return;
}

void audit_map_destroy(audit_map_t *table)
{
    __audit_map_clear(table);
    hashtb_destroy(&table->lookupTB);
    return;
}

static void __audit_map_update(audit_map_t *table, audit_cache_hdr_t *item)
{
    __list_remove_item(&item->chead);
    __list_add_tail(&table->cacheList, &item->chead);
    return;
}

audit_cache_hdr_t *audit_map_lookup(audit_map_t *table, void *val_ptr)
{
    audit_cache_hdr_t *item = NULL;

    LIST_HEAD *entry;

    spin_lock_bh(&table->lock);

    entry = hashtb_find(&table->lookupTB, (unsigned int *)val_ptr);
    if (entry) {
        item = OBJECT_PTR(entry, audit_cache_hdr_t, hhead);
        comm_object_addref(item);
        __audit_map_update(table, item);
    }

    // delete only one item
    if (hash_get_item_count(&table->lookupTB) > __MAX_CACHE_COUNT) {
        audit_cache_hdr_t *abortItem = NULL;
        LIST_HEAD *__abort_entry;
        __list_remove_head(&table->cacheList, __abort_entry);

        abortItem = OBJECT_PTR(__abort_entry, audit_cache_hdr_t, chead);
        hashtb_remove_item_entry(&table->lookupTB, &abortItem->hhead);
        comm_object_release(abortItem);
    }

    spin_unlock_bh(&table->lock);

    return item;
}

int audit_map_insert(audit_map_t *table, audit_cache_hdr_t *item)
{
    int retval = -1;
    LIST_HEAD *entry;
    audit_cache_hdr_t *oldItem = NULL;
    void *val_ptr = table->lookupTB.get_value(&item->hhead);

    spin_lock_bh(&table->lock);
    entry = hashtb_find(&table->lookupTB, (unsigned int *)val_ptr);
    if (entry) {
        oldItem = OBJECT_PTR(entry, audit_cache_hdr_t, hhead);
        
        // release a ref-count
        hashtb_remove_item_entry(&table->lookupTB, entry);  

        // a ref-count keep for oldItem itself!
        __list_remove_item(&oldItem->chead);  
    }

    if (!hashtb_insert_item(&table->lookupTB, &item->hhead,
                            (unsigned int *)val_ptr)) {
        // object_release(item);
        // item = NULL;
        goto DONE;
    } else {
        comm_object_addref(item); 
    }

    __list_add_tail(&table->cacheList, &item->chead);
    comm_object_addref(item); 

    retval = 0;
DONE:
    spin_unlock_bh(&table->lock);

    if (oldItem) 
        comm_object_release(oldItem);  // ref for cacheList
    
    return retval;
}
