
#include <linux/kernel.h>
#include "common/general/include/audit_mod.h"

audit_str_map_t audit_type_map[] = {
    {AUDIT_TYPE_IMA, "ima"},
    {AUDIT_TYPE_AC, "ac"},
    {AUDIT_TYPE_PROTECT, "protect"},
};
const char* audit_type_str(int type) 
{
    int i;

    for (i = 0; i < ARRAY_SIZE(audit_type_map); i++) {
        if (audit_type_map[i].type == type) {
            return audit_type_map[i].str;
        }
    }
    return "Unknown";
}

audit_str_map_t audit_subtype_map[] = {
    {AUDIT_SUBTYPE_STATIC_MEASURE, "Static Measure"},
    {AUDIT_SUBTYPE_KERNEL_MEASURE, "Kernel Measure"},
    {AUDIT_SUBTYPE_DYNAMIC_MEASURE, "Dynamic Measure"},

    {AUDIT_SUBTYPE_MAC, "Mandatory access control"},
    {AUDIT_SUBTYPE_DAC, "Discretionary access control"},
    {AUDIT_SUBTYPE_PAC, "Peripheral access control"},
    {AUDIT_SUBTYPE_NAC, "Network access control"},

    {AUDIT_SUBTYPE_PROC_PROTECT, "Process Protection"},
    {AUDIT_SUBTYPE_KMOD_PROTECT, "Kernel Module Protection"},
    {AUDIT_SUBTYPE_FILE_PROTECT, "File Protection"},
    {AUDIT_SUBTYPE_DIR_PROTECT, "Directory Protection"},
    {AUDIT_SUBTYPE_SELF_PROTECT, "Self Protection"},
    {AUDIT_SUBTYPE_APP_VERIFY, "Application Verification"},
    {AUDIT_SUBTYPE_KMOD_VERIFY, "Kernel Module Verification"},

    {AUDIT_SUBTYPE_SET_REINFORE, "Set Reinforcement"},
    {AUDIT_SUBTYPE_SET_BASELINE, "Set Baseline"},
    {AUDIT_SUBTYPE_SET_SECSWITCH, "Set Security Switch"},
};

const char* audit_subtype_str(int subtype) 
{
    int i;

    for (i = 0; i < ARRAY_SIZE(audit_subtype_map); i++) {
        if (audit_subtype_map[i].type == subtype) {
            return audit_subtype_map[i].str;
        }
    }
    return "Unknown";
}


audit_str_map_t audit_ope_map[] = {
    {AUDIT_OPE_FILE_EXEC, "file exec"},
    {AUDIT_OPE_FILE_READ, "file read"},
    {AUDIT_OPE_FILE_WRITE, "file write"},
    {AUDIT_OPE_FILE_APPEND, "file append"},
    {AUDIT_OPE_FILE_RENAME, "file rename"},
    {AUDIT_OPE_FILE_UNLINK, "file unlink"},
    {AUDIT_OPE_FILE_MKDIR, "mkdir"},
    {AUDIT_OPE_FILE_RMDIR, "rmdir"},
    {AUDIT_OPE_FILE_NEW, "file new"},
    {AUDIT_OPE_FILE_LOAD_LIB, "load lib"},
    {AUDIT_OPE_FILE_INSMOD, "insmod kmod"},
    {AUDIT_OPE_FILE_KILL, "task kill"},
    {AUDIT_OPE_FILE_RMMOD, "rmmod"},
};

const char* audit_ope_str(int ope)
{
    int i;

    for (i = 0; i < ARRAY_SIZE(audit_ope_map); i++) {
        if (audit_ope_map[i].type == ope) {
            return audit_ope_map[i].str;
        }
    }
    return "Unknown";
}