#include "sec_ply_if.h"
#include "common/general/include/path_def.h"
#include "common/kmod_check_table.h"
#include "common/config/ply_switch.h"
#include "common/ac_table.h"
#include "common/sec_user_table.h"
#include "common/file_access_table.h"
#include "common/image_table.h"
#include "common/sec_ext_table.h"
#include "common/proc_check_table.h"
#include "common/service_check_table.h"
#include "common/nac_ply.h"

int ply_head_check(sec_ply_if_t * ply_if)
{
    int retval = CFG_CHECK_ERR;
    int ret = 0;

    WL_FILE_HDR hdr;

    CIPHER_FILE * fp = NULL;
    LONGLONG file_len = 0;

    tag_pr_debug("ply head check path[%s]\n", ply_if->path);

    fp = kmalloc(sizeof(*fp), GFP_KERNEL);
    if (fp == NULL)
        goto DONE;

    memset(fp, 0, sizeof(*fp));

    if (0 != cipher_file_open(fp, ply_if->path, "rb"))
        goto DONE;

    if (0 != cipher_file_get_length(fp, &file_len))
        goto DONE;

    file_len = file_len - sizeof(WL_FILE_TAIL) - sizeof(hdr);
    if (file_len < 0) {
        retval = CFG_CHECK_WRONG;
        goto DONE;
    }

    ret = cipher_file_read((char *)&hdr, sizeof(hdr), 1, fp);
    if (1 != ret) {
        retval = CFG_CHECK_WRONG;
        goto DONE;
    }    

    if (hdr.fileMagic != MAGIC_FILE_HDR) {
        retval = CFG_CHECK_WRONG;
        goto DONE;
    }

    ply_if->file_len = file_len;
    ply_if->fp = fp;

    retval = CFG_CHECK_RIGHT;
DONE:
    return retval;
}

int ply_tail_check(sec_ply_if_t * ply_if)
{
    int retval = CFG_CHECK_ERR;
    int ret = 0;

    unsigned char sum[16] = {0};
    WL_FILE_TAIL tail;

    cipher_file_fetch_md5sum(sum, ply_if->fp);

    ret = cipher_file_read((char *)&tail, sizeof(tail), 1, ply_if->fp);
    if (1 != ret) {
        retval = CFG_CHECK_WRONG;
        goto DONE;
    }

    if (tail.magic != MAGIC_FILE_HDR) {
        retval = CFG_CHECK_WRONG;
        goto DONE;
    }

    if (0 != memcmp(sum, tail.md5sum, 16)) {
        retval = CFG_CHECK_WRONG;
        goto DONE;
    }

    retval = CFG_CHECK_RIGHT;
DONE:
    if (ply_if->fp) {
        cipher_file_close(ply_if->fp);
        kfree(ply_if->fp);
    }

    return retval;
}

void ply_post_handle(sec_ply_if_t * ply_if, int ret, void ** new_trie)
{
    // 成功 释放旧策略，切换为新策略
    if (ret == 0) {
        tag_pr_debug("=========\n");
        mutex_lock(&ply_if->sec_ply_lock);

        // notice: 这里处理不当（如直接释放）的话，会导致内核崩溃，且挂在别处
        if (ply_if->ply_head) {
            ac_table_uninit(ply_if->ply_head);
        }
        ply_if->ply_head = *new_trie;
        ply_if->ply_cnt = ply_if->__item_cnt;
        mutex_unlock(&ply_if->sec_ply_lock);

    // 失败，释放新策略
    // todo，是否需要审计或报错
    } else { 
        tag_pr_debug("=========\n");

        if (new_trie && *new_trie) {
            ac_table_uninit(*new_trie);
        }
    }
}

// todo 这里的用户处理是否正确？还是需要改sec_user_table？？？？
void ply_post_handle_user(sec_ply_if_t * ply_if, int ret, void ** new_ply)
{
    // 成功,释放旧策略，切换为新策略
    if (ret == 0) {
        mutex_lock(&ply_if->sec_ply_lock);

        // todo: update
        // if (ply_if->ply_head) {
        //     sec_user_table_update_hash_tb(*new_ply);
        //     // todo 如何释放旧的
        // }
        ply_if->ply_head = *new_ply;
        ply_if->ply_cnt = ply_if->__item_cnt;
        mutex_unlock(&ply_if->sec_ply_lock);

        tag_pr_debug("new_ply[%p]\n", *new_ply);
        tag_pr_debug("ply_if->ply_head[%p]\n", (unsigned int *)ply_if->ply_head);
    // 失败，释放新策略
    // todo，是否需要审计或报错
    } else { 
        // todo !!!!!!
        if (new_ply && *new_ply) {
            sec_user_table_destroy_hash_tb(*new_ply);
            kfree(*new_ply);            
            *new_ply = NULL;
        }
    }
}

void ply_post_handle_ima(sec_ply_if_t * ply_if, int ret, void ** new_trie)
{
    // 成功, 释放旧策略，切换为新策略
    if (ret == 0) {
        mutex_lock(&ply_if->sec_ply_lock);
        if (ply_if->ply_head) {
            wlImageHashTBDestroy(ply_if->ply_head);
        }
        ply_if->ply_head = *new_trie;
        ply_if->ply_cnt = ply_if->__item_cnt;
        mutex_unlock(&ply_if->sec_ply_lock);
        tag_pr_debug("ply_if->ply_cnt[%d]\n", ply_if->ply_cnt);
        tag_pr_debug("ply_if->ply_head[%x]\n", (unsigned int*)ply_if->ply_head);

    // 失败，释放新策略
    } else { 
        if (new_trie && *new_trie) {
            wlImageHashTBDestroy(*new_trie);
        }
    }
}

void ply_post_handle_ima_ext(sec_ply_if_t * ply_if, int ret, void ** new_trie)
{
    // 成功, 释放旧策略，切换为新策略
    if (ret == 0) {
        mutex_lock(&ply_if->sec_ply_lock);
        if (ply_if->ply_head) {
            sec_ext_table_destroy(ply_if->ply_head);
            kfree(ply_if->ply_head);
        }
        ply_if->ply_head = *new_trie;
        ply_if->ply_cnt = ply_if->__item_cnt;
        mutex_unlock(&ply_if->sec_ply_lock);

    // 失败，释放新策略
    } else { 
        if (new_trie && *new_trie) {
            sec_ext_table_destroy(*new_trie);
            kfree(*new_trie);            
            *new_trie = NULL;
        }
    }
}

void ply_post_handle_proc(sec_ply_if_t * ply_if, int ret, void ** new_trie)
{
    // todo： 成功
    // 释放旧策略，切换为新策略
    if (ret == 0) {
        mutex_lock(&ply_if->sec_ply_lock);
        if (ply_if->ply_head) {
            comm_object_release(ply_if->ply_head);
        }
        ply_if->ply_head = *new_trie;
        ply_if->ply_cnt = ply_if->__item_cnt;

        if (0 == service_access_build_trie_object(ply_if->ply_head)) {
            service_access_update_trie_object(ply_if->ply_head);
        }
        mutex_unlock(&ply_if->sec_ply_lock);

    // 失败，释放新策略
    // todo，是否需要审计或报错
    } else { 
        if (new_trie && *new_trie) {
            comm_object_release(*new_trie);
            // kfree(*new_trie);            
            // *new_trie = NULL;
        }
    }
}

void ply_post_handle_kmod(sec_ply_if_t * ply_if, int ret, void ** new_trie)
{
    // todo： 成功
    // 释放旧策略，切换为新策略
    if (ret == 0) {
        mutex_lock(&ply_if->sec_ply_lock);
        if (ply_if->ply_head) {
            comm_object_release(ply_if->ply_head);
        }
        ply_if->ply_head = *new_trie;
        ply_if->ply_cnt = ply_if->__item_cnt;

        if (0 == kmod_check_build_trie_object(ply_if->ply_head)) {
            kmod_check_update_trie_object(ply_if->ply_head);
        }
        mutex_unlock(&ply_if->sec_ply_lock);

    // 失败，释放新策略
    // todo，是否需要审计或报错
    } else { 
        if (new_trie && *new_trie) {
            comm_object_release(*new_trie);
            // kfree(*new_trie);            
            // *new_trie = NULL;
        }
    }
}

void ply_post_handle_nac(sec_ply_if_t * ply_if, int ret, void ** ply_head)
{
    // todo： 成功
    // 释放旧策略，切换为新策略
    if (ret == 0) {
        mutex_lock(&ply_if->sec_ply_lock);
        nac_ply_switch(*ply_head);
        ply_if->ply_cnt = ply_if->__item_cnt;
        mutex_unlock(&ply_if->sec_ply_lock);

    // 失败，释放新策略
    // todo，是否需要审计或报错
    } else { 
        if (ply_head && *ply_head) {
            nac_ply_head_free(*ply_head);
        }
    }
}

// todo: 初始化、切换策略、释放
static int ply_load_mac_sub(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    SEC_USER_ITEM_STOR *pItem = NULL;
    SID_STOR sid;

    HASHTB * ply = NULL;

    pItem = (SEC_USER_ITEM_STOR *)kmalloc(sizeof(SEC_USER_ITEM_STOR), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    ply = sec_user_table_create_hash_tb();
    if (!ply)
        goto DONE;

    *new_ply = ply;

    count = ((unsigned int)ply_if->file_len) / sizeof(SEC_USER_ITEM_STOR);

    for (i = 0; i < count; i++) {
        sec_user_item_t *userItem = NULL;

        read_cnt =
            cipher_file_read((char *)pItem, sizeof(SEC_USER_ITEM_STOR), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != SEC_USR_ITEM_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        memset(&sid, 0, sizeof(sid));
        sid.id = pItem->sid.id;
        sid.model = pItem->sid.model;

        tag_pr_debug("=====pItem->sid.id[%d]\n", pItem->sid.id);
        tag_pr_debug("=====pItem->user_id[%d]\n", pItem->user_id);
        tag_pr_debug("=====pItem->model[%d]\n", pItem->sid.model);
        tag_pr_debug("=====pItem->rank[%d]\n", pItem->rank);

        // todo lock
        userItem = sec_user_table_lookup_hash_tb(ply, &sid);
        if (!userItem) {
            userItem = sec_user_table_insert_hash_tb(ply, pItem);
            if (userItem) {
                userItem->rank = pItem->rank;
                userItem->integrityRank = pItem->integrityRank;
                userItem->model = pItem->sid.model;
                comm_object_release(userItem);
                userItem = NULL;
            }
        } else {
            userItem->rank = pItem->rank;
            userItem->integrityRank = pItem->integrityRank;
            userItem->model = pItem->sid.model;
            comm_object_release(userItem);
            userItem = NULL;
        }
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
    // todo: 错误处理
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, __FILE_IDX_SYS_USER_INFO);

    if (pItem)
        kfree(pItem);

    return retval;
}

static int ply_load_mac_obj(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    SEC_GUARD_ITEM *pItem = NULL;

    void * ply_trie = NULL;

    pItem = (SEC_GUARD_ITEM *)kmalloc(sizeof(SEC_GUARD_ITEM), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    ply_trie = ac_object_create();
    if (!ply_trie)
        goto DONE;

    *new_ply = ply_trie;

    count = ((unsigned int)ply_if->file_len) / sizeof(SEC_GUARD_ITEM);

    for (i = 0; i < count; i++) {
        int __len;

        read_cnt =
            cipher_file_read((char *)pItem, sizeof(SEC_GUARD_ITEM), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != SEC_GUARD_ITEM_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        pItem->path[sizeof(pItem->path) / sizeof(pItem->path[0]) - 1] = 0;

        // tag_pr_debug("type[%d]\n", type);
        // tag_pr_debug("pItem->path[%s]\n", pItem->path);

        __len = strlen(pItem->path);
        if (__len <= 0) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if ('/' == pItem->path[__len - 1] || '\\' == pItem->path[__len - 1]) {
            pItem->path[__len - 1] = 0;
            __len--;
        }

        // todo: 是否需要将强访客体的完整性纳入此处？？？ 
        // if (WPFILE_SYSOBJECT_MAC == type) {
            // -- pItem->itemType:rank
            if (pItem->itemType > 0) {

                // todo : 将rank等结构封装，转换为ext_data

                mac_obj_ext_t ext_data;

                ext_data.level = pItem->itemType;
                ext_data.model = pItem->model;
                
                tag_pr_debug("insert ply mac_obj, path[%s], id[%d], model[%d]\n", pItem->path, pItem->id, pItem->model);

                ac_table_insert(ply_trie, pItem->path, 
                    pItem->id, ply_if->type,
                    pItem->sub_type, &ext_data);
            }

        // } 
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
        tag_pr_debug("\n");

    // todo: 错误处理
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, file_id);

    if (pItem)
        kfree(pItem);

    return retval;
}

static int ply_load_ima(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    SYSTEM_HASH_ITEM *pItem = NULL;
    HASHTB *hashTB = NULL;

    pItem = (SYSTEM_HASH_ITEM *)kmalloc(sizeof(SYSTEM_HASH_ITEM), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    count = ((unsigned int)ply_if->file_len) / sizeof(SYSTEM_HASH_ITEM);

    hashTB = wlImageHashTBCreateExt(count);
    if (!hashTB)
        goto DONE;

    *new_ply = hashTB;
    
    // tag_pr_debug("hashTB:[%x]\n", hashTB);

    for (i = 0; i < count; i++) {
        read_cnt =
            cipher_file_read((char *)pItem, sizeof(SYSTEM_HASH_ITEM), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != SYSHASH_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }
        // tag_pr_debug("insert ima [%s]\n", pItem->path);
        // digest_show(pItem->bhash_code, sizeof(pItem->bhash_code));

        wlImageHashTBInsert(hashTB, pItem->bhash_code, pItem->type);
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
    tag_pr_debug("\n");

    // todo: 错误处理
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, file_id);

    if (pItem)
        kfree(pItem);

    return retval;
}

static int ply_load_ima_ext(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    SEC_EXT_ITEM *pItem = NULL;
    sec_ext_table_t * ext_tb = NULL;

   pItem = (SEC_EXT_ITEM *)kmalloc(sizeof(SEC_EXT_ITEM), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    count = ((unsigned int)ply_if->file_len) / sizeof(SEC_EXT_ITEM);

    ext_tb = sec_ext_table_init(count);
    if (!ext_tb)
        goto DONE;

    *new_ply = ext_tb;

    for (i = 0; i < count; i++) {
        SEC_EXT_TYPE *extType = NULL;

        read_cnt =
            cipher_file_read((char *)pItem, sizeof(SEC_EXT_ITEM), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != SEC_EXT_ITEM_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        pItem->ext[sizeof(pItem->ext) / sizeof(pItem->ext[0]) - 1] = 0;

        extType = sec_ext_table_add(ext_tb, pItem->ext);
        if (extType) {
            comm_object_release(extType);
            extType = NULL;
        }
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
    tag_pr_debug("\n");

    // todo: 错误处理
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, file_id);

    if (pItem)
        kfree(pItem);

    return retval;
}

static int ply_load_protect_kmod(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    KMOD_ITEM *pItem = NULL;
    void * ply_trie = NULL;

    pItem = (KMOD_ITEM *)kmalloc(sizeof(KMOD_ITEM), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    ply_trie = kmod_check_create_trie_object();
    if (!ply_trie)
        goto DONE;

    *new_ply = ply_trie;

    count = ((unsigned int)ply_if->file_len) / sizeof(KMOD_ITEM);

    for (i = 0; i < count; i++) {
        read_cnt = cipher_file_read((char *)pItem, sizeof(KMOD_ITEM), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != KMOD_ITEM_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        pItem->mod_name[sizeof(pItem->mod_name) / sizeof(pItem->mod_name[0]) -
                        1] = 0;
tag_pr_debug("ply id[%d] mod_name[%s]\n", pItem->id, pItem->mod_name);

            kmod_check_insert(ply_trie, pItem->mod_name, pItem->id);
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
    // todo
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, __FILE_IDX_KMOD_CHECK_CONF);

    if (pItem)
        kfree(pItem);

    return retval;
}

static int ply_load_protect_proc(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    SEC_SRV_ITEM *pItem = NULL;
    void * ply_trie = NULL;

    ply_trie = service_access_create_trie_object();
    // ply_trie = proc_check_create_trie_object();
    if (!ply_trie)
        goto DONE;

    pItem = (SEC_SRV_ITEM *)kmalloc(sizeof(SEC_SRV_ITEM), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    *new_ply = ply_trie;

    count = ((unsigned int)ply_if->file_len) / sizeof(SEC_SRV_ITEM);

    for (i = 0; i < count; i++) {
        read_cnt = cipher_file_read((char *)pItem, sizeof(SEC_SRV_ITEM), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != SEC_SRV_ITEM_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        pItem->path[sizeof(pItem->path) / sizeof(pItem->path[0]) - 1] = 0;

        tag_pr_debug("load ply, protect proc[%d:%s]\n", pItem->id, pItem->path);

        if (ply_trie == NULL)
            tag_pr_debug("ply trie is null\n");

        tag_pr_debug("ply_tire[%x]\n", (unsigned int*)ply_trie);

        // proc_check_insert(ply_trie, pItem->path, pItem->id, ply_if->type);
        service_access_insert(ply_trie, pItem->path, pItem->id, 4);

        // todo： pItem->sub_type： PROTECT_PROCESS,4 这里应该使用什么？
        // ac_table_insert(ply_trie, pItem->path, 
        //             pItem->id, ply_if->type,
        //             4, NULL);
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
    // todo
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, __FILE_IDX_KMOD_CHECK_CONF);

    if (pItem)
        kfree(pItem);

    return retval;
}


static int ply_load_protect_dir(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    SEC_GUARD_ITEM *pItem = NULL;
    void * ply_trie = NULL;

    ply_trie = ac_object_create();
    if (!ply_trie)
        goto DONE;

    pItem = (SEC_GUARD_ITEM *)kmalloc(sizeof(SEC_GUARD_ITEM), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    *new_ply = ply_trie;

    count = ((unsigned int)ply_if->file_len) / sizeof(SEC_GUARD_ITEM);

    for (i = 0; i < count; i++) {
        protect_ext_t ext_data;
        int __len;

        read_cnt = cipher_file_read((char *)pItem, sizeof(SEC_GUARD_ITEM), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != SEC_GUARD_ITEM_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        pItem->path[sizeof(pItem->path) / sizeof(pItem->path[0]) - 1] = 0;
       
        __len = strlen(pItem->path);
        if (__len <= 0) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if ('/' == pItem->path[__len - 1]) {
            pItem->path[__len - 1] = 0;
            __len--;
        }

        ext_data.type = PROTECT_TYPE_DIR;

        // todo: 目前是够用的，所以，先使用这个字段，后续根据需求调整
        ext_data.is_recursion = pItem->model;

        tag_pr_debug("load ply, protect dir[%d:%s]\n", pItem->id, pItem->path);

        ac_table_insert(ply_trie, pItem->path,  pItem->id, ply_if->type,
                    pItem->sub_type, &ext_data);
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
    // todo
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, __FILE_IDX_KMOD_CHECK_CONF);

    if (pItem)
        kfree(pItem);

    return retval;
}

static int ply_load_protect_file(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    SEC_GUARD_ITEM *pItem = NULL;
    void * ply_trie = NULL;

    ply_trie = ac_object_create();
    if (!ply_trie)
        goto DONE;

    pItem = (SEC_GUARD_ITEM *)kmalloc(sizeof(SEC_GUARD_ITEM), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    *new_ply = ply_trie;

    count = ((unsigned int)ply_if->file_len) / sizeof(SEC_GUARD_ITEM);

    for (i = 0; i < count; i++) {
        protect_ext_t ext_data;

        read_cnt = cipher_file_read((char *)pItem, sizeof(SEC_GUARD_ITEM), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != SEC_GUARD_ITEM_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        pItem->path[sizeof(pItem->path) / sizeof(pItem->path[0]) - 1] = 0;

        ext_data.type = PROTECT_TYPE_FILE;

        tag_pr_debug("load ply, protect file[%d:%s]\n", pItem->id, pItem->path);

        ac_table_insert(ply_trie, pItem->path,  pItem->id, ply_if->type,
                    pItem->sub_type, &ext_data);
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
    // todo
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, __FILE_IDX_KMOD_CHECK_CONF);

    if (pItem)
        kfree(pItem);

    return retval;
}

static int ply_load_protect_self(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    SEC_GUARD_ITEM *pItem = NULL;
    void * ply_trie = NULL;

    ply_trie = ac_object_create();
    if (!ply_trie)
        goto DONE;

    pItem = (SEC_GUARD_ITEM *)kmalloc(sizeof(SEC_GUARD_ITEM), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    *new_ply = ply_trie;

    count = ((unsigned int)ply_if->file_len) / sizeof(SEC_GUARD_ITEM);

    for (i = 0; i < count; i++) {
        protect_ext_t ext_data;
        int __len;

        read_cnt = cipher_file_read((char *)pItem, sizeof(SEC_GUARD_ITEM), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != SEC_GUARD_ITEM_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        __len = strlen(pItem->path);
        if (__len <= 0) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if ('/' == pItem->path[__len - 1] || '\\' == pItem->path[__len - 1]) {
            pItem->path[__len - 1] = 0;
            __len--;
        }

        ext_data.type = PROTECT_TYPE_DIR; // todo
        ext_data.is_recursion = TRUE;

        tag_pr_debug("load ply, protect self[%d:%s]\n", pItem->id, pItem->path);

        ac_table_insert(ply_trie, pItem->path, pItem->id, ply_if->type,
                    pItem->sub_type, &ext_data);
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
    // todo
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, __FILE_IDX_KMOD_CHECK_CONF);

    if (pItem)
        kfree(pItem);

    return retval;
}

static int ply_load_nac(sec_ply_if_t * ply_if, void ** new_ply)
{
    int retval = -1;
    int ret = CFG_CHECK_RIGHT;

    unsigned int count = 0, i = 0, read_cnt = 0;

    nac_rule_item_t *pItem = NULL;
    nac_ply_head_t *ply_head = NULL; 
    nac_rule_node_t *nac_rule = NULL;

    nac_ip_range_t ip_range;
    nac_port_range_t port_range;

    ply_head = nac_ply_head_new();
    if (!ply_head)
        goto DONE;

    pItem = (nac_rule_item_t *)kmalloc(sizeof(nac_rule_item_t), GFP_ATOMIC);
    if (!pItem)
        goto DONE;

    *new_ply = ply_head;

    count = ((unsigned int)ply_if->file_len) / sizeof(nac_rule_item_t);

    for (i = 0; i < count; i++) {
        read_cnt = cipher_file_read((char *)pItem, sizeof(nac_rule_item_t), 1, ply_if->fp);
        if (1 != read_cnt) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        if (pItem->magic != NAC_RULE_ITEM_MAGIC) {
            ret = CFG_CHECK_WRONG;
            goto DONE;
        }

        // 将策略转化为内核层数据
        ip_range.from = ntohl(pItem->ip_from);
        ip_range.to = ntohl(pItem->ip_to);
        port_range.from = pItem->port_from;
        port_range.to = pItem->port_to;
        
    tag_pr_debug("pItem->port_from:[%d], pItem->port_to[%d]\n", pItem->port_from, pItem->port_to);
    tag_pr_debug("pItem->ip_from:[%pI4], pItem->ip_to[%pI4]\n", &ip_range.from, &ip_range.to);
    tag_pr_debug("pItem->proc:[%s]\n", pItem->proc);

        nac_rule = nac_rule_new(pItem->direction, pItem->protocol, &ip_range, &port_range, pItem->proc);
        nac_ply_add_rule(ply_head, pItem->level, pItem->action, nac_rule);
    }

    ply_if->__item_cnt = count;

    retval = 0;
DONE:
    // todo
    // if (ret == CFG_CHECK_WRONG)
    //     config_file_corrupt_notify(filePath, __FILE_IDX_KMOD_CHECK_CONF);

    if (pItem)
        kfree(pItem);

    return retval;
}

// todo: 名称和类型需要和应用层对应
// todo: 存在 一个开关 对应 多个策略  的情况
sec_ply_if_t sec_ply_ifs[] = {
    {
        .sw = {
            .type = SEC_SW_MAC, 
            .b_ctrl = 0, 
            .b_audit = 0,
            .key = "mac",
            .__on_switch_change = NULL, 
        },
        .path = PATH_MAC_USER_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_MAC_SUB, 
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_mac_sub,
        .post_handle = ply_post_handle_user,
    },
    {
        .sw = {
            .type = SEC_SW_MAC, 
            .b_ctrl = 0, 
            .b_audit = 0,
            .key = "mac", 
            .__on_switch_change = NULL, 
        },
        .path = PATH_MAC_OBJECT_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_MAC_OJB, 
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_mac_obj,
        .post_handle = ply_post_handle,
    },
    {
        .sw = {
            .type = SEC_SW_NAC, 
            .b_ctrl = 0,
            .b_audit = 0,
            .key = "nac",
            .__on_switch_change = NULL, 
        },
        .path = PATH_NAC_RULE_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_NAC,
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_nac,
        .post_handle = ply_post_handle_nac,
    },
    {
        .sw = {
            .type = SEC_SW_IMA, 
            .b_ctrl = 0, //
            .b_audit = 0,
            .key = "ima",
            .__on_switch_change = NULL, 
        },
        .path = PATH_IMA_WL_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_IMA, 
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_ima,
        .post_handle = ply_post_handle_ima,
    },
    {
        .sw = {
            .type = SEC_SW_IMA, 
            .b_ctrl = 0, 
            .b_audit = 0,
            .key = "ima", 
            .__on_switch_change = NULL, 
        },
        .path = PATH_EXT_NAME_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_IMA_EXT, 
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_ima_ext,
        .post_handle = ply_post_handle_ima_ext,
    },
    {
        .sw = {
            .type = SEC_SW_PROT_KMOD, 
            .b_ctrl = 0, 
            .b_audit = 0,
            .key = "prot_kmod",
            .__on_switch_change = NULL, 
        },
        .path = PATH_PROTECT_KMOD_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_PROTECT_KMOD,
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_protect_kmod,
        .post_handle = ply_post_handle_kmod,
    },
    {
        .sw = {
            .type = SEC_SW_PROT_PROC,
            .b_ctrl = 0, 
            .b_audit = 0,
            .key = "prot_proc", 
            .__on_switch_change = NULL, 
        },
        .path = PATH_PROTECT_PROC_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_PROTECT_PROC,
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_protect_proc,
        .post_handle = ply_post_handle_proc,
    },
    {
        .sw = {
            .type = SEC_SW_PROT_FILE,
            .b_ctrl = 0, 
            .b_audit = 0,
            .key = "prot_file",
            .__on_switch_change = NULL, 
        },
        .path = PATH_PROTECT_FILE_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_PROTECT_FILE,
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_protect_file,
        .post_handle = ply_post_handle,
    },
    {
        .sw = {
            .type = SEC_SW_PROT_DIR, 
            .b_ctrl = 0, 
            .b_audit = 0,
            .key = "prot_dir", 
            .__on_switch_change = NULL, 
        },
        .path = PATH_PROTECT_DIR_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_PROTECT_DIR,
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_protect_dir,
        .post_handle = ply_post_handle,
    },
    {
        .sw = {
            .type = SEC_SW_PROT_SELF,
            .b_ctrl = 0, //
            .b_audit = 0,
            .key = "prot_self",
            .__on_switch_change = NULL, 
        },
        .path = PATH_PROTECT_SELF_POLICY,
        .fp = NULL,
        .file_len = 0,
        .type = SEC_PLY_PROTECT_SELF,
        .__item_cnt = 0,
        .head_check = ply_head_check,
        .tail_check = ply_tail_check,
        .ply_load = ply_load_protect_self,
        .post_handle = ply_post_handle,
    },
};

// todo: 如何和原来的策略类型相对应
sec_ply_if_t * get_ply_if(int type)
{
    return  &sec_ply_ifs[type];
}

int sec_ply_load(int type)
{
    int retval = -1;
    int ret = 0;

    sec_ply_if_t * ply_if = NULL;

    void * new_ply = NULL;

    tag_pr_debug("===========get ply type[%d]\n", type);
    ply_if = get_ply_if(type);
    if (ply_if == NULL) {
        tag_pr_debug("unkown ply type[%d]\n", type);
        return retval;
    }

    // mutex_lock(&(ply_if->lock));
    ret = ply_if->head_check(ply_if);
    if (ret != CFG_CHECK_RIGHT) {
        tag_pr_debug("ply head check failed\n");
        goto DONE;
    }

    ret = ply_if->ply_load(ply_if, &new_ply);
    if (ret != 0) {
        tag_pr_debug("ply load failed\n");
        goto DONE;
    }

    ret = ply_if->tail_check(ply_if);
    if (ret != CFG_CHECK_RIGHT) {
        tag_pr_debug("ply tail check failed\n");
        goto DONE;
    }

    ply_if->post_handle(ply_if, ret, &new_ply);

    retval = 0;
DONE:
    // mutex_unlock(&(ply_if->lock));

    return retval;
}

int sec_sw_load(int type)
{
    int retval = 0;
    int ret = 0;

    int i;
    sec_ply_if_t * ply_if = NULL;
    int ply_type = 0;

    BOOLEAN old_ctrl;
    BOOLEAN old_audit;

    // 如果是开关类型的策略,先转换一下,以便于操作 ply_if
    for (i = 0; i < SEC_PLY_MAX; i++) {
            // tag_pr_debug("[%d-%d]\n", i, sec_ply_ifs[i].sw.type);

        if (sec_ply_ifs[i].sw.type == type) {
            ply_type = i;
            tag_pr_debug("=========type[%d], ply_type[%d]\n", type, ply_type);
            break;
        }
    }


tag_pr_debug("ply_type[%d]\n", ply_type);
    ply_if = get_ply_if(ply_type);
    if (ply_if == NULL) {
        tag_pr_debug("unkown ply type[%d]\n", ply_type);
        goto DONE;
    }
       
    old_ctrl = ply_if->sw.b_ctrl;
    old_audit = ply_if->sw.b_audit;
tag_pr_debug("old_ctrl[%d], old_audit[%d]\n", old_ctrl, old_audit);

    // todo: 确保应用层统一通过此方式生成开关
    // todo ： 和应用层一一对应
    // 获取开关配置，并将开关配置转换为switch
    ret = ply_switch_get(&ply_if->sw);
    if (ret)
        goto DONE;

    // 切换开关后的post处理
    if (ply_if->sw.__on_switch_change) {
        ply_if->sw.__on_switch_change(&ply_if->sw, old_ctrl, old_audit);
    }
     
    retval = 0;
DONE:
    return retval;
}

void sec_ply_init(int type)
{
    sec_ply_if_t * ply_if = NULL;

    ply_if = get_ply_if(type);
    mutex_init(&(ply_if->sec_ply_lock));    
    ply_if->ply_cnt = 0; 
}
void sec_ply_uninit(int type) // todo
{

}
