#ifndef __SEC_PLY_IF_H_
#define __SEC_PLY_IF_H_

#include <project.h>
#include "include/misc/list_head.h"
#include "common/general/encode/cipher_file.h"
#include "common/general/include/nfs_system_switch.h"

// 策略文件路径的最大长度
#define MAX_SEC_PLY_LEN 256

// todo 返回值 待整理
#define CFG_CHECK_RIGHT 0
#define CFG_CHECK_WRONG 1
#define CFG_CHECK_ERR -1

// noteict: 和sec_ply_ifs中对应
enum SEC_PLY_TYPE {
    SEC_PLY_MAC_SUB = 0,
    SEC_PLY_MAC_OJB,
    // SEC_PLY_DAC, // todo
    SEC_PLY_NAC, // todo
    SEC_PLY_IMA,
    SEC_PLY_IMA_EXT,
    // SEC_PLY_DM, // todo
    SEC_PLY_PROTECT_KMOD,
    SEC_PLY_PROTECT_PROC,
    SEC_PLY_PROTECT_FILE,
    SEC_PLY_PROTECT_DIR,
    //
    SEC_PLY_PROTECT_SELF,
    SEC_PLY_MAX,
};

// for func param
typedef struct __sec_ply_if sec_ply_if_t;
typedef struct __sec_ply_if {

    sec_ply_switch_t sw; // 开关

    // 策略文件全路径
    char path[MAX_SEC_PLY_LEN]; 
    void * fp; // 封装的文件描述符
    unsigned int file_len;
    int type;

    // 策略计数，内部使用
    int __item_cnt;   

    // 在 读、写、切换时，应该加锁
    struct mutex sec_ply_lock;  
    // ---- 临界区 ----
    void * ply_head; // 策略头
    int ply_cnt;    // 策略数量
    // ---- 临界区 ----
    
    // 文件头检查
    int (*head_check)(sec_ply_if_t * ply_if);
    // 策略加载
    int (*ply_load)(sec_ply_if_t * ply_if, void ** new_ply);   
    // 文件尾检查
    int (*tail_check)(sec_ply_if_t * ply_if);   
    // 在加载完成策略后,可做一些后续处理,如转换
    void (*post_handle)(sec_ply_if_t * ply_if, int ret, void ** new_ply);

} sec_ply_if_t;

// todo: init for ply.lock; 类似ac_table_init
// todo: 分配不同的init函数
void sec_ply_init(int type);
void sec_ply_uninit(int type); // todo


static inline int is_sec_ply_empty(sec_ply_if_t * ply_if)
{
    int ret = 0;

    mutex_lock(&ply_if->sec_ply_lock);
    ret = ply_if->ply_cnt == 0 ? 1 : 0;
    mutex_unlock(&ply_if->sec_ply_lock);

    return ret;
}

// 根据策略类型获取对应的操作接口
// todo: 可以以策略类型为索引，直接获取对应的ops
// 但是，目前，type需要和 WPFILE_SYSOBJECT_MAC 等现有宏一致，注意修改
sec_ply_if_t * get_ply_if(int type);

// 加载指定类型的策略
int sec_ply_load(int type);

// 加载指定类型的开关
int sec_sw_load(int type);


// todo 注销全部策略，释放资源 
// 释放全部策略

// todo 更新策略，更新过程需要保护
// 更新指定策略

#endif