#ifndef __COMM_HASHTABLE_H_
#define __COMM_HASHTABLE_H_

// 自定义hash表

#include "comm_object.h"
#include "include/misc/list_head.h"

typedef void (*hash_tb_release_func)(LIST_HEAD *head);
typedef char *(*hash_tb_value_func)(LIST_HEAD *head);

typedef struct __sec_hash_table {
    unsigned int bucket_count;
    hash_tb_release_func hash_release;
    hash_tb_value_func hash_value;
    long item_count;
    LIST_HEAD *hash_list;
} sec_hash_table_t;

int comm_hash_table_init(sec_hash_table_t *table, hash_tb_value_func hash_value,
                         hash_tb_release_func hash_release);
void comm_hash_table_destroy(sec_hash_table_t *table);
LIST_HEAD *comm_hash_table_find(sec_hash_table_t *table, const char *valPtr);
int comm_hash_table_insert(sec_hash_table_t *table, LIST_HEAD *item,
                           const char *valPtr);
int comm_hash_table_remove(sec_hash_table_t *table, const char *valPtr);
void comm_hash_table_remove_all(sec_hash_table_t *table);
int comm_hash_table_set_buckets(sec_hash_table_t *table, int bucket_count);
LIST_HEAD *comm_hash_table_find_ex(sec_hash_table_t *table, const char *valPtr,
                                   unsigned int length);
int comm_hash_table_remove_item(sec_hash_table_t *table, LIST_HEAD *entry);

#define comm_hash_table_get_buckets(table) ((int)(table)->bucket_count)
#define comm_hash_table_get_count(table) ((int)(table)->item_count)

#endif
