// -- elf_header.h

#ifndef __ELF_HEADER_H_
#define __ELF_HEADER_H_

#define MAGIC "\177ELF"
#define INVAL 0

#define INFO_POS 40
#define FOPEN_FAILED -1
#define FORMAT_ERROR -2
#define SUCCESS 0

#define MODTIP()                    \
    do {                            \
        puts("\tEnter new value:"); \
    } while (0)
#define CUR_FTYPE_MAX 4
#define CUR_ARCH_MAX 40

typedef struct elf_head {
    char magic[4];    // Magic Number
    char addr_width;  // Class 1 - 32bits(ELF32);2 - 64bits
    char byteorder;   // Byte order 1 - little-endian;2 - big-endian
    char hversion;    // Header version 1
    char pad[9];      // Padding bytes
    short filetype;   // 1 - relocatable;2 - executable;3 - shared object;4 -
                      // core-image
    short archtype;   // Architecture 2 - SPARC;3 - x86;4 - 68K
    int fversion;     // File version 1
    int entry;        // Entry point if executable
    int phdrpos;      // Program header position
    int shdrpos;      // Section header position
    int flags;        // Architecture relative
    short hdrsize;    // ELF header size
    short phdrent;    // Size of program headers
    short phdrcnt;    // Number of program headers
    short shdrent;    // Size of section headers
    short shdrcnt;    // Number of section headers
    short strsec;     // Section header string table index
} elf_header;

typedef struct __BIN_INFO {
    unsigned char bits;        // 位数  1:32 2:64
    unsigned char byte_order;  // 字节序  2:大端  1:小端
    unsigned char file_type;   // 文件类型 1:.o  2:exe 3:.so 4:core文件
                               // 1:可重定位文件，一般位.o文件
    unsigned char is_driver_file : 1;
    // 201:.ko
    unsigned char arch;  // 架构
} BIN_INFO;

int read_elf_header(elf_header *ehdr, const char *filename);  //读取文件头信息
int judge_bin_file(const char *filename);

#endif /*ELF_HEAD_H*/
