#include <project.h>

#include "cipher_file.h"

int cipher_file_get_length(CIPHER_FILE *file, LONGLONG *pLen)
{
    *pLen = kern_file_length(file->fp);
    return 0;
}

int cipher_file_open(CIPHER_FILE *file, const kchar *path, const char *mode)
{
    int retval = -1;

    if (0 != kern_fopen(&file->__kern_file, path, mode)) {
        // tag_pr_debug("kern_fopen [%s] failed.\n", path);
        goto DONE;
    }

    file->fp = &file->__kern_file;

    if (0 != encode_ctx_init(&file->cipher_ctx)) {
        tag_pr_debug("cipher_ctx_init failed.\n");
        kern_fclose(file->fp);
        file->fp = NULL;
        goto DONE;
    }

    MD5Init(&file->md5_ctx);

    retval = 0;
DONE:
    return retval;
}

void cipher_file_close(CIPHER_FILE *file)
{
    if (file->fp) {
        kern_fclose(file->fp);
        file->fp = NULL;
    }
    return;
}

size_t cipher_file_read(void *buffer, size_t size, size_t count,
                        CIPHER_FILE *file)
{
    size_t retval;
    size_t length;

    retval = kern_fread(buffer, size, count, file->fp);
    if (retval <= 0) {
        goto DONE;
    }

    length = retval * size;

    encode_ctx_xor(&file->cipher_ctx, (unsigned char *)buffer, length);

    MD5Update(&file->md5_ctx, (unsigned char *)buffer, length);

    // -- retval = 0;
DONE:
    return retval;
}

void cipher_file_fetch_md5sum(unsigned char sum[16], CIPHER_FILE *file)
{
    MD5Final(sum, &file->md5_ctx);
    return;
}

