#ifndef __HASHTB_H_
#define __HASHTB_H_
#include "include/misc/list_head.h"

typedef void (*HAHSTB_RELEASE)(LIST_HEAD *head);
typedef unsigned char *(*HAHSTB_VALUE)(LIST_HEAD *head);
typedef void *(*HASHTB_ALLOC)(unsigned int size);
typedef void (*HASHTB_FREE)(void *_data);

#pragma pack(1)
typedef struct __HASHTB {
    unsigned int bucket_count;
    HAHSTB_RELEASE release_item;
    HAHSTB_VALUE get_value;
    unsigned int valueLen;
    LIST_HEAD *hash_list;
    
    // 1:表示"u32val"为网络数据，需要使用 ntohl转化
    unsigned char net_flag : 1;  
    long item_count;
    HASHTB_ALLOC _alloc;
    HASHTB_FREE _free;
} HASHTB;
#pragma pack()

#define hash_get_item_count(table) ((int)(table)->item_count)

// 即使多次调用也没有问题
BOOLEAN hashtb_set_buckets(HASHTB *table, int bucket_count);  

// valueLen 必须是 sizeof(int)的整数倍；
int hashtb_init(HASHTB *table, HAHSTB_VALUE __get_value_ptr, 
    HAHSTB_RELEASE release_item, unsigned int valueLen);  

int hashtb_init_ex(HASHTB *table, HAHSTB_VALUE __get_value_ptr,
                   HAHSTB_RELEASE release_item, unsigned int valueLen,
                   HASHTB_ALLOC _alloc, HASHTB_FREE _free);

void hashtb_destroy(HASHTB *table);

LIST_HEAD *hashtb_find(HASHTB *table, unsigned int *val_ptr);
BOOLEAN hashtb_insert_item(HASHTB *table, LIST_HEAD *item,
                           unsigned int *val_ptr);
BOOLEAN hashtb_remove_item(HASHTB *table, unsigned int *val_ptr);
void hashtb_remove_all(HASHTB *table);
int hashtb_get_buckets(HASHTB *table);
BOOLEAN hashtb_remove_item_entry(HASHTB *table, LIST_HEAD *entry);

#endif
