#ifndef __AUDIT_MOD_H_
#define __AUDIT_MOD_H_

// 用于模块间交互的审计

// 审计->type
enum AuditType {
    AUDIT_TYPE_IMA = 10,      // 完整性度量
    AUDIT_TYPE_AC = 20,       // 访问控制
    AUDIT_TYPE_PROTECT = 30,  // 系统保护
    AUDIT_TYPE_REINFORE = 40  // 加固选项
};

// 审计子类型
enum AUDIT_SUBTYPE {
    AUDIT_SUBTYPE_STATIC_MEASURE = 1001,
    AUDIT_SUBTYPE_KERNEL_MEASURE,
    AUDIT_SUBTYPE_DYNAMIC_MEASURE,

    AUDIT_SUBTYPE_MAC = 2001,  // Mandatory access control
    AUDIT_SUBTYPE_DAC,         // Discretionary access control
    AUDIT_SUBTYPE_PAC,         // Peripheral access control
    AUDIT_SUBTYPE_NAC,         // Network access control

    AUDIT_SUBTYPE_PROC_PROTECT = 3001,
    AUDIT_SUBTYPE_KMOD_PROTECT,
    AUDIT_SUBTYPE_FILE_PROTECT,
    AUDIT_SUBTYPE_DIR_PROTECT,
    AUDIT_SUBTYPE_SELF_PROTECT,
    AUDIT_SUBTYPE_APP_VERIFY,
    AUDIT_SUBTYPE_KMOD_VERIFY,

    AUDIT_SUBTYPE_SOFT_PROTECT,     // 软件防卸载
    AUDIT_SUBTYPE_BAN_KMOD,         // 内核模块黑名单

    AUDIT_SUBTYPE_SET_REINFORE = 4001,
    AUDIT_SUBTYPE_SET_BASELINE,
    AUDIT_SUBTYPE_SET_SECSWITCH,

};

typedef struct __audit_str_map {
    int type;
    const char* str;
} audit_str_map_t;

extern audit_str_map_t audit_type_map[];
extern const char* audit_type_str(int type);

extern audit_str_map_t audit_subtype_map[];
extern const char* audit_subtype_str(int subtype);

extern audit_str_map_t audit_ope_map[];
extern const char* audit_ope_str(int ope);


// 审计->operation
enum AUIDT_OPE {
    // UI调用接口 start
    AUDIT_OPE_INIT_SCAN = 1,  // 初始化扫描
    AUDIT_OPE_SET_IMA_ST,     // 配置ima状态
    AUDIT_OPE_SET_DYTIME,     //配置动态度量时间

    AUDIT_OPE_SET_BASELINE_ITEM,  // 修改基线配置项
    AUDIT_OPE_SET_BASELINE_TEMP,  // 修改基线配置模板
    AUDIT_OPE_SET_SECSWITCH,      // 修改安全switch开关

    AUDIT_OPE_SET_SUBJECT,  // 配置主体
    AUDIT_OPE_SET_OBJECT,   // 配置客体
    AUDIT_OPE_SET_MAC_ST,   // 配置强访状态

    AUDIT_OPE_SET_PROTECT_KMOD,        // 配置模块保护
    AUDIT_OPE_SET_PROTECT_KMOD_ST,     // 配置模块保护状态
    AUDIT_OPE_SET_PROTECT_FOLDER,      // 配置目录保护
    AUDIT_OPE_SET_PROTECT_FOLDER_ST,   // 配置目录保护状态
    AUDIT_OPE_SET_PROTECT_PROCESS,     // 配置程序保护
    AUDIT_OPE_SET_PROTECT_PROCESS_ST,  // 配置程序保护状态
    AUDIT_OPE_SET_PROTECT_FILE,        // 配置文件保护
    AUDIT_OPE_SET_PROTECT_FILE_ST,     // 配置文件保护状态

    AUDIT_OPE_SET_NAC_RULE,             // 配置网络访问控制规则
    AUDIT_OPE_SET_NAC_DEV,              // 配置网络访问控制设备
    // UI调用接口 end

    // （内核层）文件访问 的 操作
    AUDIT_OPE_FILE_EXEC = 100,
    AUDIT_OPE_FILE_READ,
    AUDIT_OPE_FILE_WRITE,
    AUDIT_OPE_FILE_APPEND,
    AUDIT_OPE_FILE_RENAME,
    AUDIT_OPE_FILE_UNLINK,
    AUDIT_OPE_FILE_MKDIR,
    AUDIT_OPE_FILE_RMDIR,
    AUDIT_OPE_FILE_NEW,
    AUDIT_OPE_FILE_LOAD_LIB,
    AUDIT_OPE_FILE_INSMOD,

    AUDIT_OPE_FILE_KILL, // todo:暂时分类到此处,如果不合适,可以更改
    AUDIT_OPE_FILE_RMMOD, 


    AUDIT_OPE_FILE_BAN_KMOD, // 内核模块黑名单，加载
    AUDIT_OPE_SOFT_REMOVE, // 软件卸载

    // todo: nac的ope

    // todo: dm的ope

};

// todo: 待转换为AUIDT_OPE
// 审计 操作类型
#define OPTYPE_FILE (1)
#define OPTYPE_PROCESS (2)
#define OPTYPE_REGISTRY (3)
#define OPTYPE_LOAD_IMAGE (4)
#define OPTYPE_MODIFY_PASSWD (5)
#define OPTYPE_LOGIN_EVENT (6)
#define OPTYPE_FLASH_DISK (7)
#define OPTYPE_CDROM_DISK (8)
#define OPTYPE_USB_NIC (9)
#define OPTYPE_CONFIG_CORRUPT (10)

#define OPTYPE_NET_ACCESS (17)
#define OPTYPE_LOAD_DIRVER (18)
#define OPTYPE_LOAD_APPLICATION (19)
#define OPTYPE_APP_CONFIG_CHECK (20)
#define OPTYPE_LOGIN_BRUTE_FORCE (21)
#define OPTYPE_VISIT_INTERNET (22)  // -- __POLICY_TYPE_OUT_CONNECTION

#define OPTYPE_DM_KERNEL (23)
#define OPTYPE_DM_SYSCALL (24)
#define OPTYPE_DM_MODULE (25)
#define OPTYPE_DM_TASK (26)
#define OPTYPE_DM_NET (27)
#define OPTYPE_DM_FILESYSTEM (28)
#define OPTYPE_DM_SUPERBLOCK (29)

// -- OPTYPE_LOGIN_EVENT's subType
#define __SUBTYPE_LOGIN_INVALID_PIN (1)
#define __SUBTYPE_LOGIN_INVALID_PASSWORD (2)
#define __SUBTYPE_LOGIN_INVALID_KEY (3)
#define __SUBTYPE_LOGIN_DEFAULT_USER (4)
#define __SUBTYPE_LOGINGUEST_USER (5)
#define __SUBTYPE_REMOTE_LOGIN (6)

// --
// -- OPTYPE_PROCESS's subType:
#define __SUBTYPE_ACCESS (1)
#define __SUBTYPE_KILL (2)

// -- OPTYPE_FILE's subType
#define __SUBTYPE_DELETEFILE (1)
#define __SUBTYPE_RENAMEFILE (2)
#define __SUBTYPE_WRITEFILE (3)
#define __SUBTYPE_READFILE (4)
#define __SUBTYPE_CHOWN (5)
#define __SUBTYPE_CREATEFOLDER (6)
#define __SUBTYPE_DELETEFOLDER (7)
#define __SUBTYPE_CREATEFILE (8)

// -- OPTYPE_REGISTRY's subType
#define __SUBTYPE_DELETEKEY (1)
#define __SUBTYPE_DELETEVALUEKEY (2)
#define __SUBTYPE_SETVALUEKEY (3)
#define __SUBTYPE_RENAMEKEY (4)
#define __SUBTYPE_CREATEKEY (5)

// -- OPTYPE_FLASH_DISK's subType
#define __SUBTYPE_FLASH_MOUNT (1)
#define __SUBTYPE_FLASH_UMOUNT (2)
#define __SUBTYPE_FLASH_APP_RUN (3)

// -- OPTYPE_CDROM_DISK's subType
#define __SUBTYPE_CDROM_MOUNT (1)
#define __SUBTYPE_CDROM_UMOUNT (2)
#define __SUBTYPE_CDROM_APP_RUN (3)

// -- OPTYPE_USB_NIC's subType
#define __SUBTYPE_USB_NIC_PLUG (1)    // -
#define __SUBTYPE_USB_NIC_UNPLUG (2)  //
#define __SUBTYPE_USB_TRANSFER (3)    // -

// -- OPTYPE_CONFIG_CORRUPT's subType
#define __SUBTYPE_CORRUPT_CONFIG_FILE (1)
#define __SUBTYPE_CORRUPT_SETTING_FILE (2)

// -- OPTYPE_VISIT_INTERNET's SUB_TYPE
#define __OUT_CONNECT_PING (1)
#define __OUT_CONNECT_CONN (2)

#endif  // __AUDIT_MOD_H_