#ifndef __INTEGRITY_H_
#define __INTEGRITY_H_

#include <linux/uidgid.h>

#ifdef HASH_ALO_SHA1
#define INTEGRITY_LENGTH (20)
#else  // HASH_ALO_SM3
#define INTEGRITY_LENGTH (32)
#endif

#define SYSHASH_MAGIC *((unsigned int*)"SHSH")
#define INTEGRITY_MAGIC SYSHASH_MAGIC
#define MAX_PATH_LENGTH (512)

#pragma pack(1)

#define MAGIC_FILE_HDR *((unsigned int*)"ADLW")
typedef struct __WL_FILE_HDR {
    unsigned int fileMagic;
} __attribute__((packed)) WL_FILE_HDR;

typedef struct __WL_FILE_TAIL {
    unsigned int magic;
    unsigned char md5sum[16];
} __attribute__((packed)) WL_FILE_TAIL;

// -- #define  __VALID_DATA_LENGTH ((unsigned int)(char * )((struct
// __SYSTEM_HASH_ITEM *)0)->pad[0])
#define __VALID_DATA_LENGTH                                       \
    (sizeof(int) + sizeof(char) + sizeof(int) + MAX_PATH_LENGTH + \
     INTEGRITY_LENGTH + sizeof(int) + sizeof(uint64_t) + sizeof(char))

typedef struct __SYSTEM_HASH_ITEM {
    unsigned int magic;
    unsigned char type;
    unsigned int wParam;
    char path[MAX_PATH_LENGTH];
    unsigned char bhash_code[INTEGRITY_LENGTH];
    unsigned int modTime;
    uint64_t fileSize;
    unsigned char bScript;
    char guid[36];
    // unsigned char pad[((__VALID_DATA_LENGTH + 15) / 16) * 16 -
    // __VALID_DATA_LENGTH];
} __attribute__((packed)) SYSTEM_HASH_ITEM;

// -- #define __VALID_DATA_LENGTH (sizeof(unsigned int) + MAX_PATH_LENGTH +
// INTEGRITY_LENGTH)

typedef SYSTEM_HASH_ITEM INTEGRITY_ITEM;

#define SEC_GUARD_ITEM_MAGIC *((unsigned int*)"GUAR")
#define SEC_GUARD_TYPE_DIR (1)
#define SEC_GUARD_TYPE_FILE (2)

#define __XGUARD_SUB_TYPE_SELF_PR (1)
#define __XGUARD_SUB_TYPE_WL_BIN_PR (2)
#define __XGUARD_SUB_TYPE_WL_SCRIPT_PR (3)
#define __XGUARD_SUB_TYPE_TRUST_FOLDER (4)
#define __XGUARD_SUB_TYPE_SYS_FOLDER (5)
#define __XGUARD_SUB_TYPE_BOOT_FILE (6)
#define __XGUARD_SUB_TYPE_PROTECT_FOLDER (7)
#define __SEC_GUARD_SUB_TYPE_PROTECT_FILE (8)

typedef struct __GUARD_ITEM {
    unsigned int magic;     // 校验魔幻数
    unsigned int id;        // 策略id
    unsigned char itemType;
    unsigned char sub_type;
    char path[512];         // 程序路径
    unsigned int model;
#ifdef EXCEPTION_PASS
    char exception_ply[512]; // 例外放行策略名字
#endif

} __attribute__((packed)) SEC_GUARD_ITEM;

// ------------------------------------------------------------------
#define SEC_SRV_ITEM_MAGIC *((unsigned int*)"XSRV")

typedef struct __SRV_ITEM {
    unsigned int magic;
    unsigned int id;
    char path[512];
} __attribute__((packed)) SEC_SRV_ITEM;

// 内核模块保护
#define KMOD_ITEM_MAGIC *((unsigned int*)"KMOD")
typedef struct __KMOD_ITEM {
    unsigned int magic;
    unsigned int id;
    char path[512];
    char mod_name[512];
} __attribute__((packed)) KMOD_ITEM;

#define SEC_EXT_ITEM_MAGIC *((unsigned int*)"TXEX")
typedef struct __XEXT_ITEM {
    unsigned int magic;
    unsigned int id;
    char ext[20];
} __attribute__((packed)) SEC_EXT_ITEM;

// 以此作为user主体的key
typedef struct __SID_STOR {
    unsigned int id;
    unsigned int model;
} __attribute__((packed)) SID_STOR;

#define SEC_USR_ITEM_MAGIC *((unsigned int*)"XUSR")
typedef struct __SEC_USER_ITEM_STOR {
    unsigned int magic; // 校验魔幻数
    unsigned int rank;  // 等级
    unsigned int integrityRank; // 用于文件完整性保护
    unsigned int user_id;   // 用户id
    SID_STOR sid;   // 策略id
    // unsigned int model; // 模型
} __attribute__((packed)) SEC_USER_ITEM_STOR;
#pragma pack()

// 内核模块黑名单
#define BAN_KMOD_PLY_MAGIC *((unsigned int*)"BMOD")
typedef struct __BAN_KMOD_PLY {
    unsigned int magic;
    unsigned int id;
    char mod_name[256];
} __attribute__((packed)) ban_kmod_ply_t;

// 软件防卸载
#define SOFT_REMOVE_PLY_MAGIC *((unsigned int*)"SREM")
typedef struct __SOFT_REMOVE_PLY {
    unsigned int magic;
    unsigned int id;
    char soft_name[256];
    char soft_full_name[256];
} __attribute__((packed)) soft_remove_ply_t;

#ifdef EXCEPTION_PASS
// 例外放行
#define EXCETP_PASS_PLY_MAGIC *((unsigned int*)"pass")
typedef struct __EXCEPT_PASS_PLY_HEAD {
    unsigned int magic;
    unsigned int cnt;
    // char proc[128]; 
    // char fullpath[512];
} __attribute__((packed)) except_pass_ply_head_t;

typedef struct __EXCEPT_PASS_PLY {
    unsigned int len;
    // char * except;   // 程序路径
} __attribute__((packed)) except_pass_ply_t;
#endif

#define NAC_RULE_ITEM_MAGIC *((unsigned int*)"NACR")
typedef struct __NAC_RULE_ITEM {
    unsigned int magic;
    unsigned int action;
    unsigned int direction;
    unsigned int level;
    unsigned int protocol;
    unsigned int ip_from;
    unsigned int ip_to;
    unsigned int port_from;
    unsigned int port_to;
    char proc[512]; // todo
} __attribute__((packed)) nac_rule_item_t;


#endif
