#ifndef __HASH_CODE_H_
#define __HASH_CODE_H_

//---第一种hash 算法，针对 mac 地址类型；
// #define BR_HASH_BITS 8
// #define BR_HASH_SIZE (1 << BR_HASH_BITS)

#define MAC_HASH(fun_name, hash_size)                  \
    static __inline__ int fun_name(unsigned char *mac) \
    {                                                  \
        unsigned long x;                               \
        x = mac[0];                                    \
        x = (x << 2) ^ mac[1];                         \
        x = (x << 2) ^ mac[2];                         \
        x = (x << 2) ^ mac[3];                         \
        x = (x << 2) ^ mac[4];                         \
        x = (x << 2) ^ mac[5];                         \
        x ^= x >> 8;                                   \
        return x & (hash_size - 1);                    \
    }

//---上面算法被废弃的原形-----------------------
#define JXX_BR_HASH_BITS 8
#define JXX_BR_HASH_SIZE (1 << JXX_BR_HASH_BITS)
static __inline__ int jxx_br_mac_hash(unsigned char *mac)
{
    unsigned long x;

    x = mac[0];
    x = (x << 2) ^ mac[1];
    x = (x << 2) ^ mac[2];
    x = (x << 2) ^ mac[3];
    x = (x << 2) ^ mac[4];
    x = (x << 2) ^ mac[5];

    x ^= x >> 8;

    /*
     * #define BR_HASH_BITS 8
     * #define BR_HASH_SIZE (1 << BR_HASH_BITS)
     */
    return x & (JXX_BR_HASH_SIZE - 1);
}
// 等价于 : MAC_HASH(xxx_br_mac_hash,XXX_BR_HASH_SIZE)

//-----------------------------------------------
//--另一种hash算法，针对int32类型，可以用于ip地址的hash
// The golden ration: an arbitrary value
#define JHASH_GOLDEN_RATIO (0x9e3779b9)

// NOTE: Arguments are modified.
#define __x_jhash_mix(a, b, c) \
    do {                       \
        a -= b;                \
        a -= c;                \
        a ^= (c >> 13);        \
        b -= c;                \
        b -= a;                \
        b ^= (a << 8);         \
        c -= a;                \
        c -= b;                \
        c ^= (b >> 13);        \
        a -= b;                \
        a -= c;                \
        a ^= (c >> 12);        \
        b -= c;                \
        b -= a;                \
        b ^= (a << 16);        \
        c -= a;                \
        c -= b;                \
        c ^= (b >> 5);         \
        a -= b;                \
        a -= c;                \
        a ^= (c >> 3);         \
        b -= c;                \
        b -= a;                \
        b ^= (a << 10);        \
        c -= a;                \
        c -= b;                \
        c ^= (b >> 15);        \
    } while (0)

static __inline__ unsigned int _sec_hash_1word(unsigned int initval)
{
    unsigned int a = JHASH_GOLDEN_RATIO, b = JHASH_GOLDEN_RATIO;
    __x_jhash_mix(a, b, initval);

    return initval;
}

static __inline__ unsigned int
_sec_hash2(const unsigned int *k, unsigned int length, unsigned int initval)
{
    unsigned int a, b, c, len;

    a = b = JHASH_GOLDEN_RATIO;
    c = initval;
    len = length;

    while (len >= 3) {
        a += k[0];
        b += k[1];
        c += k[2];
        __x_jhash_mix(a, b, c);
        k += 3;
        len -= 3;
    }

    c += length * 4;

    switch (len) {
        case 2:
            b += k[1];
        case 1:
            a += k[0];
    };

    __x_jhash_mix(a, b, c);

    return c;
}

/* A special ultra-optimized versions that knows they are hashing exactly
 * 3, 2 or 1 word(s).
 *
 * NOTE: In partilar the "c += length; __x_jhash_mix(a,b,c);" normally
 *       done at the end is not done here.
 */

static __inline__ unsigned int _sec_hash_3words(unsigned int a, unsigned int b,
                                             unsigned int c,
                                             unsigned int initval)
{
    (a) += JHASH_GOLDEN_RATIO;
    (b) += JHASH_GOLDEN_RATIO;
    (c) += initval;
    __x_jhash_mix((a), (b), (c));
    return c;
}

#endif
