#ifndef __LIST_HEAD_H_
#define __LIST_HEAD_H_

typedef struct _LIST_HEAD LIST_HEAD;
#pragma pack(4)
struct _LIST_HEAD {
    LIST_HEAD *prev;
    LIST_HEAD *next;
};
#pragma pack()

#ifndef OBJECT_PTR
#ifndef AMD64
#define OBJECT_PTR(address, type, field)   \
    ((type *)((unsigned char *)(address) - \
              (unsigned long)(&((type *)0)->field)))

#else
#define OBJECT_PTR(address, type, field)   \
    ((type *)((unsigned char *)(address) - \
              (unsigned long)(ULONGLONG)(&((type *)0)->field)))
#endif

#endif

#ifndef OBJECT_OFFSET
#ifndef AMD64
#define OBJECT_OFFSET(type, field) ((unsigned int)(&((type *)0)->field))
#else
#define OBJECT_OFFSET(type, field) \
    ((unsigned int)(ULONGLONG)(&((type *)0)->field))
#endif

#endif

//------------------------------------------------------------------------------------------
#define __list_init(list)    \
    do {                     \
        (list)->prev = list; \
        (list)->next = list; \
    } while (0);

#define __list_is_empty(list) ((list)->prev == list)

#define __list_add_tail(head, item)     \
    do {                                \
        LIST_HEAD *prev = (head)->prev; \
        (item)->next = (head);          \
        (item)->prev = prev;            \
        prev->next = (item);            \
        (head)->prev = (item);          \
    } while (0);

#define __list_add_head(head, item)     \
    do {                                \
        LIST_HEAD *next = (head)->next; \
        (item)->prev = (head);          \
        (item)->next = next;            \
        next->prev = (item);            \
        (head)->next = (item);          \
    } while (0);

#define __list_add_tail(head, item)     \
    do {                                \
        LIST_HEAD *prev = (head)->prev; \
        (item)->next = (head);          \
        (item)->prev = prev;            \
        prev->next = (item);            \
        (head)->prev = (item);          \
    } while (0);

#define __list_remove_item(item) \
    do {                         \
        LIST_HEAD *prev;         \
        LIST_HEAD *next;         \
        prev = (item)->prev;     \
        next = (item)->next;     \
        prev->next = next;       \
        next->prev = prev;       \
    } while (0);

#define __list_remove_item_safe(item) \
    do {                              \
        LIST_HEAD *prev;              \
        LIST_HEAD *next;              \
        prev = (item)->prev;          \
        next = (item)->next;          \
        prev->next = next;            \
        next->prev = prev;            \
        (item)->next = (item);        \
        (item)->prev = (item);        \
    } while (0);

#define __list_remove_tail(list, to)         \
    do {                                     \
        (to) = (list)->prev;                 \
        __list_remove_item((LIST_HEAD *)to); \
    } while (0);

#define __list_remove_head(list, to)         \
    do {                                     \
        (to) = (list)->next;                 \
        __list_remove_item((LIST_HEAD *)to); \
    } while (0);

#endif
