#ifndef __SEC_MOD_SHARE_H_
#define __SEC_MOD_SHARE_H_
#include "audit_mod.h"
#include "format/integrity.h"
#include "ioctl_mod.h"

#pragma pack(1)

#define __XKRNL_MSG_INSTALL_FILE (1)
#define __XKRNL_MSG_UPDATE_FILE (2)

typedef struct __SEC_KRNL_MSG_HDR {
    unsigned int type;
    unsigned int totalLen;
    unsigned int wParam;
    unsigned int lParam;
} __attribute__((packed)) SEC_KRNL_MSG_HDR;

#define __SEC_KRNL_MSG_HDR_INIT(h, t, total, wp, lp) \
    do {                                          \
        (h)->type = (t);                          \
        (h)->totalLen = (total);                  \
        (h)->wParam = (wp);                       \
        (h)->lParam = (lp);                       \
    } while (0);

typedef struct __SEC_INSTALL_MESSAGE {
    SEC_KRNL_MSG_HDR hdr;
    unsigned char isWrite;
    unsigned char isCreated;
    unsigned char isRename;
    unsigned char isDelete;
    unsigned int task_id;  //

    char path[512];
} __attribute__((packed)) SEC_INSTALL_MESSAGE;

typedef struct __NFSSEC_SWITCH_VARIANT {
    int bEnabled;
    int bAudit;
    int bControl;
    unsigned int type;
} __attribute__((packed)) NFSSEC_SWITCH_VARIANT;

// ===================================================================
typedef struct __audit_time {
    ULONGLONG tv_sec;  /* seconds */
    ULONGLONG tv_usec; /* microseconds */
} __attribute__((packed)) audit_time_t;

#define AUDIT_MAGIC 0xABBCCBBA
#define AUDIT_HDR_TYPE_OPE (3) 
#define AUDIT_HDR_TYPE_NET (14)
#define AUDIT_HDR_TYPE_PAC (17)

#define AUDIT_RESULT_SUCCESS    0
#define AUDIT_RESULT_FAILED     1

typedef struct __AUDIT_HDR {
    unsigned int magic;   // AUDIT_MAGIC
    unsigned int length;  // include length of AUDIT_HDR;
    unsigned int hdr_type;  // 用于区分不同的审计结构 like AUDIT_HDR_TYPE_OPE
    unsigned int result;   // 操作结果
    unsigned int uid;       // 用户id（操作的系统用户）
    // todo 用户信息，用户id还是名字？
    audit_time_t auditTime;
} __attribute__((packed)) AUDIT_HDR;

// ===================================================================

// level
#define LEVEL_FATAL (1)
#define LEVEL_ALERT (2)
#define LEVEL_WARN (3)

// 审计项，内核模块和后台服务交互
// 用于文件操作类型的审计
// todo：如果不能满足，再拓展（如网络审计等）
typedef struct __audit_ope {
    AUDIT_HDR hdr;

    // todo ： 精简 hdr_type  ruleType sub_type
    // todo: 在hdr中，存在了审计类型，应用层够用了，所以，此处可以删除
    unsigned int ruleType;  // 审计大类 like AUDIT_TYPE_AC
    
    unsigned int ruleId;    // 目前未使用

    unsigned int level;

    char process[512];
    unsigned long processId;

    unsigned int op_class;  // 主体对客体的操作 like AUDIT_OPE_FILE_EXEC
    unsigned int sub_type;  // 审计子类型 like AUDIT_SUBTYPE_MAC
    
    unsigned char sub_rank;
    unsigned char obj_rank;

    char target[512];
    unsigned char sum[INTEGRITY_LENGTH];

    unsigned int sid;  
} __attribute__((packed)) audit_ope_t;

typedef struct ___nac_audit_data {
    AUDIT_HDR hdr;
    
    unsigned int srcIp;
    unsigned short srcPort;
    unsigned int destIp;
    unsigned short destPort;
    unsigned char protocol; 
    char action;             // -- 1: denied, 2: pass
    char direction;          // -- 1: out, 0: in;
    unsigned int ruleId;
    unsigned short ruleType;
} nac_audit_data_t;

// 外设访问控制，目前主要是usb设备
#define PAC_DEV_TYPE_USB 0001   

#define PAC_ACTION_DEV_GOT 0001     // 拦截设备

typedef struct ___pac_audit_data {
    AUDIT_HDR hdr;
    
    unsigned int audit_type;
    unsigned int sub_type;

    unsigned char dev_type;
    unsigned int pid;
    unsigned int vid;
    unsigned int clsid;
    unsigned char action;   // 预留
} pac_audit_data_t;

typedef struct __AUDIT_GEN_DATA {
    AUDIT_HDR hdr;
    unsigned char content[1];
} __attribute__((packed)) AUDIT_GEN_DATA;

typedef struct __VALIDATE_IMAGE {
    char imageName[512];
} __attribute__((packed)) VALIDATE_IMAGE;

#pragma pack()

#endif
