#ifndef __SCAN_MACHINE_H_
#define __SCAN_MACHINE_H_

#include "../misc/list_head.h"
#define SCN_CHECK_CONTINUE (0)
#define SCN_CHECK_BREAK (1)

typedef struct _SCN_PATTERN SCN_PATTERN;
typedef int (*SCAN_CHECK_ROUTINE)(SCN_PATTERN *pattern, char *pattern_pos,
                                  int offset, int left, void *search_context);

#define SCN_PATTERN_MAGIC (5678)

#pragma pack(1)
struct _SCN_PATTERN {
#ifdef __SCN_PATTERN_DEBUG__
    unsigned int magic;
#endif
    LIST_HEAD list_head;  

#ifdef __STOR_PATTERN_INFO
    unsigned char *pat;  
#endif

    unsigned short pat_len; 
    void *context;

#ifdef __STOR_PATTERN_FREE_CONTEXT
    void (*free_context)(void *);
#endif
};
#pragma pack()

#define __SCAN_GET_EXT(T, pat) ((T *)((SCN_PATTERN *)(pat) + 1))

// -- module initialize functions;
int scan_scn_am_module_init(void);
void scan_scn_am_module_destroy(void);

void *scan_scn_am_create(BOOL is_dfa, unsigned int patExtLen);
int scan_scn_am_build(void *am);

#ifdef __STOR_PATTERN_FREE_CONTEXT
SCN_PATTERN *scan_scn_am_insert_pattern(void *am, unsigned char *pat,
                                        unsigned short pat_len,
                                        void (*free_ctx)(void *), void *ctx);
#else
SCN_PATTERN *scan_scn_am_insert_pattern(void *am, unsigned char *pat,
                                        unsigned short pat_len, void *ctx);
#endif

int scan_scn_am_traverse(void *am, unsigned char *content, int len,
                         SCAN_CHECK_ROUTINE _check, void *search_ctx);

void scan_scn_machine_stat(void);

void *scan_scn_am_go(void *am_ptr, unsigned char *base, unsigned int baseOffset,
                     int len, SCAN_CHECK_ROUTINE _check, void *search_ctx,
                     void *_node);


void *scan_scn_trie_create(unsigned int patExtLen);
#ifdef __STOR_PATTERN_FREE_CONTEXT
SCN_PATTERN *scan_scn_trie_insert_key(void *am, unsigned char *pat,
                                      unsigned short pat_len,
                                      void (*free_ctx)(void *), void *ctx);
#else
SCN_PATTERN *scan_scn_trie_insert_key(void *am, unsigned char *pat,
                                      unsigned short pat_len, void *ctx);
#endif

int scan_scn_trie_nocase_traverse(void *am_ptr, unsigned char *content, int len,
                                  SCAN_CHECK_ROUTINE _check, void *search_ctx);
int scan_scn_trie_traverse(void *am_ptr, unsigned char *content, int len,
                           SCAN_CHECK_ROUTINE _check, void *search_ctx);
int scan_scn_trie_build(void *am);

#endif
