#include <project.h>
#include "include/misc/list_head.h"
#include "object_pool.h"

typedef struct __OBJECT_POOL {
    LIST_HEAD head;
    spinlock_t lock;
    LIST_HEAD free_object_head;
    unsigned int limit;
    unsigned int obj_count;
    unsigned int obj_size;
    unsigned int real_size;
    char pool_name[128];
} OB_POOL;

static LIST_HEAD s_pool_list;
static spinlock_t s_pool_lock;

static atomic_t s_pool_init_flag = ATOMIC_INIT(0);
static int volatile s_pool_destroy_flag = 0;

void __object_pool_module_init(void)
{
    int count = atomic_inc_return(&s_pool_init_flag);
    if (1 == count) {
        spin_lock_init(&s_pool_lock);
        __list_init(&s_pool_list);
    }
    return;
}

__attribute__((visibility("default"))) void object_pool_module_init()
{
    __object_pool_module_init();
    return;
}

void __object_pool_module_destroy(void)
{
    int count = atomic_dec_return(&s_pool_init_flag);
    if (0 == count) {

    }
    return;
}

__attribute__((visibility("default"))) void object_pool_module_destroy(void)
{
    __object_pool_module_destroy();
    return;
}

__attribute__((visibility("default"))) void object_destroy_pool(void *pool)
{
    OB_POOL *pl = (OB_POOL *)pool;
    LIST_HEAD *entry;

    while (!__list_is_empty(&pl->free_object_head)) {
        __list_remove_head(&pl->free_object_head, entry);
        pl->obj_count--;
        kfree((char *)entry);
    }

    spin_lock_bh(&s_pool_lock);
    __list_remove_item(&pl->head);
    spin_unlock_bh(&s_pool_lock);

    kfree(pl);
    return;
}

__attribute__((visibility("default"))) void *
object_create_pool(int uplimit, unsigned int size, char *name)
{
    OB_POOL *pl = NULL;

    __object_pool_module_init();

    pl = (OB_POOL *)kmalloc(sizeof(OB_POOL), GFP_ATOMIC);
    if (!pl)
        return NULL;
    
    memset(pl, 0, sizeof(OB_POOL));
    pl->limit = uplimit;

    __list_init(&pl->free_object_head);

    pl->real_size = size;

    if (size < sizeof(LIST_HEAD)) {
        pl->obj_size = sizeof(LIST_HEAD);
    } else {
        pl->obj_size = size;
    }
    pl->obj_count = 0;

    strncpy(pl->pool_name, name, sizeof(pl->pool_name));
    pl->pool_name[sizeof(pl->pool_name) - 1] = 0;

    spin_lock_init(&pl->lock);

    spin_lock_bh(&s_pool_lock);
    __list_add_tail(&s_pool_list, &pl->head);
    spin_unlock_bh(&s_pool_lock);

    // -- DONE:
    return pl;
}

__attribute__((visibility("default"))) void *object_pool_alloc(void *pool)
{
    OB_POOL *pl = (OB_POOL *)pool;
    void *ob = NULL;
    LIST_HEAD *entry = NULL;

    spin_lock_bh(&pl->lock);

    if (pl->obj_count > 0) {
        pl->obj_count--;
        __list_remove_head(&pl->free_object_head, entry);
        ob = entry;
    } else {
        ob = kmalloc(pl->obj_size, GFP_ATOMIC);
    }

    spin_unlock_bh(&pl->lock);

    return ob;
}

__attribute__((visibility("default"))) void object_pool_free(void *pool,
                                                             void *obj)
{
    OB_POOL *pl = (OB_POOL *)pool;
    LIST_HEAD *entry = (LIST_HEAD *)obj;

    spin_lock_bh(&pl->lock);

    if (pl->obj_count < pl->limit) {
        pl->obj_count++;
        __list_add_head(&pl->free_object_head, entry);
    } else {
        kfree(obj);
    }

    spin_unlock_bh(&pl->lock);

    return;
}
