#include <project.h>

#include "object_pool/object_pool.h"

#include "scn_machine.h"

#include "statistic.h"

__inline__ void __clean_built_list(scan_machine_t *ma)
{
    while (!__list_is_empty(&ma->built_nodes)) {
        LIST_HEAD *entry;
        scan_node_t *node;

        __list_remove_tail(&ma->built_nodes, entry);
        node = OBJECT_PTR(entry, scan_node_t, _u_.list);
        delete_scn_node(ma, node);
    }
}

__inline__ void __clean_unbuilt_list(scan_machine_t *ma)
{
    while (!__list_is_empty(&ma->unbuild_nodes)) {
        LIST_HEAD *entry;
        scan_node_t *node;
        __list_remove_tail(&ma->unbuild_nodes, entry);
        node = OBJECT_PTR(entry, scan_node_t, t.unbuilt.ub_list);
        delete_scn_node(ma, node);
    }
}

void delete_scan_machine(scan_machine_t *ma)
{
    __clean_unbuilt_list(ma);
    __clean_built_list(ma);
    kfree(ma);
    DEC_OBJ_COUNT(scan_machine);
    return;
}

scan_machine_t *new_scan_machine(void)
{
    scan_machine_t *ma = (scan_machine_t *)kmalloc(sizeof(scan_machine_t), GFP_ATOMIC);
    if (unlikely(!ma))
        return NULL;
        
    memset(ma, 0, sizeof(scan_machine_t));

    __list_init(&ma->unbuild_nodes);
    __list_init(&ma->built_nodes);
    ma->header.destroy_ptr = (destroy_atom_ptr)delete_scan_machine;
    INC_OBJ_COUNT(scan_machine);
    return ma;
}

scan_machine_t *scan_scn_create_scan_machine(int node_ext_len, int is_dfa)
{
    scan_machine_t *ma = new_scan_machine();
    if (unlikely(!ma))
        return NULL;
        
    comm_object_addref(ma);
    ma->op.node_ext_len = node_ext_len;
    ma->root = new_scn_node(ma);
    ma->is_dfa = (!!is_dfa);
    __list_add_tail(&ma->unbuild_nodes, &ma->root->t.unbuilt.ub_list);
    return ma;
}
