#include <project.h>
#include "object_pool/object_pool.h"

#include "scn_machine.h"

#include "key_tree.h"
#include "scn_content_search.h"  // -- ����ʹ��
#include "scn_multi_match.h"
#include "scn_node.h"
#include "statistic.h"

#define SCN_MUL_CONTAINER_CACHE_NAME "SCN_MUL_CONTAINER_CACHE"

static void *s_scn_container_cache = NULL;

int scan_scn_container_module_init(void)
{
    s_scn_container_cache = object_create_pool(100, sizeof(SCN_MUL_CONTAINER),
                                               SCN_MUL_CONTAINER_CACHE_NAME);
    if (unlikely(!s_scn_container_cache))
        return -1;

    return 0;
}

void scan_scn_container_module_destroy(void)
{
    if (s_scn_container_cache) {
        object_destroy_pool(s_scn_container_cache);
    }
    return;
}

static __inline__ SCN_MUL_CONTAINER *scn_mul_container_new(void)
{
    SCN_MUL_CONTAINER *contain =
        (SCN_MUL_CONTAINER *)object_pool_alloc(s_scn_container_cache);

    if (unlikely(!contain))
        return NULL;
        
    memset(contain, 0, sizeof(SCN_MUL_CONTAINER));
    INC_OBJ_COUNT(scan_scn_mul_container);
    return contain;
}

static __inline__ void scn_mul_container_delete(SCN_MUL_CONTAINER *contain)
{
    object_pool_free(s_scn_container_cache, contain);
    DEC_OBJ_COUNT(scan_scn_mul_container);
    return;
}

static int init_mul_node(void *node)
{
    SCN_MUL_NODE *mul = (SCN_MUL_NODE *)node;
    mul->match_keys = NULL;
    return 0;
}

static int destroy_mul_node(void *node)
{
    SCN_MUL_NODE *mul = (SCN_MUL_NODE *)node;
    SCN_MUL_CONTAINER *contain = mul->match_keys;
    while (contain) {
        mul->match_keys = contain->next;
        scn_mul_container_delete(contain);
        contain = mul->match_keys;
    }

    return 0;
}

int insert_mul_key(void *node, void *key_ctx)
{
    SCN_MUL_NODE *mul = (SCN_MUL_NODE *)node;
    SCN_MUL_CONTAINER *contain = scn_mul_container_new();

tag_pr_debug("\n");

    if (unlikely(!contain))
        return -1;

    contain->next = mul->match_keys;
    contain->key_ctx = key_ctx;
    mul->match_keys = contain;

    return 0;
}

scan_machine_t *scan_scn_create_multi_machine(int is_dfa)
{
    scan_machine_t *ma = scan_scn_create_scan_machine(
        sizeof(SCN_MUL_NODE) - sizeof(scan_node_t), is_dfa);

    if (unlikely(!ma))
        return NULL;
        
    ((SCN_MUL_NODE *)(ma->root))->is_root = 1;
    ma->op.init_node = init_mul_node;
    ma->op.destroy_node = destroy_mul_node;
    ma->op.insert_key = insert_mul_key;

    init_mul_node(ma->root);

    return ma;
}

// (-1):ppNode 返回到上一个失败状态
// 0:ppNode 返回到根节点，key tree中，状态转移成功，返回一个状态
static __inline__ int __multi_next_node(scan_machine_t *ma, SCN_MUL_NODE *node,
                                        unsigned char val,
                                        SCN_MUL_NODE **ppNode)
{
    int retval = -1;
    SCN_MUL_NODE *next = NULL;
    
    if (ppNode)
        *ppNode = NULL;

    if (NODE_BIT_IS_VALID(val, node->scan_scn_node.t.transfer)) {
        next = (SCN_MUL_NODE *)node->scan_scn_node.t.transfer.nodes[val];
    }

    if (next) {
        retval = 0;
    } else {
        next = node->failed_node;
    }

    if (ppNode)
        *ppNode = next;
    return retval;
}

// 创建一个新的根节点的数据结构
__inline__ int __set_failed_node(scan_machine_t *ma, SCN_MUL_NODE *parent,
                                 SCN_MUL_NODE *node)
{
    SCN_MUL_NODE *target;
    SCN_MUL_NODE *failed_node;
    unsigned char value = node->scan_scn_node.value;
    SCN_MUL_CONTAINER *contain;

    failed_node = parent->failed_node;
    while (1) {
        if (0 == __multi_next_node(ma, failed_node, value, &target)) {
            node->failed_node = target;
            failed_node = target;

            // 使用该数据结构
            break;
        } else {
            // 如果failed节点为root节点，则将failed节点设为root节点
            if (failed_node->is_root) {
                // 移除循环后，将状态自动转为0状态
                node->failed_node = failed_node;
                break;
            }
            failed_node = failed_node->failed_node;
        }
    }

    contain = node->failed_node->match_keys;
    while (contain) {
        insert_mul_key(node, contain->key_ctx);
        contain = contain->next;
    }
    return 0;
}

void __print_built_node_list(LIST_HEAD *list_head)
{
    LIST_HEAD *entry;

    entry = list_head->next;
    while (entry != list_head) {
        SCN_MUL_NODE *node =
            OBJECT_PTR(entry, SCN_MUL_NODE, scan_scn_node._u_.list);

#ifdef __SCN_NODE_DEBUG__
        // -- DbgPrint("<magic:%d>next = %p, prev = %p\n", node->scn_node.magic,
        // node->scn_node.u.list.next, node->scn_node.u.list.prev);
#else
        // -- DbgPrint("next = %p, prev = %p\n", node->scn_node.u.list.next,
        // node->scn_node.u.list.prev);
#endif
        entry = entry->next;
    }
}

static void __print_unbuilt_node_list(LIST_HEAD *list_head)
{
    LIST_HEAD *entry;

    entry = list_head->next;
    while (entry != list_head) {
        SCN_MUL_NODE *node =
            OBJECT_PTR(entry, SCN_MUL_NODE, scan_scn_node.t.unbuilt.ub_list);

        if (0 > __check_patterns((CNT_NODE *)node)) {
            // -- DbgPrint("__print_unbuilt_node_list pattern error\n");
        }
#ifdef __SCN_NODE_DEBUG__
        // -- DbgPrint("<magic:%d>next = %p, prev = %p\n", node->scn_node.magic,
        // node->scn_node.u.list.next, node->scn_node.u.list.prev);
#else
        // -- DbgPrint("next = %p, prev = %p\n", node->scn_node.u.list.next,
        // node->scn_node.u.list.prev);
#endif
        entry = entry->next;
    }
}

__inline__ int __build_nfa(scan_machine_t *ma)
{
    int i;

    SCN_MUL_NODE *root = (SCN_MUL_NODE *)ma->root;
    SCN_MUL_NODE *parent = (SCN_MUL_NODE *)root;
    LIST_HEAD node_list;
    LIST_HEAD total_list;

    __list_init(&node_list);
    __list_init(&total_list);

    for (i = 0; i < INDICES_COUNT; i++) {
        if (NODE_BIT_IS_VALID(i, parent->scan_scn_node.t.transfer)) {
            SCN_MUL_NODE *node =
                (SCN_MUL_NODE *)parent->scan_scn_node.t.transfer.nodes[i];
            __list_remove_item( &node->scan_scn_node._u_.list);  
            __list_add_tail(&node_list, &node->scan_scn_node._u_.list);
            node->failed_node = root;
        }
    }

    while (!__list_is_empty(&node_list)) {
        LIST_HEAD *entry;
        __list_remove_head(&node_list, entry);
        parent = OBJECT_PTR(entry, SCN_MUL_NODE, scan_scn_node._u_.list);
        __list_add_tail(&ma->built_nodes, &parent->scan_scn_node._u_.list);

        for (i = 0; i < INDICES_COUNT; i++) {
            if (NODE_BIT_IS_VALID(i, parent->scan_scn_node.t.transfer)) {
                SCN_MUL_NODE *node;

                node =
                    (SCN_MUL_NODE *)parent->scan_scn_node.t.transfer.nodes[i];

                __list_remove_item(&node->scan_scn_node._u_.list);  
                __list_add_tail(&node_list, &node->scan_scn_node._u_.list);

                __set_failed_node(ma, parent, node);
            }
        }
    }

    return 0;
}

static __inline__ SCN_MUL_NODE *__next_node__(SCN_MUL_NODE *failed_node,
                                              unsigned char val)
{
    SCN_MUL_NODE *node;
    node = (SCN_MUL_NODE *)failed_node->scan_scn_node.t.transfer.nodes[val];
    return node;
}

__inline__ int __nfa_to_dfa(scan_machine_t *ma)
{
    unsigned char i;

    SCN_MUL_NODE *root = (SCN_MUL_NODE *)ma->root;
    SCN_MUL_NODE *parent = (SCN_MUL_NODE *)root;
    LIST_HEAD node_list;

    __list_init(&node_list);

    // -- root->failed_node = root;
    for (i = 0; i < INDICES_COUNT; i++) {
        if (NODE_BIT_IS_VALID(i, parent->scan_scn_node.t.transfer)) {
            SCN_MUL_NODE *node =
                (SCN_MUL_NODE *)parent->scan_scn_node.t.transfer.nodes[i];

            __list_remove_item(&node->scan_scn_node._u_.list); 
            __list_add_tail(&node_list, &node->scan_scn_node._u_.list);
        } else {
            parent->scan_scn_node.t.transfer.nodes[i] = (scan_node_t *)root;
        }
    }

    while (!__list_is_empty(&node_list)) {
        LIST_HEAD *entry;

        __list_remove_head(&node_list, entry);
        parent = OBJECT_PTR(entry, SCN_MUL_NODE, scan_scn_node._u_.list);
        __list_add_tail(&ma->built_nodes, &parent->scan_scn_node._u_.list);
        for (i = 0; i < INDICES_COUNT; i++) {
            if (NODE_BIT_IS_VALID(i, parent->scan_scn_node.t.transfer)) {
                SCN_MUL_NODE *node;
                node =
                    (SCN_MUL_NODE *)parent->scan_scn_node.t.transfer.nodes[i];
                __list_remove_item(&node->scan_scn_node._u_.list);
                __list_add_tail(&node_list, &node->scan_scn_node._u_.list);
            } else {
                parent->scan_scn_node.t.transfer.nodes[i] =
                    (scan_node_t *)__next_node__(parent->failed_node, i);
            }
        }
    }

    return 0;
}

int scan_scn_multi_machine_compile(scan_machine_t *ma)
{
    if (unlikely(ma->is_built))
        return -1;

    // -- __print_unbuilt_node_list(&ma->unbuild_nodes);
    if (0 != scan_scn_key_tree_build(ma)) {
        return -1;
    }
    
    // -- __print_built_node_list(&ma->built_nodes);

    if (0 != __build_nfa(ma))
        return -1;

    if (ma->is_dfa) {
        __nfa_to_dfa(ma);
    }
    return 0;
}

int scan_scn_trie_compile(scan_machine_t *ma)
{
    if (unlikely(ma->is_built))
        return -1;

    // -- ma->is_built = 1;
    // -- return 0;

    // -- __print_unbuilt_node_list(&ma->unbuild_nodes);
    if (0 != scan_scn_key_tree_build(ma)) {
        return -1;
    }

    return 0;
}
