#ifndef __SCN_MULTI_MATCH_H_
#define __SCN_MULTI_MATCH_H_
#include "scan_rule_debug.h"
#include "scn_node.h"

typedef struct __SCN_MUL_NODE SCN_MUL_NODE;
typedef struct __SCN_MUL_CONTAINER SCN_MUL_CONTAINER;

struct __SCN_MUL_NODE {
    scan_node_t scan_scn_node;
    SCN_MUL_NODE *failed_node;
    unsigned char is_root : 1;
    SCN_MUL_CONTAINER *match_keys;
};

struct __SCN_MUL_CONTAINER {
    SCN_MUL_CONTAINER *next;
    void *key_ctx;
};

// 根据给定的值，在多状态机中进行状态转移，返回目标节点
// val: 要匹配的值（16进制）
static __inline__ SCN_MUL_NODE *
scan_scn_multi_machine_go(scan_machine_t *ma, SCN_MUL_NODE *prev_node,
                          unsigned int val)
{
    register SCN_MUL_NODE *target = NULL;
    register SCN_MUL_NODE *node = prev_node;

    while (node) {
        if (NODE_BIT_IS_VALID(val, node->scan_scn_node.t.transfer)) {
            target = (SCN_MUL_NODE *)node->scan_scn_node.t.transfer.nodes[val];
            if (target)
                goto DONE;
        }

        // 当遇到root时，说明已经遍历到根节点，此时跳出循环并返回root节点
        if (unlikely(node->is_root)) {
            target = node;
            goto DONE;
        }

        node = node->failed_node;
    }

DONE:
    return target;
}

extern int scan_scn_multi_machine_compile(scan_machine_t *ma);
extern int scan_scn_trie_compile(scan_machine_t *ma);

//extra 用于 SCN_MUL_NODE 额外添加的字段
extern scan_machine_t *scan_scn_create_multi_machine(int is_dfa);  

extern int scan_scn_container_module_init(void);
extern void scan_scn_container_module_destroy(void);


#endif
