#include "hash_if.h"

#include <crypto/hash.h>
#include <linux/async.h>
#include <linux/crypto.h>
#include <linux/err.h>
#include <linux/scatterlist.h>
#include "digest_ctx.h"

// hash实现
// 目前三种思路： 同步sm3、异步sm3、直接使用sm3算法
// 同步sm3可用
// 异步sm3不可用
// 直接使用sm3算法未验证

#define HASH_TYPE_AHASH 1  // 异步hash
#define HASH_TYPE_SHASH 2  // 同步hash
#define HASH_TYPE_SM3 3    // 直接使用sm3

char *hash_type_str(int type)
{
    if (type == HASH_TYPE_AHASH)
        return "ahash";
    else if (type == HASH_TYPE_SHASH)
        return "shash";
    else if (type == HASH_TYPE_SM3)
        return "sm3";
    else
        return "";
}

int hash_type = HASH_TYPE_SM3;
const char hash_alo[] = "sm3";

/* 异步哈希计算 */
struct hash_async_ctx {
    struct crypto_ahash *tfm;
    struct ahash_request *req;
    // unsigned char *result;
    unsigned char result[SM3_DIGEST_SIZE];
    unsigned int result_size;
    wait_queue_head_t wait;
    int err;
};


// #if LINUX_VERSION_CODE > KERNEL_VERSION(6, 6, 0)
// static void hash_complete(void *req, int err)
// #else
// static void hash_complete(struct crypto_async_request *req, int err)
// #endif
// {
//     struct hash_async_ctx *ctx = req->data;

//     if (err == -EINPROGRESS)
//         return;

//     print_hex_dump(KERN_INFO, "", DUMP_PREFIX_NONE, 16, 1, ctx->result,
//                    ctx->result_size, false);

//     ctx->err = err;
//     wake_up(&ctx->wait);
// }

// err
// 异步hash不能用，因为，当在内核为buf分配空间并写数据后，功能正常
// 但是，当通过指针数组赋值时，则会出现hash在两个值间跳跃的情况
static int cacl_buf_ahash(const unsigned char *buf, size_t buf_len,
                          unsigned char *hash)
{
    int retval = -1;
//     struct hash_async_ctx ctx = {};
//     struct scatterlist sg;
//     int ret = 0;
//     int i;

//     ctx.result_size = SM3_DIGEST_SIZE;
//     // ctx.result_size = crypto_ahash_digestsize(crypto_alloc_ahash(hash_alo, 0,
//     // 0)); ctx.result = kmalloc(ctx.result_size, GFP_KERNEL); if (!ctx.result)
//     // {
//     //     pr_err("Error allocating result buffer\n");
//     //     return -ENOMEM;
//     // }

//     ctx.tfm = crypto_alloc_ahash(hash_alo, 0, 0);
//     if (IS_ERR(ctx.tfm)) {
//         pr_err("Failed to load transform for %s: %ld\n", hash_alo,
//                PTR_ERR(ctx.tfm));
//         retval = PTR_ERR(ctx.tfm);
//         goto DONE;
//     }

//     ctx.req = ahash_request_alloc(ctx.tfm, GFP_KERNEL);
//     if (!ctx.req) {
//         pr_err("Failed to allocate request\n");
//         retval = -ENOMEM;
//         goto DONE;
//     }

//     init_waitqueue_head(&ctx.wait);

//     sg_init_one(&sg, buf, buf_len);

//     ahash_request_set_callback(ctx.req, CRYPTO_TFM_REQ_MAY_BACKLOG,
//                                hash_complete, &ctx);

//     ahash_request_set_crypt(ctx.req, &sg, ctx.result, buf_len);

//     ctx.err = 0;
//     ret = crypto_ahash_digest(ctx.req);
//     if (ret == -EINPROGRESS) {
//         wait_event(ctx.wait, ctx.err != 0);
//         ret = ctx.err;
//     }

//     ret = ctx.err;
//     if (ret) {
//         pr_err("Failed to compute hash: %d\n", ret);
//         goto DONE;
//     }

//     pr_info("Hash result: ");
//     for (i = 0; i < ctx.result_size; i++) pr_cont("%02x", ctx.result[i]);
//     pr_cont("\n");

//     memcpy(hash, ctx.result, ctx.result_size);

//     retval = 0;
// DONE:
//     // if (ctx.result)
//     //     kfree(ctx.result);
//     if (ctx.req)
//         ahash_request_free(ctx.req);
//     if (ctx.tfm)
//         crypto_free_ahash(ctx.tfm);

    return retval;
}

// 同步hash算法
static int cacl_buf_shash(const unsigned char *buf, size_t buf_len,
                          unsigned char *hash)
{
    int retval = -1;
    struct crypto_shash *tfm;
    struct shash_desc *desc;
    // struct scatterlist sg;
    int ret;

    tfm = crypto_alloc_shash(hash_alo, 0, 0);
    if (IS_ERR(tfm)) {
        pr_err("Failed to load transform for %s: %ld\n", hash_alo,
               PTR_ERR(tfm));
        retval = PTR_ERR(tfm);
        goto DONE;
    }

    desc = kmalloc(sizeof(struct shash_desc) + crypto_shash_descsize(tfm),
                   GFP_KERNEL);
    if (!desc) {
        pr_err("Failed to allocate shash desc\n");
        retval = -ENOMEM;
        goto DONE;
    }

    desc->tfm = tfm;

#if LINUX_VERSION_CODE <= KERNEL_VERSION(5, 4, 0)
    desc->flags = 0;
#endif

    // sg_init_one(&sg, buf, buf_len);

    ret = crypto_shash_digest(desc, buf, buf_len, hash);
    if (ret) {
        pr_err("Failed to compute hash: %d\n", ret);
        goto DONE;
    }

    retval = 0;
DONE:
    if (desc)
        kfree(desc);
    if (tfm)
        crypto_free_shash(tfm);

    return retval;
}

// 直接使用sm3
static int cacl_buf_sm3(const unsigned char *buf, size_t buf_len,
                        unsigned char *hash)
{
    int retval = -1;
    DIGEST_CTX ctx;

    digest_ctx_init(&ctx);
    digest_ctx_update(&ctx, (unsigned char *)buf, buf_len);
    digest_ctx_finish(&ctx, hash);

    retval = 0;
// DONE:
    return retval;
}

int cacl_buf_hash(const unsigned char *buf, size_t buf_len, unsigned char *hash)
{
    printk(KERN_INFO "cacl hash with [%s]-[%s]\n", hash_alo,
           hash_type_str(hash_type));

    if (hash_type == HASH_TYPE_AHASH)
        return cacl_buf_ahash(buf, buf_len, hash);
    else if (hash_type == HASH_TYPE_SHASH)
        return cacl_buf_shash(buf, buf_len, hash);
    else  // sm3
        return cacl_buf_sm3(buf, buf_len, hash);
}
