#include <linux/slab.h>
#include "ioctrl_param.h"
#include "project.h"

IOCTL_PARAM *ioctl_param_create(SEC_IOCTL_DATA *__data)
{
    BOOL bflag = FALSE;
    IOCTL_PARAM *param = NULL;
    void *input;

    param = (IOCTL_PARAM *)kmalloc(sizeof(IOCTL_PARAM), GFP_ATOMIC);
    if (!param)
        return NULL;

    memset(param, 0, sizeof(IOCTL_PARAM));
    param->uRawMode = __data->uRawMode;

    input = (void *)(unsigned long)__data->input;
    if (input != NULL && __data->input_len > 0) {
        if (__data->input_len > 4096 * 4)
            goto DONE;

        param->inputBuffer =
            (unsigned char *)kmalloc(__data->input_len, GFP_ATOMIC);
        if (!param->inputBuffer)
            goto DONE;

        if (unlikely(0 != copy_from_user(param->inputBuffer, input,
                                         __data->input_len))) {
            // -- retval = -EINVAL;
            goto DONE;
        }
    }

    if (__data->output != NULL && __data->outputLen > 0) {
        if (0 == param->uRawMode) {
            if (__data->outputLen > 4096 * 4) {
                // -- printk("<1> outputLen tool long....\n");
                goto DONE;
            }
            param->outputBuffer =
                (unsigned char *)kmalloc(__data->outputLen, GFP_ATOMIC);
            if (!param->outputBuffer)
                goto DONE;
        } else {
            param->outputBuffer = __data->output;
        }
    }
    param->uCode = __data->uCode;
    param->outputBufferLength = __data->outputLen;
    param->inputBufferLength = __data->input_len;
    bflag = TRUE;
DONE:
    if (!bflag) {
        if (param) {
            ioctl_param_destroy(param);
            param = NULL;
        }
    }
    return param;
}

void ioctl_param_destroy(IOCTL_PARAM *param)
{
    if (param->inputBuffer)
        kfree(param->inputBuffer);

    if (param->outputBuffer) {
        if (0 == param->uRawMode) {
            kfree(param->outputBuffer);
        }
    }
    kfree(param);
    return;
}