#include <linux/errno.h>
#include <project.h>

#include "common/general/include/ioctl_mod_prot.h"
#include "common/config/sec_ply_if.h"
#include "common/audit/sec_audit.h"
#include "ioctl_handle.h"

int ioctl_reload_ply(IOCTL_PARAM *param)
{
    int nt_status = STATUS_SUCCESS;
    int ret = 0;

    PVOID ioBuffer;
    LONG inputBufferLength;
    unsigned int type;

    ioBuffer = param->inputBuffer;
    inputBufferLength = param->inputBufferLength;

    if ((inputBufferLength < sizeof(int))) {
        nt_status = STATUS_INVALID_PARAMETER;
        goto DONE;
    }

    type = *(unsigned int *)ioBuffer;

    tag_pr_debug("sec ply load [%d]\n", type);

    // 这里可以加载 文件策略 或 开关策略
    if (type > SEC_SW_MIN && type < SEC_SW_MAX)
        ret = sec_sw_load(type);
    else 
        ret = sec_ply_load(type);

    if (ret) 
        nt_status = STATUS_INVALID_PARAMETER;
    else 
        nt_status = STATUS_SUCCESS;

DONE:
    param->uRetValue = nt_status;
    param->outputBufferLength = 0;
    return nt_status;
}

long nfssec_ioctl_handle(struct file *filep, void *__user __data, f_dispatch_ioctl dispatch)
{
    int retval = -1;
    SEC_IOCTL_DATA __ioctl;
    IOCTL_PARAM *param = NULL;

    if (unlikely(0 != copy_from_user(&__ioctl, __data, sizeof(SEC_IOCTL_DATA)))) {
        retval = -EINVAL;
        goto DONE;
    }

    param = ioctl_param_create(&__ioctl);
    if (!param) {
        retval = -EINVAL;
        goto DONE;
    }

    __ioctl.uRetValue = dispatch(filep, param);
    __ioctl.outputLen = 0;
    if (0 == __ioctl.uRetValue) {
        __ioctl.outputLen = param->outputBufferLength;
        if (param->outputBufferLength > 0) {
            if (0 == param->uRawMode) {
                if (0 != copy_to_user((void *)__ioctl.output,
                                      param->outputBuffer,
                                      param->outputBufferLength)) {
                    goto DONE;
                }
            }
        }
    }
    if (0 != copy_to_user(__data, &__ioctl, sizeof(SEC_IOCTL_DATA))) {
        goto DONE;
    }

    retval = 0;
DONE:
    if (param) {
        ioctl_param_destroy(param);
        param = NULL;
    }
    return retval;
}
