#include <project.h>
#include "kern_file.h"

ssize_t __sec_kern_read(struct file *file, loff_t off, void *buf, size_t count)
{
    ssize_t retval;
    loff_t pos;
#if LINUX_VERSION_CODE < KERNEL_VERSION(4, 14, 0)
    retval = kernel_read(file, off, buf, count);
#else
    pos = off;
    retval = kernel_read(file, buf, count, &pos);
#endif
    return retval;
}

int kern_fopen(KERN_FILE *file, const kchar *filename, const char *mode)
{
    int retval = -1;

    memset(file, 0, sizeof(KERN_FILE));

    file->offset = 0;
    file->fp = filp_open(filename, O_RDONLY | O_SYNC, 0);
    if (IS_ERR(file->fp)) {
        file->fp = NULL;
        // -- printk("[kern_fopen] open config (%s) error!\n", filename);
        retval = -EPERM;
        goto DONE;
    } else {
        retval = 0;
        // -- printk("[+] [kern_fopen] open config (%s) successfully.....\n",
        // filename);
    }

DONE:
    return retval;
}

void kern_fclose(KERN_FILE *file)
{
    if (file->fp) {
        filp_close(file->fp, NULL);
    }
    return;
}

LONGLONG kern_file_length(KERN_FILE *file)
{
    LONGLONG retval = -1;
    loff_t i_size;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 0, 0)
    i_size = i_size_read(file->fp->f_path.dentry->d_inode);
#else
    i_size = i_size_read(file->fp->f_dentry->d_inode);
#endif

    retval = i_size;

    return retval;
}

size_t kern_fread(void *buffer, size_t size, size_t count, KERN_FILE *file)
{
    size_t retval = 0;
    int rlen = 0;
    unsigned int total = size * count;

    rlen = __sec_kern_read(file->fp, file->offset, buffer, total);
    if (rlen < 0) {
        goto DONE;
    }
    retval = rlen / size;

    rlen = rlen % (unsigned int)size;

    file->offset += (retval * size);

DONE:
    return retval;
}
