#include <project.h>
#include <linux/fs.h>
#include <linux/wait.h>
#include <linux/sched.h>
#include <linux/fcntl.h>
#include <linux/file.h>
#include <linux/delay.h>
#include <linux/workqueue.h>
#include "common/kern_file.h"

static int fs_ready = 0;
static int ioctl_ready = 0;

DECLARE_WAIT_QUEUE_HEAD(fs_waitqueue);
DECLARE_WAIT_QUEUE_HEAD(ioctl_waitqueue);

static struct work_struct filesystem_work;
static struct work_struct ioctl_work;

// static struct delayed_work filesystem_work;
// static struct delayed_work ioctl_work;

struct workqueue_struct *g_check_q = NULL;

static void fs_check_work(struct work_struct *work)
{
    while (1) {
        if (current->fs) {
            fs_ready = 1;
            wake_up(&fs_waitqueue);
            break;
        }

        msleep(100);

    }
}

static void ioctl_check_work(struct work_struct *work)
{
    int ret = 0;
    KERN_FILE fp;

tag_pr_debug("==========\n");
    while (1) {
        ret = kern_fopen(&fp, "/dev/null", "rb");
        if (!ret) {
            tag_pr_debug("==========\n");

            kern_fclose(&fp);
            ioctl_ready = 1;
            wake_up(&ioctl_waitqueue);
            break;
        }

        msleep(100);
    }
    
tag_pr_debug("==========\n");


}

int kernel_check_q_init(void)
{
    int retval = -1;

    if (g_check_q != NULL) {
        retval = 0;
        goto DONE;
    }

    g_check_q = create_workqueue("kernel_check_q");
    if (!g_check_q) {
        tag_pr_debug("create workqueue\n");
        goto DONE;
    }    

    retval = 0;
DONE:
    return retval;
}

void kernel_check_q_exit(void)
{
    flush_workqueue(g_check_q);
    destroy_workqueue(g_check_q);
}

void ioctl_check_ok(void)
{
    INIT_WORK(&ioctl_work, ioctl_check_work);

    queue_work(g_check_q, &ioctl_work);

    wait_event(ioctl_waitqueue, ioctl_ready);
}

void fs_check_ok(void)
{
    INIT_WORK(&filesystem_work, fs_check_work);

    queue_work(g_check_q, &filesystem_work);

    tag_pr_debug("before wait event fs\n");
    wait_event(fs_waitqueue, fs_ready);
    tag_pr_debug("after wait event fs\n");

}


// void ioctl_check_ok(void)
// {
//     INIT_DELAYED_WORK(&ioctl_work, ioctl_check_work);

//     queue_delayed_work(g_check_q, &ioctl_work, msecs_to_jiffies(1000));
// }

// void fs_check_ok(void)
// {
//     INIT_DELAYED_WORK(&filesystem_work, fs_check_work);

//     queue_delayed_work(g_check_q, &filesystem_work, msecs_to_jiffies(1000));
// }

