#include <project.h>
#include "../common/spin_lock.h"
#include "../common/tag_print.h"
#include "common/general/comm_object.h"
#include "common/general/include/scan_machine/scan_machine.h"
#include "kmod_check_table.h"

typedef struct __KMOD_SEARCH_CTX {
    KMOD_CHECK_RESULT acitem;

    char *mod_name;
    long processId;
} KMOD_SEARCH_CTX;

typedef struct __KMOD_CHECK_EXT {
    unsigned int id;
    unsigned int type;
} KMOD_CHECK_EXT;

typedef struct __KMOD_CHECK_TRIE {
    void *trieObject;
    spinlock_t lock;
} KMOD_CHECK_TRIE;

static KMOD_CHECK_TRIE g_kmod_trie;

static int __kmod_trie_check(SCN_PATTERN *pattern, char *pattern_pos,
                             int offset, int left, void *search_context)
{
    int retval = SCN_CHECK_CONTINUE;

    KMOD_SEARCH_CTX *ctx = (KMOD_SEARCH_CTX *)search_context;
    // KMOD_CHECK_EXT *ext = __SCAN_GET_EXT(KMOD_CHECK_EXT, pattern);
    // -- #define SCN_CHECK_CONTINUE (0)
    // -- #define SCN_CHECK_BREAK (1)

    // 精确匹配，防止后续还有字符时的误拦截
    if (left != 0)
        goto DONE;

    // todo:这里需要增加一个什么条件作为判断呢？
    // if (ext->type < __POLICY_TYPE_COUNT && ext->type > 0) {
    // ctx->acitem.checkFlags[ext->type] = 1;
    ctx->acitem.checkFlags[__POLICY_TYPE_KMOD] = 1;
    // } else {
    //     goto DONE;
    // }
DONE:
    return retval;
}

void *kmod_check_create_trie_object(void)
{
    void *obj = NULL;
    obj = scan_scn_trie_create(sizeof(KMOD_CHECK_EXT));
    return obj;
}

int kmod_check_build_trie_object(void *trieObj)
{
    int retval = scan_scn_trie_build(trieObj);
    return retval;
}

void kmod_check_update_trie_object(void *__trie_obj)
{
    spin_lock_bh(&g_kmod_trie.lock);
    if (NULL != g_kmod_trie.trieObject) {
        comm_object_release(g_kmod_trie.trieObject);
        g_kmod_trie.trieObject = NULL;
    }

    if (__trie_obj) {
        g_kmod_trie.trieObject = __trie_obj;
        comm_object_addref(__trie_obj);
    }
    spin_unlock_bh(&g_kmod_trie.lock);
    return;
}

char kmod_check_insert(void *__trie_obj, const char *mod_name, unsigned int id)
{
    SCN_PATTERN *pattern = NULL;
    unsigned short pat_len;
    KMOD_CHECK_EXT *ext;

    pat_len = strlen(mod_name);
    pattern = scan_scn_trie_insert_key(__trie_obj, (unsigned char *)mod_name,
                                       pat_len, (void *)(unsigned long)id);
    if (!pattern) {
        return FALSE;
    }
    ext = __SCAN_GET_EXT(KMOD_CHECK_EXT, pattern);
    memset(ext, 0, sizeof(*ext));
    // ext->type = type;
    ext->id = id;

    return TRUE;
}

char kmod_check(void * ply_head, char *mod_name, KMOD_CHECK_RESULT *pAcitem)
{
    char retval = FALSE;
    KMOD_SEARCH_CTX ctx;
    void *__trie_obj = ply_head;

    memset(&ctx, 0, sizeof(ctx));

    // spin_lock_bh(&g_kmod_trie.lock);
    ctx.mod_name = mod_name;

    scan_scn_trie_traverse(__trie_obj, (unsigned char *)mod_name,
                           strlen(mod_name), __kmod_trie_check, &ctx);

    retval = TRUE;
    *pAcitem = ctx.acitem;

    // spin_unlock_bh(&g_kmod_trie.lock);

    return retval;
}

char kmod_table_module_init(void)
{
    char retval = FALSE;

    spin_lock_init(&g_kmod_trie.lock);

    g_kmod_trie.trieObject = kmod_check_create_trie_object();
    if (!g_kmod_trie.trieObject) {
        return FALSE;
    }

    return retval;
}

void kmod_table_module_destroy(void)
{
    if (NULL != g_kmod_trie.trieObject) {
        comm_object_release(g_kmod_trie.trieObject);
        g_kmod_trie.trieObject = NULL;
    }
    return;
}
