#include <project.h>
#include "common/tag_print.h"
#include "common/hookmem.h"
#include "n_trace_table.h"

trace_table_t g_trace_table;

static void __sec_trace_release(LIST_HEAD *entry)
{
    trace_ctx_t *ctx = OBJECT_PTR(entry, trace_ctx_t, head);
    comm_object_release(ctx);
    return;
}

static unsigned char *__sec_trace_value(LIST_HEAD *entry)
{
    trace_ctx_t *ctx = OBJECT_PTR(entry, trace_ctx_t, head);
    unsigned char *ptr = (unsigned char *)&ctx->thId;
    return (unsigned char *)ptr;
}

BOOLEAN trace_table_init(int __count)
{
    trace_table_t *table = &g_trace_table;
    BOOLEAN retval = FALSE;
    int bucket_count = __count;
    spin_lock_init(&table->lock);

    hashtb_init_ex(&table->lookupTB, __sec_trace_value, __sec_trace_release,
                   sizeof(long), sec_vmalloc, sec_vfree);

    while (bucket_count > 10) {
        
        if (hashtb_set_buckets(&table->lookupTB, bucket_count)) 
            break;
       
        bucket_count = (bucket_count / 2);
    }
    if (bucket_count <= 10)
        return FALSE;

    retval = TRUE;
    return retval;
}

void trace_table_destroy(void)
{
    trace_table_t *table = &g_trace_table;
    hashtb_destroy(&table->lookupTB);
    return;
}

BOOLEAN trace_table_remove(long thId)
{
    trace_table_t *table = &g_trace_table;
    BOOLEAN retval = FALSE;

    spin_lock(&table->lock);

    retval = hashtb_remove_item(&table->lookupTB, (unsigned int *)&thId);

    spin_unlock(&table->lock);
    return retval;
}

static void __trace_ctx_destroy(trace_ctx_t *thCtx)
{
    kfree(thCtx);
}

trace_ctx_t *trace_ctx_create(void)
{
    trace_ctx_t *ctx;

    ctx = (trace_ctx_t *)kmalloc(sizeof(trace_ctx_t), GFP_ATOMIC);
    if (!ctx)
        return NULL;

    memset(ctx, 0, sizeof(trace_ctx_t));

    ctx->objHdr.destroy_ptr = (destroy_atom_ptr)__trace_ctx_destroy;
    comm_object_addref(ctx);
    return ctx;
}

trace_ctx_t *trace_table_get(long thId, int *exist)
{
    trace_table_t *table = &g_trace_table;
    trace_ctx_t *ctx = NULL;
    LIST_HEAD *entry;

    spin_lock(&table->lock);
    entry = hashtb_find(&table->lookupTB, (unsigned int *)&thId);
    if (entry) {
        // printk("=======in trace_table_get, finded[%ld] !!!!!!!\n", thId);

        ctx = OBJECT_PTR(entry, trace_ctx_t, head);
        comm_object_addref(ctx);
        spin_unlock(&table->lock);

        *exist = 1;
        
        tag_pr_debug("=======in trace_table_get, tid[%d], ctx->trace_count[%d], exist[%d]\n", thId, ctx->trace_cnt, *exist);
        goto DONE;
    }
    spin_unlock(&table->lock);

    ctx = trace_ctx_create();
    if (!ctx)
        goto DONE;

    spin_lock(&table->lock);
    ctx->thId = thId;
    if (!hashtb_insert_item(&table->lookupTB, &ctx->head,
                            (unsigned int *)&ctx->thId)) {
        comm_object_release(ctx);
        ctx = NULL;
        spin_unlock(&table->lock);
        goto DONE;
    }
    // printk("=======in trace_table_get, insert[%ld] !!!!!!!\n", thId);

    comm_object_addref(ctx);
    spin_unlock(&table->lock);

DONE:
    return ctx;
}

// 遍历表，如果存在，则设置其ret值
void trace_table_set_ret(long thId, int ret)
{
    trace_table_t *table = &g_trace_table;
    trace_ctx_t *ctx = NULL;
    LIST_HEAD *entry;

    spin_lock(&table->lock);
    entry = hashtb_find(&table->lookupTB, (unsigned int *)&thId);
    if (entry) {
        ctx = OBJECT_PTR(entry, trace_ctx_t, head);
        atomic_set(&ctx->ret, ret);
        // printk("=======in trace_table_get, set ret[%d] !!!!!!!\n",
        // ctx->ret);
    }
    spin_unlock(&table->lock);
}
