#ifndef __TH_TRACE_TABLE_H_
#define __TH_TRACE_TABLE_H_

#include "common/spin_lock.h"
#include "common/general/comm_object.h"
#include "common/general/hashtb.h"
#include "include/misc/list_head.h"

typedef struct __TRACE_CTX {
    SEC_OBJECT objHdr;
    LIST_HEAD head;

    long thId;

    atomic_t trace_cnt;
    atomic_t ret;
} trace_ctx_t;

#define __ATOM_INC_TRACE(__p) atomic_inc_return(&(__p)->trace_cnt)
#define __ATOM_DEC_TRACE(__p) atomic_dec_return(&(__p)->trace_cnt)
#define __IS_IN_TRACE(__p) (0 != atomic_read(&(__p)->trace_cnt))

#define TRACE_DEFINE()                \
    trace_ctx_t *__trace_ctx = NULL; \
    int __trace_flag = 0;           \
    int exist = 0;                    \
    static unsigned int up_cnt = 0;

// 遍历表，如果存在，则设置其ret值
void trace_table_set_ret(long thId, int ret);

// 目前似乎效果不明显，有必要时再使用
// 检查当前进程是否已经在跟踪中
// 返回值： -1 失败
//          0 不在跟踪中
//          1 在跟踪中
#define CHECK_IN_TRACE()                                               \
    ({                                                                     \
        int cnt = 0, ret;                                               \
        __trace_ctx = trace_table_get(current->pid, &exist);            \
        cnt = __trace_ctx ? atomic_inc_return(&__trace_ctx->trace_cnt) : 0; \
        __trace_flag = (__trace_ctx != NULL);                          \
        if (cnt == 0)                                                        \
            ret = -1;                                                        \
        else                                                               \
            ret = (exist == 0) ? 0 : 1;                                      \
        ret;                                                                 \
    })

#define EXIT_TRACE(ret)                         \
    do {                                            \
        if (__trace_flag) {                       \
            trace_table_set_ret(current->pid, ret); \
            __ATOM_DEC_TRACE(__trace_ctx);        \
        }                                           \
        if (__trace_ctx) {                        \
            comm_object_release(__trace_ctx);     \
        }                                           \
    } while (0);

// 当策略更新时（如何判断？），需要继续后续的检查
// 是否需要？？？如果没有这个，当进程检查全完成后，策略更新完成时，还是能正常检查的；还是说存在这个空档期？
// #define IN_TRACE_HANDLE()                                   \
//     do {                                                    \
//         if (__trace_flag == 0)                            \
//             break;                                          \
//         if (exist == 0)                                     \
//             break;                                          \
//         if (up_cnt != g_globals.policy_up_cnt) {            \
//             up_cnt = g_globals.policy_up_cnt;               \
//             break;                                          \
//         }                                                   \
//         printk("============in trace[%d]\n", current->pid); \
//         retval = atomic_read(&__trace_ctx->ret);          \
//         goto DONE;                                          \
//     } while (0);


typedef struct __TRACE_TABLE {
    spinlock_t lock;
    HASHTB lookupTB;
} trace_table_t;

BOOLEAN trace_table_init(int __count);

void trace_table_destroy(void);

trace_ctx_t *trace_table_get(long thId, int *exist);
BOOLEAN trace_table_remove(long thId);

#endif
