#include <project.h>
#include "nac_ply.h"

// 全局nac表
static nac_ply_head_t * g_nac_ply = NULL;

// 切换全局nac表时所用的锁
spinlock_t nac_ply_lock;


// check if the direction is equal to rule_direction
// return 1/0 yes/no
static int is_direction_equal(unsigned int direction, unsigned int rule_direction)
{
    if (rule_direction == NAC_DIRECTION_ALL)
        return 1;

    if (direction == rule_direction) 
        return 1;
    else
        return 0;
}

// check if the proc is equal to rule_proto
// return 1/0 yes/no
static int is_protocol_equal(unsigned int proto, unsigned int rule_proto)
{
    if (rule_proto == NAC_PROTO_ALL)
        return 1;

    if (proto == rule_proto) 
        return 1;
    else
        return 0;
}

// check if the ip is in the ip_range
// return 1/0 yes/no
static int is_ip_in_range(unsigned int ip, nac_ip_range_t * ip_range)
{
    if (ip_range->from == NAC_IP_ALL && ip_range->to == NAC_IP_ALL)
        return 1;

    if (ip >= ip_range->from && ip <= ip_range->to) 
        return 1;
    else
        return 0;
}

// check if the port is in the port_range
// return 1/0 yes/no
static int is_port_in_range(unsigned int port, nac_port_range_t * port_range)
{
    if (port_range->from == NAC_PORT_ALL && port_range->to == NAC_PORT_ALL)
        return 1;

    if (port >= port_range->from && port <= port_range->to) 
        return 1;
    else
        return 0;
}

// check if the proc is equal to rule_proc
// return 1/0 yes/no
static int is_proc_equal(const char * proc, const char * rule_proc)
{
    if (!strcmp(rule_proc, NAC_APP_ALL))
        return 1;

    if (!strncmp(proc, rule_proc, strlen(rule_proc)))
        return 1;
    else
        return 0;
}

// void nac_update_rule(nac_ply_node_t *ply, int type, nac_rule_node_t *rule) 
// {
//     nac_rule_node_t *tmp_rule;
//     struct hlist_node *pos, *q;

//     spin_lock(&ply->lock);
//     if (type == NAC_PLY_TYPE_ALLOW) {
//         hlist_for_each_entry_safe(tmp_rule, pos, q, &ply->allow_list, hnode) {
//             if (tmp_rule->protocol == rule->protocol && 
//                 tmp_rule->ip.start == rule->ip.start && 
//                 tmp_rule->ip.end == rule->ip.end &&
//                 tmp_rule->port.start == rule->port.start &&
//                 tmp_rule->port.end == rule->port.end &&
//                 strcmp(tmp_rule->proc, rule->proc) == 0) {
//                 hlist_del(&tmp_rule->hnode);
//                 kfree(tmp_rule);
//                 break;
//             }
//         }
//         hlist_add_head(&rule->hnode, &ply->allow_list);

//     } else if (type == NAC_PLY_TYPE_DENY) {
//         hlist_for_each_entry_safe(tmp_rule, pos, q, &ply->deny_list, hnode) {
//             if (tmp_rule->protocol == rule->protocol && 
//                 tmp_rule->ip.start == rule->ip.start && 
//                 tmp_rule->ip.end == rule->ip.end &&
//                 tmp_rule->port.start == rule->port.start &&
//                 tmp_rule->port.end == rule->port.end &&
//                 strcmp(tmp_rule->proc, rule->proc) == 0) {
//                 hlist_del(&tmp_rule->hnode);
//                 kfree(tmp_rule);
//                 break;
//             }
//         }
//         hlist_add_head(&rule->hnode, &ply->deny_list);
//     }
//     spin_unlock(&ply->lock);
// }

//  ======================================================================
nac_rule_node_t * nac_rule_new(unsigned int direction, unsigned int protocol, nac_ip_range_t * ip,
                  nac_port_range_t * port, char *proc)
{
    nac_rule_node_t *new_rule = NULL;

    new_rule = (nac_rule_node_t *)kmalloc(sizeof(nac_rule_node_t), GFP_KERNEL);
    if (!new_rule)
        return NULL;

    memset(new_rule, 0, sizeof(nac_rule_node_t));

    new_rule->protocol = protocol; 
    memcpy(&new_rule->ip, ip, sizeof(*ip));  
    memcpy(&new_rule->port, port, sizeof(*port));  
    strncpy(new_rule->proc, proc, MAX_PROC_LEN);
    INIT_LIST_HEAD(&new_rule->node);

    return new_rule;
}

void nac_ply_node_free(nac_ply_node_t * ply_node)
{
    nac_rule_node_t *rule_pos, *rule_tmp;

    list_for_each_entry_safe(rule_pos, rule_tmp, &ply_node->allow_list, node) {
        list_del(&rule_pos->node);
        kfree(rule_pos);
    }

     list_for_each_entry_safe(rule_pos, rule_tmp, &ply_node->deny_list, node) {
        list_del(&rule_pos->node);
        kfree(rule_pos);
    }
}

// find if the item in nac_ply
// return rule/NULL : yes/no
nac_rule_node_t *nac_find_rule(nac_ply_node_t *ply, unsigned int type, 
            unsigned int direction, int protocol,
            unsigned int ip, unsigned int port, char *proc)
{
    nac_rule_node_t *tmp_rule;
    struct list_head * rule_list;

    if (type == NAC_PLY_TYPE_ALLOW)
        rule_list =  &ply->allow_list;
    else // type == NAC_PLY_TYPE_DENY
        rule_list =  &ply->deny_list;

    // if (strcmp(proc, "nc") == 0) 
    //     tag_pr_debug("nc ======  nac check direction[%d] ip[%u-%pI4] port[%d] protocol[%d] proc[%s]\n", 
    //         direction, ip, &ip, port, protocol, proc);

    // tag_pr_debug("nac check direction[%d] ip[%u-%pI4] port[%d] protocol[%d] proc[%s]\n", 
    //     direction, ip, &ip, port, protocol, proc);

    // spin_lock(&ply->lock);
    list_for_each_entry(tmp_rule, rule_list, node)
    {
        // tag_pr_debug("rule ip[%pI4-%pI4] port[%d-%d] protocol[%d] proc[%s]\n", 
        //     &tmp_rule->ip.from, &tmp_rule->ip.to, tmp_rule->port.from, tmp_rule->port.to,
        //     tmp_rule->protocol, tmp_rule->proc);

        if (is_direction_equal(direction, tmp_rule->direction) \
            && is_protocol_equal(protocol, tmp_rule->protocol) \
            && is_ip_in_range(ip, &tmp_rule->ip) \
            && is_port_in_range(port, &tmp_rule->port) \
            && is_proc_equal(proc, tmp_rule->proc)) {
                // spin_unlock(&ply->lock);
                return tmp_rule;
        }
    }
    // spin_unlock(&ply->lock);

    return NULL;
}

int is_nac_ply_empty(void)
{
    if (g_nac_ply == NULL || g_nac_ply->cnt == 0)
        return TRUE;
    else
        return FALSE;
}

// check if the item is in g_nac_ply
// type: out, NAC_PLY_TYPE_ALLOW/NAC_PLY_TYPE_DNEY
// return rule/NULL : yes/no
nac_rule_node_t * nac_check_item(unsigned int direction, unsigned int protocol,
                               unsigned int ip, unsigned int port,
                               char *proc, unsigned int * type)
{
    int i;
    nac_rule_node_t *tmp_rule;
    nac_ply_node_t *nac_ply = NULL;
    unsigned int ply_type = NAC_PLY_TYPE_ALLOW;

    // no nac ply found
    if (g_nac_ply == NULL)
        return NULL;

    spin_lock(&g_nac_ply->lock);

    //按照level，依次 遍历 全局nac 的每个策略
    for (i = 0; i < NAC_TABLE_SIZE; i++) {
       if (hlist_empty(&g_nac_ply->head[i]))
            continue;

        nac_ply = hlist_entry(g_nac_ply->head[i].first, nac_ply_node_t, hnode);

        // 在允许表中查找
        tmp_rule = nac_find_rule(nac_ply, NAC_PLY_TYPE_ALLOW, direction, protocol, ip, port, proc);   
        if (tmp_rule) {
            tag_pr_debug("nac find rule [allow]\n");
            ply_type = NAC_PLY_TYPE_ALLOW;
            break;
        }

        // 在禁止表中查找
        tmp_rule = nac_find_rule(nac_ply, NAC_PLY_TYPE_DENY, direction, protocol, ip, port, proc);   
        if (tmp_rule) {
            tag_pr_debug("nac find rule [deny]\n");
            ply_type = NAC_PLY_TYPE_DENY;
            break;
        }

        // 遍历完成，没找到
        tmp_rule = NULL;
    }

    spin_unlock(&g_nac_ply->lock);

    *type = ply_type;

    return tmp_rule;
}

// void nac_del_rule(nac_ply_node_t *ply, nac_rule_node_t *rule)
// {
//     spin_lock(&ply->lock);
//     hlist_del(&rule->hnode);
//     kfree(rule);
//     spin_unlock(&ply->lock);
// }

// //  ======================================================================

nac_ply_node_t * nac_ply_new(unsigned int level)
{
    nac_ply_node_t *ply_new = NULL;

    ply_new = (nac_ply_node_t *)kmalloc(sizeof(nac_ply_node_t), GFP_KERNEL);
    if (!ply_new)
        return NULL;
        
    memset(ply_new, 0, sizeof(nac_ply_node_t));

    ply_new->level = level;
    // spin_lock_init(&ply_new->lock);
    INIT_LIST_HEAD(&ply_new->allow_list);
    INIT_LIST_HEAD(&ply_new->deny_list);
    INIT_HLIST_NODE(&ply_new->hnode);    

    return ply_new;
}

int nac_ply_add_rule(nac_ply_head_t * ply_head, unsigned int level, int action, nac_rule_node_t *rule) 
{
    int retval = -1;
    nac_ply_node_t *nac_ply = NULL;

    // 如果该level已经存在，则使用此节点
    spin_lock(&ply_head->lock);
    if (!hlist_empty(&ply_head->head[level])) {
        nac_ply = hlist_entry(ply_head->head[level].first, nac_ply_node_t, hnode);
    }
    spin_unlock(&ply_head->lock);

    // todo： 似乎不存在new的情况，而且，上面的判空是否正确？
    if (nac_ply == NULL) {
       nac_ply = nac_ply_new(level);
        if (nac_ply == NULL)
            goto DONE;
    }

    spin_lock(&ply_head->lock);
    if (action == NAC_PLY_TYPE_ALLOW) {
        tag_pr_debug("add to allow_list\n");
        list_add_tail(&rule->node, &nac_ply->allow_list);
    } else if (action == NAC_PLY_TYPE_DENY) {
        tag_pr_debug("add to deny_list\n");
        list_add_tail(&rule->node, &nac_ply->deny_list);
    }
    hlist_add_head(&nac_ply->hnode, &ply_head->head[level]);
    ply_head->cnt++;
    spin_unlock(&ply_head->lock);

    retval = 0;
DONE:
    return retval;
}

// void nac_ply_del(unsigned int level)
// {
//     nac_ply_node_t *tmp_ply;
//     struct hlist_node *pos, *q;

//     spin_lock(&g_nac_ply->lock);
//     hlist_for_each_entry_safe(tmp_ply, pos, q, g_nac_ply, hnode)
//     {
//         if (tmp_ply->level == level) {
//             hlist_del(&tmp_ply->hnode);
//             kfree(tmp_ply);
//             break;
//         }
//     }
//     spin_unlock(&g_nac_ply->lock);
// }

// void nac_ply_del(nac_ply_head_t *ply_head, unsigned int level)
// {
//     nac_ply_node_t *tmp_ply;
//     struct hlist_node *pos, *q;
//     nac_rule_node_t *rule, *tmp_rule;

//     spin_lock(&ply_head->lock);
//     if (!hlist_empty(&ply_head->head[level])) {
//         // 释放这个节点上的allow_list和deny_list
//         tmp_ply = hlist_entry(ply_head->head[level].first, nac_ply_node_t, hnode);

//         hlist_del(&tmp_ply->hnode);

//         // 释放allow_list和deny_list
//         hlist_for_each_entry_safe(rule, tmp_rule, &tmp_ply->allow_list, hnode) {
//             hlist_del(&rule->hnode);
//             kfree(rule);
//         }
//         hlist_for_each_entry_safe(rule, tmp_rule, &tmp_ply->deny_list, hnode) {
//             hlist_del(&rule->hnode);
//             kfree(rule);
//         }

//         kfree(tmp_ply); // 释放nac_ply_node_t
//     }
//     spin_unlock(&ply_head->lock);
// }


// void nac_ply_update(unsigned int level, int type, nac_rule_node_t *rule) 
// {
//     nac_ply_del(level);
//     nac_add_ply(level, type, rule);
// }

nac_ply_head_t * nac_ply_head_new(void) 
{
    nac_ply_head_t * nac_ply = NULL;
    int i;

    nac_ply = (nac_ply_head_t *)kmalloc(sizeof(nac_ply_head_t), GFP_KERNEL);
    if (nac_ply == NULL) 
        return NULL;

    memset(nac_ply, 0, sizeof(nac_ply_head_t));

    spin_lock_init(&nac_ply->lock);
    for (i = 0; i < NAC_TABLE_SIZE; i++) {
        INIT_HLIST_HEAD(&nac_ply->head[i]);
    }

    return nac_ply;
}

void nac_ply_head_free(nac_ply_head_t * nac_ply) 
{
    int i = 0;
    nac_ply_node_t *pos;

    if (nac_ply == NULL)
        return;

    spin_lock(&nac_ply->lock);
    for (i = 0; i < NAC_TABLE_SIZE; i++) {
        if (!hlist_empty(&nac_ply->head[i])) {
            pos = hlist_entry(nac_ply->head[i].first, nac_ply_node_t, hnode);
            nac_ply_node_free(pos);

            hlist_del(&pos->hnode);
            kfree(pos);
        }
    }
    spin_unlock(&nac_ply->lock);
}

void nac_ply_switch(nac_ply_head_t * ply_head)
{
    // 将旧的g_nac_ply释放，将新的ply_head赋给他
    spin_lock(&nac_ply_lock);
    if (g_nac_ply != NULL)
        nac_ply_head_free(g_nac_ply);
    g_nac_ply = ply_head;
    spin_unlock(&nac_ply_lock);
}
// ==========================================================
void nac_ply_init(void)
{
    spin_lock_init(&nac_ply_lock);
    // g_nac_ply = nac_ply_head_new();
    g_nac_ply = NULL;
}

void nac_ply_uninit(void)
{
    nac_ply_head_free(g_nac_ply);
    kfree(g_nac_ply); 
    g_nac_ply = NULL;
}

