#ifndef ___NAC_PLY_H_
#define ___NAC_PLY_H_
/**
 * @file nac_ply.h
 * @author wy 
 * @brief net access ctrl policy
 * @version 0.1
 * @date 2024-01-19
 * 
 * @copyright Copyright (c) 2024
 * 
 * 基于优先级，定义hash链表，每个优先级下，存在 “允许表” 和 “禁止表”
 * 查表时，按照优先级次序，从高到低，依次查询
 * 优先查询 允许表，如果存在则放行，不存在，则查询 禁止表，如果存在则禁止；其他情况默认放行
 * 
 * 操作逻辑：
 *  全局init、uninit
 *  
 *  加载策略时，add
 * 
 *  策略更新时，new_ply, add, switch
 * 
 */

// todo: ip str to int
// 策略hash表，是在创建时new好，还是在添加时再new？


#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/slab.h>
#include <linux/spinlock.h>
#include <linux/types.h>

// 策略优先级 范围
#define NAC_LEVEL_MIN 1
#define NAC_LEVEL_MAX 100
#define NAC_TABLE_SIZE 100 

#define NAC_PROTOCOL_ALL 0
#define NAC_PROTOCOL_TCP 1
#define NAC_PROTOCOL_UDP 2

// #define NAC_PORT_ALL    "*"
// #define NAC_IP_ALL      "*"
#define NAC_PORT_ALL    0
#define NAC_IP_ALL      0
#define NAC_APP_ALL     ""
#define NAC_PROTO_ALL   0

#define NAC_PLY_TYPE_DENY   0
#define NAC_PLY_TYPE_ALLOW  1

#define NAC_DIRECTION_ALL   0
#define NAC_DIRECTION_IN    1
#define NAC_DIRECTION_OUT   2

// todo: comm
#define MAX_PROC_LEN 512

// 端口（范围值）
typedef struct __nac_port_range {
    unsigned int from;
    unsigned int to;
} nac_port_range_t;

// IP（范围值）
typedef struct __nac_ip_range {
    unsigned int from;
    unsigned int to;
} nac_ip_range_t;

// nac 策略表 (头)
typedef struct __nac_ply_head {
    struct hlist_head head[NAC_TABLE_SIZE];
    spinlock_t lock;

    unsigned int cnt; // 策略数量，如果为0，可直接不检查

} nac_ply_head_t;

// nac 策略节点
typedef struct __nac_ply_node {
    unsigned int level;  // 作为主键

    // 挂着 nac_rule_node_t 表
    struct list_head  allow_list;  // 允许规则链表
    struct list_head  deny_list;   // 禁止规则链表

    // hash表，挂着ply_node
    struct hlist_node hnode;

} nac_ply_node_t;

// nac 规则节点
typedef struct __nac_rule_node {
    struct list_head  node;  // 挂载 allow_list 或 deny_list 上

    unsigned int direction;
    unsigned int protocol;
    nac_ip_range_t ip;
    nac_port_range_t port;
    char proc[MAX_PROC_LEN];
} nac_rule_node_t;


// nac_ply_node_t * nac_ply_new(unsigned int level);
// 似乎不需要delete，因为在整体的uninit时会整体删
// void nac_ply_del(unsigned int level);
// void nac_ply_update(unsigned int level, int type, nac_rule_node_t *rule); 

// void nac_ply_node_free(nac_ply_node_t * rule_node);

// void nac_del_rule(nac_ply_node_t *ply, nac_rule_node_t *rule);


// ==========================================================
void nac_ply_init(void);
void nac_ply_uninit(void);

nac_rule_node_t * nac_rule_new(unsigned int direction, unsigned int protocol, nac_ip_range_t * ip,
                  nac_port_range_t * port, char *proc);

nac_ply_head_t * nac_ply_head_new(void);
void nac_ply_head_free(nac_ply_head_t * nac_ply);


int nac_ply_add_rule(nac_ply_head_t * ply_head, unsigned int level, int action, nac_rule_node_t *rule);

// switch golbal nac ply to this
void nac_ply_switch(nac_ply_head_t * ply_head);

// check if the item is in g_nac_ply
// type: out, NAC_PLY_TYPE_ALLOW/NAC_PLY_TYPE_DENY
// return rule/NULL : yes/no
nac_rule_node_t * nac_check_item(unsigned int direction, unsigned int protocol,
                               unsigned int ip, unsigned int port,
                               char *proc, unsigned int * type);

// check if the nac ply is empty
// return: FALSE/TRUE no/yes
int is_nac_ply_empty(void);


#endif