#include "nfs_mnt.h"

struct fake_mount {
    struct list_head mnt_hash;  // -- 4.9 =>hlist_node
    struct fake_mount *mnt_parent;
    struct dentry *mnt_mountpoint;
    struct vfsmount mnt;

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 0, 0)
    // -- 4.2.0 && 4.9.0 && 4.4.0 && 4.18.0
    union {
        struct rcu_head mnt_rcu;
        struct llist_node mnt_llist;
    };
#endif

#ifdef CONFIG_SMP
    struct mnt_pcp __percpu *mnt_pcp;
#else
    int mnt_count;
    int mnt_writers;
#endif
    struct list_head mnt_mounts; /* list of children, anchored here */
    struct list_head mnt_child;  /* and going through their mnt_child */
};

static inline struct fake_mount *fake_real_mount(struct vfsmount *mnt)
{
    return container_of(mnt, struct fake_mount, mnt);
}

static struct vfsmount *next_mnt(struct vfsmount *__p, struct vfsmount *root)
{
    struct fake_mount *p = fake_real_mount(__p);

    struct list_head *next = p->mnt_mounts.next;

    if (next == &p->mnt_mounts) {
        while (1) {
            if (&p->mnt == root) {
                return NULL;
            }

            next = p->mnt_child.next;
            if (next != &p->mnt_parent->mnt_mounts) {
                break;
            }
            p = p->mnt_parent;
        }
    }
    p = list_entry(next, struct fake_mount, mnt_child);
    if (p) {
        return &p->mnt;
    } else {
        return NULL;
    }
}

struct vfsmount *get_dentry_mnt(struct dentry *dentry)
{
    struct vfsmount *p, *root, *retval = NULL;
    struct fs_struct *fs = current->fs;

    spin_lock_bh(&fs->lock);
    root = fs->root.mnt;
    if (root) {
        mntget(root);
    }
    spin_unlock_bh(&fs->lock);

    if (!root) {
        // -- printk("failed to call get_dentry_mnt root == NULL\n");
        return NULL;
    }

    rcu_read_lock();
    for (p = root; p; p = next_mnt(p, root)) {
        if (p->mnt_sb == dentry->d_sb) {
            mntget(p);
            retval = p;
            break;
        }
    }
    if (!retval) {
        // -- printk("failed to call get_dentry_mnt\n");
    }
    rcu_read_unlock();

    mntput(root);

    return retval;
}
