#include <linux/sched.h>
#include <linux/syscalls.h>
#include <project.h>

#include "hookmem.h"
#include "nfs_table.h"
#include "pid_ctx.h"

// -----------------------------------------------------------------------------------------
sec_gen_table_t g_nfs_table;

// -----------------------------------------------------------------------------------------
int nfs_lock_init(NFS_LOCK *__nfs_lock)
{
    spin_lock_init(&__nfs_lock->lock);
    // -- mutex_init(&xgenLock->lock);

    return 0;
}

void nfs_lock_destroy(NFS_LOCK *__nfs_lock)
{
    // -- mutex_destroy(&table->lock);

    return;
}

int nfs_lock(NFS_LOCK *__nfs_lock)
{

    spin_lock_bh(&__nfs_lock->lock);
    // -- mutex_lock(&xgenLock->lock);

    return 0;
}

int nfs_unlock(NFS_LOCK *__nfs_lock)
{
    spin_unlock_bh(&__nfs_lock->lock);
    // -- mutex_unlock(&xgenLock->lock);
    return 0;
}

static void __pid_ctx_release(LIST_HEAD *entry)
{
    PID_CONTEXT *ctx = OBJECT_PTR(entry, PID_CONTEXT, head);
    comm_object_release(ctx);
    return;
}

static unsigned char *__pid_ctx_value(LIST_HEAD *entry)
{
    PID_CONTEXT *ctx = OBJECT_PTR(entry, PID_CONTEXT, head);
    unsigned char *ptr = (unsigned char *)&ctx->PID;
    return (unsigned char *)ptr;
}

struct task_struct *find_task(sec_pid_t pid)
{
    struct task_struct *task;

    rcu_read_lock();

#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 18)

#ifdef __USE_PID_STRUCT_AS_KEY
    task = pid_task(pid, PIDTYPE_PID);
#else
    task = pid_task(find_pid_ns(pid, &init_pid_ns), PIDTYPE_PID);
#endif
#else
    // -- task = pid_task(pid, PIDTYPE_PID);
    task = find_task_by_pid_type(PIDTYPE_PID, pid);
#endif
    if (task) {
        get_task_struct(task);
        // -- put_task_struct
    }

    rcu_read_unlock();

    return task;
}

#ifdef __PID_GARGAGE_COLLECTION__
int nfs_table_gc(sec_gen_table_t *table)
{
    extern void sec_put_task_struct(struct task_struct * t);

    int del_count = 0;
    LIST_HEAD *entry;
    int count = 0;
    unsigned long __now = ktime_get_seconds();
    int moveCount = 0;
    int i;
    PID_CONTEXT *pidCtx;

    for (entry = table->gcList.next; entry != &table->gcList;) {
        struct task_struct *tsk = NULL;

        pidCtx = OBJECT_PTR(entry, PID_CONTEXT, gchead);
        entry = entry->next;

        tsk = find_task(pidCtx->PID);

        // -- if(NULL == pid_task(__GET pidCtx->PID, PIDTYPE_PID)){
        if (NULL == tsk) {
            // -- printk("[+] process exit (ref:%d):%s\n",
            // atomic_read(&pidCtx->objHdr.lRefCount), pidCtx->appFullName);

            __list_remove_item_safe(&pidCtx->gchead);

            comm_object_addref(pidCtx);
            __list_add_tail(&table->deleteList, &pidCtx->gchead);

            hashtb_remove_item_entry(&table->lookupTB, &pidCtx->head);
            pidCtx = NULL;
            del_count++;
            if (moveCount != 0) {
                moveCount = (count + 1) - del_count;
            }
        } else {
            sec_put_task_struct(tsk);
            if (__now > pidCtx->lastAccessTime) {
                if (__now < (pidCtx->lastAccessTime + 5)) {
                    break;
                }
            } else {
                break;
            }
        }

        count++;
    }

    if (0 == moveCount) {
        if (count > 10) {
            moveCount = 10;
        }
    } else if (moveCount > 10) {
        moveCount = 10;
    }

    for (i = 0, entry = table->gcList.next;
         (i < moveCount) && (entry != &table->gcList); i++) {
        pidCtx = OBJECT_PTR(entry, PID_CONTEXT, gchead);
        entry = entry->next;
        __list_remove_item_safe(&pidCtx->gchead);
        __list_add_tail(&table->gcList, &pidCtx->gchead);
    }

    return del_count;
}

#endif

#ifdef __PID_GARGAGE_COLLECTION__
BOOLEAN __nfs_table_query_delete_items(sec_gen_table_t *table, LIST_HEAD *list)
{
    BOOLEAN retval = FALSE;
    __list_init(list);

    nfs_lock(&table->nfs_lock);
    while (!__list_is_empty(&table->deleteList)) {
        LIST_HEAD *entry;

        __list_remove_tail(&table->deleteList, entry);

        __list_add_tail(list, entry);
        retval = TRUE;
    }
    nfs_unlock(&table->nfs_lock);
    return retval;
}

BOOLEAN gen_table_query_delete_items(LIST_HEAD *list)
{
    return __nfs_table_query_delete_items(&g_nfs_table, list);
}

#endif

BOOLEAN __gen_table_init(sec_gen_table_t *table)
{
    BOOLEAN retval = FALSE;
    int bucket_count = (4096 * 4);

    nfs_lock_init(&table->nfs_lock);

    hashtb_init_ex(&table->lookupTB, __pid_ctx_value, __pid_ctx_release,
                   sizeof(sec_pid_t), sec_vmalloc, sec_vfree);

#ifdef __PID_GARGAGE_COLLECTION__
    __list_init(&table->gcList);
    __list_init(&table->deleteList);
#endif

    while (bucket_count > 10) {
        if (!hashtb_set_buckets(&table->lookupTB, bucket_count)) {
        } else {
            break;
        }
        bucket_count = (bucket_count / 2);
    }
    if (bucket_count <= 10)
        return FALSE;
    retval = TRUE;
    return retval;
}

void nfs_table_destroy(sec_gen_table_t *table)
{
    nfs_lock_destroy(&table->nfs_lock);

#ifdef __PID_GARGAGE_COLLECTION__
    do {
        LIST_HEAD *entry;
        while (!__list_is_empty(&table->gcList)) {
            PID_CONTEXT *pidCtx;
            __list_remove_tail(&table->gcList, entry);

            pidCtx = OBJECT_PTR(entry, PID_CONTEXT, gchead);
            __list_init(entry);
        }
        // -- __list_init(&table->gcList);
    } while (0);
#endif

    hashtb_destroy(&table->lookupTB);
    return;
}

PID_CONTEXT *__nfs_lookup(sec_gen_table_t *table, sec_pid_t pid)
{
    PID_CONTEXT *ctx = NULL;

    LIST_HEAD *entry;

    nfs_lock(&table->nfs_lock);

    entry = hashtb_find(&table->lookupTB, (unsigned int *)&pid);
    if (entry) {
        ctx = OBJECT_PTR(entry, PID_CONTEXT, head);
        comm_object_addref(ctx);

#ifdef __PID_GARGAGE_COLLECTION__
        __list_remove_item(&ctx->gchead);
        __list_add_tail(&table->gcList, &ctx->gchead);
        ctx->lastAccessTime = ktime_get_seconds();
#endif
    }

    nfs_unlock(&table->nfs_lock);

    return ctx;
}

BOOLEAN __nfs_remove(sec_gen_table_t *table, sec_pid_t pid)
{
    BOOLEAN retval = FALSE;

#ifdef __PID_GARGAGE_COLLECTION__
    LIST_HEAD *entry;
#endif

    nfs_lock(&table->nfs_lock);

#ifdef __PID_GARGAGE_COLLECTION__
    entry = hashtb_find(&table->lookupTB, (unsigned int *)&pid);
    if (entry) {
        PID_CONTEXT *pidCtx;
        pidCtx = OBJECT_PTR(entry, PID_CONTEXT, head);
        // -- xobject_addref(ctx);
        __list_remove_item_safe(&pidCtx->gchead);
        hashtb_remove_item_entry(&table->lookupTB, entry);
        retval = TRUE;
    } else {
        retval = FALSE;
    }
#else
    retval = hashtb_remove_item(&table->lookupTB, (unsigned int *)&pid);
    if (retval) {
    }
#endif

    nfs_unlock(&table->nfs_lock);
    return retval;
}

PID_CONTEXT *__nfs_insert(sec_gen_table_t *table, sec_pid_t pid)
{
    PID_CONTEXT *ctx = NULL;

    nfs_lock(&table->nfs_lock);

    if (NULL != hashtb_find(&table->lookupTB, (unsigned int *)&pid))
        goto DONE;

    ctx = pid_ctx_new();
    if (!ctx)
        goto DONE;
        
    ctx->PID = __SEC_COPY_PID(pid);

    if (!hashtb_insert_item(&table->lookupTB, &ctx->head,
                            (unsigned int *)&ctx->PID)) {
        comm_object_release(ctx);
        ctx = NULL;
        goto DONE;
    }

#ifdef __PID_GARGAGE_COLLECTION__
    __list_add_tail(&table->gcList, &ctx->gchead);
    nfs_table_gc(table);
#endif

    comm_object_addref(ctx);  // -- 给lookupTB使用;
DONE:
    nfs_unlock(&table->nfs_lock);
    return ctx;
}

BOOLEAN __nfs_insert_ctx(sec_gen_table_t *table, PID_CONTEXT *ctx)
{
    BOOLEAN retval = FALSE;

    nfs_lock(&table->nfs_lock);

    if (NULL != hashtb_find(&table->lookupTB, (unsigned int *)&ctx->PID))
        goto DONE;

    if (!hashtb_insert_item(&table->lookupTB, &ctx->head,
                            (unsigned int *)&ctx->PID)) {
        ctx = NULL;
        goto DONE;
    }

#ifdef __PID_GARGAGE_COLLECTION__
    __list_add_tail(&table->gcList, &ctx->gchead);
    nfs_table_gc(table);
#endif

    comm_object_addref(ctx);  // -- 给lookupTB使用;
    retval = TRUE;
DONE:
    nfs_unlock(&table->nfs_lock);
    return retval;
}

BOOLEAN sec_gen_insert_ctx(PID_CONTEXT *ctx)
{
    return __nfs_insert_ctx(&g_nfs_table, ctx);
}

PID_CONTEXT *sec_gen_insert(sec_pid_t pid)
{
    return __nfs_insert(&g_nfs_table, pid);
}

PID_CONTEXT *sec_gen_lookup(sec_pid_t pid)
{
    return __nfs_lookup(&g_nfs_table, pid);
}

BOOLEAN sec_gen_remove(sec_pid_t pid)
{
    return __nfs_remove(&g_nfs_table, pid);
}

BOOLEAN gen_module_init(void)
{
    return __gen_table_init(&g_nfs_table);
}

void gen_module_destroy(void)
{
    nfs_table_destroy(&g_nfs_table);
}
