#include <project.h>
#include <linux/sched/signal.h>
#include <linux/pid_namespace.h>
#include <linux/spinlock_types.h>

static DEFINE_SPINLOCK(child_proc_lock);

int is_child_proc_by_name(const char * root_proc, struct task_struct * task, pid_t * root_pid)
{
    int retval = 0;

    struct task_struct * iter_task = NULL;
    pid_t pid = 0;

    if (root_proc == NULL || task == NULL || task->comm == NULL)
        return retval;

    // tag_pr_debug("root_proc:[%s]\n", root_proc);
    // tag_pr_debug("task->comm:[%s]\n", task->comm);

    if (!strncmp(task->comm, root_proc, strlen(root_proc))) {
        *root_pid = task_pid_nr(task);
        retval = 1;
        return retval;
    }

    spin_lock(&child_proc_lock);
    for (iter_task = task, pid = task_pid_nr(task); pid != 0 && pid != 1 && pid != 2;
         iter_task = iter_task->real_parent, pid = task_pid_nr(iter_task)) {

        tag_pr_debug("check comm[%s], group_leader->comm[%s]\n", iter_task->comm, iter_task->group_leader->comm);

        if (!strncmp(iter_task->comm, root_proc, strlen(iter_task->comm))) {
            tag_pr_debug("find proc_tree:[%s-%d]\n", iter_task->comm, task_pid_nr(iter_task));
            *root_pid = task_pid_nr(iter_task);
            retval = 1;
            goto DONE;
        }

        if (!strncmp(iter_task->group_leader->comm, root_proc,
                     strlen(iter_task->group_leader->comm))) {
            tag_pr_debug("find proc_tree:[%s-%d]\n",
                   iter_task->group_leader->comm, task_pid_nr(iter_task->group_leader));
            *root_pid = task_pid_nr(task->group_leader);
            retval = 1;
            goto DONE;
        }
    }

DONE:
    spin_unlock(&child_proc_lock);
    return retval;
}

int is_child_proc(pid_t parent, struct task_struct * task)
{
    int retval = 0;

    struct task_struct * iter_task = task;
    pid_t pid = task_pid_nr(task);

    if (parent == task_pid_nr(iter_task->real_parent) || parent == task_pid_nr(iter_task->group_leader)) {
        tag_pr_debug("find proc_tree:[%d-%d]\n", task_pid_nr(iter_task->group_leader), parent);
        retval = 1;
        goto DONE;
    }

    for (; pid != 1 && pid != 2; iter_task = iter_task->real_parent, pid = task_pid_nr(iter_task)) {
        if (task_pid_nr(iter_task) == parent || task_pid_nr(iter_task->group_leader) == parent) {
            tag_pr_debug("find proc_tree:[%d-%d]\n", task_pid_nr(iter_task->real_parent), parent);
            retval = 1;
            goto DONE;
        }
    }

DONE:
    return retval;
}
