#include <project.h>
#include "pid_ctx.h"
#include "sec_krnl_misc.h"
#include "nfs_table.h"
#include "common/fullpath.h"
#include "version_type.h"

void pid_ctx_module_init(void)
{
    return;
}

void pid_ctx_module_destroy(void)
{
    return;
}

void pidCopy(PID_CONTEXT *src, PID_CONTEXT *dest)
{
    dest->PID = __SEC_COPY_PID(src->PID);  //
    dest->isNamed = src->isNamed;           //
    dest->bStartup = src->bStartup;         // -- 第一个线程启动标识

    dest->createTime = src->createTime;  // -- 进程创建的时刻
    dest->startUpTime = src->startUpTime;  // -- 进程启动(进入main函数)的时刻

    // -- char name[PROCNAME_LEN];
    // -- LIST_HEAD imageList;
    dest->isAppLoaded = src->isAppLoaded;  // -- 应用程序是否己经加载
    dest->isBit64 = src->isBit64;          //
    dest->isWow64 =
        src->isWow64;  // -- BOOLEAN IsWow64(HANDLE procHandle) 返回值;

    dest->isThread = src->isThread;  // -- linux pthread .. vfork .. clone;
    dest->processID =
        src->processID;  // -- to thread, processID is real process id;
    // -- processID 比如说 fork 出来的带 (CLONE_VM)
    // 参数的进程(实际就是线程)，它与父进程的processID一致;

    dest->isTarget = src->isTarget;
    dest->isPrivilege = src->isPrivilege;
    dest->taskDesc = src->taskDesc;

    // -- if(dest->taskDesc){
    // -- 	xobject_addref(dest->taskDesc);
    // -- }
    dest->taskDesc.deep_level = src->taskDesc.deep_level;

    strcpy(dest->appFullName, src->appFullName);

    // -- char * __nameCache;
    // -- BOOLEAN isUnlinkBypassed;
}

void PidCtxDbgPrint(PID_CONTEXT *ctx)
{
    // -- LIST_HEAD * entry;
    // -- IMAGE_DESC * image = NULL;
    // -- int i;

    spin_lock(&ctx->lock);

    // -- DbgPrint("[+] <<\n");

    spin_unlock(&ctx->lock);

    return;
}

IMAGE_DESC *createImageDesc(char *name, unsigned int length)
{
    unsigned int total = sizeof(IMAGE_DESC) + length;
    IMAGE_DESC *image = NULL;

    image = (IMAGE_DESC *)kmalloc(total, GFP_ATOMIC);
    if (!image)
        return NULL;

    memset(image, 0, sizeof(IMAGE_DESC));
    memcpy(image->name, name, length);
    image->nameLength = length;
    image->name[length / 2] = 0;

    __list_init(&image->head);
    return image;
}

void destroyImageDesc(IMAGE_DESC *desc)
{
    kfree(desc);
    return;
}

void __destroy_pid_context(PID_CONTEXT *ctx)
{
    if (ctx->PID) {
        __SEC_FREE_PID(ctx->PID);
        ctx->PID = NULL;
    }

    kfree(ctx);
    return;
}

// -- 返回值带一个引用计数；
PID_CONTEXT *pid_ctx_new(void)
{
    PID_CONTEXT *ctx;

    ctx = (PID_CONTEXT *)kmalloc(sizeof(PID_CONTEXT), GFP_ATOMIC);
    if (!ctx)
        return NULL;

    memset(ctx, 0, sizeof(PID_CONTEXT));

#ifdef __PID_GARGAGE_COLLECTION__
    __list_init(&ctx->gchead);  // -- Auto Garbage Collection
#endif

    ctx->isWow64 = 0;
#ifdef __x86_64__
    ctx->isBit64 = 1;
#else
    ctx->isBit64 = 0;
#endif

    spin_lock_init(&ctx->lock);

    ctx->objHdr.destroy_ptr = (destroy_atom_ptr)__destroy_pid_context;
    comm_object_addref(ctx);
    return ctx;
}

int pid_ctx_update(PID_CONTEXT *ctx, long pid, const char *filename, unsigned long i_ino)
{
    spin_lock(&ctx->lock);
    ctx->isThread = 0;
    ctx->processID = pid;
    ctx->sid.id = __get_current_sid();
    ctx->i_ino = i_ino;

    if (filename != NULL)
        strncpy(ctx->appFullName, filename, sizeof(ctx->appFullName));
    spin_unlock(&ctx->lock);

    return 0;
}

#ifdef __PID_GARGAGE_COLLECTION__

static inline void clear_deleted_pid_context(void)
{
    LIST_HEAD pidList, *entry;

    if (!gen_table_query_delete_items(&pidList)) {
        return;
    }

    while (!__list_is_empty(&pidList)) {
        PID_CONTEXT *pidCtx;
        __list_remove_tail(&pidList, entry);
        pidCtx = OBJECT_PTR(entry, PID_CONTEXT, gchead);

        // mark_table_remove(pidCtx->processID);
        // protect_proc_remove_pid(pidCtx->processID);
        // tcb_remove_pid(pidCtx->processID);
        sec_gen_remove(pidCtx->PID);

        comm_object_release(pidCtx);
    }
    return;
}
#endif

// 更新pid相关上下文
void update_pid_context(struct file * file, struct task_struct * task)
{
    long pid = (long)task->pid;

    PID_CONTEXT *ctx = NULL;
    PID_CONTEXT *ppctx = NULL;

    char *filename = NULL;

    if (!file || !get_file_dentry(file) || !get_file_dentry(file)->d_inode)
        goto DONE;

    if (!S_ISREG(get_file_dentry(file)->d_inode->i_mode))
        goto DONE;

    // todo: 这层封装的全局结构需要拆分为参数
    ctx = sec_gen_lookup(__SEC_PID_GET(task));
    if (!ctx) {
        struct task_struct *parent = task->parent;
        int count = 1;

        parent = task;

        // tag_pr_debug("pid[%d]\n", task->pid);
        
        while (parent) {
            if (parent == parent->parent) 
                break;

            parent = parent->parent;
            if (!parent) 
                break;

            ppctx = sec_gen_lookup(__SEC_PID_GET(parent));
            if (ppctx) {
                tag_pr_debug("got parent pid [%d-%s]\n", ppctx->processID, ppctx->appFullName);
                break;
            } 

            ppctx = sec_gen_lookup(__SEC_PGID_GET(parent));
            if (ppctx) {
                tag_pr_debug("got parent pgid [%d-%s]\n", ppctx->processID, ppctx->appFullName);
                break;
            }
            
            count++;
        }
    }

    // 如果 ctx 为空， 或者， ctx中的文件信息和当前文件信息不一致，则从新获取文件信息并更新
    if (!ctx || ctx->i_ino != file->f_inode->i_ino) {
//        tag_pr_debug("ctx is null\n");

        if (!ctx) {
            ctx = sec_gen_insert(__SEC_PID_GET(task));
            if (!ctx)
                goto DONE;        
//            tag_pr_debug("ctx insert\n");
        }

        filename = get_fullpath_by_file(file);
        if (!filename) 
            goto DONE;
        
//        tag_pr_info("fullpath[%s]\n", filename);

        if (ctx) {
            pid_ctx_update(ctx, pid, filename, file->f_inode->i_ino);
        }
    }
 
DONE:
    if (ppctx) 
        comm_object_release(ppctx);
    if (ctx)
        comm_object_release(ctx);

#ifdef __PID_GARGAGE_COLLECTION__
    clear_deleted_pid_context();
#endif

    return;
}

