#ifndef __PID_CTX_H_
#define __PID_CTX_H_
#include "common/general/comm_object.h"
#include "common/general/include/format/integrity.h"
#include "include/misc/list_head.h"
#include "pid_port.h"
#include "task_desc.h"

#define PROCNAME_LEN 100
#define PROCESSMON_POOL_TAG 'PRCM'

typedef struct __IMAGE_DESC {
    LIST_HEAD head;
    unsigned int nameLength;
    char name[1];
} IMAGE_DESC;

typedef struct __PID_CONTEXT {
    SEC_OBJECT objHdr;
    LIST_HEAD head;

#ifdef __PID_GARGAGE_COLLECTION__
    LIST_HEAD gchead;  // -- Auto Garbage Collection
    unsigned long lastAccessTime;
#endif

    sec_pid_t PID;

    spinlock_t lock;
    BOOLEAN isNamed;   //
    BOOLEAN bStartup;  // -- 第一个线程启动标识

    long long createTime;  // -- 进程创建时间
    long long startUpTime;  // -- 启动时间(进入main函数)时间
    char name[PROCNAME_LEN];
    // -- LIST_HEAD imageList;

    BOOLEAN isAppLoaded;  // -- 应用是否已加载完成
    BOOLEAN isBit64;      // --
    BOOLEAN isWow64;  // -- BOOLEAN IsWow64(HANDLE procHandle) 返回值;

    SID_STOR sid;  // 

    BOOLEAN isThread;  // -- linux pthread .. vfork .. clone;
    long processID;    // -- to thread, processID is real process id;
    BOOLEAN isTarget;

    // -- BOOLEAN isMarked; // -- 是否为标记对象
    BOOLEAN isPrivilege;  // -- 是否为权限对象
    TASK_DESC taskDesc;

    unsigned int rank;  // 安全等级
    unsigned int user_id;
    BOOLEAN isService;
    BOOLEAN isWhiteList;

    BOOLEAN isCheckNamed;
    BOOLEAN isUserChecked;
    unsigned char hashValue[INTEGRITY_LENGTH];

    char appFullName[512];

    unsigned long		i_ino; // 以此变化，作为更新ctx的依据

} PID_CONTEXT, *PPID_CONTEXT;

void pid_ctx_module_destroy(void);
void pid_ctx_module_init(void);

void destroyImageDesc(IMAGE_DESC *desc);

IMAGE_DESC *createImageDesc(char *name, unsigned int length);

PID_CONTEXT *pid_ctx_new(void);
void PidCtxDbgPrint(PID_CONTEXT *ctx);
void pidCopy(PID_CONTEXT *src, PID_CONTEXT *dest);

void update_pid_context(struct file * file, struct task_struct * task);


#endif
