#include <project.h>
#include "common/general/comm_object.h"
#include "common/general/include/scan_machine/scan_machine.h"
#include "proc_check_table.h"

typedef struct __PROCESS_CHECK_TRIE {
    void *trieObject;
    spinlock_t lock;
} proc_check_trie_t;

typedef struct __PROCESS_CHECK_EXT {
    unsigned int id;
    unsigned int type;  // -- one as "__POLICY_TYPE_SYSFOLDER_FILE" .etc.
} proc_check_ext_t;

typedef struct __PROCESS_SEARCH_CTX {
    PROC_CHECK_RESULT_T acitem;
    // -- BOOLEAN bValid;

    char *processPath;
    long processId;
} proc_search_ctx_t;

// --------------------------------------------------------------------------------------------
static proc_check_trie_t g_process_trie;

static int __process_trie_check(SCN_PATTERN *pattern, char *pattern_pos,
                                int offset, int left, void *search_context)
{
    int retval = SCN_CHECK_CONTINUE;
    proc_search_ctx_t *ctx = (proc_search_ctx_t *)search_context;
    proc_check_ext_t *ext = __SCAN_GET_EXT(proc_check_ext_t, pattern);

    tag_pr_debug("ext->type[%d]\n", ext->type);

    if (ext->type == __POLICY_TYPE_PROCESS_FILE || ext->type == __POLICY_TYPE_SERVICE_FILE)
        ctx->acitem.is_ban = 1;

    return retval;
}

BOOLEAN proc_check_insert(void *__trie_obj, const char *path,
                              unsigned int id, unsigned int type)
{
    SCN_PATTERN *pattern = NULL;
    unsigned short pat_len;
    proc_check_ext_t *ext;

    pat_len = strlen(path);
    pattern = scan_scn_trie_insert_key(__trie_obj, (unsigned char *)path,
                                       pat_len, (void *)(unsigned long)id);
    if (!pattern) {
        return FALSE;
    }
    
    ext = __SCAN_GET_EXT(proc_check_ext_t, pattern);
    memset(ext, 0, sizeof(*ext));
    ext->type = type;
    ext->id = id;

    return TRUE;
}

BOOLEAN proc_check_do(void * ply_head, char *tftPath, PROC_CHECK_RESULT_T *pAcitem)
{
    BOOLEAN retval = FALSE;
    proc_search_ctx_t ctx;

    memset(&ctx, 0, sizeof(ctx));
    // -- ctx.bValid = FALSE;

    // spin_lock_bh(&g_process_trie.lock);
    ctx.processPath = tftPath;

    scan_scn_trie_traverse(ply_head, (unsigned char *)tftPath,
                           strlen(tftPath), __process_trie_check, &ctx);

    retval = TRUE;
    *pAcitem = ctx.acitem;

    // spin_unlock_bh(&g_process_trie.lock);
    return retval;
}

// void proc_check_update_trie_object(void *__trie_obj)
// {
//     spin_lock_bh(&g_process_trie.lock);
//     if (NULL != g_process_trie.trieObject) {
//         comm_object_release(g_process_trie.trieObject);
//         g_process_trie.trieObject = NULL;
//     }
//     if (__trie_obj) {
//         g_process_trie.trieObject = __trie_obj;
//         comm_object_addref(__trie_obj);
//     }

//     spin_unlock_bh(&g_process_trie.lock);
//     return;
// }

void *proc_check_create_trie_object(void)
{
    void *obj = NULL;
    obj = scan_scn_trie_create(sizeof(proc_check_ext_t));
    return obj;
}

int proc_check_build_trie_object(void *trieObj)
{
    int retval = scan_scn_trie_build(trieObj);
    return retval;
}

BOOLEAN proc_check_module_init(void)
{
    BOOLEAN retval = FALSE;
    spin_lock_init(&g_process_trie.lock);

    g_process_trie.trieObject = proc_check_create_trie_object();
    if (!g_process_trie.trieObject) {
        return FALSE;
    }
    return retval;
}

void proc_check_module_destroy(void)
{
    if (NULL != g_process_trie.trieObject) {
        comm_object_release(g_process_trie.trieObject);
        g_process_trie.trieObject = NULL;
    }
    return;
}
