#include <project.h>

#include "sec_ext_table.h"

// -------------------------------------------------------------------------------
static char *__x_file_ext_value(LIST_HEAD *head);
static void __x_file_ext_release(LIST_HEAD *head);
static SEC_EXT_TYPE *createXExtTypeItem(void);
// -------------------------------------------------------------------------------

sec_ext_table_t* sec_ext_table_init( unsigned int __count)
{
    int bucket_count = __count;  // -- (4096 * 4);

    sec_ext_table_t *table = NULL;

    if (0 == bucket_count) 
        bucket_count = 100;


    table = (sec_ext_table_t *)kmalloc(sizeof(sec_ext_table_t), GFP_ATOMIC);
    if (!table) {
        return NULL;
    }

    memset(table, 0, sizeof(sec_ext_table_t));

    if (0 != comm_hash_table_init(table, __x_file_ext_value,
                                  __x_file_ext_release)) {
        return FALSE;
    }

    do {
        if (0 != comm_hash_table_set_buckets(table, bucket_count)) {
        } else {
            break;
        }

        bucket_count = (bucket_count / 2);
    } while (bucket_count > 10);

    return table;
}

void sec_ext_table_destroy(sec_ext_table_t *table)
{
    comm_hash_table_destroy(table);
}

void sec_ext_table_clear(sec_ext_table_t *table)
{
    comm_hash_table_remove_all(table);
}

SEC_EXT_TYPE *sec_ext_table_add(sec_ext_table_t *table, const char *ext)
{
    LIST_HEAD *entry;
    SEC_EXT_TYPE *retval = NULL;

    if (strlen(ext) >= __MAX_EXT_NAME_LENGTH)
        return NULL;


    entry = comm_hash_table_find(table, ext);
    if (entry) {
        retval = OBJECT_PTR(entry, SEC_EXT_TYPE, head);
        comm_object_addref(retval);
    } else {
        retval = createXExtTypeItem();
        if (retval) {
            // -- retval->setExtData(ext.c_str());
            strncpy(retval->ext, ext, sizeof(retval->ext));
            if (0 ==
                comm_hash_table_insert(table, &retval->head, ext)) {
                comm_object_addref(retval);
            } else {
                comm_object_release(retval);
                retval = NULL;
            }
        }
    }

    return retval;
}

SEC_EXT_TYPE *sec_ext_table_lookup(sec_ext_table_t *table, const char *ext)
{
    LIST_HEAD *entry;
    SEC_EXT_TYPE *retval = NULL;

    entry = comm_hash_table_find(table, ext);
    if (entry) {
        retval = OBJECT_PTR(entry, SEC_EXT_TYPE, head);
        comm_object_addref(retval);
    }

    return retval;
}

BOOLEAN sec_ext_table_is_empty(sec_ext_table_t *table)
{
    unsigned int count;

    count = comm_hash_table_get_count(table);

    return (0 == count);
}

static void __x_file_ext_release(LIST_HEAD *head)
{
    SEC_EXT_TYPE *base = OBJECT_PTR(head, SEC_EXT_TYPE, head);
    comm_object_release(base);
    return;
}

static char *__x_file_ext_value(LIST_HEAD *head)
{
    SEC_EXT_TYPE *base = OBJECT_PTR(head, SEC_EXT_TYPE, head);
    return base->ext;
}

void __destroy_xext_type_item(SEC_EXT_TYPE *extItem)
{
    kfree(extItem);
    return;
}

static SEC_EXT_TYPE *createXExtTypeItem(void)
{
    SEC_EXT_TYPE *extItem;

    extItem = (SEC_EXT_TYPE *)kmalloc(sizeof(SEC_EXT_TYPE), GFP_ATOMIC);

    if (!extItem)
        return NULL;

    memset(extItem, 0, sizeof(SEC_EXT_TYPE));

    extItem->objHdr.destroy_ptr = (destroy_atom_ptr)__destroy_xext_type_item;
    comm_object_addref(extItem);
    return extItem;
}
