#include <project.h>
#include "common/general/comm_object.h"
#include "common/general/include/scan_machine/scan_machine.h"
#include "service_check_table.h"
#include "spin_lock.h"

typedef struct __SERVICE_ACCESS_TRIE {
    void *trieObject;
    spinlock_t lock;
} srv_access_trie_t;

typedef struct __SRV_ACCESS_EXT {
    unsigned int id;

    unsigned char isSysService : 1;
    unsigned char isNormalProcess : 1;
} srv_access_ext_t;

typedef struct __SRV_SEARCH_CTX {
    srv_access_item_t acitem;
    BOOLEAN bValid;
    long processId;
} srv_search_ctx_t;

// --------------------------------------------------------------------------------------------
static srv_access_trie_t g_srv_trie;

static int __service_trie_check(SCN_PATTERN *pattern, char *pattern_pos,
                                int offset, int left, void *search_context)
{
    int retval = SCN_CHECK_CONTINUE;
    srv_search_ctx_t *ctx = (srv_search_ctx_t *)search_context;
    srv_access_ext_t *ext = __SCAN_GET_EXT(srv_access_ext_t, pattern);

    tag_pr_debug("pattern[%s]\n", pattern_pos);
    tag_pr_debug("ext->isSysService[%d], ext->isNormalProcess[%d], left[%d]\n",
                 ext->isSysService, ext->isNormalProcess, left);
    if (ext->isSysService) {
        if (0 == left) {
            ctx->bValid = 1;
            ctx->acitem.isSysService = 1;
        }
    } else if (ext->isNormalProcess) {
        if (0 == left) {
            ctx->bValid = 1;
            ctx->acitem.isNormalProcess = 1;
        }
    } else {
        goto DONE;
    }
DONE:
    return retval;
}

BOOLEAN service_access_insert(void *__trie_obj, const char *path,
                               unsigned int id, unsigned int type)
{
    SCN_PATTERN *pattern = NULL;
    unsigned short pat_len;
    srv_access_ext_t *ext;

    pat_len = strlen(path);
    tag_pr_debug("\n");
    pattern = scan_scn_trie_insert_key(__trie_obj, (unsigned char *)path,
                                       pat_len, (void *)(unsigned long)id);
     tag_pr_debug("\n");
   if (!pattern) {
        return FALSE;
    }
     tag_pr_debug("\n");
   ext = __SCAN_GET_EXT(srv_access_ext_t, pattern);
     tag_pr_debug("\n");
   memset(ext, 0, sizeof(*ext));
    switch (type) {
        case PROTECT_SERVICE:
            ext->isSysService = 1;
            break;
        case PROTECT_PROCESS:
            ext->isNormalProcess = 1;
            break;
    }

    return TRUE;
}

BOOLEAN service_access_check(char *tftPath, unsigned int user_id,
                              srv_access_item_t *pAcitem)
{
    BOOLEAN retval = FALSE;
    srv_search_ctx_t ctx;

    memset(&ctx, 0, sizeof(ctx));
    ctx.bValid = FALSE;

    spin_lock_bh(&g_srv_trie.lock);

    scan_scn_trie_traverse(g_srv_trie.trieObject, (unsigned char *)tftPath,
                           strlen(tftPath), __service_trie_check, &ctx);

    if (ctx.bValid) {
        retval = TRUE;
        *pAcitem = ctx.acitem;
    }

    spin_unlock_bh(&g_srv_trie.lock);

    return retval;
}

void service_access_update_trie_object(void *__trie_obj)
{
    spin_lock_bh(&g_srv_trie.lock);

    if (NULL != g_srv_trie.trieObject) {
        comm_object_release(g_srv_trie.trieObject);
        g_srv_trie.trieObject = NULL;
    }
    if (__trie_obj) {
        g_srv_trie.trieObject = __trie_obj;
        comm_object_addref(__trie_obj);
    }

    spin_unlock_bh(&g_srv_trie.lock);

    return;
}

void *service_access_create_trie_object(void)
{
    void *obj = NULL;
    obj = scan_scn_trie_create(sizeof(srv_access_ext_t));
    return obj;
}

int service_access_build_trie_object(void *trieObj)
{
    int retval = scan_scn_trie_build(trieObj);
    return retval;
}

BOOLEAN service_table_module_init(void)
{
    BOOLEAN retval = FALSE;

    spin_lock_init(&g_srv_trie.lock);

    g_srv_trie.trieObject = service_access_create_trie_object();

    if (!g_srv_trie.trieObject) {
        return FALSE;
    }
    return retval;
}

void service_table_module_destroy(void)
{
    if (NULL != g_srv_trie.trieObject) {
        comm_object_release(g_srv_trie.trieObject);
        g_srv_trie.trieObject = NULL;
    }
    return;
}
