#include <project.h>
#include <linux/version.h>
#if LINUX_VERSION_CODE < KERNEL_VERSION(3, 10, 0)
#include <linux/syscalls.h>
#endif

#include "module.h"
#include "kern_symbol.h"

// some way for get symbol
// use target way(first) or judge by current version
// #define __GET_SYMBOL_BY_LOOKUP_NAME // use kallsyms_lookup_name
// #define __GET_SYMBOL_BY_CALLBACK // use kallsyms_on_each_symbol，set callback
// #define __GET_SYMBOL_BY_FILE // by parse file (kallsyms)

extern void * probe_find_symbol(const char *keyline);

void * get_symbol(const char * symbol)
{
	void * retval = NULL;
	int ret = 0;
	struct kernsym sym;

#if defined(__GET_SYMBOL_BY_LOOKUP_NAME)
	retval = (void *)kallsyms_lookup_name(symbol);

#elif defined(__GET_SYMBOL_BY_CALLBACK)
	memset(&sym, 0, sizeof(sym));
	ret = find_symbol_address(&sym, symbol);
	if (IN_ERR(ret)) {
		goto DONE;
	}
	retval = sym.run;

#elif defined(__GET_SYMBOL_BY_FILE)
	retval = probe_find_symbol(symbol);

#else // no target, by version
#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 6, 0)
	retval = (void *)kallsyms_lookup_name(symbol);
#else // 4.19 ok
	memset(&sym, 0, sizeof(sym));
	ret = find_symbol_address(&sym, symbol);
	if (IN_ERR(ret)) {
		goto DONE;
	}
	retval = sym.run;
#endif
#endif // __GET_SYMBOL_BY_

DONE:	
	if (retval != NULL)
        tag_pr_debug("get symbol[%s-%lx]\n", symbol, retval);
	return retval;
}

