#include <asm/unistd.h>
#include <linux/kernel.h>
#include <linux/list.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/slab.h>
#include <linux/syscalls.h>
#include <linux/version.h>
#include <project.h>

#if defined(__aarch64__)
#include <linux/vmalloc.h>
#endif

#include "common/sym_hook/kern_symbol.h"
#include "writable.h"

#if !defined(NR_syscalls)
#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 18)
#include <asm/asm-offsets.h>
#else
#define NR_syscalls (__NR_syscall_max + 1)
#endif
#endif

#if defined(__arm__) && defined(CONFIG_STRICT_MEMORY_RWX)
#include <asm/mmu_writeable.h>
#endif

#if defined(__aarch64__)
void (*update_mapping_prot)(phys_addr_t phys, unsigned long virt,
                            phys_addr_t size, pgprot_t prot);
unsigned long start_rodata, end_rodata;
#endif

static DEFINE_SPINLOCK(syscall_hook_lock);
static struct hlist_head syscall_links[NR_syscalls];
static unsigned long *origin_syscall_table;
static unsigned long *writable_syscall_table;

struct syscall_link {
    struct hlist_node node;
    unsigned long **oldcallholder;
};

static void *get_writable_syscall_table(void *sct_addr)
{
#if defined(__aarch64__)
    return sct_addr;
#else
    struct page *p[2];
    void *sct;
    unsigned long addr = (unsigned long)sct_addr & PAGE_MASK;

    // printk("syscall address = %p\n" , sct_addr);
    if (sct_addr == NULL)
        return NULL;

#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 22) && defined(__x86_64__)
    p[0] = pfn_to_page(__pa_symbol(addr) >> PAGE_SHIFT);
    p[1] = pfn_to_page(__pa_symbol(addr + PAGE_SIZE) >> PAGE_SHIFT);
#else
    p[0] = virt_to_page(addr);
    p[1] = virt_to_page(addr + PAGE_SIZE);
#endif
    sct = vmap(p, 2, VM_MAP, PAGE_KERNEL);
    if (sct == NULL)
        return NULL;
    // printk("writeable syscall adress = %p\n" , sct +
    // offset_in_page(sct_addr));
    return sct + offset_in_page(sct_addr);
#endif
}

unsigned long get_syscall_table(void)
{
    unsigned long addr = 0;

    addr = get_symbol("sys_call_table");

    // printk("sys_call_table at:%lx \n", addr);
    return addr;
}

/*define but not used
static void (*pmem_text_write_kernel_word)(unsigned long addrss,unsigned long
value);
*/
static void wirte_syscall_enrty(int num, unsigned long *syscall,
                                unsigned long *sys_call_table,
                                unsigned long *ori_syscall_table)
{
    // #if defined(__arm__) // && defined(CONFIG_STRICT_MEMORY_RWX)

    // 	#if defined(CONFIG_STRICT_MEMORY_RWX)
    // 		tag_pr_debug("1 Syscall table
    // %p,sys_call_table[%d]=%p\n",sys_call_table,num,sys_call_table[num]);
    // 		mem_text_write_kernel_word(&sys_call_table[num], syscall);
    // 		tag_pr_debug("2 Syscall table
    // %p,sys_call_table[%d]=%p\n",sys_call_table,num,sys_call_table[num]);
    // 	#else
    // 		set_kernel_text_rw();
    // 		tag_pr_debug("3 Syscall table
    // %p,sys_call_table[%d]=%p\n",sys_call_table,num,sys_call_table[num]);
    // 		sys_call_table[num] = syscall;
    // 		tag_pr_debug("4 Syscall table
    // %p,sys_call_table[%d]=%p\n",sys_call_table,num,sys_call_table[num]);
    // 		set_kernel_text_ro();
    // 	#endif
    // #else

    if (sys_call_table == ori_syscall_table) {
        set_kernel_writable();
    }

    sys_call_table[num] = (unsigned long)syscall;

    if (sys_call_table == ori_syscall_table) {
        set_kernel_readonly();
    }

    // #endif
}

int hook_restore_system_call(int num, unsigned long *syscall,
                             unsigned long **oldcallholder)
{
    int r = 0;
    struct syscall_link *tpos, *me = 0, *useme = 0;
    // struct hlist_node *pos;

    if (!*oldcallholder)
        return -1;

    if (!writable_syscall_table) {
        return -1;
    }
    //	tag_pr_debug("Old syscall %lx\n,",sys_call_table[num]);
    //	tag_pr_debug("set  syscall %lx\n",*oldcallholder);
    //	tag_pr_debug("Old syscall %lx\n,",sys_call_table[num]);

    spin_lock(&syscall_hook_lock);
    hlist_for_each_entry(tpos, syscall_links + num, node)
    {
        if (tpos->oldcallholder == oldcallholder) {
            me = tpos;
        } else if (*tpos->oldcallholder == syscall) {  //
            useme = tpos;
        }
    }
    //	tag_pr_debug("me =%p,useme=%p\n",me,useme);
    //	wirte_syscall_enrty(num,*oldcallholder,sys_call_table);//restore to old
    if (me) {
        if (useme)
            *useme->oldcallholder = *me->oldcallholder;
        else {
            // set_kernel_writable();
            // sys_call_table[num] = (unsigned long)*oldcallholder;
            wirte_syscall_enrty(num, *oldcallholder, writable_syscall_table,
                                origin_syscall_table);  // restore to old
            // set_kernel_readonly();
        }
        mb();
        hlist_del(&me->node);
        kfree(me);
    } else {
        r = -1;
    }
    spin_unlock(&syscall_hook_lock);

    tag_pr_debug("restore-Old syscall [%d-%lx]\n", num,
                 (unsigned long)*oldcallholder);
    tag_pr_debug("restore-New syscall [%d-%lx]\n", num,
                 (unsigned long)writable_syscall_table[num]);

    return r;
}

int hook_replace_system_call(int num, unsigned long *syscall,
                             unsigned long **oldcallholder)
{
    struct syscall_link *link;

    link = kmalloc(sizeof(struct syscall_link), GFP_KERNEL);
    if (!link)
        return -ENOMEM;

    if (!writable_syscall_table) {
        kfree(link);
        return -1;
    }

    //	set_kernel_writable();

    spin_lock(&syscall_hook_lock);
    *oldcallholder = (unsigned long *)writable_syscall_table[num];
    link->oldcallholder = oldcallholder;

    hlist_add_head(&link->node, syscall_links + num);
    mb();
    // set_kernel_writable();

    wirte_syscall_enrty(num, syscall, writable_syscall_table,
                        origin_syscall_table);

    // sys_call_table[num] = (unsigned long)syscall;
    // set_kernel_readonly();
    spin_unlock(&syscall_hook_lock);

    tag_pr_debug("Old syscall [%d-%lx]\n", num, (unsigned long)*oldcallholder);
    tag_pr_debug("New syscall [%d-%lx]\n", num,
                 (unsigned long)writable_syscall_table[num]);
    //	set_kernel_readonly();
    //
    return 0;
}

int syscall_hook_init(void)
{
    origin_syscall_table = (unsigned long *)get_syscall_table();
    
    if (!origin_syscall_table)
        return -1;

    tag_pr_debug("get syscall_table[%lx]\n", origin_syscall_table);

    writable_syscall_table =
        (unsigned long *)get_writable_syscall_table(origin_syscall_table);
    if (!writable_syscall_table)
        return -1;

    tag_pr_debug("get writable syscall_table[%lx]\n", writable_syscall_table);

#if defined(__aarch64__)
    update_mapping_prot = (void *)kallsyms_lookup_name("update_mapping_prot");
    start_rodata = (unsigned long)kallsyms_lookup_name("__start_rodata");
    end_rodata = (unsigned long)kallsyms_lookup_name("__end_rodata");

    tag_pr_debug("update_mapping_prot:%lx, start_rodata:%lx, end_rodata:%lx.\n",
                 update_mapping_prot, start_rodata, end_rodata);
#endif

    return 0;
}

void syscall_hook_exit(void)
{
    if (writable_syscall_table &&
        writable_syscall_table != origin_syscall_table)
        vunmap(
            (const void *)((unsigned long)writable_syscall_table & PAGE_MASK));
}
