#ifndef _TAG_PRINT_H_
#define _TAG_PRINT_H_

#include <linux/kern_levels.h>
#include <linux/printk.h>
#include <linux/string.h>

#define MODULE_TAG "nfssec"

extern char log_level_sys[10];

// 日志输出级别：
// error > warning > info > debug(dev) > none
//
// error：严重错误
// waring：警告
// info：提示信息
// debug：调试信息，不做输出频率的限制
// dev：调试信息，限制了输出频率，默认5秒10次
// none： 不打印

#define LOG_LEVEL_NONE "none"

// 默认日志输出级别
#define DEFAULT_LOG_LEVEL KERN_INFO
#define DEFAULT_LOG_LEVEL_S "info"

// 转换日志输出级别：可读 -> 系统可认
static inline int get_log_level(const char *log_level, char *log_level_sys)
{
    int retval = -1;

    if (!strcmp(log_level, "debug")) {
        strcpy(log_level_sys, KERN_DEBUG);

    } else if (!strcmp(log_level, "info")) {
        strcpy(log_level_sys, KERN_INFO);

    } else if (!strcmp(log_level, "warning")) {
        strcpy(log_level_sys, KERN_WARNING);

    } else if (!strcmp(log_level, "error")) {
        strcpy(log_level_sys, KERN_ERR);

    } else if (!strcmp(log_level, "none")) {
        strcpy(log_level_sys, LOG_LEVEL_NONE);

    } else {
        strcpy(log_level_sys, DEFAULT_LOG_LEVEL);
        goto DONE;
    }

    retval = 0;
DONE:
    printk("[%s]: log level: [%s]\n", MODULE_TAG, log_level);
    return retval;
}

static inline const char *get_filename(const char *filepath)
{
    const char *ptr = strrchr(filepath, '/');

    if (ptr)
        return ptr + 1;
    else
        return filepath;
}

#define tag_pr_fmt(fmt, arg) MODULE_TAG "-%s: " fmt, arg
#define tag_pr_fmt_dbg(fmt, arg) \
    MODULE_TAG "-%s: [%s:%d] " fmt, arg, get_filename(__FILE__), __LINE__

#define tag_pr_err(fmt, ...)                    \
    if (!strcmp(log_level_sys, KERN_ERR) ||     \
        !strcmp(log_level_sys, KERN_WARNING) || \
        !strcmp(log_level_sys, KERN_INFO) ||    \
        !strcmp(log_level_sys, KERN_DEBUG))     \
    printk(KERN_ERR tag_pr_fmt(fmt, "ERR"), ##__VA_ARGS__)

#define tag_pr_warning(fmt, ...)                \
    if (!strcmp(log_level_sys, KERN_WARNING) || \
        !strcmp(log_level_sys, KERN_INFO) ||    \
        !strcmp(log_level_sys, KERN_DEBUG))     \
    printk(KERN_WARNING tag_pr_fmt(fmt, "WARNING"), ##__VA_ARGS__)

#define tag_pr_info(fmt, ...)                \
    if (!strcmp(log_level_sys, KERN_INFO) || \
        !strcmp(log_level_sys, KERN_DEBUG))  \
    printk(KERN_INFO tag_pr_fmt(fmt, "INFO"), ##__VA_ARGS__)

#define tag_pr_debug(fmt, ...)              \
    if (!strcmp(log_level_sys, KERN_DEBUG)) \
    printk(KERN_DEBUG tag_pr_fmt_dbg(fmt, "DEBUG"), ##__VA_ARGS__)

#define tag_pr_dev(fmt, ...)                    \
    if (printk_ratelimit())                     \
        if (!strcmp(log_level_sys, KERN_DEBUG)) \
    printk(KERN_DEBUG tag_pr_fmt_dbg(fmt, "DEBUG"), ##__VA_ARGS__)

#endif
