#ifndef __VERSION_TYPE_H_
#define __VERSION_TYPE_H_

// 对不同版本的类型进行封装，尽量在此处将差异封装，以防止代码中过多分支
// 注意：因为该文件会包含较多的头文件，所有，如果不是基础的数据类型就不要放在这里

#include <linux/version.h>

#include <linux/device.h>
#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 18)
typedef struct device nfssec_class_device_t;
#else
typedef struct class_device nfssec_class_device_t;
#endif

#include <linux/fs.h>
#include <linux/mount.h>
static inline struct dentry *get_file_dentry(struct file *file)
{
    // this can be found since 2.6.0
    return file_dentry(file);

    //  it can also be like this
    //  (infact this is diffrent)
    // #if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 0, 0)
    //     return file->f_path.dentry;
    // #else
    //     return file->f_dentry
    // #endif
}
static inline struct vfsmount *get_file_vfsmount(struct file *file)
{
    // to be check
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0)
    return file->f_path.mnt;
#else
    return file->f_vfsmnt;
#endif
}

#include <linux/time.h>
#include <linux/timekeeping.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 0, 0)
struct timeval {
    int64_t tv_sec;  /* seconds */
    int64_t tv_usec; /* and microseconds */
};
static inline void do_gettimeofday(struct timeval *tv)
{
    struct timespec64 tv_real;

    ktime_get_real_ts64(&tv_real);
    tv->tv_sec = tv_real.tv_sec;
    tv->tv_usec = tv_real.tv_nsec / 1000;
}
#endif

#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 18, 0)
typedef struct timespec64 timespec_t;
#else
typedef struct timespec timespec_t;
#endif

#include <linux/mm_types.h>
static inline struct rw_semaphore *get_mm_rw_sema(struct mm_struct *mm)
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 10, 0)
    return &mm->mmap_lock;
#else
    return &mm->mmap_sem;
#endif
}

#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
static inline timespec_t inode_get_ctime(const struct inode *inode)
{
	return inode->i_ctime;
}

static inline timespec_t inode_get_mtime(const struct inode *inode)
{
	return inode->i_mtime;
}
#endif

#endif  // __VERSION_TYPE_H_