#ifndef __INODE_CACHE_H_
#define __INODE_CACHE_H_

#include "common/general/comm_object.h"
#include "common/general/hashtb.h"
#include "common/version_type.h"

#define __MAX_CACHE_COUNT (1024 * 10)

typedef struct __inode_key {
    dev_t diskDev;
    unsigned long i_ino;
} __attribute__((__packed__)) inode_key_t;

typedef struct __inode_cache_item {
    SEC_OBJECT objHdr;
    LIST_HEAD hhead;

    LIST_HEAD chead;
    spinlock_t lock;

    loff_t i_size;

    timespec_t i_mtime;
    timespec_t i_ctime;

    unsigned char bytes[INTEGRITY_LENGTH];
    inode_key_t key;
} inode_cache_item_t;

typedef struct __inode_map {
    spinlock_t lock;
    HASHTB lookupTB;
    LIST_HEAD cacheList;
} inode_map_t;

extern inode_map_t g_inode_map;

BOOLEAN inode_cache_module_init(void);
void inode_cache_module_destroy(void);

inode_cache_item_t *inode_cache_lookup(dev_t diskDev, unsigned long i_ino);

inode_cache_item_t *inode_cache_insert(dev_t diskDev, unsigned long i_ino,
                                     loff_t i_size, timespec_t i_mtime,
                                     timespec_t i_ctime,
                                     unsigned char bytes[INTEGRITY_LENGTH]);

#define __IS_TIMESPEC_EQUAL(t1, t2) \
    (((t1)->tv_sec == (t2)->tv_sec) && ((t1)->tv_nsec == (t2)->tv_nsec))


#endif  // __INODE_CACHE_H_
