#include <linux/module.h>
#include <linux/init.h>
#include <linux/moduleparam.h>

#include <project.h>
#include "common/sym_hook/lsm_if.h"
#include "common/config/sec_ply_if.h"
#include "common/audit/audit_queue.h"
#include "ipc/ipc_ioctl.h"
#include "inode_cache.h"

extern struct security_hook_list nfssec_ima_hooks[];
extern int nfssec_ima_hook_size;

audit_q_t g_audit_ima;

#ifdef NFSSEC_KMOD
// 模块参数： /sys/module/module_name/parameters/param_name
char log_level_sys[10];

char * log_level = DEFAULT_LOG_LEVEL_S; // 日志输出级别
module_param(log_level, charp, 0644);
MODULE_PARM_DESC(log_level, "level of log output!");
#else 
extern char log_level_sys[10];
extern char * log_level;
#endif

static int __init __nfssec_ima_init(void)
{
	int retval = -1;
	int ret = 0;

	get_log_level(log_level, log_level_sys); // 初始化日志输出级别
	// printk("log_level_sys:[%s]\n", log_level_sys);

#ifdef __USE_IMAGE_TABLE__
    inode_cache_module_init();
#endif

#ifndef IMA_PLY_BY_ATTR
	sec_ply_init(SEC_PLY_IMA);
	sec_ply_load(SEC_PLY_IMA);

	sec_ply_init(SEC_PLY_IMA_EXT);
	sec_ply_load(SEC_PLY_IMA_EXT);
#endif

	// todo： 临时注释
	// sec_sw_load(SEC_SW_IMA);

#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
	ret = lsm_if_init(nfssec_ima_hooks, nfssec_ima_hook_size, "nfssecima");
	if (ret) {
		tag_pr_err("init for nfssecima lsm error!\n");
		goto DONE;
	}
#endif

	audit_q_init(&g_audit_ima, 200);

    ret = ipc_init_ima();
    if (ret) {
        tag_pr_err("nac ipc_init_mac error.\n");
        goto DONE;
    }

	tag_pr_info("After nfssecima module init...\n");

    retval = 0;
DONE:
	return 0;
}

static void __exit nfssec_ima_exit(void)
{
    ipc_exit_ima();

	lsm_if_exit(nfssec_ima_hooks, nfssec_ima_hook_size);

#ifdef __USE_IMAGE_TABLE__
	inode_cache_module_destroy();
#endif

	audit_q_clear(&g_audit_ima);

	tag_pr_info("After nfssecima module exit...\n");
}

#ifdef NFSSEC_KMOD
module_init(__nfssec_ima_init);
module_exit(nfssec_ima_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("nfssec corp");
MODULE_DESCRIPTION("nfssec integrity measurement architecture");
MODULE_VERSION("1.0.1");
MODULE_ALIAS("nfssec ima ");
#else

static struct delayed_work __delay_work;
static struct workqueue_struct *wq;
#define MY_FILE "/opt/nfssec/script/loadService.sh"

static void __nfssec_ima_init_delay(struct work_struct *work_req)
{
	struct file *filep = NULL;

	filep = filp_open(MY_FILE, O_RDWR | O_APPEND | O_CREAT, 0644);
	if (IS_ERR_OR_NULL(filep)) {
		mod_delayed_work(wq, &__delay_work, 100);
	} else {
		filp_close(filep, NULL);
		__nfssec_ima_init();
	}
}

int __init nfssec_ima_init(void)
{
	INIT_DELAYED_WORK(&__delay_work, __nfssec_ima_init_delay);
	wq = alloc_ordered_workqueue("nfssec_wq_ima", WQ_MEM_RECLAIM);
	queue_delayed_work(wq, &__delay_work, 100);

	return 0;
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 6, 0)
// 这里应该制作lsm的初始化，要么通过这种方式，要么仿造一个宏，能够实现类似的架构
static int __init nfssec_ima_init_hook(void)
{
    security_add_hooks(nfssec_ima_hooks, nfssec_ima_hook_size, "nfssecima");
    // printk(KERN_INFO "=== nfssecima LSM initialized\n");
    return 0;
}

DEFINE_LSM(nfssecima) = {
	.name = "nfssecima",
	.init = nfssec_ima_init_hook,
};
#endif


#endif