#include <linux/slab.h>
#include <linux/sched.h>
#include <project.h>
#include "common/sym_hook/lsm_if.h"
#include "lsm_impl.h"
#include "common/version_type.h" 
#include "common/fullpath.h" 
#include "common/sec_krnl_misc.h" 
#include "common/file_access_table.h" 
#include "common/nfs_table.h"
#include "common/sec_user_table.h"
#include "common/vfs_mount.h"
#include "common/ac_table.h"
#include "common/ac_result.h"
#include "common/config/sec_ply_if.h"
#include "common/audit/sec_audit.h"
#include "common/n_trace_table.h"
#include "common/nfs_mnt.h"

// todo: can be common
// mac的模型
#define MAC_MODEL_ALL   0
#define MAC_MODEL_BLP   1
#define MAC_MODEL_BIBA  2


// todo 安全模式
// op_class 对应 OPTYPE_FILE等，主要用于审计，相当于大的审计类型
// sub_type 有用到，相当于ope，但是不多，看看是否有必要(这个名字要改)
// level 表示审计的警告等级，可用可扔

// todo: 用于扩展属性的操作
// name即为 setfattr中的-n字段，所以可以依次作为依据
// 然后使用current做为进程主体
static int impl_inode_setxattr(struct dentry *dentry, const char *name,
                                const void *value, size_t size, int flags)
{
    tag_pr_debug("setxattr name[%s]\n", name);
    return 0;
}                                

static int impl_inode_getxattr(struct dentry *dentry, const char *name)
{
    tag_pr_debug("getxattr name[%s]\n", name);
    return 0;
}

static int impl_inode_removexattr(struct dentry *dentry, const char *name)
{
    tag_pr_debug("removexattr name[%s]\n", name);
    return 0;    
}

// todo： mac的完整性 
static int mac_trie_check(path_trie_pattern_t *pattern, char *pattern_pos,
                               int offset, int left, void *search_context)
{
    int retval = PATH_TRIE_CHECK_CONTINUE;

    ac_ctx_t *ctx = (ac_ctx_t *)search_context;
    ac_trie_ext_t *ext;
    unsigned ruleType = 0;
    unsigned int sub_rank = 0;
    unsigned int obj_rank = 0;
    // unsigned int integrityRank = 0;
    unsigned int model = 0;

    mac_result_t mac_result;

    tag_pr_debug("\n");

    if (ctx->userItem) {
        sub_rank = ctx->userItem->rank;
        // integrityRank = ctx->userItem->integrityRank; // todo
    }

    ext = __PATH_TRIE_GET_EXT(ac_trie_ext_t, pattern);

    if (left != 0) {
        if (pattern_pos[pattern->nPathCount] != '/') {
            goto DONE;
        }
    }

    // -- 0 means has not security mark
    if ((NULL == ctx->userItem) || 0 == sub_rank)
        goto DONE;

    ruleType = ctx->acitem.rule_type = __POLICY_TYPE_MAC;

    // todo: 需要将ext中数据转化为具体格式，然后再赋值
    obj_rank = ((mac_obj_ext_t*)(ext->ext_data))->level;
    model = ((mac_obj_ext_t*)(ext->ext_data))->model;

    // (rank:0 is highest)->0 is lowest
    if (sub_rank == obj_rank) {
        goto DONE;
    }

    // 防止主体和客体分别来自于两种模型
    if (model != ctx->userItem->model)
        goto DONE;

    tag_pr_debug("model[%d], ctx->model[%d]\n", model, ctx->userItem->model);
    tag_pr_debug("sub_rank[%d] obj_rank[%d]\n", sub_rank, obj_rank);
    tag_pr_debug("ctx->bReading[%d] ctx->bModifying[%d]\n", ctx->bReading, ctx->bModifying);

    if (model == MAC_MODEL_BLP) {
        if (sub_rank < obj_rank && ctx->bReading) {
            ctx->bValid = TRUE;
        } else if (sub_rank > obj_rank && ctx->bModifying) {
            ctx->bValid = TRUE;
        }
    } else if (model == MAC_MODEL_BIBA) {
         if (sub_rank > obj_rank && ctx->bReading) {
            ctx->bValid = TRUE;
        } else if (sub_rank < obj_rank && ctx->bModifying) {
            ctx->bValid = TRUE;
        }       
    }

    if (ctx->bValid) {
        if (ctx->bReading) {
            ctx->acitem.ope = AUDIT_OPE_FILE_READ;
        } else if (ctx->bModifying) {
            ctx->acitem.ope = AUDIT_OPE_FILE_WRITE;
        }

        ctx->acitem.rule_id = ext->id;
        ctx->acitem.sub_type = ext->sub_type;

        ctx->acitem.is_ban = TRUE;
        ctx->acitem.is_audit = TRUE;

        memset(&mac_result, 0, sizeof(mac_result));
        mac_result.sub_rank = sub_rank;
        mac_result.obj_rank = obj_rank;

        memcpy(ctx->acitem.private, &mac_result, sizeof(mac_result));

        retval = PATH_TRIE_CHECK_BREAK;
    }

    tag_pr_debug("retval[%d]\n", retval);

DONE:
    if (ctx->acitem.is_ban) {
        tag_pr_debug("ctx->acitem.ruleType: %d\n", (int)ctx->acitem.rule_type);
        tag_pr_debug("file-access: path : %s\n", ctx->path);
        tag_pr_debug("file-access: retval : %d\n", retval);
    }
    return retval;
}

// 文件访问检查
// 返回值： -1/0 拒绝/允许
BOOLEAN file_acl_check_mac(struct dentry * dentry, struct vfsmount * mnt,
                BOOLEAN b_modify, BOOLEAN b_read, unsigned int sub_type)
{
    BOOLEAN retval = 0;
    int ret = 0;

    char *fullpath = NULL;

    ac_result_t ac_result;
    PID_CONTEXT *pid_ctx = NULL;

    sec_user_item_t *blp_user = NULL;
    sec_user_item_t *biba_user = NULL;

    sec_ply_if_t * ply_sub = NULL;
    sec_ply_if_t * ply_obj = NULL;

    BOOLEAN b_audit = FALSE;   

    SID_STOR sid;

    // 因为必然使用ac_result，未防止脏数据，需先清除
    memset(&ac_result, 0, sizeof(ac_result));

    ply_sub = get_ply_if(SEC_PLY_MAC_SUB);
    ply_obj = get_ply_if(SEC_PLY_MAC_OJB);

    // 策略检查 
    if (!ply_sub || !is_sw_enable(&ply_sub->sw) || is_sec_ply_empty(ply_sub))
        goto DONE;

    if (!ply_obj || is_sec_ply_empty(ply_obj))
        goto DONE;

    pid_ctx = sec_gen_lookup(__SEC_PID_GET(current));
    if (!pid_ctx) {
        pid_ctx = attachProcess(__SEC_PID_GET(current));
        if (!pid_ctx) {
            goto DONE;
        }
    }

    memset(&sid, 0, sizeof(sid));
    sid.id = pid_ctx->sid.id;

    // 获取blp user， 获取biba user
    // 然后分别用blp user和 biba user 查询客体

    mutex_lock(&ply_sub->sec_ply_lock);
    sid.model = MAC_MODEL_BLP;
    blp_user = sec_user_table_lookup_hash_tb( (HASHTB *)(ply_sub->ply_head), &sid);
    sid.model = MAC_MODEL_BIBA;
    biba_user = sec_user_table_lookup_hash_tb( (HASHTB *)(ply_sub->ply_head), &sid);
    mutex_unlock(&ply_sub->sec_ply_lock);
    if (!blp_user && !biba_user) {
        tag_pr_debug("useritem is null\n");
        goto DONE;
    }

    // 调整get path 位置，直到此刻，才开始需要获取path
    fullpath = get_fullpath_by_dentry_mnt(dentry, mnt);
    if (!fullpath)
        goto DONE;

    // 根据blp user， 查询客体
    memset(&ac_result, 0, sizeof(ac_result));

    mutex_lock(&ply_obj->sec_ply_lock);
    ret = ac_table_check(ply_obj->ply_head, fullpath, blp_user, b_modify, b_read, pid_ctx,
                               sub_type, &ac_result, mac_trie_check);
    mutex_unlock(&ply_obj->sec_ply_lock);

    if (ac_result.is_audit && ply_sub->sw.b_audit) 
        b_audit = TRUE;

    // todo : 信任进程，不受强访限制

    // 控制模式下，如果反馈终止，则退出
    if (ac_result.is_ban && ply_sub->sw.b_ctrl) {
        retval = -1;
        goto DONE;
    }

    // 需要审计，证明不需要后续处理
    // 但是，这里又不能放到 ban 判断的前面，否则会导致ban结果的错误
    if (b_audit == TRUE)
        goto DONE;

    // 如果blp没有查询到，则查询biba
    // 根据biba user， 查询客体
    memset(&ac_result, 0, sizeof(ac_result));

    mutex_lock(&ply_obj->sec_ply_lock);
    ret = ac_table_check(ply_obj->ply_head, fullpath, biba_user, b_modify, b_read, pid_ctx,
                               sub_type, &ac_result, mac_trie_check);
    mutex_unlock(&ply_obj->sec_ply_lock);

    if (ac_result.is_audit && ply_sub->sw.b_audit) 
        b_audit = TRUE;

    // todo : 信任进程，不受强访限制

    // 控制模式下，如果反馈终止，则退出
    if (ac_result.is_ban && ply_sub->sw.b_ctrl) {
        retval = -1;
        goto DONE;
    }
  
DONE:
    if (b_audit == TRUE) {
        AUDIT_FILE_MAC(pid_ctx->appFullName, pid_ctx->processID, ac_result.ope,  
           retval ? AUDIT_RESULT_FAILED : AUDIT_RESULT_SUCCESS, fullpath, 
           ((mac_result_t*)ac_result.private)->sub_rank, 
           ((mac_result_t*)ac_result.private)->obj_rank);
    }

    if (blp_user)
        comm_object_release(blp_user);

    if (biba_user)
        comm_object_release(biba_user);

    if (pid_ctx)
        comm_object_release(pid_ctx);

    return retval;
}

int mac_file_open(struct dentry * dentry, struct vfsmount * mnt, int flags)
{
    int retval = 0;

    BOOLEAN b_modify = FALSE;
    BOOLEAN b_read = FALSE;

    int acc_mode = (flags & O_ACCMODE); 

    unsigned int sub_type = __SUBTYPE_READFILE;

    // 当某些标志时，直接获取acc_mode是不准确的，因为不会置位而被当作读操作
    // O_NOATIME  01000000 读取文件时不更新最后访问时间 (atime)，以减少磁盘写入，提高性能，尤其是SSD
    // O_LARGEFILE	00100000 允许打开大文件。这通常用于 32 位系统，以支持超过 2 GB 的文件
    // O_DSYNC:[00010000], O_DIRECT[00040000]
    if (flags & O_NOATIME)
        return 0;

    if (flags & (O_APPEND | O_TRUNC | O_CREAT)) {
        b_modify = TRUE;
        sub_type = __SUBTYPE_WRITEFILE;
    } else {
        // can just be one of them
        switch (acc_mode) {
            case O_RDONLY:
                b_read = TRUE;
                sub_type = __SUBTYPE_READFILE;
                break;
            case O_WRONLY:
                b_modify = TRUE;
                sub_type = __SUBTYPE_WRITEFILE;
                break;
            case O_RDWR:
                b_read = TRUE;
                b_modify = TRUE;
                sub_type = __SUBTYPE_WRITEFILE;
                break;
            default:
                return 0;
        }
    }

    // todo: 这里，需要使用read、modify
    // 但是，ope其实也表示此含义
    // sub_type 其实也是，但是，因为在ac表中使用所以不好去除

    retval = file_acl_check_mac(dentry, mnt, b_modify, b_read, sub_type);

    return retval;
}

// 检查mac的策略，如果为空或开关未开，则退出
#define DO_MAC_PLY_CHECK() \
do {    \
    sec_ply_if_t * ply_sub = NULL;  \
    sec_ply_if_t * ply_obj = NULL;  \
    ply_sub = get_ply_if(SEC_PLY_MAC_SUB);  \
    if (!ply_sub)   \
        goto DONE;  \
    ply_obj = get_ply_if(SEC_PLY_MAC_OJB);  \
    if (!ply_obj)   \
        goto DONE;  \
    if (!ply_sub || !is_sw_enable(&ply_sub->sw) || is_sec_ply_empty(ply_sub))   \
        goto DONE;  \
} while (0);

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 32)
static int impl_file_open(struct dentry *dentry, struct vfsmount *mnt, const int flags)
#else
static int impl_file_open(struct file *file)
#endif
{
    int retval = 0;
    int ret = 0;

    struct dentry * __dentry = NULL;
    struct vfsmount * __mnt = NULL;
    int __flags = 0;
    
    // TRACE_DEFINE();

    DO_MAC_PLY_CHECK();

#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 32)
    __dentry = dentry;
    __mnt = mnt;
    __flags = flags;
#else
    __mnt = get_file_vfsmount(file);
    __dentry = get_file_dentry(file);
    __flags = file->f_flags;
#endif

    if (!__dentry)
        goto DONE;

    if (!__mnt)
        goto DONE;

    if (!is_mount_need_check(__mnt))
        goto DONE;

    // if (CHECK_IN_TRACE() == TRUE)
    //     goto DONE;

    ret = mac_file_open(__dentry, __mnt, __flags);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

DONE:
    // EXIT_TRACE(retval);
    return retval;
}

static int impl_inode_create(struct inode *dir, struct dentry *dentry, umode_t mode)
{
    int retval = 0;
    int ret = 0;
    struct vfsmount *mnt = NULL;

    DO_MAC_PLY_CHECK();

#ifndef CONFIG_SECURITY_PATH
    mnt = get_dentry_mnt(dentry);
#endif

    if (!dentry || !mnt || !is_mount_need_check(mnt))
        goto DONE;

    ret = mac_file_open(dentry, mnt, O_TRUNC);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

DONE:
    if (mnt)
        mntput(mnt);
    return retval;
}

static int impl_inode_rename(struct inode *old_dir, struct dentry *old_dentry,
                            struct inode *new_dir, struct dentry *new_dentry)
{
    int retval = 0;
    int ret = 0;

    struct vfsmount * mnt = NULL;

    DO_MAC_PLY_CHECK();

#ifndef CONFIG_SECURITY_PATH
    mnt = get_dentry_mnt(new_dentry);
#endif

    if (!mnt || !old_dentry || !old_dentry->d_inode)
        goto DONE;

    if ((!S_ISREG(old_dentry->d_inode->i_mode)) &&
        (!S_ISDIR(old_dentry->d_inode->i_mode)) &&
        (!S_ISLNK(old_dentry->d_inode->i_mode)))
        goto DONE;

    if (!new_dentry)
        goto DONE;

    if (!old_dentry  || !is_mount_need_check(mnt))
        goto DONE;

    if (!new_dentry || !mnt || !is_mount_need_check(mnt))
        goto DONE;

    ret = mac_file_open(old_dentry, mnt, O_TRUNC);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

    ret = mac_file_open(new_dentry, mnt, O_TRUNC);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

DONE:
    if (mnt)
        mntput(mnt);    
    return retval;
}

static int impl_inode_mkdir(struct inode *dir, struct dentry *dentry,
                           umode_t mode)
{
    int retval = 0;
    int ret = 0;
    struct vfsmount *mnt = NULL;

    DO_MAC_PLY_CHECK();

#ifndef CONFIG_SECURITY_PATH
    mnt = get_dentry_mnt(dentry);
#endif

    if (!mnt || !dentry)
        goto DONE;

    ret = mac_file_open(dentry, mnt, O_CREAT);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

DONE:
    if (mnt)
        mntput(mnt);
    return retval;
}

static int impl_inode_rmdir(struct inode *dir, struct dentry *dentry)
{
    int retval = 0;
    int ret = 0;
    struct vfsmount *mnt = NULL;

    DO_MAC_PLY_CHECK();

#ifndef CONFIG_SECURITY_PATH
    mnt = get_dentry_mnt(dentry);
#endif

    if (!mnt || !dentry || !dentry->d_inode)
        goto DONE;

    ret = mac_file_open(dentry, mnt, O_CREAT);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

DONE:
    if (mnt)
        mntput(mnt);
    return retval;
}

static int impl_inode_symlink(struct inode *dir, struct dentry *dentry,
                             const char *old_name)
{
    int retval = 0;
    int ret = 0;
    struct vfsmount *mnt = NULL;

    DO_MAC_PLY_CHECK();

#ifndef CONFIG_SECURITY_PATH
    mnt = get_dentry_mnt(dentry);
#endif

    if (!mnt || !dentry)
        goto DONE;

    ret = mac_file_open(dentry, mnt, O_CREAT);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

DONE:
    if (mnt)
        mntput(mnt);
    return retval;
}

static int impl_inode_link(struct dentry *old_dentry, struct inode *dir,
                          struct dentry *new_dentry)
{
    int retval = 0;
    int ret = 0;

    struct vfsmount * old_mnt = NULL;
    struct vfsmount * new_mnt = NULL;

    DO_MAC_PLY_CHECK();

#ifndef CONFIG_SECURITY_PATH
    old_mnt = get_dentry_mnt(old_dentry);
    new_mnt = get_dentry_mnt(new_dentry);
#endif

    if (!old_dentry || !old_mnt || !is_mount_need_check(old_mnt))
        goto DONE;

    if (!new_dentry || !new_mnt || !is_mount_need_check(new_mnt))
        goto DONE;

    ret = mac_file_open(old_dentry, old_mnt, O_CREAT);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

    ret = mac_file_open(new_dentry, new_mnt, O_CREAT);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

DONE:
    if (old_mnt)
        mntput(old_mnt);
    if (new_mnt)
        mntput(new_mnt);    
    return retval;
}

static int impl_inode_unlink(struct inode *dir, struct dentry *dentry)
{
    int retval = 0;
    int ret = 0;
    struct vfsmount *mnt = NULL;

    DO_MAC_PLY_CHECK();

#ifndef CONFIG_SECURITY_PATH
    mnt = get_dentry_mnt(dentry);
#endif

    if (!mnt || !dentry || !dentry->d_inode)
        goto DONE;

    if ((!S_ISREG(dentry->d_inode->i_mode)) &&
        (!S_ISDIR(dentry->d_inode->i_mode)) &&
        (!S_ISLNK(dentry->d_inode->i_mode)))
        goto DONE;

    ret = mac_file_open(dentry, mnt, O_TRUNC);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

DONE:
    if (mnt)
        mntput(mnt);
    return retval;
}

static int impl_file_permission(struct file *file, int mask)
{
    int retval = 0;
    int ret = 0;

    struct dentry * __dentry = NULL;
    struct vfsmount * __mnt = NULL;
    int __flags = 0;
    
    sec_ply_if_t * ply_sub = NULL;
    sec_ply_if_t * ply_obj = NULL;

    // TRACE_DEFINE();

    // 获取全局的plf_if
    ply_sub = get_ply_if(SEC_PLY_MAC_SUB);
    if (!ply_sub) {
        tag_pr_debug("sub ply null \n");
        goto DONE;
    }

    ply_obj = get_ply_if(SEC_PLY_MAC_OJB);
    if (!ply_obj) {
        tag_pr_debug("obj ply null \n");
        goto DONE;
    }

    // 策略检查 
    if (!ply_sub || !is_sw_enable(&ply_sub->sw) || is_sec_ply_empty(ply_sub))
        goto DONE;

    if (!ply_obj || !is_sw_enable(&ply_obj->sw) || is_sec_ply_empty(ply_obj))
        goto DONE;

    __mnt = get_file_vfsmount(file);
    __dentry = get_file_dentry(file);
    __flags = file->f_flags;

    if (!__dentry)
        goto DONE;

    if (!__mnt)
        goto DONE;

    if (!is_mount_need_check(__mnt))
        goto DONE;

    // if (CHECK_IN_TRACE() == TRUE)
    //     goto DONE;

    ret = mac_file_open(__dentry, __mnt, __flags);
    if (0 != ret) {
        retval = -1;
        goto DONE;
    }

DONE:
    // EXIT_TRACE(retval);
    return retval;    
}

static int impl_bprm_check_security(struct linux_binprm *bprm)
{
    int retval = 0;
    int ret = 0;

    // sec_ply_if_t * ply_sub = NULL;
    // sec_ply_if_t * ply_obj = NULL;

    // char *file_path = NULL;

    DO_MAC_PLY_CHECK();

    // todo: 判断, 当对进程执行时,当作客体写操作,进行mac的判断
    // 但是存在个问题,程序执行时会读文件,这个时候已经被拦截了
    // 到底是读拦截,还是写拦截???

    // file_path = get_fullpath_by_file(bprm->file);
    // if (!file_path) {
    //     goto DONE;
    // }

    // tag_pr_debug("brpm file_path[%s]\n", file_path);
    // ret = file_acl_check_mac(file_path, TRUE, FALSE, __SUBTYPE_WRITEFILE);
    // if (ret) {
    //     retval = -1;
    //     goto DONE;
    // }

    update_pid_context(bprm->file, current);

DONE:
    // if (file_path)
    //     kfree(file_path);
    return retval;
}


struct security_hook_list nfssec_mac_hooks[] = {

    // 通过file_open，可简单判断对文件的写操作
    // 通过file_permission,可判断的更具体（改名、删除）
    // notice： in some version，when file_open is using, file access maybe forbiden by ima
    // becase the flags is write even just cat file
    // so, use file_permission to replace it
    // but, file_permission cost too much resources
    // file_open is ok in current version (temp bug in some version?)
    MY_HOOK_INIT(file_open, impl_file_open), 

    // MY_HOOK_INIT(file_permission, impl_file_permission),
    
    MY_HOOK_INIT(inode_create, impl_inode_create),
    MY_HOOK_INIT(inode_rename, impl_inode_rename),
    MY_HOOK_INIT(inode_mkdir, impl_inode_mkdir),
    MY_HOOK_INIT(inode_rmdir, impl_inode_rmdir),
    MY_HOOK_INIT(inode_symlink, impl_inode_symlink),
    MY_HOOK_INIT(inode_link, impl_inode_link),
    MY_HOOK_INIT(inode_unlink, impl_inode_unlink),

    // MY_HOOK_INIT(task_alloc, impl_task_create),
	MY_HOOK_INIT(bprm_check_security, impl_bprm_check_security),

	// MY_HOOK_INIT(inode_setxattr, impl_inode_setxattr),
	// MY_HOOK_INIT(inode_getxattr, impl_inode_getxattr),
	// MY_HOOK_INIT(inode_removexattr, impl_inode_removexattr),
   
};

int nfssec_mac_hook_size = sizeof(nfssec_mac_hooks) / sizeof(nfssec_mac_hooks[0]);
