#include <linux/module.h>
#include <linux/init.h>
#include <linux/moduleparam.h>
#include <project.h>
#include "common/sym_hook/lsm_if.h"
#include "common/ac_table.h"
#include "common/config/sec_ply_if.h"
#include "common/audit/audit_queue.h"
#include "common/audit/sec_audit.h"
#include "ipc/ipc_ioctl.h"
#include "common/n_trace_table.h"
#include "common/nfs_table.h"

extern struct security_hook_list nfssec_mac_hooks[];
extern int nfssec_mac_hook_size;


#ifdef NFSSEC_KMOD
// 模块参数： /sys/module/module_name/parameters/param_name
char log_level_sys[10];
char * log_level = DEFAULT_LOG_LEVEL_S; // 日志输出级别
module_param(log_level, charp, 0644);
MODULE_PARM_DESC(log_level, "level of log output!");
#else 
extern char log_level_sys[10];
extern char * log_level;
#endif

audit_q_t g_audit_mac;

static int __nfssec_mac_init(void)
{
	int retval = -1;
	int ret = 0;

	get_log_level(log_level, log_level_sys); // 初始化日志输出级别
	// printk("log_level_sys:[%s]\n", log_level_sys);

    trace_table_init(256 * 16);

	gen_module_init();


	sec_ply_init(SEC_PLY_MAC_SUB);
	sec_ply_load(SEC_PLY_MAC_SUB);

	sec_ply_init(SEC_PLY_MAC_OJB);
	sec_ply_load(SEC_PLY_MAC_OJB);
	
	// todo：临时注释
	// sec_sw_load(SEC_SW_MAC);

#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
	ret = lsm_if_init(nfssec_mac_hooks, nfssec_mac_hook_size, "nfssecmac");
	if (ret) {
		tag_pr_err("init for nfssecmac lsm error!\n");
		goto DONE;
	}
#endif

	audit_q_init(&g_audit_mac, 200);

    ret = ipc_init_mac();
    if (ret) {
        tag_pr_err("nac ipc_init_mac error.\n");
        goto DONE;
    }

	tag_pr_info("After nfssecmac module init...\n");

    retval = 0;
DONE:
    return retval;
}

static void __exit nfssec_mac_exit(void)
{
    ipc_exit_mac();

	gen_module_destroy();

	trace_table_destroy();

	lsm_if_exit(nfssec_mac_hooks, nfssec_mac_hook_size);

	audit_q_clear(&g_audit_mac);

	tag_pr_info("After nfssecmac module exit...\n");
}

#ifdef NFSSEC_KMOD
module_init(__nfssec_mac_init);
module_exit(nfssec_mac_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("nfssec corp");
MODULE_DESCRIPTION("nfssec mandatory access ctrl");
MODULE_VERSION("1.0.1");
MODULE_ALIAS("nfssec mac");
#else

static struct delayed_work __delay_work;
static struct workqueue_struct *wq;
#define MY_FILE "/opt/nfssec/script/loadService.sh"

static void __nfssec_mac_init_delay(struct work_struct *work_req)
{
	struct file *filep = NULL;

	filep = filp_open(MY_FILE, O_RDWR | O_APPEND | O_CREAT, 0644);
	if (IS_ERR_OR_NULL(filep)) {
		mod_delayed_work(wq, &__delay_work, 100);
	} else {
		filp_close(filep, NULL);
		__nfssec_mac_init();
	}
}

int __init nfssec_mac_init(void)
{
	INIT_DELAYED_WORK(&__delay_work, __nfssec_mac_init_delay);
	wq = alloc_ordered_workqueue("nfssec_wq_mac", WQ_MEM_RECLAIM);
	queue_delayed_work(wq, &__delay_work, 100);

	return 0;
}

#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 6, 0)
static int __init nfssec_mac_init_hook(void)
{
    security_add_hooks(nfssec_mac_hooks, nfssec_mac_hook_size, "nfssecmac");
    // printk(KERN_INFO "=== nfssecmac LSM initialized\n");
    return 0;
}

DEFINE_LSM(nfssecmac) = {
	.name = "nfssecmac",
	.init = nfssec_mac_init_hook,
};
#endif

#endif