#include <project.h>
#include "nac_audit_map.h"
#include "common/nac_ply.h"
#include "common/audit/audit_map.c"


const char * nac_direction_str(unsigned int direction)
{
    if (direction == NAC_DIRECTION_IN)
        return "in";
    else if (direction == NAC_DIRECTION_OUT) 
        return "out";
    else 
        return "err";        
}

const char * nac_protocol_str(unsigned int protocol)
{
    if (protocol == NAC_PROTOCOL_TCP)
        return "tcp";
    else if (protocol == NAC_PROTOCOL_UDP) 
        return "udp";
    else 
        return "err";        
}

void nac_audit_item_show(nac_audit_item_t * item)
{
    tag_pr_info("audit item: proto[%d]\n", item->key.protocol);
    tag_pr_info("proto[%s] direction[%s] [%pI4:%d -> %pI4:%d] ", 
        nac_protocol_str(item->key.protocol),
        nac_direction_str(item->key.direction),
        &item->key.srcIp, item->key.srcPort, &item->key.destIp, item->key.destPort);
}

static void nac_audit_item_free(nac_audit_item_t *item)
{
    kfree(item);
    return;
}

nac_audit_item_t *nac_audit_item_new(void)
{
    nac_audit_item_t *item = NULL;

    item = (nac_audit_item_t *)kmalloc(sizeof(nac_audit_item_t), GFP_ATOMIC);
    if (!item)
        return NULL;

    memset(item, 0, sizeof(*item));
    item->hdr.objHdr.destroy_ptr = (destroy_atom_ptr)nac_audit_item_free;

    spin_lock_init(&item->hdr.lock);

    comm_object_addref(item);
    return item;
}

static void __net_audit_item_release(LIST_HEAD *entry)
{
    nac_audit_item_t *ctx = OBJECT_PTR(entry, nac_audit_item_t, hdr.hhead);
    comm_object_release(ctx);
    return;
}

static unsigned char *__net_audit_item_value(LIST_HEAD *entry)
{
    nac_audit_item_t *ctx = OBJECT_PTR(entry, nac_audit_item_t, hdr.hhead);
    unsigned char *ptr = (unsigned char *)&ctx->key;

    return (unsigned char *)ptr;
}

nac_audit_item_t *nac_audit_map_lookup(nac_aduit_map_t *audit_map, unsigned int srcIp,
                                 unsigned short srcPort, unsigned int destIp,
                                 unsigned short destPort,
                                 unsigned char protocol)
{
    nac_aduit_key_t key;
    nac_audit_item_t *item = NULL;
    audit_cache_hdr_t *hdr;
    key.srcIp = srcIp;
    key.srcPort = srcPort;
    key.destIp = destIp;
    key.destPort = destPort;
    key.protocol = protocol;

    hdr = audit_map_lookup(&audit_map->map, &key);
    if (hdr) 
        item = OBJECT_PTR(hdr, nac_audit_item_t, hdr);
   
    return item;
}

int nac_audit_map_init(nac_aduit_map_t *audit_map)
{
    int retval = -1;
    BOOLEAN bflag;

    bflag = audit_map_init(&audit_map->map, __net_audit_item_value,
                         __net_audit_item_release, sizeof(nac_aduit_key_t));
    if (!bflag) 
        goto DONE;
   
    retval = 0;
DONE:
    return retval;
}

void nac_aduit_map_destroy(nac_aduit_map_t *audit_map)
{
    audit_map_destroy(&audit_map->map);
    return;
}

int nac_audit_map_clear(nac_aduit_map_t *audit_map)
{
    audit_map_clear(&audit_map->map);
    return 0;
}

int nac_audit_map_insert(nac_aduit_map_t *audit_map, nac_audit_item_t *item)
{
    return audit_map_insert(&audit_map->map, &item->hdr);
}
