#include <linux/errno.h>
#include <linux/fs.h>
#include <project.h>

#include "common/ioctrl_param.h"
#include "common/audit/sec_audit.h"
#include "ipc_handle.h"
#include "../cfg_nac_ply.h"
#include "../nac_switch.h"
#include "audit/nac_audit.h"
#include "common/config/sec_ply_if.h"

#include "common/general/include/ioctl_mod_nac.h"
// #include "common/config/cfg_switch.h"

NTSTATUS __ioctl_read_audit(struct file *filep, IOCTL_PARAM *param)
{
    NTSTATUS nt_status = STATUS_SUCCESS;

    nt_status = nac_audit_read(filep, param);
    return nt_status;
}

NTSTATUS __nac_set_switch(struct file *filep, IOCTL_PARAM *param)
{
    NTSTATUS nt_status = STATUS_SUCCESS;
    PVOID ioBuffer;
    LONG inputBufferLength;
    NFSSEC_SWITCH_VARIANT *val;

    ioBuffer = param->inputBuffer;
    inputBufferLength = param->inputBufferLength;

    val = (NFSSEC_SWITCH_VARIANT *)ioBuffer;

    nac_set_switch(val);

    param->uRetValue = nt_status;
    param->outputBufferLength = 0;
    return nt_status;
}

int __ioctl_reload_ply(IOCTL_PARAM *param)
{
    int nt_status = STATUS_SUCCESS;
    int ret = 0;

    PVOID ioBuffer;
    LONG inputBufferLength;
    unsigned int type;

    ioBuffer = param->inputBuffer;
    inputBufferLength = param->inputBufferLength;

    if ((inputBufferLength < sizeof(int))) {
        nt_status = STATUS_INVALID_PARAMETER;
        goto DONE;
    }

    type = *(unsigned int *)ioBuffer;

    tag_pr_debug("sec ply load [%d]\n", type);

    // 这里可以加载 文件策略 或 开关策略
    if (type > SEC_SW_MIN && type < SEC_SW_MAX)
        ret = sec_sw_load(type);
    else 
        ret = sec_ply_load(type);

    if (ret) 
        nt_status = STATUS_SUCCESS;
    else 
        nt_status = STATUS_INVALID_PARAMETER;

DONE:
    param->uRetValue = nt_status;
    param->outputBufferLength = 0;
    return nt_status;
}

int dispatch_ioctl(struct file *filep, IOCTL_PARAM *param)
{
    int retval = -1;

    // g_globals.policy_up_cnt++; // todo

    // tag_pr_info("nac got ioctl[%d]\n", param->uCode);

    switch (param->uCode) {
        case NFSSEC_CODE_READ_AUDIT:
            __ioctl_read_audit(filep, param);
            break;


        case NFSSEC_CODE_RELOAD_PLY:
        tag_pr_debug("==============load ply\n");
            __ioctl_reload_ply(param); //todo

            // sec_ply_load(SEC_PLY_PROTECT_KMOD);
            // sec_ply_load(SEC_PLY_PROTECT_PROC);
            // sec_ply_load(SEC_PLY_PROTECT_FILE);
            // sec_ply_load(SEC_PLY_PROTECT_DIR);
            break;

        // case NFSSEC_CODE_RELOAD_PLY:
        //     // cfg_nac_ply_load();
        //     sec_ply_load(SEC_PLY_NAC);
        //     break;
        
        case NAC_CODE_SET_SWITCH:
            __nac_set_switch(filep, param);
            break;

        default:
            tag_pr_debug("default\n");
            break;
    }
    retval = param->uRetValue;
DONE:
    return retval;
}

long __ipc_handle(struct file *filep, void *__user __data)
{
    int retval = -1;
    SEC_IOCTL_DATA __ioctl;
    IOCTL_PARAM *param = NULL;

    if (unlikely(0 != copy_from_user(&__ioctl, __data, sizeof(SEC_IOCTL_DATA)))) {
        retval = -EINVAL;
        goto DONE;
    }

    param = ioctl_param_create(&__ioctl);
    if (!param) {
        retval = -EINVAL;
        goto DONE;
    }

    __ioctl.uRetValue = dispatch_ioctl(filep, param);
    __ioctl.outputLen = 0;
    if (0 == __ioctl.uRetValue) {
        __ioctl.outputLen = param->outputBufferLength;
        if (param->outputBufferLength > 0) {
            if (0 == param->uRawMode) {
                if (0 != copy_to_user((void *)__ioctl.output,
                                      param->outputBuffer,
                                      param->outputBufferLength)) {
                    goto DONE;
                }
            }
        }
    }
    if (0 != copy_to_user(__data, &__ioctl, sizeof(SEC_IOCTL_DATA))) {
        goto DONE;
    }

    retval = 0;
DONE:
    if (param) {
        ioctl_param_destroy(param);
        param = NULL;
    }
    return retval;
}

long ipc_handle_nac(struct file *filep, unsigned int cmd, unsigned long arg)
{
    int retval = 0;
    switch (cmd) {
        
        case NFSSEC_CODE_KMOD_IOCTL:
            retval = __ipc_handle(filep, (void *__user)arg);
            break;
        default:
            retval = -EINVAL;
    }

    return retval;
}
