#include <project.h>
#include <linux/init.h>
#include <linux/module.h>
#include "common/nac_ply.h"
#include "ipc/ipc_ioctl.h"
#include "nac_filter.h"
#include "nac_switch.h"
#include "audit/nac_audit.h"
#include "common/config/sec_ply_if.h"

nac_switch_t g_nac_switch;

extern int cfg_nac_ply_load(void); // todo

#ifdef NFSSEC_KMOD
// 模块参数： /sys/module/module_name/parameters/param_name
char log_level_sys[10];
char * log_level = DEFAULT_LOG_LEVEL_S; // 日志输出级别
module_param(log_level, charp, 0644);
MODULE_PARM_DESC(log_level, "level of log output!");
#else 
extern char log_level_sys[10];
extern char * log_level;
#endif

static int __init __nfssec_nac_init(void)
{
    int retval = -1;
    int ret;

    get_log_level(log_level, log_level_sys);  // 初始化日志输出级别

    nac_audit_init();

    nac_switch_init();

    nac_ply_init();

    // cfg_nac_ply_load(); 
    sec_ply_init(SEC_PLY_NAC);
	sec_ply_load(SEC_PLY_NAC);
	sec_sw_load(SEC_SW_NAC);

    nac_filter_init();

    ret = ipc_init_nac();
    if (ret) {
        tag_pr_err("nac ipc_init error.\n");
        goto DONE;
    }

	tag_pr_info("After nfssecnac module init...\n");

    retval = 0;
DONE:
    return retval;
}

static void __exit nfssec_nac_exit(void) 
{
    ipc_exit_nac();

    nac_filter_uninit();

    nac_ply_uninit();

    nac_switch_uninit();

    nac_audit_uninit();

    tag_pr_info("After nfssecnac module exit...\n");
}

#ifdef NFSSEC_KMOD
module_init(nfssec_nac_init);
module_exit(nfssec_nac_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("nfssec corp");
MODULE_DESCRIPTION("nfssec net access ctrl");
MODULE_VERSION("2.0.0");
MODULE_ALIAS("nfssec nac 1");
#else

static struct delayed_work __delay_work;
static struct workqueue_struct *wq;
#define MY_FILE "/opt/nfssec/script/loadService.sh"

static void __nfssec_nac_init_delay(struct work_struct *work_req)
{
	struct file *filep = NULL;

	filep = filp_open(MY_FILE, O_RDWR | O_APPEND | O_CREAT, 0644);
	if (IS_ERR_OR_NULL(filep)) {
		mod_delayed_work(wq, &__delay_work, 100);
	} else {
		filp_close(filep, NULL);
		__nfssec_nac_init();
	}
}

int __init nfssec_nac_init(void)
{
	INIT_DELAYED_WORK(&__delay_work, __nfssec_nac_init_delay);
	wq = alloc_ordered_workqueue("nfssec_wq_nac", WQ_MEM_RECLAIM);
	queue_delayed_work(wq, &__delay_work, 100);

	return 0;
}

#endif
