#include <project.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include "nac_switch.h"
#include <linux/netfilter.h>
#include <uapi/linux/netfilter_ipv4.h>
#include <linux/skbuff.h>
#include <net/sock.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <net/ip.h>
#include <linux/fdtable.h>
#include <linux/sched/task.h>
#include <linux/sched.h>
extern rwlock_t tasklist_lock;

// for ip4 netfilter, ipv6 -> linux/netfilter_ipv6.h
#include <linux/netfilter_ipv4.h>  
#include "common/nac_ply.h"
#include "common/audit/sec_audit.h"
#include "audit/nac_audit.h"
#include "common/config/sec_ply_if.h"

#define nf_register_hooks(net_hooks, count) \
    nf_register_net_hooks(&init_net, net_hooks, count)
#define nf_unregister_hooks(net_hooks, count) \
    nf_unregister_net_hooks(&init_net, net_hooks, count)


static struct task_struct *for_process(struct sk_buff *skb)
{
	struct task_struct *p = NULL;
	struct files_struct *files;
	struct fdtable *fdtab;
	int i;
	int file_flag;

	if (IS_ERR_OR_NULL(skb))
		return 0;
	struct sock *sk = skb->sk;
	if (IS_ERR_OR_NULL(sk))
		return 0;
	read_lock(&tasklist_lock);
	for_each_process(p)
	{
		if (IS_ERR_OR_NULL(p))
		{
			break;
		}
		if (IS_ERR_OR_NULL(p->files))
		{
			if (IS_ERR_OR_NULL(p->comm))
			{
				break;
			}
		}
		else
		{
			files = p->files;
			if (!IS_ERR_OR_NULL(files))
			{
				rcu_read_lock();
				fdtab = rcu_dereference(files->fdt);
				for (i = 0; i < fdtab->max_fds; i++) {
					struct file *file;
					file = fcheck_files(files, i);
					file_flag = 0;
					if (!IS_ERR_OR_NULL(file)) {
						if (!IS_ERR_OR_NULL(file->f_inode) && !IS_ERR_OR_NULL(sk->sk_socket) && !IS_ERR_OR_NULL(sk->sk_socket->file) && !IS_ERR_OR_NULL(sk->sk_socket->file->f_inode) && file->f_inode == sk->sk_socket->file->f_inode)
						{
							file_flag = 1;
							break;
						}
					}
					else
						break;
				}
				rcu_read_unlock();
			}
		}
		if (file_flag == 1)
			break;
	}
	read_unlock(&tasklist_lock);
	if (IS_ERR_OR_NULL(p))
		p = current;
	return p;
}

static unsigned int check_current_sk(struct sk_buff *skb)
{
	struct task_struct *p;
	p = current;
	struct files_struct *files;
	struct fdtable *fdtab;
	int i;
	int file_flag;
	file_flag = 0;
	if (IS_ERR_OR_NULL(skb))
		return 0;
	struct sock *sk = skb->sk;
	if (IS_ERR_OR_NULL(sk))
		return 0;
	if (!IS_ERR_OR_NULL(p->files))
	{
		files = NULL;
		files = p->files;
		if (!IS_ERR_OR_NULL(files)) {
			rcu_read_lock();
			struct file *file;
			fdtab = rcu_dereference(files->fdt);
			for (i = 0; i < fdtab->max_fds; i++) {
				file = NULL;
				file = fcheck_files(files, i);
				if (IS_ERR_OR_NULL(file))
					break;
				if (file) {
					if (!IS_ERR_OR_NULL(file->f_inode) && !IS_ERR_OR_NULL(sk->sk_socket) && !IS_ERR_OR_NULL(sk->sk_socket->file) && !IS_ERR_OR_NULL(sk->sk_socket->file->f_inode) && file->f_inode == sk->sk_socket->file->f_inode)
					{
						file_flag = 1;
						break;
					}
				}
			}
			rcu_read_unlock();
		}
	}
	//if (file_flag == 0)
	//{
	//	printk("%s need check \n", current->comm);
	//	for_process(sk);
	//}
	return file_flag;
}



static int filter_sk_buff(unsigned int direction, struct sk_buff *pskb)
{
    // 默认放行
    int retval = NF_ACCEPT; 

    struct sk_buff *skb = pskb;

    struct iphdr *iph;

    unsigned char protocol = 0;

    struct tcphdr *tcp = NULL;
    struct udphdr *udp = NULL;

    unsigned int src_ip = 0, dest_ip = 0;
    unsigned int src_port = 0, dest_port = 0;

    unsigned int check_ip = 0;
    unsigned int check_port = 0;

    char * proc = NULL;

    nac_rule_node_t * rule_node = NULL;
    unsigned int type = 0;

    sec_ply_if_t * ply = NULL;

    // 获取全局的plf_if
    ply = get_ply_if(SEC_PLY_NAC);
    if (!ply || !is_sw_enable(&ply->sw) || is_sec_ply_empty(ply))
        goto DONE;

    // 直接使用current时，似乎不能准确获得进程，可能实际为ac，但获得为swapper
    // 此处尝试通过其他方式获取进程信息，未验证完
    // {
    //     struct pid * kpid = NULL;
    //     struct task_struct * task =  NULL;
    //     struct task_struct * got_task =  NULL;

    //     kpid = find_get_pid(current->pid);
    //     task = pid_task(kpid, PIDTYPE_PID);

    //     if (kpid && task) {
    //         pid_t result = task_tgid_nr_ns(task, kpid->numbers[kpid->level].ns);
            
    //         printk(KERN_INFO "pid[%d] result[%d]\n", kpid->numbers[kpid->level].nr, result);
    //         got_task = get_pid_task(kpid->numbers[kpid->level].nr, PIDTYPE_PID);
    //         if (got_task != NULL) {
    //                 tag_pr_debug("get proc: [%s]\n", got_task->comm);
    //         }
    //     }
    // }

    // todo： 不稳定，可能导致死机
    // 当前进程
    // if (check_current_sk(skb))
    //     proc = current->comm;
    // else
	// proc = for_process(skb)->comm;

    // 获取ip头
    iph = ip_hdr(skb);  
    if (!iph)
        goto DONE;

    // ip地址
    src_ip = iph->saddr;
    dest_ip = iph->daddr;

    // 协议类型
    protocol = iph->protocol;

    if (protocol == IPPROTO_TCP) {

        tcp = tcp_hdr(skb);
        // tcp = (struct tcphdr *)((u8 *)iph + (iph->ihl << 2));
        if (!tcp)
            goto DONE;

        if (!tcp->syn) 
            goto DONE;

        // if (tcp->ack) 
        //     goto DONE;

        dest_port = ntohs(tcp->dest);
        src_port = ntohs(tcp->source);

        // if (!bIn) 
        //     goto DONE;

    } else if (protocol == IPPROTO_UDP) {
        // udp = udp_hdr(skb);
        udp = (struct udphdr *)((u8 *)iph + (iph->ihl << 2));
        if(!udp) 
            goto DONE;
        
        dest_port = ntohs(udp->dest);
        src_port = ntohs(udp->source);
    }

    // 映射proto到策略类型
    if (protocol == IPPROTO_TCP)
        protocol = NAC_PROTOCOL_TCP;
    else if (protocol == IPPROTO_UDP)
        protocol = NAC_PROTOCOL_UDP;
    else 
        protocol = NAC_PROTOCOL_ALL;

    if (direction == NAC_DIRECTION_IN) {
        check_ip = src_ip;
        check_port = dest_port;
    } else { // direction == NAC_DIRECTION_OUT
        check_ip = dest_ip;
        check_port = src_port;
    }

    // printk(KERN_INFO "src_port[%d] dest_port[%d]\n", 
    //     src_port, dest_port);

tag_pr_debug("\n");
    rule_node = nac_check_item(direction, protocol, check_ip, check_port, proc, &type);
    if (rule_node && type == NAC_PLY_TYPE_DENY) {
        tag_pr_debug("===============ply->sw.b_audit[%d] ply->sw.b_ctrl[%d]\n", ply->sw.b_audit, ply->sw.b_ctrl);
        if (ply->sw.b_audit) {
            nac_audit_write(&g_audit_nac, src_ip, src_port, dest_ip, dest_port, protocol, direction, retval);
        }

        if (ply->sw.b_ctrl) {
            tag_pr_debug("nac drop item!\n");
            retval = NF_DROP;
            goto DONE;
        }
    }

DONE:
    return retval;
}

static unsigned int nf_hook_local_in(void *priv, struct sk_buff *pskb,
                                    const struct nf_hook_state *state)
{
    return filter_sk_buff(NAC_DIRECTION_IN, pskb);
}

static unsigned int net_hook_local_out(void *priv, struct sk_buff *pskb,
                                     const struct nf_hook_state *state)
{
    return filter_sk_buff(NAC_DIRECTION_OUT, pskb);
}

static unsigned int net_hook_post_routing(void *priv, struct sk_buff *pskb,
                                        const struct nf_hook_state *state)
{
    // return filter_sk_buff(NAC_DIRECTION_IN, pskb);
    return NF_ACCEPT;
}

static unsigned int net_hook_forward(void *priv, struct sk_buff *pskb,
                                    const struct nf_hook_state *state)
{
    // return filter_sk_buff(NAC_DIRECTION_IN, pskb);
    return NF_ACCEPT;
}

static unsigned int net_hook_pre_routing(void *priv, struct sk_buff *pskb,
                                       const struct nf_hook_state *state)
{
    // return filter_sk_buff(NAC_DIRECTION_IN, pskb);
    return NF_ACCEPT;
}

// 钩子数组
static struct nf_hook_ops net_hooks[] = {
    {
        .hook = nf_hook_local_in,  // 发往本地数据包
        .pf = PF_INET,
        .hooknum = NF_INET_LOCAL_IN, 
        .priority = NF_IP_PRI_FILTER - 1,
    },
    {
        .hook = net_hook_local_out,  // 本地发出数据包
        .pf = PF_INET,
        .hooknum = NF_INET_LOCAL_OUT,
        .priority = NF_IP_PRI_FILTER - 1,
    },
    {
        .hook = net_hook_forward,  // 转发的数据包
        .pf = PF_INET,
        .hooknum = NF_INET_FORWARD,
        .priority = NF_IP_PRI_FILTER - 1,
    },
    {
        .hook = net_hook_pre_routing,  // 进入本机路由前
        .pf = PF_INET,
        .hooknum = NF_INET_PRE_ROUTING,
        .priority = NF_IP_PRI_FILTER - 1,
    },
    {
        .hook = net_hook_post_routing,  // 本机发出包经路由后
        .pf = PF_INET,
        .hooknum = NF_INET_POST_ROUTING,
        .priority = NF_IP_PRI_FILTER - 1,
    },
};

int nac_filter_init(void)
{
    return nf_register_hooks(net_hooks, ARRAY_SIZE(net_hooks));  // 安装钩子
}

void nac_filter_uninit(void)
{
    nf_unregister_hooks(net_hooks, ARRAY_SIZE(net_hooks));  // 卸载钩子
}

