
#ifndef __XC_LIST_H
#define __XC_LIST_H

#include <linux/types.h>
#include <linux/string.h>
#include <linux/kernel.h>
#include <linux/spinlock_types.h>


struct xc_list_node
{
    char *data_;
    struct xc_list_node *next_;
    struct xc_list_node *rear_;
};

struct xc_list
{
    unsigned int maximum_;
    unsigned int len_;
    int is_thread_safe_;
    struct xc_list_node *head_;
    spinlock_t lock_;
    char name_[64];
};

typedef int (*xc_list_fun)(struct xc_list *list, void *arg);
typedef int (*xc_list_find_fun)(const char *data, const char *condition);
typedef int (*xc_list_for_each_fun)(const char *data, char *opt);

int xc_list_init(struct xc_list *list
                , unsigned int maximum);
int xc_list_set_name(struct xc_list *list, const char *name);
int xc_list_set_thread_safe(struct xc_list *list, int enable);
int xc_list_push(struct xc_list *list, const void *data, unsigned int len);
int xc_list_push_shallow_copy(struct xc_list *list, void *data);
int xc_list_push_str(struct xc_list *list, const char *str);
int xc_list_pop(struct xc_list *list, char **data);
int xc_list_clear(struct xc_list *list);
int xc_list_remove_if(struct xc_list *list, xc_list_find_fun fun, const char *condition);
int xc_list_remove_if_all(struct xc_list *list, xc_list_find_fun fun, const char *condition);
int xc_list_remove_back(struct xc_list *list);
int xc_list_for_each(struct xc_list *list, xc_list_for_each_fun fun
                    , char *opt, unsigned int *len);
int xc_list_destroy(struct xc_list *list);

#endif
