#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/version.h>
 
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/udp.h>
#include <linux/netfilter.h>
#include <linux/netfilter_ipv4.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/uaccess.h>
#include <linux/proc_fs.h>
#include <linux/string.h>
#include <linux/module.h>  
#include <linux/kernel.h>  
#include <linux/jiffies.h>  
#include <net/ipv6.h>
#include <linux/time.h>

#include "list.h"
#include "debug_log_queue.h"
#include "file_opt.h"
#include "net_audit.h"
#include "test.h"
#include "util.h"
#include "net_drop.h"

#define DEVICE_CNT			1		  	/* 设备号个数 */
#define DEVICE_NAME			"net_flow_audit_policy"	/* 名字 */

#define USE_DEBUG_GET_TUPLE_HEAD (0)
#define USE_DEBUG_GET_TUPLE_TCP (0)
#define USE_DEBUG_GET_TUPLE_UDP (0)
#define USE_DEBUG_VERIFY_TUPLE (0)
#define USE_DEBUG_AUDIT (0)
#define USE_DEBUG_DROP  (0)
#define USE_DEBUG_NET_HOOK  (0)
#define USE_DEBUG_HEARTBEAT (0)

#define USE_FILE_OPT  (0)
#define USE_SELF_TEST (0)
#define USE_READ_BLOCK (1)

#define ABS(x) (((x) < 0) ? (-(x)) : (x))

// 验证超出边界
#define VERIFY_LEFT_LIMIT(x, y)  if (x < y) return -1
#define VERIFY_RIGHT_LIMIT(x, y) if (x > y) return -1
#define VERIFY_IPV6_LEFT_LIMIT(x, y) if (0 > ipv6_addr_cmp(x, y)) return -1;
#define VERIFY_IPV6_RIGHT_LIMIT(x, y) if (0 < ipv6_addr_cmp(x, y)) return -1;



/* 设备结构体 */
struct net_audit_dev{
	dev_t devid;			    /* 设备号 	 */
	struct cdev cdev;		    /* cdev 	*/
	struct class *class;	    /* 类 		*/
	struct device *device;	    /* 设备 	 */
	int major;				    /* 主设备号	  */
	int minor;				    /* 次设备号   */
	struct device_node *nd;     /* 设备节点 */
};

/* 审计参数 */
struct net_audit_param
{
    void *param1;
    void *param2;
    void *param3;
    void *param4;
    void *param5;
    int ret;
    netflow_dir_t dir;
    struct net_five_tuple tuple;
};

/* 进程审计服务心跳定时器 */
struct audit_proc_heartbeat_timer
{
    int timeperiod; 		            // 定时时间ms
    struct timer_list timer;            // 定时器
    atomic_t enable_audit_proc;         // 使能进程审计
    atomic_t time_remain;               // 剩余时间
    int timeout;                        // 超时时间
} audit_proc_heartbeat;

// 审计和阻断日志
struct xc_list audit_log, drop_log;

// 设备
static struct net_audit_dev device;

// 文件操作
#if USE_FILE_OPT
static struct file_opt file_fd;
#endif


// 虚拟文件系统
static struct proc_dir_entry *proc_netflow_root = NULL;
static struct proc_dir_entry *proc_netflow_audit_policy_flie = NULL;
static struct proc_dir_entry *proc_netflow_audit_policy_log_file = NULL;
static struct proc_dir_entry *proc_netflow_drop_plicy_file = NULL;
static struct proc_dir_entry *proc_netflow_drop_policy_log_file = NULL;
static struct proc_dir_entry *proc_debug_log_file = NULL;

static const char *  proc_netflow_root_name = "netflow_module";
static const char *  proc_netflow_audit_policy_name = "netflow_audit_policy.data";
static const char *  proc_netflow_audit_policy_log_name = "netflow_audit_policy.log";
static const char *  proc_netflow_drop_policy_name = "netflow_drop_policy.data";
static const char *  proc_netflow_drop_policy_log_name = "netflow_drop_policy.log";
static const char *  proc_debug_log_name = "debug.log";

// 等待队列头
#if USE_READ_BLOCK
static wait_queue_head_t audit_policy_log_wait_head;
static wait_queue_head_t drop_policy_log_wait_head;
#endif


// 流量阻断开关是否开启   0  关闭    1  开启
static int g_netflow_drop_log_flag = 0;
static int g_netflow_audit_log_flag = 0;

static int start_audit_proc(void); // 开启进程审计


// 设置是否开启审计策略保存日志开启
int set_audit_log_open_flag(void)
{
    g_netflow_audit_log_flag = 1;
    return 0;
}
// 设置是否开启审计策略保存日志关闭
int set_audit_log_close_flag(void)
{
    g_netflow_audit_log_flag = 0;
    return 0;
}


// 设置是否开启阻断策略保存日志开启
int set_drop_log_open_flag(void)
{
    g_netflow_drop_log_flag = 1;
    return 0;
}

// 设置是否开启阻断策略保存日志关闭
int set_drop_log_close_flag(void)
{
    g_netflow_drop_log_flag = 0;
    return 0;
}



/*
 * @description		: 打开设备
 * @param - inode 	: 传递给驱动的inode
 * @param - filp 	: 设备文件，file结构体有个叫做private_data的成员变量
 * 					  一般在open的时候将private_data指向设备结构体。
 * @return 			: 0 成功;其他 失败
 */
static int net_flow_open(struct inode *inode, struct file *filp)
{
	filp->private_data = &device; /* 设置私有数据 */

	return 0;
}

/*
 * @description		: 从设备读取数据 
 * @param - filp 	: 要打开的设备文件(文件描述符)
 * @param - buf 	: 返回给用户空间的数据缓冲区
 * @param - cnt 	: 要读取的数据长度
 * @param - offt 	: 相对于文件首地址的偏移
 * @return 			: 读取的字节数，如果为负值，表示读取失败
 */
static ssize_t net_flow_read(struct file *filp, char __user *buf, size_t cnt, loff_t *offt)
{
	return 0;
}

/*
 * @description		: 向设备写数据 
 * @param - filp 	: 设备文件，表示打开的文件描述符
 * @param - buf 	: 要写给设备写入的数据
 * @param - cnt 	: 要写入的数据长度
 * @param - offt 	: 相对于文件首地址的偏移
 * @return 			: 写入的字节数，如果为负值，表示写入失败
 */
static ssize_t net_flow_write(struct file *filp, const char __user *buf, size_t cnt, loff_t *offt)
{
	int ret = 0;
	struct unified_param para;
    struct netflow_audit_policy policy;
    memset(&policy,0,sizeof(struct netflow_audit_policy));
    memset(&para,0,sizeof(struct unified_param));
    ret = copy_from_user((void *)&para, buf ,sizeof(para));
    if(ret == 0){
        // 根据业务的操作添加到不同的队列种进行后续业务处理
        switch(para.type) 
        {
            case drop_add_policy:
            // 添加网络数据阻断策略
                policy.direction = para.param.drop_policy.direction;
                policy.tuple = para.param.drop_policy.tuple;
                policy.type = 2;
                policy.policy_id = para.param.drop_policy.policy_id;
                ret = netflow_drop_add_policy(&policy);
                break;
            case drop_delete_policy:
                // 删除网络数据阻断策略
                // printk("---------drop_delete_policy------------\n");
                ret = netflow_drop_delete_policy(para.param.drop_policy_id);
                break;
            case drop_delete_all:
                // 清空网络数据阻断策略
                ret = netflow_drop_clear_policy();
                break;
            case drop_policy_counts:
                // proc中处理
                break; 

            case drop_socket_open:
                ret = set_drop_log_open_flag();
                break;
            case drop_socket_close:
                ret = set_drop_log_close_flag();
                break;   

            case audit_add_policy:
                ret = netflow_audit_add_policy(&para.param.audit_policy);
                break;

            case audit_add_proc_policy:
                ret = netflow_audit_add_proc_policy(&para.param.audit_proc_policy);
                start_audit_proc();
                break;

            case audit_add_proc_policy_complete:
                ret = netflow_audit_add_policy_complete();
                start_audit_proc();
                break;

            case audit_add_proc_policy_heartbeat:
                ret = start_audit_proc();
                break;

            case audit_delete_policy:
                ret = netflow_audit_delete_policy(para.param.audit_policy_id);
                break;

            case audit_delete_all:
                ret = netflow_audit_clear_policy();
                break;

            case audit_policy_counts:
                // 在proc中
                break;
            case audit_socket_open:
                ret = set_audit_log_open_flag();
                break;
            case audit_socket_close:
                ret = set_audit_log_close_flag();
                break;  

            default:
            printk("kernel net_flow_policy not unified param type data failed,please check parameter!\n");    
            break;
        }

        if (ret) {
            return -1;
        }

    } 
    else {
        printk("kernel net_flow_policy recv data failed!\n");
    }

	return 0;
}

/*
 * @description		: 关闭/释放设备
 * @param - filp 	: 要关闭的设备文件(文件描述符)
 * @return 			: 0 成功;其他 失败
 */
static int net_flow_release(struct inode *inode, struct file *filp)
{
	return 0;
}

/* 设备操作函数 */
static struct file_operations net_flow_fops = {
	.owner = THIS_MODULE,
	.open = net_flow_open,
	.read = net_flow_read,
	.write = net_flow_write,
	.release = 	net_flow_release,
};



static int netflow_audit_policy_open(struct inode *inode, struct file *file)
{
    // printk("proc netflow_audit_policy_open,fileName:%s\n",file->f_path.dentry->d_iname);
    return 0;
}

/*
** This function will be called when we close the procfs file
*/
static int netflow_audit_policy_release(struct inode *inode, struct file *file)
{
    // printk("proc netflow_audit_policy_release,fileName:%s\n",file->f_path.dentry->d_iname);
    return 0;
}

static ssize_t netflow_audit_policy_write(struct file *file, const char *buff, size_t len, loff_t * off)
{
    // printk("proc netflow_audit_policy_write,fileName:%s\n",file->f_path.dentry->d_iname);
    
    return len;
}

static ssize_t netflow_audit_policy_read(struct file *file , char __user *buffer, size_t length, loff_t * offset)
{  
    int ret = 0;
    unsigned int count = 0;
    struct netflow_audit_policy *audit_policy = NULL;
    // 获取审计策略
    audit_policy = netflow_audit_policies(&count);
    if (audit_policy==NULL)
    {
        // printk("[netflow_drop_policy_read] audit_policy==NULL\n");
        return 0;
    }
    if (count > length / sizeof(struct netflow_audit_policy))
    {
        printk("[netflow_audit_policy_read] read buffer size not enough\n");
        if(audit_policy != NULL) kfree(audit_policy);
        return -1;
    }
    ret = copy_to_user(buffer, (char *)audit_policy, count*sizeof(struct netflow_audit_policy));
    if (ret !=0)
    {
        if(audit_policy != NULL) kfree(audit_policy);
        printk("[netflow_audit_policy_read] copy_to_user is failed!\n");
        return -2;
    }
    kfree(audit_policy);
    return count*sizeof(struct netflow_audit_policy);
}



static int netflow_audit_policy_log_open(struct inode *inode, struct file *file)
{
    // printk("proc netflow_audit_policy_log_open,fileName:%s\n",file->f_path.dentry->d_iname);
    return 0;
}
/*
** This function will be called when we close the procfs file
*/
static int netflow_audit_policy_log_release(struct inode *inode, struct file *file)
{
    // printk("proc netflow_audit_policy_log_release,fileName:%s\n",file->f_path.dentry->d_iname);
    return 0;
}

static ssize_t netflow_audit_policy_log_write(struct file *file, const char *buff, size_t len, loff_t * off)
{
    // printk("proc netflow_audit_policy_log_write,fileName:%s\n",file->f_path.dentry->d_iname);
    
    return len;
}



static ssize_t netflow_audit_policy_log_read(struct file *file , char __user *buffer, size_t length, loff_t * offset)
{  
    int ret = 0;
    unsigned int count = sizeof(struct netflow_audit_pkt);// 这个是审计策略日志结构体大小sizeof(xxxx)
    char *audit_policy_log = NULL;
#if USE_READ_BLOCK
    // 判断队列中数据是否为空,为空时进行切换进程休眠
    ret = wait_event_interruptible(audit_policy_log_wait_head,xc_list_pop(&audit_log,&audit_policy_log) == 0);
    if (ret)
    {
        printk("netflow_audit_policy_log_read wait_event_interruptible return ret: %d\n",ret);
        count = 0;
        goto out;
    }
#endif
    // count审计策略日志结构体大小,每次只返回一条数据
    if (!audit_policy_log) {
        return 0;
    }
    ret = copy_to_user(buffer,(char *)audit_policy_log, count);
    if (ret !=0)
    {
        printk("[netflow_audit_policy_log_read] copy_to_user is failed!\n");
        count = 0;
        goto out;
    }
  out:
    if(audit_policy_log != NULL) kfree(audit_policy_log);
    return count;
}


static int netflow_drop_policy_open(struct inode *inode, struct file *file)
{
    // printk("proc netflow_drop_policy_open,fileName:%s\n",file->f_path.dentry->d_iname);
    return 0;
}
/*
** This function will be called when we close the procfs file
*/
static int netflow_drop_policy_release(struct inode *inode, struct file *file)
{
    // printk("proc netflow_drop_policy_release,fileName:%s\n",file->f_path.dentry->d_iname);
    return 0;
}

static ssize_t netflow_drop_policy_write(struct file *file, const char *buff, size_t len, loff_t * off)
{
    // printk("proc netflow_drop_policy_write,fileName:%s\n",file->f_path.dentry->d_iname);
    
    return len;
}

static ssize_t netflow_drop_policy_read(struct file *file , char __user *buffer, size_t length, loff_t * offset)
{  
    int ret = 0;
    unsigned int count = 0;
    struct netflow_drop_policy *drop_policy = NULL;
    struct netflow_audit_policy *audit_policy = NULL;
    int i = 0;
    // 这里需要把netflow_audit_policy结构替换成netflow_drop_policy
    audit_policy = netflow_drop_policies(&count);
    if (audit_policy==NULL)
    {
        printk("[netflow_drop_policy_read] audit_policy==NULL\n");
        return 0;
    }
    drop_policy = kmalloc(count * sizeof(struct netflow_drop_policy ), GFP_ATOMIC);
    if (drop_policy == NULL) {
        kfree(audit_policy);
        printk("[netflow_drop_policy_read] drop_policy == NULL\n");
        return -1;
    }

    for(i = 0; i < count; i++)
    {
        drop_policy[i].direction = audit_policy[i].direction;
        drop_policy[i].tuple = audit_policy[i].tuple;
        drop_policy[i].policy_id = audit_policy[i].policy_id;
    }
    kfree(audit_policy);
    if (count > length / sizeof(struct netflow_drop_policy))
    {
        printk("[netflow_drop_policy_read] read buffer size not enough\n");
        kfree(drop_policy);
        return -1;        
    }
    ret = copy_to_user(buffer, (char *)drop_policy, count * sizeof(struct netflow_drop_policy));
    if (ret !=0)
    {
        printk("[netflow_drop_policy_read] copy_to_user is failed!\n");
        kfree(drop_policy);
        return -2;
    }
    kfree(drop_policy);
    return count * sizeof(struct netflow_drop_policy);
}


static int netflow_drop_policy_log_open(struct inode *inode, struct file *file)
{
    // printk("proc netflow_drop_policy_log_open,fileName:%s\n",file->f_path.dentry->d_iname);
    return 0;
}
/*
** This function will be called when we close the procfs file
*/
static int netflow_drop_policy_log_release(struct inode *inode, struct file *file)
{
    // printk("proc netflow_drop_policy_log_release,fileName:%s\n",file->f_path.dentry->d_iname);
    return 0;
}

static ssize_t netflow_drop_policy_log_write(struct file *file, const char *buff, size_t len, loff_t * off)
{
    // printk("proc netflow_drop_policy_log_write,fileName:%s\n",file->f_path.dentry->d_iname);
    
    return len;
}

static ssize_t netflow_drop_policy_log_read(struct file *file , char __user *buffer, size_t length, loff_t * offset)
{  
    int ret = 0;
    unsigned int count = sizeof(struct netflow_drop_log);
    char *drop_policy_log = NULL;
#if USE_READ_BLOCK
    // 判断队列中数据是否为空,为空时进行切换进程休眠
    ret = wait_event_interruptible(drop_policy_log_wait_head,xc_list_pop(&drop_log,&drop_policy_log) == 0);
    if (ret)
    {
        printk("netflow_drop_policy_log_read wait_event_interruptible return ret: %d\n",ret);
        count = 0;
        goto out;
    }
#endif
    if (!drop_policy_log) {
        return 0;
    }
    ret = copy_to_user(buffer, (char *)drop_policy_log, count);
    if (ret !=0)
    {
        printk("[netflow_drop_policy_log_read] copy_to_user is failed!\n");
        count = 0;
        goto out;
    }
  out:
    if (drop_policy_log != NULL) kfree(drop_policy_log);
    return count;
}

static int debug_log_open_proc(struct inode *inode, struct file *file)
{
    // pr_info("proc file opend.....\t");
    return 0;
}
/*
** This function will be called when we close the procfs file
*/
static int debug_log_release_proc(struct inode *inode, struct file *file)
{
    // pr_info("proc file released.....\n");
    return 0;
}

static ssize_t debug_log_write_proc(struct file *filp, const char *buff, size_t len, loff_t * off)
{
    // pr_info("proc file wrote.....\n");
    
    return len;
}

static ssize_t debug_log_read_proc(struct file *filp 
                            , char __user *buffer
                            , size_t length
                            , loff_t * offset)  
{  
    int len = 0;  
    char *txt = NULL;
    int ret = 0;

    ret = debug_log_queue_pop(&txt);
    if (!ret && *txt) {
        len = strlen(txt);
        ret = copy_to_user(buffer, txt, len);
        kfree(txt);
    }

    return len;  
}

static int init_proc(void)
{
    static struct file_operations proc_fops[] = {
        {
            .owner = THIS_MODULE,
            .open = netflow_audit_policy_open,
            .read = netflow_audit_policy_read,
            .write = netflow_audit_policy_write,
            .release = 	netflow_audit_policy_release,
        },
        {
            .owner = THIS_MODULE,
            .open = netflow_audit_policy_log_open,
            .read = netflow_audit_policy_log_read,
            .write = netflow_audit_policy_log_write,
            .release = 	netflow_audit_policy_log_release,
        },
        {
            .owner = THIS_MODULE,
            .open = netflow_drop_policy_open,
            .read = netflow_drop_policy_read,
            .write = netflow_drop_policy_write,
            .release = 	netflow_drop_policy_release,
        },
        {
            .owner = THIS_MODULE,
            .open = netflow_drop_policy_log_open,
            .read = netflow_drop_policy_log_read,
            .write = netflow_drop_policy_log_write,
            .release = 	netflow_drop_policy_log_release,
        },
        {
            .owner = THIS_MODULE,
            .open = debug_log_open_proc,
            .read = debug_log_read_proc,
            .write = debug_log_write_proc,
            .release = 	debug_log_release_proc,
        },
    };
    // 初始化队列头

#if USE_READ_BLOCK
    init_waitqueue_head(&audit_policy_log_wait_head);
    init_waitqueue_head(&drop_policy_log_wait_head);
#endif
    proc_netflow_root = proc_mkdir(proc_netflow_root_name, NULL);
    if (!proc_netflow_root) {
        printk("Cannot create directory under /proc/%s.",proc_netflow_root_name);
        return -1;
    }

    proc_netflow_audit_policy_flie = proc_create(proc_netflow_audit_policy_name, 0444, proc_netflow_root, &proc_fops[0]);
    if(!proc_netflow_audit_policy_flie)
    {
        printk("Cannot create file under /proc/%s/%s.",proc_netflow_root_name,proc_netflow_audit_policy_name);
        return -1;
    }

    
    proc_netflow_audit_policy_log_file = proc_create(proc_netflow_audit_policy_log_name, 0444, proc_netflow_root, &proc_fops[1]);
    if(!proc_netflow_audit_policy_log_file)
    {
        printk("Cannot create file under /proc/%s/%s.",proc_netflow_root_name,proc_netflow_audit_policy_log_name);
        return -1;
    }


    proc_netflow_drop_plicy_file = proc_create(proc_netflow_drop_policy_name, 0444, proc_netflow_root, &proc_fops[2]);
    if(!proc_netflow_drop_plicy_file)
    {
        printk("Cannot create file under /proc/%s/%s.",proc_netflow_root_name,proc_netflow_drop_policy_name);
        return -1;
    }

        
    proc_netflow_drop_policy_log_file = proc_create(proc_netflow_drop_policy_log_name, 0444, proc_netflow_root, &proc_fops[3]);
    if(!proc_netflow_drop_policy_log_file)
    {
        printk("Cannot create file under /proc/%s/%s.",proc_netflow_root_name,proc_netflow_drop_policy_log_name);
        return -1;
    }

    proc_debug_log_file = proc_create(proc_debug_log_name, 0444, proc_netflow_root, &proc_fops[4]);
    if(!proc_debug_log_file)
    {
        printk("Cannot create file under /proc/%s/%s.",proc_netflow_root_name,proc_debug_log_name);
        return -1;
    }


    printk("init_proc success.\n");  

    return 0;
}

static int exit_proc(void)
{

    if (proc_netflow_audit_policy_flie)
    {
        remove_proc_entry(proc_netflow_audit_policy_name, proc_netflow_root);
    }
    if (proc_netflow_audit_policy_log_file)
    {
        remove_proc_entry(proc_netflow_audit_policy_log_name, proc_netflow_root);
    }
    if (proc_netflow_drop_plicy_file)
    {
        remove_proc_entry(proc_netflow_drop_policy_name, proc_netflow_root);
    }
    if (proc_netflow_drop_policy_log_file)
    {
        remove_proc_entry(proc_netflow_drop_policy_log_name, proc_netflow_root);
    }
    if (proc_debug_log_file) {
        remove_proc_entry(proc_debug_log_name, proc_netflow_root);
    }

    if (proc_netflow_root)
    {
        remove_proc_entry(proc_netflow_root_name, NULL);
    }

    return 0;
}

// 填充审计日志
int fill_audit_log(struct netflow_audit_pkt *audit_pkt
, const struct net_audit_param *audit_param)
{
#if USE_DEBUG_AUDIT
    char txt[123];
#endif

    // 审计
    char payload[256 + 1] = {0};
    struct sk_buff *skb = audit_param->param3;

    int data_len = (int)(unsigned long)audit_param->param4;
    int transport_offset = (int)(unsigned long)audit_param->param5;
    if (data_len <= 0) {
#if USE_DEBUG_AUDIT
        snprintf(txt, sizeof (txt) - 1, "Payload less than or equal 0! len:%d\n", data_len);
        debug_log_queue_push_add_title(txt);
#endif
        return -1;
    }
    data_len = ((data_len <= sizeof (audit_pkt->payload)) ? (data_len) : (sizeof (audit_pkt->payload)));

#if 1
    int ret = skb_copy_bits(skb, transport_offset, &payload, data_len);
    if (ret) {
        debug_log_queue_push_add_title("ERROR: skb_copy_bits failed!\n");
        return -1;
    }

    payload[data_len] = 0;
#else
    strcpy(payload, "audit test.");
#endif

#if USE_DEBUG_AUDIT
    snprintf(txt, sizeof (txt) - 1, "data_len:%d capture: %s\n", data_len, payload);
    debug_log_queue_push_add_title(txt);
#endif

    // 保存审计日志  通过开关开始是否输出日志到队列
    memcpy(&audit_pkt->tuple, &audit_param->tuple, sizeof (struct net_five_tuple));
    audit_pkt->pkt_len = (int)(unsigned long)audit_param->param4;
    memset(audit_pkt->payload, 0, sizeof (audit_pkt->payload));
    memcpy(audit_pkt->payload, payload, data_len);
    audit_pkt->direction = audit_param->dir;

    return 0;
}

// 验证五元组
static int verify_tuple(const struct netflow_audit_policy *policy
                        , const struct net_audit_param *audit_param)
{
#if USE_DEBUG_VERIFY_TUPLE
    char txt[128];
#endif

    if (IP_VERSION_IPV4 == audit_param->tuple.ip_version) {
        // 验证源IP
        u32 net_ip   = audit_param->tuple.src_ip_range.start.ipv4.s_addr;
        u32 audit_ip = policy->tuple.src_ip_range.start.ipv4.s_addr;
#if USE_DEBUG_VERIFY_TUPLE
        snprintf(txt, sizeof (txt), "verify_tuple: ipv4 src start: net_ip:0x%x audit_ip:0x%x\n"
        , net_ip, audit_ip);
        debug_log_queue_push_add_title(txt);
#endif
        VERIFY_LEFT_LIMIT(net_ip, audit_ip);

        audit_ip    = policy->tuple.src_ip_range.end.ipv4.s_addr;
#if USE_DEBUG_VERIFY_TUPLE
        snprintf(txt, sizeof (txt), "verify_tuple: ipv4 src end: net_ip:0x%x audit_ip:0x%x\n"
        , net_ip, audit_ip);
        debug_log_queue_push_add_title(txt);
#endif
        VERIFY_RIGHT_LIMIT(net_ip, audit_ip);

        // 验证目的IP
        net_ip      = audit_param->tuple.dst_ip_range.start.ipv4.s_addr;
        audit_ip    = policy->tuple.dst_ip_range.start.ipv4.s_addr;
#if USE_DEBUG_VERIFY_TUPLE
        snprintf(txt, sizeof (txt), "verify_tuple: ipv4 dst start: net_ip:0x%x audit_ip:0x%x\n"
        , net_ip, audit_ip);
        debug_log_queue_push_add_title(txt);
#endif
        VERIFY_LEFT_LIMIT(net_ip, audit_ip);

        audit_ip    = policy->tuple.dst_ip_range.end.ipv4.s_addr;
#if USE_DEBUG_VERIFY_TUPLE
        snprintf(txt, sizeof (txt), "verify_tuple: ipv4 dst end: net_ip:0x%x audit_ip:0x%x\n"
        , net_ip, audit_ip);
        debug_log_queue_push_add_title(txt);
#endif
        VERIFY_RIGHT_LIMIT(net_ip, audit_ip);
    }
    else if (IP_VERSION_IPV6 == audit_param->tuple.ip_version) {
        // 验证源IP
        const struct in6_addr *net_ip   = &audit_param->tuple.src_ip_range.start.ipv6;
        const struct in6_addr *audit_ip = &policy->tuple.src_ip_range.start.ipv6;
        VERIFY_IPV6_LEFT_LIMIT(net_ip, audit_ip);

        audit_ip = &policy->tuple.src_ip_range.end.ipv6;
        VERIFY_IPV6_RIGHT_LIMIT(net_ip, audit_ip);

        // 验证目的IP
        net_ip   = &audit_param->tuple.dst_ip_range.start.ipv6;
        audit_ip = &policy->tuple.dst_ip_range.start.ipv6;
        VERIFY_IPV6_LEFT_LIMIT(net_ip, audit_ip);

        audit_ip = &policy->tuple.dst_ip_range.end.ipv6;
        VERIFY_IPV6_RIGHT_LIMIT(net_ip, audit_ip);
    }
    else {
        return -1;
    }

    // 验证源端口
    u16 net_port   = audit_param->tuple.src_port_range.start;
    u16 audit_port = policy->tuple.src_port_range.start;
#if USE_DEBUG_VERIFY_TUPLE
        snprintf(txt, sizeof (txt), "verify_tuple: src start: net_port:%u audit_port:%u\n"
        , net_port, audit_port);
        debug_log_queue_push_add_title(txt);
#endif
    VERIFY_LEFT_LIMIT(net_port, audit_port);

    audit_port  = policy->tuple.src_port_range.end;
#if USE_DEBUG_VERIFY_TUPLE
        snprintf(txt, sizeof (txt), "verify_tuple: src end: net_port:%u audit_port:%u\n"
        , net_port, audit_port);
        debug_log_queue_push_add_title(txt);
#endif
    VERIFY_RIGHT_LIMIT(net_port, audit_port);

    // 验证目的端口
    net_port   = audit_param->tuple.dst_port_range.start;
    audit_port = policy->tuple.dst_port_range.start;
#if USE_DEBUG_VERIFY_TUPLE
        snprintf(txt, sizeof (txt), "verify_tuple: dst start: net_port:%u audit_port:%u\n"
        , net_port, audit_port);
        debug_log_queue_push_add_title(txt);
#endif
    VERIFY_LEFT_LIMIT(net_port, audit_port);

    audit_port  = policy->tuple.dst_port_range.end;
#if USE_DEBUG_VERIFY_TUPLE
        snprintf(txt, sizeof (txt), "verify_tuple: dst end: net_port:%u audit_port:%u\n"
        , net_port, audit_port);
        debug_log_queue_push_add_title(txt);
#endif
    VERIFY_RIGHT_LIMIT(net_port, audit_port);

#if USE_DEBUG_VERIFY_TUPLE
    debug_log_queue_push_add_title("verify_tuple: ok.\n\n");
#endif

    return 0;
}

// 五元组审计
static int audit_tuple(const char *data, char *opt)
{
    const struct netflow_audit_policy *policy = (const struct netflow_audit_policy *)data;
    struct net_audit_param *audit_param = (struct net_audit_param *)opt;

    // 不处理进程
    if (NETFLOW_PROCESS == policy->type) {
        return -1;
    }

    // 验证五元组
    int ret = verify_tuple(policy, audit_param);
    if(ret) {
        return ret;
    }

    // 设置返回值
    audit_param->ret = 0;

    // 填充审计日志
    if (!g_netflow_audit_log_flag) {
        return 0; // 返回0, 结束链表遍历
    }

    struct netflow_audit_pkt *audit_pkt = kmalloc(sizeof(struct netflow_audit_pkt), GFP_ATOMIC);
    if(!audit_pkt) {
        printk(KERN_WARNING "[net_audit_module] Failed to allocate memory.\n");
        return 0; // 返回0, 结束链表遍历
    };

    ret = fill_audit_log(audit_pkt, audit_param);
    if (ret) {
        kfree(audit_pkt);
        return 0; // 返回0, 结束链表遍历
    }
    audit_pkt->policy_id = policy->policy_id;
    audit_pkt->pid = -1;
    audit_pkt->process_name[0] = 0;
    ret = xc_list_push_shallow_copy(&audit_log, audit_pkt);
    if (ret) {
        kfree(audit_pkt);
    }

#if USE_READ_BLOCK
    // 唤醒进程
    wake_up_interruptible(&audit_policy_log_wait_head);
#endif

    return 0;
}

// 验证进程审计策略
int verify_proc(const struct netflow_audit_proc_policy *policy
, const struct net_audit_param *audit_param)
{
    uint16_t net_port = 0;
    if (NETFLOW_INBOUND == audit_param->dir) {
        net_port = audit_param->tuple.dst_port_range.start;
    }
    else if (NETFLOW_OUTBOUND == audit_param->dir) {
        net_port = audit_param->tuple.src_port_range.start;
    }
    else {
        // 不会出现这种情况
        debug_log_queue_push_add_title("Error:abnormal flow direction!\n");
        return -1;
    }

    if (net_port != policy->port) {
        return -1;
    }

    if (audit_param->tuple.protocol != policy->protocol) {
        return -1;
    }

    return 0;
}

// 进程审计
static int audit_proc(const char *data, char *opt)
{
    const struct netflow_audit_proc_policy *policy = (const struct netflow_audit_proc_policy *)data;
    struct net_audit_param *audit_param = (struct net_audit_param *)opt;
    int ret = 0;

    // 验证策略
    ret = verify_proc(policy, audit_param);
    if (ret) {
        return -1;
    }

    // 设置返回值
    audit_param->ret = 0;

    // 填充审计日志
    if (!g_netflow_audit_log_flag) {
        return 0;  // 返回0, 结束链表遍历
    }

    struct netflow_audit_pkt *audit_pkt = kmalloc(sizeof(struct netflow_audit_pkt), GFP_ATOMIC);
    if(!audit_pkt) {
        printk(KERN_WARNING "[net_audit_module] Failed to allocate memory.\n");
        return 0; // 返回0, 结束链表遍历
    };

    ret = fill_audit_log(audit_pkt, audit_param);
    if (ret) {
        kfree(audit_pkt);
        return 0; // 返回0, 结束链表遍历
    }
    audit_pkt->policy_id = policy->policy_id;
    audit_pkt->pid = policy->pid;
    memcpy(audit_pkt->process_name, policy->process_name
    , sizeof (policy->process_name));
    xc_list_push_shallow_copy(&audit_log, audit_pkt);
#if USE_READ_BLOCK
    // 唤醒进程
    wake_up_interruptible(&audit_policy_log_wait_head);
#endif

    return 0;
}

// 阻断
static int drop(const char *data, char *opt)
{
#if USE_DEBUG_DROP
    char txt[128];
#endif
    const struct netflow_audit_policy *policy = (const struct netflow_audit_policy *)data;
    struct net_audit_param *audit_param = (struct net_audit_param *)opt;

    // 验证五元组
    int ret = verify_tuple(policy, audit_param);
    if(ret) {
        return ret;
    }

    // 保存阻断日志 通过日志开关进行设置
    if(g_netflow_drop_log_flag !=0)
    {
        int data_len = (int)(unsigned long)audit_param->param4;
        struct netflow_drop_log *drop_pkt = kmalloc(sizeof(struct netflow_drop_log), GFP_ATOMIC);
        if(!drop_pkt) {
            printk(KERN_WARNING "[net_audit_module] Failed to allocate memory.\n");
            return -1;
        };

        memcpy(&drop_pkt->tuple_info, &audit_param->tuple, sizeof (struct net_five_tuple));
        drop_pkt->pkt_len = data_len;
        drop_pkt->direction = audit_param->dir;
        drop_pkt->policy_id = policy->policy_id;
        drop_pkt->timestamp = ktime_get_real();
        drop_pkt->pid = -1;
        drop_pkt->process_name[0] = 0;
        ret = xc_list_push_shallow_copy(&drop_log, drop_pkt);
        if (ret) {
            kfree(drop_pkt);
        }
        // 唤醒进程
#if USE_READ_BLOCK
        wake_up_interruptible(&drop_policy_log_wait_head);
#endif
#if USE_DEBUG_AUDIT
        snprintf(txt, sizeof (txt) - 1, "Drop successfully, data_len:%d\n", data_len);
        debug_log_queue_push_add_title(txt);
#endif
    }

    // 设置返回值
    audit_param->ret = 0;

    return 0;
}

// 提取五元组信息
int get_tuple(struct sk_buff *skb, struct iphdr **iph, struct ipv6hdr **iph6
            , struct tcphdr **tcph, struct udphdr **udph, struct policy_table_id *table_id
            , struct net_audit_param *audit_param, const netflow_dir_t dir, const ip_version_t ip_version)
{
#if (USE_DEBUG_GET_TUPLE_HEAD | USE_DEBUG_GET_TUPLE_TCP | USE_DEBUG_GET_TUPLE_UDP)
    char txt[128];
#endif

    int is_ipv4 = 0;
    int is_ipv6 = 0;
    unsigned int ipv4_head_len = 0;
    unsigned int ipv6_head_len = sizeof(struct ipv6hdr); // 40

    // 提取IP报首
    audit_param->tuple.ip_version = ip_version;
    if (IP_VERSION_IPV4 == ip_version) {
        *iph = ip_hdr(skb);
        if(!(*iph)) {
#if USE_DEBUG_GET_TUPLE_HEAD
            snprintf(txt, sizeof (txt) - 1, "Warn: get ip head failed!\n");
            debug_log_queue_push_add_title(txt);
#endif
            return -1;
        }

        is_ipv4 = 1;
        ipv4_head_len = (*iph)->ihl * 4;
        audit_param->tuple.ip_version = IP_VERSION_IPV4;
        audit_param->tuple.src_ip_range.start.ipv4.s_addr = ntohl((*iph)->saddr);
        audit_param->tuple.dst_ip_range.start.ipv4.s_addr = ntohl((*iph)->daddr);
#if USE_DEBUG_GET_TUPLE_HEAD
        snprintf(txt, sizeof (txt) - 1, "get_tuple: ipv4, s_addr:0x%x daddr:0x%x\n"
        , ntohl((*iph)->saddr), ntohl((*iph)->daddr));
        debug_log_queue_push_add_title(txt);
#endif
    }
    else if (IP_VERSION_IPV6 == ip_version) {
        *iph6 = ipv6_hdr(skb);
        if (!(*iph6)) {
#if USE_DEBUG_GET_TUPLE_HEAD
            snprintf(txt, sizeof (txt) - 1, "Warn: get ip head failed!\n");
            debug_log_queue_push_add_title(txt);
#endif
            return -1;
        }
        is_ipv6 = 1;
        audit_param->tuple.ip_version = IP_VERSION_IPV6;
        memcpy(&audit_param->tuple.src_ip_range.start.ipv6
        , &((*iph6)->saddr), sizeof ((*iph6)->saddr));
        memcpy(&audit_param->tuple.dst_ip_range.start.ipv6
        , &((*iph6)->daddr), sizeof ((*iph6)->daddr));
        ipv6_endianness_conversion(&audit_param->tuple.src_ip_range.start.ipv6);
        ipv6_endianness_conversion(&audit_param->tuple.dst_ip_range.start.ipv6);
    }
    else {
#if USE_DEBUG_GET_TUPLE_HEAD
        snprintf(txt, sizeof (txt) - 1, "Warn: abnormal IP version:%d!\n", ip_version);
        debug_log_queue_push_add_title(txt);
#endif
        return -1;
    }

    // 得到协议类型
    int is_tcp = 0;
    int is_udp = 0;

    if (is_ipv4) {
        is_tcp = (IPPROTO_TCP == (*iph)->protocol);
        is_udp = (IPPROTO_UDP == (*iph)->protocol);
    }
    else if (is_ipv6) {
        is_tcp = (NEXTHDR_TCP == (*iph6)->nexthdr);
        is_udp = (NEXTHDR_UDP == (*iph6)->nexthdr);
    }
    else {
#if USE_DEBUG_GET_TUPLE_TCP
        snprintf(txt, sizeof (txt) - 1, "Warn: Unknown IP version!\n");
        debug_log_queue_push_add_title(txt);
#endif
        return -1;
    }

    // 只处理TCP和UDP
    if (!is_tcp && !is_udp) {
        return -1;
    }

    // 设置传输层报头偏移
    if (is_ipv4) {
        skb_set_transport_header(skb, ipv4_head_len);
    }
    else {
        skb_set_transport_header(skb, ipv6_head_len);
    }

    // 提取IP和端口，以及传输层报头
    if (is_tcp) {
        (*tcph) = tcp_hdr(skb);
        if (!(*tcph)) {
#if USE_DEBUG_GET_TUPLE_TCP
            snprintf(txt, sizeof (txt) - 1, "Warn: get tcp head failed!\n");
            debug_log_queue_push_add_title(txt);
#endif
            return -1;
        }

        // 过滤空包
        int iph_len = (is_ipv4 ? ipv4_head_len : ipv6_head_len);
        int tcp_head_len = (*tcph)->doff * 4;
        int data_len = skb->len - iph_len - tcp_head_len;
        if (data_len <= 0) {
            return -1;
        }

        audit_param->tuple.protocol = IPPROTO_TCP;
        audit_param->tuple.src_port_range.start = ntohs((*tcph)->source);
        audit_param->tuple.dst_port_range.start = ntohs((*tcph)->dest);
        audit_param->param2 = (*tcph);
        audit_param->param4 = (void *)(unsigned long)data_len;
        audit_param->param5 = (void *)(unsigned long)(iph_len + tcp_head_len);
#if USE_DEBUG_GET_TUPLE_TCP
        snprintf(txt, sizeof (txt) - 1, "get_tuple: tcp, src_port:%u dst_port:%u\n"
        , ntohs((*tcph)->source), ntohs((*tcph)->dest));
        debug_log_queue_push_add_title(txt);
#endif
    }
    else if (is_udp) {
        (*udph) = udp_hdr(skb);
        if (!(*udph)) {
#if USE_DEBUG_GET_TUPLE_UDP
            snprintf(txt, sizeof (txt) - 1, "Warn: get udp head failed!\n");
            debug_log_queue_push_add_title(txt);
#endif
            return -1;
        }

        // 过滤空包
        int iph_len = (is_ipv4 ? ipv4_head_len : ipv6_head_len);
        int udp_head_len = sizeof (struct udphdr);
        int data_len = skb->len - iph_len - udp_head_len;
        if (data_len <= 0) {
            return -1;
        }

        audit_param->tuple.protocol = IPPROTO_UDP;
        audit_param->tuple.src_port_range.start = ntohs((*udph)->source);
        audit_param->tuple.dst_port_range.start = ntohs((*udph)->dest);
        audit_param->param2 = (*udph);
        audit_param->param4 = (void *)(unsigned long)data_len;
        audit_param->param5 = (void *)(unsigned long)(iph_len + udp_head_len);
#if USE_DEBUG_GET_TUPLE_UDP
        snprintf(txt, sizeof (txt) - 1, "get_tuple: udp, src_port:%u dst_port:%u\n"
        , ntohs((*udph)->source), ntohs((*udph)->dest));
        debug_log_queue_push_add_title(txt);
#endif
    }
    else {
        return -1;
    }

    table_id->direction = dir;
    table_id->protocol = ((IPPROTO_TCP == audit_param->tuple.protocol) ? (PROTOCOL_TCP) : (PROTOCOL_UDP));
    table_id->ip_version = audit_param->tuple.ip_version;
    audit_param->param1 = (is_ipv4 ? ((void *)(*iph)) : ((void *)(*iph6)));
    audit_param->param3 = skb;
    audit_param->dir = dir;

#if (USE_DEBUG_GET_TUPLE_TCP | USE_DEBUG_GET_TUPLE_UDP)
    snprintf(txt, sizeof (txt) - 1, "get_tuple: table_id: protocol:%d direction:%d ip_version:%d\n"
    , table_id->protocol, table_id->direction, table_id->ip_version);
    debug_log_queue_push_add_title(txt);
#endif

    return 0;
}

int is_enable_audit_enable(void)
{
    return atomic_read(&audit_proc_heartbeat.enable_audit_proc);
}

static unsigned int packet_filter(void *priv,
			       struct sk_buff *skb,
			       const struct nf_hook_state *state
                   , const ip_version_t ip_version)
{
    // IPv4协议头部
    struct iphdr *iph       = NULL;
    // IPv6协议头部
    struct ipv6hdr *iph6    = NULL;
    // tcp协议头部
    struct tcphdr *tcph     = NULL;
    // udp协议头部
    struct udphdr *udph     = NULL;

    // 如socket buffer指针为空，则当前hook点允许该数据包继续在协议栈中流转。 
    if(skb == NULL) {
        return NF_ACCEPT;
    }
    
    struct policy_table_id table_id = {0};
    struct net_audit_param audit_param = {0};
    netflow_dir_t dir = NETFLOW_INBOUND;
    if (NF_INET_POST_ROUTING == (unsigned long)priv) {
#if USE_DEBUG_NET_HOOK
        debug_log_queue_push_add_title("hook: NF_INET_POST_ROUTING.\n");
#endif
        dir = NETFLOW_OUTBOUND;
    }
    
    // 提取五元组信息
    int ret = get_tuple(skb, &iph, &iph6, &tcph, &udph
                        , &table_id, &audit_param, dir, ip_version);
    if (ret) {
        return NF_ACCEPT;
    }

    // 遍历审计策略表
    if (g_netflow_audit_log_flag) {
        // 遍历五元组策略（单方向）
        audit_param.ret = -1;
        table_id.direction = dir;
        netflow_audit_policies_tables_for_each(&table_id
                                                , audit_tuple
                                                , &audit_param);

        // 遍历五元组策略（全方向）
        if (audit_param.ret) {
            table_id.direction = NETFLOW_ALL;
            netflow_audit_policies_tables_for_each(&table_id
                                                , audit_tuple
                                                , &audit_param);
        }

        // 遍历进程策略（单方向）
        if (audit_param.ret && is_enable_audit_enable()) {
            netflow_audit_proc_policies_tables_for_each(dir
                                                        , audit_proc
                                                        , &audit_param);
        }

        // 遍历进程策略（全方向）
        if (audit_param.ret && is_enable_audit_enable()) {
            netflow_audit_proc_policies_tables_for_each(NETFLOW_ALL
                                                        , audit_proc
                                                        , &audit_param);
        }
    }

    // 遍历阻断策略表（单方向）
    audit_param.ret = -1;
    table_id.direction = dir;
    netflow_drop_policies_tables_for_each(&table_id
                                            , drop
                                            , &audit_param);

    //  遍历阻断策略表（全方向）
    if (audit_param.ret) {
        table_id.direction = NETFLOW_ALL;
        netflow_drop_policies_tables_for_each(&table_id
                                                , drop
                                                , &audit_param);
    }

    if (audit_param.ret) {
        return NF_ACCEPT;
    }
    else {
        return NF_DROP;
    }
};

static unsigned int packet_filter_ipv6(void *priv,
			       struct sk_buff *skb,
			       const struct nf_hook_state *state)
{
    return packet_filter(priv, skb, state, IP_VERSION_IPV6);
}

static unsigned int packet_filter_ipv4(void *priv,
			       struct sk_buff *skb,
			       const struct nf_hook_state *state)
{
    return packet_filter(priv, skb, state, IP_VERSION_IPV4);
}

// 复位进程审计剩余时间
static int reset_audit_proc_time_remain(void)
{
    atomic_set(&audit_proc_heartbeat.time_remain
    , audit_proc_heartbeat.timeout / audit_proc_heartbeat.timeperiod);

    return 0;
}

// 开启进程审计
static int start_audit_proc(void)
{
    reset_audit_proc_time_remain();
    atomic_set(&audit_proc_heartbeat.enable_audit_proc, 1);

    return 0;
}

// 关闭进程审计
static int close_audit_proc(void)
{
    reset_audit_proc_time_remain();
    atomic_set(&audit_proc_heartbeat.enable_audit_proc, 0);

    return 0;
}

// 开启进程审计心跳定时器
static int start_audit_proc_heartbeat_timer(void)
{
    mod_timer(&audit_proc_heartbeat.timer
    , jiffies + msecs_to_jiffies(audit_proc_heartbeat.timeperiod));

    return 0;
}

/* 心跳定时器回调函数 */
void heartbeat_timer_function(struct timer_list *timer)
{	
    (void) timer;

#if USE_DEBUG_HEARTBEAT
    // debug_log_queue_push_add_title("timer timeout.\n");
#endif

    int time_remain = atomic_dec_return(&audit_proc_heartbeat.time_remain);
    if (0 >= time_remain) {
#if USE_DEBUG_HEARTBEAT
    debug_log_queue_push_add_title("close audit proc!\n");
#endif
        close_audit_proc();
    }

	/* 重启定时器 */
	start_audit_proc_heartbeat_timer();
}

// 初始化进程审计心跳定时器
static int init_audit_proc_heartbeat_timer(void)
{
    timer_setup(&audit_proc_heartbeat.timer, heartbeat_timer_function, 0);
    audit_proc_heartbeat.timeperiod = 100;
    audit_proc_heartbeat.timeout    = 2000;
    close_audit_proc();
    
    // 开启定时器
    start_audit_proc_heartbeat_timer();

    return 0;
}

static int exit_audit_proc_heartbeat_timer(void)
{
    del_timer_sync(&audit_proc_heartbeat.timer);		/* 删除timer */

    return 0;
}

//初始化hook参数
static struct nf_hook_ops net_flow_opt[] = {
{
        .hook = (nf_hookfn *)packet_filter_ipv4,
        .priv = (void *)NF_INET_PRE_ROUTING,
        .pf = NFPROTO_IPV4,
        .hooknum = NF_INET_PRE_ROUTING,
        .priority = NF_IP_PRI_FIRST,
},

{
        .hook = (nf_hookfn *)packet_filter_ipv4,
        .priv = (void *)NF_INET_POST_ROUTING,
        .pf = NFPROTO_IPV4,
        .hooknum = NF_INET_POST_ROUTING,
        .priority = NF_IP_PRI_FIRST,
},

{
        .hook = (nf_hookfn *)packet_filter_ipv6,
        .priv = (void *)NF_INET_PRE_ROUTING,
        .pf = NFPROTO_IPV6,
        .hooknum = NF_INET_PRE_ROUTING,
        .priority = NF_IP_PRI_FIRST,
},

{
        .hook = (nf_hookfn *)packet_filter_ipv6,
        .priv = (void *)NF_INET_POST_ROUTING,
        .pf = NFPROTO_IPV6,
        .hooknum = NF_INET_POST_ROUTING,
        .priority = NF_IP_PRI_FIRST,
}
};

static int simple_nf_init(void)
{   
    //注册函数钩子，内核版本大于4.3.0的内核函数API为nf_register_net_hook，否则为nf_register_hook
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,3,0) 
    //int nf_register_net_hook(struct net *net, const struct nf_hook_ops *reg)
    //网络系统在初始化的时候会初始化一个初始网络命名空间，即init_net命名空间。后续创建的net namespace命名空间会和init_net一起通过list项组织起来，且每个网络设备都对应一个命名空间
    for (int i = 0; i < 4; ++i) {
        nf_register_net_hook(&init_net, &net_flow_opt[i]);
    }
#else
    for (int i = 0; i < 4; ++i) {
        nf_register_hook(&net_flow_opt[i]);
    }
#endif

    const int log_size_max = 512;
    debug_log_queue_init(log_size_max);
    xc_list_init(&audit_log, log_size_max); // 审计日志初始化
    xc_list_set_name(&audit_log, "audit_log");
    xc_list_init(&drop_log, log_size_max); // 阻断日志初始化
    xc_list_set_name(&drop_log, "drop_log");
#if USE_FILE_OPT
    file_opt_init(&file_fd, "/tmp/test_capture");
#endif
    init_proc();
    netflow_audit_init();
    netflow_drop_init();
    init_audit_proc_heartbeat_timer();

    // test
#if USE_SELF_TEST
    test_netflow_audit_add_policy();
    test_netflow_drop_add_policy();
    test_netflow_enable(&g_netflow_audit_log_flag);
    test_netflow_enable(&g_netflow_drop_log_flag);
#endif

    // 注册设备
    alloc_chrdev_region(&device.devid, 0, DEVICE_CNT, DEVICE_NAME);	/* 申请设备号 */
    device.major = MAJOR(device.devid);	/* 获取分配号的主设备号 */
    device.minor = MINOR(device.devid);	/* 获取分配号的次设备号 */

    /* 2、初始化cdev */
	device.cdev.owner = THIS_MODULE;
	cdev_init(&device.cdev, &net_flow_fops);
	
	/* 3、添加一个cdev */
	cdev_add(&device.cdev, device.devid, DEVICE_CNT);

	/* 4、创建类 */
	device.class = class_create(THIS_MODULE, DEVICE_NAME);
	if (IS_ERR(device.class)) {
		return PTR_ERR(device.class);
	}

	/* 5、创建设备 */
	device.device = device_create(device.class, NULL, device.devid, NULL, DEVICE_NAME);
	if (IS_ERR(device.device)) {
		return PTR_ERR(device.device);
	}

    printk("[net_audit_module] network hooks success.\n");

    return 0;
 
};

static void simple_nf_exit(void)
{
    //卸载hook函数
#if LINUX_VERSION_CODE >= KERNEL_VERSION(4,3,0)
    for (int i = 0; i < 4; ++i) {
        nf_unregister_net_hook(&init_net, &net_flow_opt[i]);
    }
#else
    for (int i = 0; i < 4; ++i) {
        nf_unregister_hook(&net_flow_opt[i]);
    }
#endif

    // 卸载定时器
    exit_audit_proc_heartbeat_timer();

    /* 注销字符设备驱动 */
	cdev_del(&device.cdev);/*  删除cdev */
	unregister_chrdev_region(device.devid, DEVICE_CNT); /* 注销设备号 */

	device_destroy(device.class, device.devid);
	class_destroy(device.class);

    exit_proc();
#if USE_FILE_OPT
    file_opt_destroy(&file_fd);
#endif
    printk("[net_audit_module] remove hook lkm success!\n");
};

//内核模块加载和卸载时，执行的函数
// module_init(simple_nf_init);
// module_exit(simple_nf_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("zkfd");

static int __init _simple_nf_init(void) 
{
    return simple_nf_init();
}

postcore_initcall(_simple_nf_init);