/**
 * 网络阻断
 */

#ifndef __XC_NET_DROP_H
#define __XC_NET_DROP_H

#include "base_types.h"
#include "list.h"

/**
 * @brief 初始化网络阻断
 * 
 * @return 0：成功 -1：失败
 */
int netflow_drop_init(void);

/**
 * @brief 添加基于进程或五元组的网络流量阻断策略
 * 
 * @param policy 
 * @return 0：成功 -1：失败
 */
int netflow_drop_add_policy(struct netflow_audit_policy *policy);

/**
 * @brief 删除（单条）网络流量阻断策略
 * 
 * @param policy_id 策略编号
 * @return 0：成功 -1：失败 
 */
int netflow_drop_delete_policy(uint16_t policy_id);

/**
 * @brief 清除所有网络流量阻断策略
 * 
 * @return 0：成功 -1：失败 
 */
int netflow_drop_clear_policy(void);

/**
 * @brief 查询网络流量阻断所有策略
 * 
 * @param count 返回的策略个数
 * @return struct netflow_audit_policy* 
 */
struct netflow_audit_policy *netflow_drop_policies(int *count);

/**
 * @brief 策略表遍历，用在netfilter回调函数
 * @param table_id 表ID
 * @param fun 处理数据的函数
 * @param arg fun的参数
 * @return 0：成功 -1：失败 
 */
int netflow_drop_policies_tables_for_each(struct policy_table_id *table_id
                                            , xc_list_for_each_fun fun
                                            , void *arg);

#endif
