
#include "test.h"
#include "debug_log_queue.h"
#include "util.h"
#include <linux/inet.h>


int test_netflow_audit_add_policy(void)
{
    static int count = 0;
    struct netflow_audit_policy policy;
    int ret = 0;

#if 0

    /* 添加TCP审计策略 -- IPv4 */
    policy.policy_id = ++count;
    policy.direction = NETFLOW_INBOUND;             // 流入
    policy.type = NETFLOW_TUPLE;                    // 五元组
    policy.tuple.src_ip_range.start.ipv4.s_addr = 0x28280d2f; // 主机字节序（小端）40.40.13.47
    policy.tuple.src_ip_range.end.ipv4.s_addr   = 0x28280d33; // 主机字节序（小端）40.40.13.51
    policy.tuple.dst_ip_range.start.ipv4.s_addr = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.dst_ip_range.end.ipv4.s_addr   = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.src_port_range.start           = 8888;
    policy.tuple.src_port_range.end             = 8888;
    policy.tuple.dst_port_range.start           = 0;
    policy.tuple.dst_port_range.end             = 0xffff;
    policy.tuple.protocol                       = IPPROTO_TCP;
    policy.tuple.ip_version                     = IP_VERSION_IPV4;

    ret = netflow_audit_add_policy(&policy);
    if (ret) {
        return ret;
    }

    policy.policy_id = ++count;
    policy.direction = NETFLOW_OUTBOUND;            // 流出
    policy.type = NETFLOW_TUPLE;                    // 五元组
    policy.tuple.src_ip_range.start.ipv4.s_addr = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.src_ip_range.end.ipv4.s_addr   = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.dst_ip_range.start.ipv4.s_addr = 0x28280d2f; // 主机字节序（小端）40.40.13.47
    policy.tuple.dst_ip_range.end.ipv4.s_addr   = 0x28280d33; // 主机字节序（小端）40.40.13.51
    policy.tuple.src_port_range.start           = 0;
    policy.tuple.src_port_range.end             = 0xffff;
    policy.tuple.dst_port_range.start           = 8888;
    policy.tuple.dst_port_range.end             = 8888;

    ret = netflow_audit_add_policy(&policy);
    if (ret) {
        return ret;
    }

    /* 添加TCP审计策略 -- IPv6 */
    struct in6_addr ipv6_saddr;
    const char ipv6_addr_str[] = "2001:da8:e000::1:1:1";
    // const char ipv6_addr_str[] = "::1";
    ret = in6_pton(ipv6_addr_str, -1
    , ipv6_saddr.s6_addr, -1, NULL);
    if (1 != ret) {
        debug_log_queue_push_add_title("Error: Failed to convert IPv6.\n");
        return ret;
    }

    ipv6_endianness_conversion(&ipv6_saddr);

    memcpy(&policy.tuple.src_ip_range.start.ipv6
    , &ipv6_saddr, sizeof (struct in6_addr));
    memcpy(&policy.tuple.src_ip_range.end.ipv6
    , &ipv6_saddr, sizeof (struct in6_addr));
    memcpy(&policy.tuple.dst_ip_range.start.ipv6
    , &ipv6_saddr, sizeof (struct in6_addr));
    memcpy(&policy.tuple.dst_ip_range.end.ipv6
    , &ipv6_saddr, sizeof (struct in6_addr));

    policy.policy_id = ++count;
    policy.direction = NETFLOW_INBOUND;             // 流入
    policy.type = NETFLOW_TUPLE;                    // 五元组
    policy.tuple.src_port_range.start           = 9999;
    policy.tuple.src_port_range.end             = 9999;
    policy.tuple.dst_port_range.start           = 0;
    policy.tuple.dst_port_range.end             = 0xffff;
    policy.tuple.protocol                       = IPPROTO_TCP;
    policy.tuple.ip_version                     = IP_VERSION_IPV6;

    ret = netflow_audit_add_policy(&policy);
    if (ret) {
        return ret;
    }

    /* 添加UDP审计策略 -- IPv4 */
    policy.policy_id = ++count;
    policy.direction = NETFLOW_INBOUND;             // 流入
    policy.type = NETFLOW_TUPLE;                    // 五元组
    policy.tuple.src_ip_range.start.ipv4.s_addr = 0x28280d2f; // 主机字节序（小端）40.40.13.47
    policy.tuple.src_ip_range.end.ipv4.s_addr   = 0x28280d33; // 主机字节序（小端）40.40.13.51
    policy.tuple.dst_ip_range.start.ipv4.s_addr = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.dst_ip_range.end.ipv4.s_addr   = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.src_port_range.start           = 0;
    policy.tuple.src_port_range.end             = 0xffff;
    policy.tuple.dst_port_range.start           = 8888;
    policy.tuple.dst_port_range.end             = 8888;
    policy.tuple.protocol                       = IPPROTO_UDP;
    policy.tuple.ip_version                     = IP_VERSION_IPV4;

    ret = netflow_audit_add_policy(&policy);
    if (ret) {
        return ret;
    }

    policy.policy_id = ++count;
    policy.direction = NETFLOW_OUTBOUND;            // 流出
    policy.type = NETFLOW_TUPLE;                    // 五元组
    policy.tuple.src_ip_range.start.ipv4.s_addr = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.src_ip_range.end.ipv4.s_addr   = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.dst_ip_range.start.ipv4.s_addr = 0x28280d2f; // 主机字节序（小端）40.40.13.47
    policy.tuple.dst_ip_range.end.ipv4.s_addr   = 0x28280d33; // 主机字节序（小端）40.40.13.51
    policy.tuple.src_port_range.start           = 0;
    policy.tuple.src_port_range.end             = 0xffff;
    policy.tuple.dst_port_range.start           = 8888;
    policy.tuple.dst_port_range.end             = 8888;
    policy.tuple.protocol                       = IPPROTO_UDP;
    policy.tuple.ip_version                     = IP_VERSION_IPV4;

    ret = netflow_audit_add_policy(&policy);
    if (ret) {
        return ret;
    }

#endif

    /* 添加进程审计策略 */
    policy.policy_id = ++count;
    policy.type = NETFLOW_PROCESS;                    // 进程
    strncpy(policy.process_name, "client"
    , sizeof (policy.process_name) - 1);
    ret = netflow_audit_add_policy(&policy);
    if (ret) {
        return ret;
    }

    policy.policy_id = ++count;
    policy.type = NETFLOW_PROCESS;                    // 进程
    strncpy(policy.process_name, "udp_recv"
    , sizeof (policy.process_name) - 1);
    ret = netflow_audit_add_policy(&policy);
    if (ret) {
        return ret;
    }

    policy.policy_id = ++count;
    policy.type = NETFLOW_PROCESS;                    // 进程
    strncpy(policy.process_name, "server"
    , sizeof (policy.process_name) - 1);
    ret = netflow_audit_add_policy(&policy);
    if (ret) {
        return ret;
    }

    policy.policy_id = ++count;
    policy.type = NETFLOW_PROCESS;                    // 进程
    strncpy(policy.process_name, "firefox"
    , sizeof (policy.process_name) - 1);
    ret = netflow_audit_add_policy(&policy);
    if (ret) {
        return ret;
    }


    return ret;
}

int test_netflow_drop_add_policy(void)
{
    static int count = 0;
    struct netflow_audit_policy policy;
    int ret = 0;

    /* 添加UDP审计策略 -- IPv4 */
    policy.policy_id = ++count;
    policy.direction = NETFLOW_INBOUND;             // 流入
    policy.type = NETFLOW_TUPLE;                    // 五元组
    policy.tuple.src_ip_range.start.ipv4.s_addr = 0x28280d2f; // 主机字节序（小端）40.40.13.47
    policy.tuple.src_ip_range.end.ipv4.s_addr   = 0x28280d33; // 主机字节序（小端）40.40.13.51
    policy.tuple.dst_ip_range.start.ipv4.s_addr = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.dst_ip_range.end.ipv4.s_addr   = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.src_port_range.start           = 0;
    policy.tuple.src_port_range.end             = 0xffff;
    policy.tuple.dst_port_range.start           = 8888;
    policy.tuple.dst_port_range.end             = 8888;
    policy.tuple.protocol                       = IPPROTO_UDP;
    policy.tuple.ip_version                     = IP_VERSION_IPV4;

    ret = netflow_drop_add_policy(&policy);
    if (ret) {
        return ret;
    }

    policy.policy_id = ++count;
    policy.direction = NETFLOW_OUTBOUND;            // 流出
    policy.type = NETFLOW_TUPLE;                    // 五元组
    policy.tuple.src_ip_range.start.ipv4.s_addr = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.src_ip_range.end.ipv4.s_addr   = 0x28280d35; // 主机字节序（小端）40.40.13.53
    policy.tuple.dst_ip_range.start.ipv4.s_addr = 0x28280d2f; // 主机字节序（小端）40.40.13.47
    policy.tuple.dst_ip_range.end.ipv4.s_addr   = 0x28280d33; // 主机字节序（小端）40.40.13.51
    policy.tuple.src_port_range.start           = 0;
    policy.tuple.src_port_range.end             = 0xffff;
    policy.tuple.dst_port_range.start           = 8888;
    policy.tuple.dst_port_range.end             = 8888;
    policy.tuple.protocol                       = IPPROTO_UDP;
    policy.tuple.ip_version                     = IP_VERSION_IPV4;

    ret = netflow_drop_add_policy(&policy);
    if (ret) {
        return ret;
    }

    return ret;
}

int test_netflow_enable(int *audit_flag)
{
    *audit_flag = 1;

    return 0;
}
