#include <linux/slab.h>
#include <project.h>
#include "usb_sw.h"
#include "usb_ctrl.h"

// interface对应子类的开关，默认功能关闭
static struct type_switch intf_type_switch[] = {
    {INTF_SERIAL, 0},      {INTF_PARALLEL, 0},
    {INTF_1394, 0},        {INTF_PCMCIA, 0},
    {INTF_TYPEC, 0},       {INTF_HDMI, 0},
    {INTF_DISPLAYPORT, 0}, {-1, 0}};

// device对应子类的开关，默认功能开启
static struct type_switch device_type_switch[] = {{DEVICE_BLUETOOTH, 0},
                                                  {DEVICE_INFRARED, 0},
                                                  {DEVICE_MODEM, 0},
                                                  {DEVICE_STORAGE_DRIVE, 0},
                                                  {DEVICE_SCSI_RAID, 0},
                                                  {DEVICE_WINCE, 0},
                                                  {DEVICE_FLOPPY_DRIVE, 0},
                                                  {DEVICE_IMAGE, 0},
                                                  {DEVICE_MULTI_SERIAL_CARD, 0},
                                                  {DEVICE_ETHERNET, 0},
                                                  {DEVICE_WIRELESS, 0},
                                                  {DEVICE_PRINTER, 0},
                                                  {DEVICE_PRINTER_LOCAL, 0},
                                                  {DEVICE_CDROM, 0},
                                                  {DEVICE_VOLUMESHADOW, 0},
                                                  {DEVICE_TAPE, 0},
                                                  {-1, 0}};

// usb对应子类的开关，默认功能开启
static struct type_switch usb_type_switch[] = {
    {USB_CLASS_PER_INTERFACE, 0},
    {USB_CLASS_AUDIO, 0},
    {USB_CLASS_COMM, 0},
    {USB_CLASS_HID, 0},
    {USB_CLASS_PHYSICAL, 0},
    {USB_CLASS_STILL_IMAGE, 0},
    {USB_CLASS_PRINTER, 0},
    {USB_CLASS_MASS_STORAGE, 0},
    {USB_CLASS_HUB, 0},
    {USB_CLASS_CDC_DATA, 0},
    {USB_CLASS_CSCID, 0},
    {USB_CLASS_CONTENT_SEC, 0},
    {USB_CLASS_VIDEO, 0},
    {USB_CLASS_WIRELESS_CONTROLLER, 0},
    {USB_CLASS_PERSONAL_HEALTHCARE, 0},
    {USB_CLASS_AUDIO_VIDEO, 0},
    {USB_CLASS_BILLBOARD, 0},
    {USB_CLASS_USB_TYPE_C_BRIDGE, 0},
    {USB_CLASS_MISC, 0},
    {USB_CLASS_APP_SPEC, 0},
    {USB_CLASS_VENDOR_SPEC, 0},
    {USB_CLASS_VENDOR_SPEC, 0},
    {-1, 0},
};

int type_switch_get(struct dev_switch *switch_data)
{
    struct type_switch *arr;
    int ix;

    if (!switch_data)
        return -EFAULT;

    switch (switch_data->type) {
        case DEV_TYPE_USB:
            arr = usb_type_switch;
            break;
        case DEV_TYPE_DEVICE:
            arr = device_type_switch;
            break;
        case DEV_TYPE_INTF:
            arr = intf_type_switch;
            break;
        default:
            return -EFAULT;
    }

    for (ix = 0; arr[ix].desc != -1; ix++) {
		if (arr[ix].desc == switch_data->sub_type)
            break;
	}

    switch_data->mode = arr[ix].mode;

    return arr[ix].mode;
}

int type_switch_set(struct dev_switch *switch_data)
{
    struct type_switch *arr;
    int ix;
    int flag = 0;

    if (!switch_data)
        return -EFAULT;

    switch (switch_data->type) {
        case DEV_TYPE_USB:
            arr = usb_type_switch;
            break;
        case DEV_TYPE_DEVICE:
            arr = device_type_switch;
            break;
        case DEV_TYPE_INTF:
            arr = intf_type_switch;
            break;
        default:
            return -EFAULT;
    }

    for (ix = 0; arr[ix].desc != -1; ix++) {
        if (arr[ix].desc == switch_data->sub_type) {
			tag_pr_debug("set sw mode[%d]\n", switch_data->mode);
            arr[ix].mode = switch_data->mode;
            flag = 1;
        }
	}

    if (flag)
        return 0;
    else
        return -EFAULT;
}

