#include <linux/slab.h>
#include <project.h>
#include "usb_wlist.h"

// usb白名单链表
static LIST_HEAD(usb_wlist);
// usb白名单链表读写所
DEFINE_RWLOCK(usb_wlist_rwlock);

int usb_wlist_add(struct usb_attr *attr)
{
    struct usb_wlist_node *info, *new;
    struct list_head *pos = NULL;

    tag_pr_debug("[usb_ctrl] add vid=%04x pid=%04x clsid=%04x to white list\n", 
        attr->vid, attr->pid, attr->clsid);

    new = kzalloc(sizeof(struct usb_wlist_node), GFP_KERNEL);
    if (!new) 
        return -ENOMEM;

    new->vid = attr->vid;
    new->pid = attr->pid;
    new->clsid = attr->clsid;

    write_lock(&usb_wlist_rwlock);
    list_for_each(pos, &usb_wlist) {
        info = list_entry(pos, struct usb_wlist_node, list);
        if (info->vid == attr->vid && info->pid == attr->pid &&
            info->clsid == attr->clsid) {
            kfree(new);
            write_unlock(&usb_wlist_rwlock);
            return 0;
        }
    }
    list_add(&new->list, &usb_wlist);
    write_unlock(&usb_wlist_rwlock);

    return 0;
}

int usb_wlist_del(struct usb_attr *attr)
{
    struct usb_wlist_node *item = NULL, *tmp = NULL;

    tag_pr_debug("[usb_ctrl] remove vid=%04x pid=%04x clsid=%04x to white list\n", 
        attr->vid, attr->pid, attr->clsid);

    write_lock(&usb_wlist_rwlock);
    list_for_each_entry_safe (item, tmp, &usb_wlist, list) {
        if (attr->vid == item->vid && attr->pid == item->pid &&
            attr->clsid == item->clsid) {
            list_del(&item->list);
            kfree(item);
            break;
        }
    }
    write_unlock(&usb_wlist_rwlock);

    return 0;
}

int usb_wlist_find(struct usb_attr *attr)
{
    struct list_head *pos = NULL;
    struct usb_wlist_node *info;
    int ret = 0;

    read_lock(&usb_wlist_rwlock);
    list_for_each(pos, &usb_wlist) {
        info = list_entry(pos, struct usb_wlist_node, list);
        if (info->vid == attr->vid && info->pid == attr->pid &&
            info->clsid == attr->clsid) {
            ret = 1;
            goto output;
        }
    }

output:
    read_unlock(&usb_wlist_rwlock);
    return ret;
}

void usb_wlist_free()
{
    struct usb_wlist_node *item = NULL, *tmp = NULL;

    write_lock(&usb_wlist_rwlock);
    list_for_each_entry_safe (item, tmp, &usb_wlist, list) {
        list_del(&item->list);
        kfree(item);
    }
    write_unlock(&usb_wlist_rwlock);
}

