#include <linux/module.h>
#include <linux/init.h>
#include <linux/moduleparam.h>
#include <linux/kthread.h>

#include <project.h>
#include "common/sym_hook/lsm_if.h"
#include "common/config/sec_ply_if.h"
#include "nfs_security_ops.h"
#include "common/general/include/scan_machine/scan_machine.h"
#include "nfs_syscall_hook.h"	
#include "common/audit/audit_queue.h"
#include "ipc/ipc_ioctl.h"
#include "common/kernel_check_q.h"

extern void set_sw_handle(void);

extern struct security_hook_list nfssec_prot_hooks[];
extern int nfssec_prot_hook_size;

audit_q_t g_audit_prot;

#ifdef NFSSEC_KMOD
// 模块参数： /sys/module/module_name/parameters/param_name
char log_level_sys[10];
char * log_level = DEFAULT_LOG_LEVEL_S; // 日志输出级别
module_param(log_level, charp, 0644);
MODULE_PARM_DESC(log_level, "level of log output!");
#else 
extern char log_level_sys[10];
extern char * log_level;
#endif

static int __nfssec_prot_init(void)
{
	int retval = -1;
	int ret = 0;
 
	// 初始化日志输出级别
	get_log_level(log_level, log_level_sys);
	// printk("log_level_sys:[%s]\n", log_level_sys);

	// kernel_check_q_init();

	// fs_check_ok();

	// todo: 进程保护需要这个，有优化余地
    scan_scn_am_module_init();

	set_sw_handle();

	// todo：临时注释开关
	sec_ply_init(SEC_PLY_PROTECT_KMOD);
	sec_ply_load(SEC_PLY_PROTECT_KMOD);
	// sec_sw_load(SEC_SW_PROT_KMOD);

	sec_ply_init(SEC_PLY_PROTECT_PROC);
	sec_ply_load(SEC_PLY_PROTECT_PROC);
	// sec_sw_load(SEC_SW_PROT_PROC);

	sec_ply_init(SEC_PLY_PROTECT_FILE);
	sec_ply_load(SEC_PLY_PROTECT_FILE);
	// sec_sw_load(SEC_SW_PROT_FILE);

	sec_ply_init(SEC_PLY_PROTECT_DIR);
	sec_ply_load(SEC_PLY_PROTECT_DIR);
	// sec_sw_load(SEC_SW_PROT_DIR);

	sec_ply_init(SEC_PLY_PROTECT_SELF);
	sec_ply_load(SEC_PLY_PROTECT_SELF);
	// sec_sw_load(SEC_SW_PROT_SELF);
	
	nfs_syscall_hook_init();

	nfs_security_ops_init();

#if LINUX_VERSION_CODE < KERNEL_VERSION(6, 6, 0)
	ret = lsm_if_init(nfssec_prot_hooks, nfssec_prot_hook_size, "prot");
	if (ret) {
		tag_pr_err("init for nfssecprot lsm error!\n");
		goto DONE;
	}
#endif

	audit_q_init(&g_audit_prot, 200);

    // ioctl_check_ok();

    ret = ipc_init_prot();
    if (ret) {
        tag_pr_err("prot ipc_init error.\n");
        goto DONE;
    }
	
	tag_pr_info("After nfssecprot module init...\n");

    retval = 0;
DONE:
    return retval;
}

static void __exit nfssec_prot_exit(void)
{
	// kernel_check_q_exit();

    ipc_exit_prot();

	scan_scn_am_module_destroy();

	nfs_syscall_hook_exit();

	lsm_if_exit(nfssec_prot_hooks, nfssec_prot_hook_size);

	audit_q_clear(&g_audit_prot);

	tag_pr_info("After nfssecprot module exit...\n");
}

// 编译成模块方式，或者编译进内核（直接调用）
#ifdef NFSSEC_KMOD
module_init(__nfssec_prot_init);
module_exit(nfssec_prot_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("nfssec corp");
MODULE_DESCRIPTION("nfssec system protect module");
MODULE_VERSION("1.0.1");
MODULE_ALIAS("nfssec prot ");
#else
// static int prot_thread(void *data)
// {
//    __nfssec_prot_init();

//     return 0;
// }

// int __init nfssec_prot_init(void)
// {
// 	struct task_struct * prot_task;	// todo： 线程的销毁？

//     prot_task = kthread_run(prot_thread, NULL, "prot_task");
//     if (IS_ERR(prot_task)) {
//         printk(KERN_ERR "无法创建kthread\n");
//         return PTR_ERR(prot_task);
//     }

// 	return 0;
// }

static struct delayed_work __delay_work;
static struct workqueue_struct *wq;
#define MY_FILE "/opt/nfssec/script/loadService.sh"

static void __nfssec_prot_init_delay(struct work_struct *work_req)
{
	struct file *filep = NULL;

	filep = filp_open(MY_FILE, O_RDWR | O_APPEND | O_CREAT, 0644);
	if (IS_ERR_OR_NULL(filep)) {
		mod_delayed_work(wq, &__delay_work, 100);
	} else {
		filp_close(filep, NULL);
		__nfssec_prot_init();

		// cancel_delayed_work(&__delay_work); 
		// flush_workqueue(wq);
		// destroy_workqueue(wq);
		// cancel_work = true;
		// printk(KERN_INFO "__nfssec_prot_init_delay 4\n");
	}
}
int __init nfssec_prot_init(void)
{
	// if (!security_module_enable("prot")) {
	// 	printk(KERN_DEBUG "==== prot not eable\n");

	// 	return 0;
	// }

	INIT_DELAYED_WORK(&__delay_work, __nfssec_prot_init_delay);
	wq = alloc_ordered_workqueue("nfssec_wq_prot", WQ_MEM_RECLAIM);
	queue_delayed_work(wq, &__delay_work, 100);

	return 0;
}

// security_initcall(nfssec_prot_init);

#if LINUX_VERSION_CODE >= KERNEL_VERSION(6, 6, 0)
static int __init nfssec_prot_init_hook(void)
{
    security_add_hooks(nfssec_prot_hooks, nfssec_prot_hook_size, "nfssecprot");
    // printk(KERN_INFO "=== nfssecprot LSM initialized\n");
    return 0;
}

DEFINE_LSM(nfssecprot) = {
	.name = "nfssecprot",
	.init = nfssec_prot_init_hook,
};
#endif


#endif
