#include <project.h>
#include <linux/binfmts.h>
#include "protect_check.h"

#include "common/file_access_table.h"
#include "common/proc_check_table.h"
#include "common/sec_krnl_misc.h" 
#include "common/sec_path_trie.h"
#include "common/vfs_mount.h"
#include "common/ac_result.h"
#include "common/ac_table.h"
#include "common/nfs_table.h"
#include "common/fullpath.h" 
#include "common/config/sec_ply_if.h"
#include "common/proc_check_table.h"
#include "common/audit/sec_audit.h"
#include "common/nfs_task_tree.h"

int bprm_check(struct linux_binprm *bprm)
{
    sec_ply_if_t * ply_file = NULL;
    sec_ply_if_t * ply_dir = NULL;
    sec_ply_if_t * ply_proc = NULL;
    int check_file = 0;
    int check_dir = 0;
    int check_proc = 0;

    ply_file = get_ply_if(SEC_PLY_PROTECT_FILE);
    ply_dir = get_ply_if(SEC_PLY_PROTECT_DIR);
    ply_proc = get_ply_if(SEC_PLY_PROTECT_PROC);

    if (ply_file && is_sw_enable(&ply_file->sw) && ! is_sec_ply_empty(ply_file))
        check_file = 1;

    if (ply_dir && is_sw_enable(&ply_dir->sw) && ! is_sec_ply_empty(ply_dir))
        check_dir = 1;

    if (ply_proc && is_sw_enable(&ply_proc->sw) && ! is_sec_ply_empty(ply_proc))
        check_proc = 1;

    if (!check_file && !check_dir && !check_proc)
        goto DONE;

    update_pid_context(bprm->file, current);

DONE:
    return 0;    
}

static int protect_trie_check(path_trie_pattern_t *pattern, char *pattern_pos,
                               int offset, int left, void *search_context)
{
    int retval = PATH_TRIE_CHECK_CONTINUE;
    ac_ctx_t *ctx = (ac_ctx_t *)search_context;
    ac_trie_ext_t *ext;

    protect_ext_t * p_ext = NULL;

    ext = __PATH_TRIE_GET_EXT(ac_trie_ext_t, pattern);

    if (left != 0) {
        if (pattern_pos[pattern->nPathCount] != '/') {
            goto DONE;
        }
    }

    if (!ctx->bModifying)
        goto DONE;

    tag_pr_debug("protect_trie_check[%s]\n", ctx->path);

    p_ext = (protect_ext_t*)(ext->ext_data);

    if (p_ext->type == PROTECT_TYPE_DIR) {
        ctx->bValid = TRUE;
        ctx->acitem.rule_id = ext->id;
        ctx->acitem.ope = AUDIT_OPE_FILE_WRITE; // todo
        ctx->acitem.is_ban = TRUE;

        ctx->acitem.rule_type = __POLICY_TYPE_FOLDER_PROTECTION;
        ctx->acitem.sub_type = AUDIT_SUBTYPE_DIR_PROTECT;

        retval = PATH_TRIE_CHECK_BREAK;

    } else if (p_ext->type == PROTECT_TYPE_FILE) {
        ctx->bValid = TRUE;
        ctx->acitem.rule_id = ext->id;
        ctx->acitem.ope = AUDIT_OPE_FILE_WRITE; // todo
        ctx->acitem.is_ban = TRUE;

        ctx->acitem.rule_type = __POLICY_TYPE_FILE;
        ctx->acitem.sub_type = AUDIT_SUBTYPE_FILE_PROTECT;

        retval = PATH_TRIE_CHECK_BREAK;
    }

DONE:
    return retval;
}

// todo: 这里的modify，read，其实可以根据sub_type判断，但是这样又比较省事，后续调整吧
// todo: 这里的sub_type 还是 换成 ope 比较合适
BOOLEAN file_acl_check(struct dentry *dentry, struct vfsmount *mnt, BOOLEAN b_modify,
                               BOOLEAN b_read, unsigned int sub_type)
{
    BOOLEAN retval = 0;
    int ret = 0;

    sec_ply_if_t * ply_file = NULL;
    sec_ply_if_t * ply_dir = NULL;
    sec_ply_if_t * ply_self = NULL;
    int check_file = 0;
    int check_dir = 0;
    int check_self = 0;

    PID_CONTEXT *pid_ctx = NULL;
    char *fullpath = NULL;

    ac_result_t ac_result;
    BOOLEAN is_audit = 0;

    pid_t root_pid = 0;

    memset(&ac_result, 0, sizeof(ac_result));

    ply_file = get_ply_if(SEC_PLY_PROTECT_FILE);
    ply_dir = get_ply_if(SEC_PLY_PROTECT_DIR);
    ply_self = get_ply_if(SEC_PLY_PROTECT_SELF);

    if (ply_file && is_sw_enable(&ply_file->sw) && ! is_sec_ply_empty(ply_file))
        check_file = 1;

    if (ply_dir && is_sw_enable(&ply_dir->sw) && ! is_sec_ply_empty(ply_dir))
        check_dir = 1;

    if (ply_self && is_sw_enable(&ply_self->sw) && ! is_sec_ply_empty(ply_self))
        check_self = 1;

    if (!check_file && !check_dir && !check_self)
        goto DONE;

    if (!dentry || !mnt || !is_mount_need_check(mnt))
        goto DONE;

    pid_ctx = sec_gen_lookup(__SEC_PID_GET(current));
    if (!pid_ctx) {
        pid_ctx = attachProcess(__SEC_PID_GET(current));
        if (!pid_ctx) {
            goto DONE;
        }
    }

    fullpath = get_fullpath_by_dentry_mnt(dentry, mnt);
    if (!fullpath)
        goto DONE;

    if (check_file) {
        mutex_lock(&ply_file->sec_ply_lock);
        ret = ac_table_check(ply_file->ply_head, fullpath, NULL, b_modify, b_read, pid_ctx,
                                sub_type, &ac_result, protect_trie_check);
        mutex_unlock(&ply_file->sec_ply_lock);

        if (ret == TRUE && ac_result.is_ban && ply_file->sw.b_audit)
            is_audit = 1;

        if (ret == TRUE && ac_result.is_ban && ply_file->sw.b_ctrl) {
                    tag_pr_debug("\n");

            retval = -1;
            goto DONE;
        }
    }

    if (check_dir) {
        // todo: 这里可以有一层是否递归的判断
        // 递归：检查路径，如果策略为递归，则存在公共前缀，算命中
        // 不递归：检查路径，如果策略为不递归，则路径完全匹配，算命中
        mutex_lock(&ply_dir->sec_ply_lock);
        // tag_pr_debug("ply_dir cnt[%d], fullpath[%s]\n", ply_dir->ply_cnt, fullpath);
        ret = ac_table_check(ply_dir->ply_head, fullpath, NULL, b_modify, b_read, pid_ctx,
                                sub_type, &ac_result, protect_trie_check);
        mutex_unlock(&ply_dir->sec_ply_lock);
        if (ret == TRUE && ac_result.is_ban && ply_dir->sw.b_audit)
            is_audit = 1;

        if (ret == TRUE && ac_result.is_ban && ply_dir->sw.b_ctrl) {
            retval = -1;
            goto DONE;
        }
    }    

    if (check_self) {
        mutex_lock(&ply_self->sec_ply_lock);
        ret = ac_table_check(ply_self->ply_head, fullpath, NULL, b_modify, b_read, pid_ctx,
                                sub_type, &ac_result, protect_trie_check);
        mutex_unlock(&ply_self->sec_ply_lock);

        if (ret == TRUE && ac_result.is_ban && ply_self->sw.b_audit)
            is_audit = 1;

        if (ret == TRUE && ac_result.is_ban && ply_self->sw.b_ctrl) {
            retval = -1;
            goto DONE;
        }
    }

DONE:
    if (retval) {
        // todo: 可在此基础上，实现例外放行！！！！！！！！
        ret = is_child_proc_by_name("nfssecd", current, &root_pid);
        if (ret) {
            tag_pr_debug("self proc pass\n");
            retval = 0;
            is_audit = 0;
        }
    }

    if (is_audit) {
        tag_pr_debug("==================protect ban, ac_result.sub_type[%d]\n", ac_result.sub_type);
        AUDIT_SYS_PROTECT(pid_ctx->appFullName, pid_ctx->processID, 
            ac_result.ope, ac_result.sub_type, retval ? AUDIT_RESULT_FAILED : AUDIT_RESULT_SUCCESS, fullpath);
    }

    if (pid_ctx)
        comm_object_release(pid_ctx);

    return retval;
}

int task_kill_check(pid_t pid, int sig)
{
    int retval = 0;

    sec_ply_if_t * ply_proc = NULL;
    sec_ply_if_t * ply_self = NULL;

    int check_proc = 0;
    int check_self = 0;

    PROC_CHECK_RESULT_T result;
    BOOLEAN is_audit = 0;

    PID_CONTEXT *pid_ctx = NULL;
#ifdef __USE_PID_STRUCT_AS_KEY
    struct pid *pidObj = NULL;
#endif

    ply_proc = get_ply_if(SEC_PLY_PROTECT_PROC);
    ply_self = get_ply_if(SEC_PLY_PROTECT_SELF);

    if (ply_proc && is_sw_enable(&ply_proc->sw) && ! is_sec_ply_empty(ply_proc))
        check_proc = 1;

    if (ply_self && is_sw_enable(&ply_self->sw) && ! is_sec_ply_empty(ply_self))
        check_self = 1;

    if (!check_proc && !check_self)
        goto DONE;

#ifdef __USE_PID_STRUCT_AS_KEY
    pidObj = find_get_pid(pid);
    if (!pidObj)
        goto DONE;
    pid_ctx = sec_gen_lookup(pidObj);
#else
    pid_ctx = sec_gen_lookup(pid);
#endif
    if (!pid_ctx) {
        pid_ctx = attachProcess(pid);
        if (!pid_ctx) {
            goto DONE;
        }
    }

    if (0 == pid_ctx->appFullName[0]) {
        goto DONE;
    }

    memset(&result, 0, sizeof(result));
    tag_pr_debug("pid_ctx->appFullName[%s]\n", pid_ctx->appFullName);

    if (check_proc) {
        mutex_lock(&ply_proc->sec_ply_lock);
        proc_check_do(ply_proc->ply_head, pid_ctx->appFullName, &result);
        mutex_unlock(&ply_proc->sec_ply_lock);

        tag_pr_debug("\n");
        if (result.is_ban && ply_proc->sw.b_audit)
            is_audit = 1;

        if (result.is_ban && ply_proc->sw.b_ctrl) {
            retval = -1;
            goto DONE;
        } 
    }

    // todo: 对自保护程序的检查
    // memset(&result, 0, sizeof(result));

    // if (check_self) {
    //     mutex_lock(&ply_self->sec_ply_lock);
    //     proc_check_do(ply_self->ply_head, pid_ctx->appFullName, &result);
    //     mutex_unlock(&ply_self->sec_ply_lock);

    //     tag_pr_debug("\n");
    //     if (result.is_ban && ply_self->sw.b_audit)
    //         is_audit = 1;

    //     if (result.is_ban && ply_self->sw.b_ctrl) {
    //         retval = -1;
    //         goto DONE;
    //     } 
    // }


    retval = 0;
DONE:
    if (is_audit) {
        // todo: 这里简单使用current,如果不够用,再处理
        AUDIT_SYS_PROTECT(current->comm, current->pid, 
            AUDIT_OPE_FILE_KILL, AUDIT_SUBTYPE_PROC_PROTECT , retval ? AUDIT_RESULT_FAILED : AUDIT_RESULT_SUCCESS, pid_ctx->appFullName);
    }

    return retval;
}

