#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include "common/file_ops/file_ops.h"

#include <linux/slab.h>
#include <linux/string.h>
#include <linux/types.h>

#include "common/hash/hash_if.h"
#include "common/tag_print.h"

#include <crypto/hash_info.h>
#include <linux/uaccess.h>
#include <linux/slab.h>
#include <linux/parser.h>
#include <linux/string.h>
#include <linux/err.h>
#include <keys/user-type.h>
#include <keys/trusted-type.h>
#include <linux/key-type.h>
#include <linux/rcupdate.h>
#include <linux/crypto.h>
#include <crypto/hash.h>
#include <crypto/sha.h>
#include <linux/capability.h>
#include <linux/tpm.h>
#include <linux/tpm_command.h>

#define TPM2_SM3_256_DIGEST_SIZE 32

#define DIGEST_STRUCT 1

#ifdef CONFIG_TC_CHECK_M
char log_level_sys[10];
char * log_level = DEFAULT_LOG_LEVEL_S;
module_param(log_level, charp, 0644);
MODULE_PARM_DESC(log_level, "level of log output!");
#else
extern char log_level_sys[10];
extern char * log_level;
#endif

unsigned char hash[SM3_DIGEST_SIZE];
unsigned char hash_result[SM3_DIGEST_SIZE*2+1];
static int check_number = 100;

int str_to_hex(char *out_string,int length,char *in_string)
{
        int index;
        char *fmt = "%02X";

        for(index = 0;index < length;index++)
        {
                snprintf((char *)&out_string[index<<1],3,fmt,(unsigned char)in_string[index]);
        }
        return (length<<1);
}

void update_str(char *dst, int str_len, char *src, char *des)
{
        str_to_hex(dst, str_len, src);
        *((char *)dst+(int)str_len*2) = '\0';
}

static int dump_hash(char *string)
{
	update_str(hash_result, SM3_DIGEST_SIZE, string, "get random:");
	return 0;
}
static int single_hashtest(char *str)
{
	int ret = 0;
        ret = cacl_buf_hash(str, strlen(str), hash);
        if (ret) {
            pr_err("Failed to compute hash for '%s': %d\n", str, ret);
        }

	dump_hash(hash);
	return 0;
}

#if DIGEST_STRUCT
static int extend_pcr(char *sm3_hash, int pcr)
{
	struct tpm_chip *chip = NULL;
	char *hash_value;
	int extend_result = -1;
	struct tpm_digest digest_struct[1] = {
		{.alg_id = TPM_ALG_SM3_256},
	};

	chip = tpm_default_chip();
	if (IS_ERR_OR_NULL(chip))
		return -ENODEV;
	hash_value = kmalloc(strlen(sm3_hash)+1, GFP_KERNEL);
	memset(hash_value, '\0', strlen(sm3_hash));
	strncpy(hash_value, sm3_hash, TPM2_SM3_256_DIGEST_SIZE);
	strncpy(((digest_struct[0]).digest), sm3_hash, TPM2_SM3_256_DIGEST_SIZE);
	extend_result = tpm_pcr_extend(chip, pcr, digest_struct);
	if (extend_result == 0)
		return 0;
	else
		return extend_result;
}
#else
static int extend_pcr(char *sm3_hash, int pcr)
{
	struct tpm_chip *chip = NULL;
	char *hash_value;
	int extend_result = -1;

	chip = tpm_default_chip();
	if (IS_ERR_OR_NULL(chip))
		return -ENODEV;
	hash_value = kmalloc(strlen(sm3_hash)+1, GFP_KERNEL);
	memset(hash_value, '\0', strlen(sm3_hash));
	strncpy(hash_value, sm3_hash, TPM_DIGEST_SIZE);
	extend_result = tpm_pcr_extend(chip, pcr, hash_value);
	printk("extend hash(times:%d): %s.\n", check_number, sm3_hash);
	if (extend_result == 0)
		return 0;
	else
		return extend_result;
}
#endif

static int sum_file(char *f_name, int pcr)
{
	struct file *fp = NULL;
	int ret = 0;
	int extend_ret = -1;
	loff_t pos = 0;

	char *file_str = NULL;
	unsigned long file_size = 0;
	fp = file_ops_open(f_name, O_RDONLY, 0);
	if (IS_ERR_OR_NULL(fp))
		goto DONE;

	file_size = fp->f_inode->i_size;

	file_str = kmalloc(file_size+1, GFP_KERNEL);
	if (!file_str)
		goto DONE;
	memset(file_str, '\0', file_size+1);
	ret = kernel_read(fp, file_str, file_size, &pos);

	single_hashtest(file_str);
	extend_ret = extend_pcr(hash_result, pcr);
	if (extend_ret != 0)
		printk(KERN_ERR "nfssec_check: can't extend %s to pcr[%d]! error!\n", f_name, pcr);
DONE:
	if (fp)
	file_ops_close(fp, NULL);

	return ret;
}

static int check_fileexit(char *f_name)
{
	struct file *filp = NULL;
	filp = filp_open(f_name, O_RDONLY, 0);
	if (IS_ERR(filp)) {
		return 1;
	}
	else
		filp_close(filp, NULL);
	return 0;
}

static void nfssec_check_check(char *filename, int pcr)
{
	int file_flag = -1;
	file_flag = check_fileexit(filename);
	if (file_flag != 0) {
		printk("nfssec_check_check: can't open %s.\n", filename);
	}
	else
	{
		printk("check file: %s.", filename);
		sum_file(filename, pcr);
	}
}

static void nfssec_check_checks(void)
{
	//nfssec_check_check("/opt/nfssec/bin/nfsseck.ko", 11);
	nfssec_check_check("/opt/nfssec/bin/nfssecd", 11);
	nfssec_check_check("/usr/lib/systemd/systemd", 11);
	//nfssec_check_check("/opt/nfssec/bin/nfsseck.ko", 23);
	nfssec_check_check("/opt/nfssec/bin/nfssecd", 23);
	nfssec_check_check("/usr/lib/systemd/systemd", 23);
}

static int __init nfssec_check_init(void)
{
	nfssec_check_checks();
	printk("nfssec tc init ok!");
	return 0;
}
static void __exit nfssec_check_exit(void)
{
	printk("nfssec tc exit!");
}

static struct delayed_work __delay_work;
static struct workqueue_struct *wq;

static void __nfssec_tc_init_delay(struct work_struct *work_req)
{
       struct file *filep = NULL;

       if (check_number)
       {
	       check_number--;
	       if (extend_pcr("0000000000000000000000", 23) != 0) {
	       mod_delayed_work(wq, &__delay_work, 1000);
	       } else {
	       extend_pcr("0000000000000000000000", 11);
	       nfssec_check_init();
	       }
	}
       else
       {
	       // mod_delayed_work(wq, &__delay_work, 100000);
	       check_number = 0;
       }

}

int __init nfssec_tc_init(void)
{
       INIT_DELAYED_WORK(&__delay_work, __nfssec_tc_init_delay);
       wq = alloc_ordered_workqueue("nfssec_wq_tc", WQ_MEM_RECLAIM);
       queue_delayed_work(wq, &__delay_work, 1000);

       return 0;
}

device_initcall_sync(nfssec_tc_init);
module_exit(nfssec_check_exit);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("zhanghui1@nfschina.com");
MODULE_DESCRIPTION("save hash to tcm2");

